@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/base/Calculator.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "Calculator.h"
+#pragma implementation "Calculator.h"
 #endif
 
 #include "Calculator.h"
@@ -119,7 +119,9 @@ void CalculatorSwe::init()
 * \brief constants for built-in Ayanamsas
 *
 ******************************************************/
-struct AyaInit {double t0, ayan_t0;};
+struct AyaInit {
+	double t0, ayan_t0;
+};
 static const struct AyaInit Ayanamsa[NB_AYANAMSAS] = {
 	{2415020.0, 360 - 337.53953},   /* 1: Lahiri (Robert Hand) */
 	{2415020.0, 360 - 338.98556},   /* 3: Raman (Robert Hand) */
@@ -210,22 +212,22 @@ double CalculatorSwe::calcNextSolarEvent( const int &type, const double &jd, con
 	if ( ! config->sunrise_refrac ) flag |= SE_BIT_NO_REFRACTION;
 
 	//int event_flag = 0;
-	switch( type )
+	switch ( type )
 	{
-		case SOLAR_EVENT_SUNRISE:
-			flag |= SE_CALC_RISE;
+	case SOLAR_EVENT_SUNRISE:
+		flag |= SE_CALC_RISE;
 		break;
-		case SOLAR_EVENT_SUNSET:
-			flag |= SE_CALC_SET;
+	case SOLAR_EVENT_SUNSET:
+		flag |= SE_CALC_SET;
 		break;
-		case SOLAR_EVENT_NOON:
-			flag |= SE_CALC_ITRANSIT;
+	case SOLAR_EVENT_NOON:
+		flag |= SE_CALC_ITRANSIT;
 		break;
-		case SOLAR_EVENT_MIDNIGHT:
-			flag |= SE_CALC_MTRANSIT;
+	case SOLAR_EVENT_MIDNIGHT:
+		flag |= SE_CALC_MTRANSIT;
 		break;
-		default:
-			assert( false );
+	default:
+		assert( false );
 		break;
 	}
 
@@ -246,7 +248,7 @@ double CalculatorSwe::calcNextSolarEvent( const int &type, const double &jd, con
 bool CalculatorSwe::calcSunRiseSunSet( DataSet *d, double *sunrise, double *sunset, double *noon, double *midnight )
 {
 	char err[256];
-	bool daybirth = true, beforemidnight = true;
+	bool daybirth = true;
 	double r[6];
 	double cusp[13], ascmc[10];
 	double rsmi[3], tret[3];
@@ -265,9 +267,7 @@ bool CalculatorSwe::calcSunRiseSunSet( DataSet *d, double *sunrise, double *suns
 	if ( diff_ascsun > 180 ) daybirth = false;
 
 	double diff_icsun = red_deg( ascmc[1] + 180 - r[0] );  // Sun and IC
-	if ( diff_icsun < 180 ) beforemidnight = false;
 
-	int quadrant = 0;
 	startjdrise = startjdset = startjdnoon = startjdmidnight = jd;
 
 	if ( daybirth )
@@ -275,14 +275,11 @@ bool CalculatorSwe::calcSunRiseSunSet( DataSet *d, double *sunrise, double *suns
 		if ( diff_icsun < 180 )
 		{
 			// forenoon
-			quadrant = 3;
 			startjdrise--;
-			//startjdmidnight--;
 		}
 		else
 		{
 			// afternoon
-			quadrant = 2;
 			startjdrise--;
 			startjdnoon--;
 		}
@@ -292,7 +289,6 @@ bool CalculatorSwe::calcSunRiseSunSet( DataSet *d, double *sunrise, double *suns
 		if ( diff_icsun < 180 )
 		{
 			// morning before sunrise
-			quadrant = 0;
 			startjdrise--;
 			startjdset--;
 			startjdnoon--;
@@ -301,7 +297,6 @@ bool CalculatorSwe::calcSunRiseSunSet( DataSet *d, double *sunrise, double *suns
 		else
 		{
 			// evening after sunset
-			quadrant = 1;
 			startjdrise--;
 			startjdset--;
 			startjdnoon--;
@@ -444,26 +439,26 @@ long CalculatorSwe::calcIFlag( DataSet *d )
 {
 	long iflag = 0;
 	if ( config->useTruePositions ) iflag |= SEFLG_TRUEPOS;
-	switch( config->ephemCoordinateSystem )
+	switch ( config->ephemCoordinateSystem )
 	{
-		case EPHEM_COORDINATES_GEOCENTRIC:
-			// do nothing
+	case EPHEM_COORDINATES_GEOCENTRIC:
+		// do nothing
 		break;
-		case EPHEM_COORDINATES_TOPOCENTRIC:
-			swe_set_topo( d->getLocation()->getLongitude(), d->getLocation()->getLatitude(), 0 );
-			iflag |= SEFLG_TOPOCTR;
+	case EPHEM_COORDINATES_TOPOCENTRIC:
+		swe_set_topo( d->getLocation()->getLongitude(), d->getLocation()->getLatitude(), 0 );
+		iflag |= SEFLG_TOPOCTR;
 		break;
-		case EPHEM_COORDINATES_EQUATORIAL:
-			iflag |= SEFLG_EQUATORIAL;
+	case EPHEM_COORDINATES_EQUATORIAL:
+		iflag |= SEFLG_EQUATORIAL;
 		break;
-		case EPHEM_COORDINATES_BARYCENTRIC:
-			iflag |= SEFLG_BARYCTR;
+	case EPHEM_COORDINATES_BARYCENTRIC:
+		iflag |= SEFLG_BARYCTR;
 		break;
-		case EPHEM_COORDINATES_HELIOCENTRIC:
-			iflag |= SEFLG_HELCTR ;
+	case EPHEM_COORDINATES_HELIOCENTRIC:
+		iflag |= SEFLG_HELCTR ;
 		break;
-		default:
-			assert( false );
+	default:
+		assert( false );
 		break;
 	}
 	return iflag;
@@ -475,7 +470,7 @@ long CalculatorSwe::calcIFlag( DataSet *d )
 **
 ******************************************************/
 void CalculatorSwe::calcPositionSpeed( DataSet *d, const int &planet, double &slen, double &slat, double &sspeed,
-		const bool aya, const bool indian )
+	const bool aya, const bool indian )
 {
 
 	double r[6];
@@ -494,14 +489,19 @@ void CalculatorSwe::calcPositionSpeed( DataSet *d, const int &planet, double &sl
 	else swe_calc( calcJd( d->getJD() ), mapOToS( planet ), calcIFlag( d ) | SEFLG_SPEED, r, err );
 	PRINT_PEDANTIC
 
-	if ( fabs( r[0] ) > .00000001 )
+	if ( r[0] != 0.0 )
 	{
 		slen = red_deg( r[0] - ( aya ? calcAyanamsa( d->getJD(), ( indian ? config->vAyanamsa : config->wAyanamsa )) : 0 ));
 		slat = r[1];
 		sspeed = r[3];
+		if ( sspeed == 0 )
+		{
+			printf( "WARN CalculatorSwe::calcPositionSpeed speed is null planet %d len %.12f\n", planet, slen );
+		}
 	}
 	else
 	{
+		// happens if data files are missing (planetoids, Chiron)
 		slen = 0;
 		slat = 0;
 		sspeed = 0;
@@ -514,7 +514,7 @@ void CalculatorSwe::calcPositionSpeed( DataSet *d, const int &planet, double &sl
 **
 ******************************************************/
 void CalculatorSwe::calcPosition( DataSet *d, const int &planet, double &slen, double &slat,
-		const bool aya, const bool indian )
+                                  const bool aya, const bool indian )
 {
 	double r[6];
 	char err[256];
@@ -532,50 +532,50 @@ void CalculatorSwe::calcPosition( DataSet *d, const int &planet, double &slen, d
 int CalculatorSwe::getHouseSystemCode( const int &hsys )
 {
 	int c = 'P';
-	switch( hsys )
+	switch ( hsys )
 	{
-		case HOUSE_SYSTEM_NONE:
-		case HOUSE_SYSTEM_PLACIDUS:
-			c = 'P';
+	case HOUSE_SYSTEM_NONE:
+	case HOUSE_SYSTEM_PLACIDUS:
+		c = 'P';
 		break;
-		case HOUSE_SYSTEM_KOCH:
-			c = 'K';
+	case HOUSE_SYSTEM_KOCH:
+		c = 'K';
 		break;
-		case HOUSE_SYSTEM_REGIOMONTANUS:
-			c = 'R';
+	case HOUSE_SYSTEM_REGIOMONTANUS:
+		c = 'R';
 		break;
-		case HOUSE_SYSTEM_CAMPANUS:
-			c = 'C';
+	case HOUSE_SYSTEM_CAMPANUS:
+		c = 'C';
 		break;
-		case HOUSE_SYSTEM_PORPHYRY:
-			c = 'O';
+	case HOUSE_SYSTEM_PORPHYRY:
+		c = 'O';
 		break;
-		case HOUSE_SYSTEM_EQUAL:
-			c = 'E';
+	case HOUSE_SYSTEM_EQUAL:
+		c = 'E';
 		break;
-		case HOUSE_SYSTEM_VEHLOW:
-			c = 'V';
+	case HOUSE_SYSTEM_VEHLOW:
+		c = 'V';
 		break;
-		case HOUSE_SYSTEM_AXIAL:
-			c = 'X';
+	case HOUSE_SYSTEM_AXIAL:
+		c = 'X';
 		break;
-		case HOUSE_SYSTEM_AZIMUTAL:
-			c = 'H';
+	case HOUSE_SYSTEM_AZIMUTAL:
+		c = 'H';
 		break;
-		case HOUSE_SYSTEM_POLICH:
-			c = 'T';
+	case HOUSE_SYSTEM_POLICH:
+		c = 'T';
 		break;
-		case HOUSE_SYSTEM_ALCABITUS:
-			c = 'B';
+	case HOUSE_SYSTEM_ALCABITUS:
+		c = 'B';
 		break;
-		case HOUSE_SYSTEM_MORINUS:
-			c = 'M';
+	case HOUSE_SYSTEM_MORINUS:
+		c = 'M';
 		break;
-		case HOUSE_SYSTEM_KRUSINSKI:
-			c = 'U';
+	case HOUSE_SYSTEM_KRUSINSKI:
+		c = 'U';
 		break;
-		default:
-			assert( false );
+	default:
+		assert( false );
 		break;
 	}
 	return c;
@@ -590,7 +590,7 @@ void CalculatorSwe::calcHouses( const double &jd, const double &lon, const doubl
 {
 	double cusp[13], ascmc[10];
 	swe_houses( jd, lat, lon, getHouseSystemCode( hsys ), cusp, ascmc );
-	for( int i = 0; i < 12; i++ ) r[i] = cusp[1+i];
+	for ( int i = 0; i < 12; i++ ) r[i] = cusp[1+i];
 }
 
 /*****************************************************
@@ -709,7 +709,7 @@ double CalculatorSwe::calcAscendantByArmc( const double &armc, const double &eps
 	swe_houses_armc( armc, lat, eps, getHouseSystemCode( hsys ), cusp, ascmc );
 	if ( r != 0 )
 	{
-		for( i = 0; i < 12; i++ ) r[i] = cusp[1+i];
+		for ( i = 0; i < 12; i++ ) r[i] = cusp[1+i];
 	}
 	return ascmc[0];
 }
@@ -800,7 +800,7 @@ double CalculatorSwe::calcMcFromAsc( const double &ascpos, const double &jd, con
 **
 ******************************************************/
 double CalculatorSwe::calcPlanetaryEvent( const DataSet *d, const double &targetlen, const int &planet,
-	const bool &vedic )
+        const bool &vedic )
 {
 	double diff, len, dummy, speed, tlen;
 	DataSet dataset( *d );
@@ -828,8 +828,8 @@ double CalculatorSwe::calcPlanetaryEvent( const DataSet *d, const double &target
 		// bugfix 6.0: found ingress event with speed=0 (Month 1/11, planet Jupiter, western ephemeris)
 		if ( speed == 0 )
 		{
-			printf( "CalculatorSwe::calcPlanetaryEvent WARN: planetary speed is null: planet %d, targetlen %f date %f\n",
-				planet, targetlen, dataset.getJD());
+			printf( "CalculatorSwe::calcPlanetaryEvent WARN: planetary speed is null: planet %d, targetlen %f date %f calculated len %.12f\n",
+			        planet, targetlen, dataset.getJD(), len );
 			speed = .0000000001;
 		}
 
@@ -893,36 +893,92 @@ double CalculatorSwe::calcJd( const double &jd )
 ******************************************************/
 int CalculatorSwe::mapOToS( const int &i )
 {
-	switch( i )
+	switch ( i )
 	{
-		case OSUN      : return SE_SUN;       break;
-		case OMOON     : return SE_MOON;      break;
-		case OMERCURY  : return SE_MERCURY;   break;
-		case OVENUS    : return SE_VENUS;     break;
-		case OMARS     : return SE_MARS;      break;
-		case OJUPITER  : return SE_JUPITER;   break;
-		case OSATURN   : return SE_SATURN;    break;
-		case OURANUS   : return SE_URANUS;    break;
-		case ONEPTUNE  : return SE_NEPTUNE;   break;
-		case OPLUTO    : return SE_PLUTO;     break;
-		case OMEANNODE : return SE_MEAN_NODE; break;
-		case OTRUENODE : return SE_TRUE_NODE; break;
-		case OCUPIDO   : return SE_CUPIDO;    break;
-		case OHADES    : return SE_HADES;     break;
-		case OZEUS     : return SE_ZEUS;      break;
-		case OKRONOS   : return SE_KRONOS;    break;
-		case OAPOLLON  : return SE_APOLLON;   break;
-		case OADMETOS  : return SE_ADMETOS;   break;
-		case OVULKANUS : return SE_VULKANUS;  break;
-		case OPOSEIDON : return SE_POSEIDON;  break;
-		case OCHIRON   : return SE_CHIRON;    break;
-		case OPHOLUS   : return SE_PHOLUS;    break;
-		case OCERES    : return SE_CERES;     break;
-		case OPALLAS   : return SE_PALLAS;    break;
-		case OJUNO     : return SE_JUNO;      break;
-		case OVESTA    : return SE_VESTA;     break;
-		case OLILITH   : return SE_MEAN_APOG; break;
-		default        : assert( 0 );         break;
+	case OSUN      :
+		return SE_SUN;
+		break;
+	case OMOON     :
+		return SE_MOON;
+		break;
+	case OMERCURY  :
+		return SE_MERCURY;
+		break;
+	case OVENUS    :
+		return SE_VENUS;
+		break;
+	case OMARS     :
+		return SE_MARS;
+		break;
+	case OJUPITER  :
+		return SE_JUPITER;
+		break;
+	case OSATURN   :
+		return SE_SATURN;
+		break;
+	case OURANUS   :
+		return SE_URANUS;
+		break;
+	case ONEPTUNE  :
+		return SE_NEPTUNE;
+		break;
+	case OPLUTO    :
+		return SE_PLUTO;
+		break;
+	case OMEANNODE :
+		return SE_MEAN_NODE;
+		break;
+	case OTRUENODE :
+		return SE_TRUE_NODE;
+		break;
+	case OCUPIDO   :
+		return SE_CUPIDO;
+		break;
+	case OHADES    :
+		return SE_HADES;
+		break;
+	case OZEUS     :
+		return SE_ZEUS;
+		break;
+	case OKRONOS   :
+		return SE_KRONOS;
+		break;
+	case OAPOLLON  :
+		return SE_APOLLON;
+		break;
+	case OADMETOS  :
+		return SE_ADMETOS;
+		break;
+	case OVULKANUS :
+		return SE_VULKANUS;
+		break;
+	case OPOSEIDON :
+		return SE_POSEIDON;
+		break;
+	case OCHIRON   :
+		return SE_CHIRON;
+		break;
+	case OPHOLUS   :
+		return SE_PHOLUS;
+		break;
+	case OCERES    :
+		return SE_CERES;
+		break;
+	case OPALLAS   :
+		return SE_PALLAS;
+		break;
+	case OJUNO     :
+		return SE_JUNO;
+		break;
+	case OVESTA    :
+		return SE_VESTA;
+		break;
+	case OLILITH   :
+		return SE_MEAN_APOG;
+		break;
+	default        :
+		assert( 0 );
+		break;
 	}
 	return -1;
 }
