@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/base/Conf.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,15 +19,15 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "Conf.h"
+#pragma implementation "Conf.h"
 #endif
 
 #include "Conf.h"
 
-#include "ColorConfig.h" 
-#include "FileConfig.h" 
-#include "FontConfig.h" 
-#include "mathbase.h" 
+#include "ColorConfig.h"
+#include "FileConfig.h"
+#include "FontConfig.h"
+#include "mathbase.h"
 
 #include <wx/defs.h>
 #include <wx/fileconf.h>
@@ -94,6 +94,27 @@ double Config::readDoubleEntry( wxString path, const double def )
 #define DO_READ( a, b, c ) if ( ! cfg->Read( a, b, c )) { printf( "Warn: config parameter not found: %s\n", str2char( a )); }
 //#define DO_READ( a, b, c ) cfg->Read( a, b, c );
 
+/*****************************************************
+**
+**   Config   ---   readIntArray
+**
+******************************************************/
+void Config::readIntArray( wxArrayInt &array, wxString key, wxString defaultValue = wxEmptyString )
+{
+	wxString s, token;
+	long l;
+
+	DO_READ( key, &s, defaultValue );
+	wxStringTokenizer tk( s, wxT( ";" ));
+	array.Clear();
+	while ( tk.HasMoreTokens())
+	{
+		token = tk.GetNextToken();
+		token.ToLong( &l );
+		array.Add( l );
+	}
+}
+
 /*****************************************************
 **
 **   Config   ---   readAll
@@ -102,9 +123,8 @@ double Config::readDoubleEntry( wxString path, const double def )
 void Config::readAll()
 {
 	wxString s, locationName;
-	double lon, lat, tz, dst;
+	double lon, lat;
 	unsigned int i;
-	long l;
 
 	DO_READ( wxT( "/General/LogMode" ), &logMode, 0l );
 	DO_READ( wxT( "/General/PreferVedic" ), &preferVedic, true );
@@ -116,8 +136,7 @@ void Config::readAll()
 	DO_READ( wxT( "/General/Lang" ), &lang, wxT( "en" ));
 	DO_READ( wxT( "/General/LangList" ), &langList, wxT( "de ru" ));
 	DO_READ( wxT( "/General/BackupFileMode" ), &backupFileMode, 1 );
-	DO_READ( wxT( "/General/FavouriteCountries" ), &favouriteCountries, wxEmptyString );
-	DO_READ( wxT( "/General/MainToolbarItems" ), &mainToolbarItems, wxEmptyString );
+	DO_READ( wxT( "/General/ToolbarItems" ), &mainToolbarItems, wxEmptyString );
 
 	// Window Preferences
 	DO_READ( wxT( "/WindowPreferences/UseMultipleViews" ), &useMultipleViews, false );
@@ -145,10 +164,17 @@ void Config::readAll()
 	DO_READ( wxT( "/WindowPreferences/CircleType" ), &circleType, CIRCLE_TYPE_45 );
 	DO_READ( wxT( "/WindowPreferences/PlanetSortOrder" ), &planetSortOrder, 0 );
 	DO_READ( wxT( "/WindowPreferences/TransitModus" ), &transitModus, 0 );
+	DO_READ( wxT( "/WindowPreferences/TransitTimezone" ), &transitTimezone, 0 );
 
-	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle" ), &vGraphicStyle,
-		VGRAPHIC_SHOW_RETRO | VGRAPHIC_SHOW_PLANET_COLORS | VGRAPHIC_SHOW_AFFLICTIONS
-		| VGRAPHIC_SOUTH_INDIAN | VGRAPHIC_CHART_CENTER_VARGA| VGRAPHIC_NORTH_INDIAN_NUMBER );
+	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle/ShowArudhas" ), &vGraphicStyle.showArudhas, false );
+	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle/ShowRetro" ), &vGraphicStyle.showRetro, true );
+	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle/ShowPlanetColors" ), &vGraphicStyle.showPlanetColors, true );
+	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle/IndianChartType" ), &vGraphicStyle.indianChartType, VGRAPHIC_SOUTH_INDIAN );
+	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle/CenterInfoType" ), &vGraphicStyle.centerInfoType, VGRAPHIC_CHART_CENTER_VARGA );
+	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle/NorthIndianSignDisplayType" ), &vGraphicStyle.northIndianSignDisplayType, VGRAPHIC_NORTH_INDIAN_NUMBER );
+	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle/ShowSbcSanskritSymbols" ), &vGraphicStyle.showSbcSanskritSymbols, false );
+	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle/ShowSbcAfflictions" ), &vGraphicStyle.showSbcAfflictions, true );
+	DO_READ( wxT( "/WindowPreferences/VedicGraphicStyle/ShowSbcNakshatraQuality" ), &vGraphicStyle.showSbcNakshatraQuality, true );
 
 	DO_READ( wxT( "/WindowPreferences/VedicMainChildStyle" ), &vMainChildStyle, MAIN_CHILD_SHOW_NAVAMSA | MAIN_CHILD_SHOW_NAKSHATRA );
 	DO_READ( wxT( "/WindowPreferences/VedicDefaultDasaList" ), &vDefaultDasaList, 7 );
@@ -162,11 +188,20 @@ void Config::readAll()
 	DO_READ( wxT( "/WindowPreferences/YogaCustomFilter" ), &vYogaCustomFilter, 0 );
 	DO_READ( wxT( "/WindowPreferences/LastYogaConfigFile" ), &lastYogaConfigFile, wxEmptyString );
 
-	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle" ), &wGraphicStyle,
-		WGRAPHIC_SHOW_HOUSES | WGRAPHIC_SHOW_ASPECTS | WGRAPHIC_SHOW_RETRO | WGRAPHIC_SHOW_PLANET_COLORS
-		| WGRAPHIC_SHOW_SIGN_COLORS | WGRAPHIC_SHOW_HOUSE_COLORS |  WGRAPHIC_SHOW_ASPECT_COLORS | WGRAPHIC_SHOW_LEFT_ASC
-		);
+	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle/ShowHouses" ), &wGraphicStyle.showHouses, true );
+	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle/ShowAspects"), &wGraphicStyle.showAspects, true );
+	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle/ShowRetro" ), &wGraphicStyle.showRetro, true );
+	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle/ShowPlanetColors" ), &wGraphicStyle.showPlanetColors, true );
+	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle/ShowSignColors" ), &wGraphicStyle.showSignColors, true );
+	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle/ShowHouseColors" ), &wGraphicStyle.showHouseColors, true );
+	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle/ShowAspectColors" ), &wGraphicStyle.showAspectColors, true );
+	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle/ShowAspectSymbols" ), &wGraphicStyle.showAspectSymbols, false );
+	DO_READ( wxT( "/WindowPreferences/WesternGraphicStyle/TransitStyle" ), &wGraphicStyle.transitStyle, 0 );
+
+	DO_READ( wxT( "/WindowPreferences/WesternChartOrientation" ), &wChartOrientation, 0 );
 	DO_READ( wxT( "/WindowPreferences/WesternMainChildStyle" ), &wMainChildStyle, MAIN_CHILD_SHOW_HOUSEPOS );
+	DO_READ( wxT( "/WindowPreferences/ConfigDialogWChartPreviewTransits" ), &configDialogWChartPreviewTransits, 0 );
+	DO_READ( wxT( "/WindowPreferences/ConfigDialogVChartPreviewTransits" ), &configDialogVChartPreviewTransits, 0 );
 
 	// Notebooks
 	DO_READ( wxT( "Notebooks/AshtakaVarga" ), &activePageAshtakaVarga, 0 );
@@ -189,12 +224,12 @@ void Config::readAll()
 	DO_READ( wxT( "/Signs/VedicPositions" ), &useVedicPositions, false );
 	DO_READ( wxT( "/Signs/SignPrecision" ), &signPrecision, TLARGE );
 	DO_READ( wxT( "/Signs/UseSignSymbols" ), &useSignSymbols, true );
-	DO_READ( wxT( "/Signs/UsePlanetSymbols" ), &usePlanetSymbols, true );
+	DO_READ( wxT( "/Signs/UsePlanetSymbols" ), &usePlanetSymbols, false );
 	DO_READ( wxT( "/Signs/UranusSymbol" ), &uranusSymbol, 0 );
 	DO_READ( wxT( "/Signs/PlutoSymbol" ), &plutoSymbol, 0 );
 	DO_READ( wxT( "/Signs/CapricornSymbol" ), &capricornSymbol, 0 );
 
-	// Paths	
+	// Paths
 	DO_READ( wxT( "/Paths/DefaultOpenFiletype" ), &defOpenFiletype, 0 );
 	DO_READ( wxT( "/Paths/DefaultTextExportType" ), &defTextExportType, 0 );
 	DO_READ( wxT( "/Paths/DefaultGraphicExportType" ), &defGraphicExportType, 0 );
@@ -203,6 +238,7 @@ void Config::readAll()
 
 	DO_READ( wxT( "/Paths/DefaultOpenPath" ), &defOpenPath, wxGetHomeDir() );
 	DO_READ( wxT( "/Paths/DefaultSavePath" ), &defSavePath, wxGetHomeDir() );
+	DO_READ( wxT( "/Paths/DefaultPdfSavePath" ), &defPdfSavePath, wxGetHomeDir() );
 	DO_READ( wxT( "/Paths/DefaultExportPath" ), &defExportPath, wxGetHomeDir() );
 	DO_READ( wxT( "/Paths/SwissEphemerisPath" ), &sePath, wxGetHomeDir() );
 
@@ -210,9 +246,7 @@ void Config::readAll()
 	DO_READ( wxT( "/DefaultLocation/LocationName" ), &locationName, wxEmptyString );
 	lon = readDoubleEntry( wxT( "/DefaultLocation/Longitude" ));
 	lat = readDoubleEntry( wxT( "/DefaultLocation/Latitude" ));
-	tz = readDoubleEntry( wxT( "/DefaultLocation/TimeZone" ));
-	dst = readDoubleEntry( wxT( "/DefaultLocation/Dst" ));
-  defaultLocation.setLocation( locationName, lon, lat, tz, dst );
+	defaultLocation.setLocation( locationName, lon, lat, 0, 0 );
 
 	// Animation
 	DO_READ( wxT( "/Animation/AnimateInterval" ), &animateInterval, 5 );
@@ -233,8 +267,8 @@ void Config::readAll()
 
 	DO_READ( wxT( "/Vedic/Ayanamsa" ), &vAyanamsa, 1 );
 	DO_READ( wxT( "/Vedic/VedicObjects" ), &vObjects,
-		OBJECTS_INCLUDE_DRAGONTAIL | OBJECTS_INCLUDE_DRAGONHEAD | OBJECTS_INCLUDE_ASCENDANT  );
-	DO_READ( wxT( "/Vedic/OrderLagna" ), &vOrderLagna, 0 );	
+	         OBJECTS_INCLUDE_DRAGONTAIL | OBJECTS_INCLUDE_DRAGONHEAD | OBJECTS_INCLUDE_ASCENDANT  );
+	DO_READ( wxT( "/Vedic/OrderLagna" ), &vOrderLagna, 0 );
 	DO_READ( wxT( "/Vedic/VGraphicSkin" ), &vGraphicSkin, 0 );
 	DO_READ( wxT( "/Vedic/HouseSystem" ), &vHouseSystem, HOUSE_SYSTEM_PLACIDUS );
 	DO_READ( wxT( "/Vedic/HouseWestern" ), &vHouseWestern, true );
@@ -259,15 +293,7 @@ void Config::readAll()
 	DO_READ( wxT( "/Vedic/MercuryBenefic" ), &vMercuryBenefic, 1 ); // without malefic
 
 	// Arabic Parts
-	DO_READ( wxT( "/Vedic/ArabicParts" ), &s, wxEmptyString );
-	wxStringTokenizer tk1( s, wxT( ";" ));
-	vArabicParts.Clear();
-	while( tk1.HasMoreTokens())
-	{
-		s = tk1.GetNextToken();
-		s.ToLong( &l );
-		vArabicParts.Add( l );
-	}
+	readIntArray( vArabicParts, wxT( "/Vedic/ArabicParts" ));
 
 	// Kala Velas
 	DO_READ( wxT( "/KalaVelas/KalaLord" ), &kalaVelaLord[0], KALAVELA_LORD1 );         // Sun
@@ -281,13 +307,13 @@ void Config::readAll()
 	DO_READ( wxT( "/KalaVelas/MrityuPortion" ), &kalaVelaPortion[1], KALAVELA_PORTION_START );
 	DO_READ( wxT( "/KalaVelas/ArdhapraharaPortion" ), &kalaVelaPortion[2], KALAVELA_PORTION_START );
 	DO_READ( wxT( "/KalaVelas/YamaghantakaPortion" ), &kalaVelaPortion[3], KALAVELA_PORTION_START );
-	DO_READ( wxT( "/KalaVelas/GulikaPortion" ), &kalaVelaPortion[4], KALAVELA_PORTION_START); 
+	DO_READ( wxT( "/KalaVelas/GulikaPortion" ), &kalaVelaPortion[4], KALAVELA_PORTION_START);
 	DO_READ( wxT( "/KalaVelas/MandiPortion" ), &kalaVelaPortion[5], KALAVELA_PORTION_START );
 
 	// Western
 	DO_READ( wxT( "/Western/Ayanamsa" ), &wAyanamsa, 0 );
 	DO_READ( wxT( "/Western/WesternObjects" ), &wObjects,
-		OBJECTS_INCLUDE_OUTER | OBJECTS_INCLUDE_DRAGONTAIL | OBJECTS_INCLUDE_DRAGONHEAD | OBJECTS_INCLUDE_ASCENDANT );
+	         OBJECTS_INCLUDE_OUTER | OBJECTS_INCLUDE_DRAGONTAIL | OBJECTS_INCLUDE_DRAGONHEAD | OBJECTS_INCLUDE_ASCENDANT );
 	DO_READ( wxT( "/Western/PlanetOrder" ), &wPlanetOrder, 0 );
 	DO_READ( wxT( "/Western/HouseSystem" ), &wHouseSystem, HOUSE_SYSTEM_PLACIDUS );
 	DO_READ( wxT( "/Western/LunarNodeMode" ), &wLunarNodeMode, LUNAR_NODE_TRUE );
@@ -297,55 +323,15 @@ void Config::readAll()
 	wCustomYearLength = readDoubleEntry( wxT( "/Western/CustomYearLength" ), b );
 
 	// Arabic Parts
-	DO_READ( wxT( "/Western/ArabicParts" ), &s, wxEmptyString );
-	wxStringTokenizer tk2( s, wxT( ";" ));
-	wArabicParts.Clear();
-	while( tk2.HasMoreTokens())
-	{
-		s = tk2.GetNextToken();
-		s.ToLong( &l );
-		wArabicParts.Add( l );
-	}
+	readIntArray( wArabicParts, wxT( "/Western/ArabicParts" ));
 
 	// Aspects
-	for( i = 0; i < MAX_ASPECT_TYPES; i++ )
+	for ( i = 0; i < MAX_ASPECT_TYPES; i++ )
 	{
 		s.Printf( wxT( "/Aspects/AspectTypes/%02d" ), i );
 		DO_READ( s, &wAspectConfig[i], wxEmptyString );
 	}
 
-	// Base Colors
-	DO_READ( wxT( "/Colors/FgColor" ), &fgColor, wxEmptyString );
-	DO_READ( wxT( "/Colors/SbcPlanetFgColor" ), &sbcPlanetFgColor, wxEmptyString );
-	DO_READ( wxT( "/Colors/SbcTransitFgColor" ), &sbcTransitFgColor, wxEmptyString );
-	DO_READ( wxT( "/Colors/BgColor" ), &bgColor, wxEmptyString );
-	DO_READ( wxT( "/Colors/TableHeaderBgColor" ), &tableHeaderBgColor, wxEmptyString );
-	DO_READ( wxT( "/Colors/SbcMarkedFieldsBgColor" ), &sbcMarkedFieldsBgColor, wxEmptyString );
-	DO_READ( wxT( "/Colors/SbcBeneficBgColor" ), &sbcBeneficBgColor, wxEmptyString );
-	DO_READ( wxT( "/Colors/SbcMaleficBgColor" ), &sbcMaleficBgColor, wxEmptyString );
-
-	for( i = 0; i < 12; i++ )
-	{
-		s.Printf( wxT( "/Colors/SignFgColor/%02d" ), i );
-		DO_READ( s, &signFgColor[i], wxEmptyString );
-		s.Printf( wxT( "/Colors/SignBgColor/%02d" ), i );
-		DO_READ( s, &signBgColor[i], wxEmptyString );
-		s.Printf( wxT( "/Colors/HouseFgColor/%02d" ), i );
-		DO_READ( s, &houseFgColor[i], wxEmptyString );
-		s.Printf( wxT( "/Colors/HouseBgColor/%02d" ), i );
-		DO_READ( s, &houseBgColor[i], wxEmptyString );
-	}
-	for( i = 0; i < MAX_EPHEM_OBJECTS; i++ )
-	{
-		s.Printf( wxT( "/Colors/PlanetColor/%02d" ), i );
-		DO_READ( s, &planetFgColor[i], wxEmptyString );
-	}
-	for( i = 0; i < MAX_DASA_COLORS; i++ )
-	{
-		s.Printf( wxT( "/Colors/Dasas/%02d" ), i );
-		DO_READ( s, &dasaColor[i], wxEmptyString );
-	}
-	
 	// Window Sizes
 	DO_READ( wxT( "/WindowSizes/xMasterWindow" ), &xMasterWindow, 720 );
 	DO_READ( wxT( "/WindowSizes/yMasterWindow" ), &yMasterWindow, 200 );
@@ -385,7 +371,7 @@ void Config::readAll()
 
 	DO_READ( wxT( "/WindowSizes/xYogaWindow" ), &xYogaWindow, 800 );
 	DO_READ( wxT( "/WindowSizes/yYogaWindow" ), &yYogaWindow, 600 );
-	
+
 	DO_READ( wxT( "/WindowSizes/xVargaSummaryWindow" ), &xVargaSummaryWindow, 800 );
 	DO_READ( wxT( "/WindowSizes/yVargaSummaryWindow" ), &yVargaSummaryWindow, 600 );
 
@@ -407,8 +393,11 @@ void Config::readAll()
 	DO_READ( wxT( "/WindowSizes/xPartnerWindow" ), &xPartnerWindow, 800 );
 	DO_READ( wxT( "/WindowSizes/yPartnerWindow" ), &yPartnerWindow, 600 );
 
-	DO_READ( wxT( "/WindowSizes/xConfigDialog" ), &xConfigDialog, 740 );
-	DO_READ( wxT( "/WindowSizes/yConfigDialog" ), &yConfigDialog, 400 );
+	DO_READ( wxT( "/WindowSizes/xConfigDialog" ), &xConfigDialog, 800 );
+	DO_READ( wxT( "/WindowSizes/yConfigDialog" ), &yConfigDialog, 600 );
+
+	DO_READ( wxT( "/WindowSizes/xSimpleConfigDialog" ), &xSimpleConfigDialog, 600 );
+	DO_READ( wxT( "/WindowSizes/ySimpleConfigDialog" ), &ySimpleConfigDialog, 500 );
 
 	DO_READ( wxT( "/WindowSizes/xYogaEditor" ), &xYogaEditor, 600 );
 	DO_READ( wxT( "/WindowSizes/yYogaEditor" ), &yYogaEditor, 400 );
@@ -421,11 +410,18 @@ void Config::readAll()
 
 	// Print
 	DO_READ( wxT( "/Print/ShowPageFrame" ), &printShowPageFrame, true );
-	DO_READ( wxT( "/Print/UseColors" ), &printUseColors, false );
+	DO_READ( wxT( "/Print/SaveFileOption" ), &printSaveFileOption, 0 );
 	DO_READ( wxT( "/Print/ShowSelectDialog" ), &printShowSelectDialog, true );
 	DO_READ( wxT( "/Print/AskOnOverwrite" ), &printAskOverwrite, true );
+	DO_READ( wxT( "/Print/CompressPdf" ), &printCompressPdf, true );
 	DO_READ( wxT( "/Print/LaunchPdfViewer" ), &printLaunchPdfViewer, true );
-	DO_READ( wxT( "/Print/PdfViewerCommand" ), &printPdfViewerCommand, wxEmptyString );
+
+	wxString pdfStartCommand = wxEmptyString;
+#ifdef __WXMAC__
+	pdfStartCommand = MAC_PDF_START_COMMAND;
+#endif
+	DO_READ( wxT( "/Print/PdfViewerCommand" ), &printPdfViewerCommand, pdfStartCommand );
+
 	DO_READ( wxT( "/Print/CustomFooter" ), &printCustomFooter, false );
 	DO_READ( wxT( "/Print/CustomFooterText" ), &printCustomFooterText, DEFAULT_CUSTOM_FOOTER );
 	DO_READ( wxT( "/Print/DefaultPrintout" ), &printDefaultPrintout, 0 );
@@ -440,11 +436,50 @@ void Config::readAll()
 	DO_READ( wxT( "/Fonts/Symbol" ), &fontSymbol, wxEmptyString );
 	DO_READ( wxT( "/Fonts/SymbolFontSize" ), &symbolFontSize, 1 );
 
-	colors->readConfigParams();
+	DO_READ( wxT( "/Fonts/PdfDefault" ), &fontPdfDefault, wxEmptyString );
+	DO_READ( wxT( "/Fonts/PdfHeader" ), &fontPdfHeader, wxEmptyString );
+	DO_READ( wxT( "/Fonts/PdfGraphic" ), &fontPdfGraphic, wxEmptyString );
+	DO_READ( wxT( "/Fonts/PdfSymbol" ), &fontPdfSymbol, wxEmptyString );
+	DO_READ( wxT( "/Fonts/PdfSymbolFontSize" ), &pdfSymbolFontSize, 1 );
+
+	// Atlas
+	DO_READ( wxT( "/Atlas/DatabaseFile" ), &databaseFile, wxEmptyString );
+	DO_READ( wxT( "/Atlas/AtlasFavouriteCountries" ), &atlasFavouriteCountries, wxT( "IN;US;DE;RU;FR;IT;GB;CA" ));
+	DO_READ( wxT( "/Atlas/AtlasFilterMode" ), &atlasFilterMode, 0 );
+	DO_READ( wxT( "/Atlas/AtlasLastSelectedCountry" ), &atlasLastSelectedCountry, wxEmptyString );
+	DO_READ( wxT( "/Atlas/AtlasGridColumnSizes" ), &atlasGridColumnSizes, wxEmptyString );
+	DO_READ( wxT( "/Atlas/AtlasFilterCaseSensitive" ), &atlasFilterCaseSensitive, false );
+	DO_READ( wxT( "/Atlas/AtlasCacheSize" ), &atlasCacheSize, 2000 );
+	DO_READ( wxT( "/Atlas/xAtlasDialog" ), &xAtlasDialog, 800 );
+	DO_READ( wxT( "/Atlas/yAtlasDialog" ), &yAtlasDialog, 750 );
+
+	// show warning dialogs
+	DO_READ( wxT( "/WarningDialogs/DoNotShowAgainWarningDFonts" ), &doNotShowAgainWarningDFonts, false );
+	DO_READ( wxT( "/WarningDialogs/DoNotShowAgainWarningEphemFiles" ), &doNotShowAgainWarningEphemFiles, false );
+	DO_READ( wxT( "/WarningDialogs/DoNotShowAgainAfterLanguageChange" ), &doNotShowAgainAfterLanguageChange, false );
+	DO_READ( wxT( "/WarningDialogs/DoNotShowAgainEphemConfigChange" ), &doNotShowAgainEphemConfigChange, false );
+
+	colors->readConfigParams( cfg );
 }
 
 #undef DO_READ
 
+/*****************************************************
+**
+**   Config   ---   writeIntArray
+**
+******************************************************/
+void Config::writeIntArray( wxString key, const wxArrayInt &array )
+{
+	wxString s;
+	for ( unsigned int i = 0; i < array.size(); i++ )
+	{
+		if ( ! s.IsEmpty() ) s << wxT( ";" );
+		s << array[i];
+	}
+	cfg->Write( key, s );
+}
+
 /*****************************************************
 **
 **   Config   ---   writeAll
@@ -455,7 +490,7 @@ void Config::writeAll()
 	unsigned int i;
 	wxString s;
 
-	colors->writeConfigParams();
+	colors->writeConfigParams( cfg );
 
 	cfg->Write( wxT( "/General/LogMode" ), logMode );
 	cfg->Write( wxT( "/General/PreferVedic" ), preferVedic );
@@ -467,8 +502,7 @@ void Config::writeAll()
 	cfg->Write( wxT( "/General/Lang" ), lang );
 	cfg->Write( wxT( "/General/LangList" ), langList );
 	cfg->Write( wxT( "/General/BackupFileMode" ), backupFileMode );
-	cfg->Write( wxT( "/General/FavouriteCountries" ), favouriteCountries );
-	cfg->Write( wxT( "/General/MainToolbarItems" ), mainToolbarItems );
+	cfg->Write( wxT( "/General/ToolbarItems" ), mainToolbarItems );
 
 	// Window Preferences
 	cfg->Write( wxT( "/WindowPreferences/UseMultipleViews" ), useMultipleViews );
@@ -490,7 +524,15 @@ void Config::writeAll()
 	cfg->Write( wxT( "/WindowPreferences/EphemCircleType" ), ephemCircleType );
 	cfg->Write( wxT( "/WindowPreferences/EphemTimezone" ), ephemTimezone );
 
-	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle" ), vGraphicStyle );
+	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle/ShowArudhas" ), vGraphicStyle.showArudhas );
+	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle/ShowRetro" ), vGraphicStyle.showRetro );
+	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle/ShowPlanetColors" ), vGraphicStyle.showPlanetColors );
+	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle/IndianChartType" ), vGraphicStyle.indianChartType );
+	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle/CenterInfoType" ), vGraphicStyle.centerInfoType );
+	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle/NorthIndianSignDisplayType" ), vGraphicStyle.northIndianSignDisplayType );
+	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle/ShowSbcSanskritSymbols" ), vGraphicStyle.showSbcSanskritSymbols );
+	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle/ShowSbcAfflictions" ), vGraphicStyle.showSbcAfflictions );
+	cfg->Write( wxT( "/WindowPreferences/VedicGraphicStyle/ShowSbcNakshatraQuality" ), vGraphicStyle.showSbcNakshatraQuality );
 
 	cfg->Write( wxT( "/WindowPreferences/VedicMainChildStyle" ), vMainChildStyle );
 	cfg->Write( wxT( "/WindowPreferences/VedicDefaultDasaList" ), vDefaultDasaList );
@@ -504,7 +546,16 @@ void Config::writeAll()
 	cfg->Write( wxT( "/WindowPreferences/YogaCustomFilter" ), vYogaCustomFilter );
 	cfg->Write( wxT( "/WindowPreferences/LastYogaConfigFile" ), lastYogaConfigFile );
 
-	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle" ), wGraphicStyle );
+	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle/ShowHouses" ), wGraphicStyle.showHouses );
+	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle/ShowAspects"), wGraphicStyle.showAspects );
+	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle/ShowRetro" ), wGraphicStyle.showRetro );
+	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle/ShowPlanetColors" ), wGraphicStyle.showPlanetColors );
+	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle/ShowSignColors" ), wGraphicStyle.showSignColors );
+	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle/ShowHouseColors" ), wGraphicStyle.showHouseColors );
+	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle/ShowAspectColors" ), wGraphicStyle.showAspectColors );
+	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle/ShowAspectSymbols" ), wGraphicStyle.showAspectSymbols );
+	cfg->Write( wxT( "/WindowPreferences/WesternGraphicStyle/TransitStyle" ), wGraphicStyle.transitStyle );
+	cfg->Write( wxT( "/WindowPreferences/WesternChartOrientation" ), wChartOrientation );
 	cfg->Write( wxT( "/WindowPreferences/WesternMainChildStyle" ), wMainChildStyle );
 
 	cfg->Write( wxT( "/WindowPreferences/OrbisRadix" ), orbisRadix );
@@ -513,6 +564,9 @@ void Config::writeAll()
 	cfg->Write( wxT( "/WindowPreferences/CircleType" ), circleType );
 	cfg->Write( wxT( "/WindowPreferences/PlanetSortOrder" ), planetSortOrder );
 	cfg->Write( wxT( "/WindowPreferences/TransitModus" ), transitModus );
+	cfg->Write( wxT( "/WindowPreferences/TransitTimezone" ), transitTimezone );
+	cfg->Write( wxT( "/WindowPreferences/ConfigDialogWChartPreviewTransits" ), configDialogWChartPreviewTransits );
+	cfg->Write( wxT( "/WindowPreferences/ConfigDialogVChartPreviewTransits" ), configDialogVChartPreviewTransits );
 
 	// Notebooks
 	cfg->Write( wxT( "Notebooks/AshtakaVarga" ), activePageAshtakaVarga );
@@ -544,6 +598,7 @@ void Config::writeAll()
 
 	cfg->Write( wxT( "/Paths/DefaultOpenPath" ), defOpenPath );
 	cfg->Write( wxT( "/Paths/DefaultSavePath" ), defSavePath );
+	cfg->Write( wxT( "/Paths/DefaultPdfSavePath" ), defPdfSavePath );
 	cfg->Write( wxT( "/Paths/DefaultExportPath" ), defExportPath );
 	cfg->Write( wxT( "/Paths/SwissEphemerisPath" ), sePath );
 
@@ -551,8 +606,6 @@ void Config::writeAll()
 	cfg->Write( wxT( "/DefaultLocation/LocationName" ), defaultLocation.getLocName() );
 	writeDoubleEntry( wxT( "/DefaultLocation/Longitude" ), defaultLocation.getLongitude() );
 	writeDoubleEntry( wxT( "/DefaultLocation/Latitude" ), defaultLocation.getLatitude() );
-	writeDoubleEntry( wxT( "/DefaultLocation/TimeZone" ), defaultLocation.getTimeZone() );
-	writeDoubleEntry( wxT( "/DefaultLocation/Dst" ), defaultLocation.getDST() );
 
 	// Animation
 	cfg->Write( wxT( "/Animation/AnimateInterval" ), animateInterval );
@@ -573,7 +626,7 @@ void Config::writeAll()
 
 	cfg->Write( wxT( "/Vedic/Ayanamsa" ), vAyanamsa );
 	cfg->Write( wxT( "/Vedic/VedicObjects" ), vObjects );
-	cfg->Write( wxT( "/Vedic/OrderLagna" ), vOrderLagna );	
+	cfg->Write( wxT( "/Vedic/OrderLagna" ), vOrderLagna );
 	cfg->Write( wxT( "/Vedic/VGraphicSkin" ), vGraphicSkin );
 	cfg->Write( wxT( "/Vedic/HouseSystem" ), vHouseSystem );
 	cfg->Write( wxT( "/Vedic/HouseWestern" ), vHouseWestern );
@@ -597,13 +650,7 @@ void Config::writeAll()
 	cfg->Write( wxT( "/Vedic/MercuryBenefic" ), vMercuryBenefic );
 
 	// Arabic Parts
-	s.Clear();
-	for( i = 0; i < vArabicParts.size(); i++ )
-	{
-		if ( ! s.IsEmpty() ) s << wxT( ";" );
-		s << vArabicParts[i];
-	}
-	cfg->Write( wxT( "/Vedic/ArabicParts" ), s );
+	writeIntArray( wxT( "/Vedic/ArabicParts" ), vArabicParts );
 
 	// Kala Velas
 	cfg->Write( wxT( "/KalaVelas/KalaLord" ), kalaVelaLord[0] );         // Sun
@@ -617,7 +664,7 @@ void Config::writeAll()
 	cfg->Write( wxT( "/KalaVelas/MrityuPortion" ), kalaVelaPortion[1] );
 	cfg->Write( wxT( "/KalaVelas/ArdhapraharaPortion" ), kalaVelaPortion[2] );
 	cfg->Write( wxT( "/KalaVelas/YamaghantakaPortion" ), kalaVelaPortion[3] );
-	cfg->Write( wxT( "/KalaVelas/GulikaPortion" ), kalaVelaPortion[4]); 
+	cfg->Write( wxT( "/KalaVelas/GulikaPortion" ), kalaVelaPortion[4]);
 	cfg->Write( wxT( "/KalaVelas/MandiPortion" ), kalaVelaPortion[5] );
 
 	// Western
@@ -631,52 +678,14 @@ void Config::writeAll()
 	cfg->Write( wxT( "/Western/CustomYearLength" ), wCustomYearLength );
 
 	// Arabic Parts
-	s.Clear();
-	for( i = 0; i < wArabicParts.size(); i++ )
-	{
-		if ( ! s.IsEmpty() ) s << wxT( ";" );
-		s << wArabicParts[i];
-	}
-	cfg->Write( wxT( "/Western/ArabicParts" ), s );
+	writeIntArray( wxT( "/Western/ArabicParts" ), wArabicParts );
 
-	// Aspects
-	for( i = 0; i < MAX_ASPECT_TYPES; i++ )
-	{
-		s.Printf( wxT( "/Aspects/AspectTypes/%02d" ), i );
-		cfg->Write( s, wAspectConfig[i] );
-	}
-
-	// Base Colors
-	cfg->Write( wxT( "/Colors/FgColor" ), fgColor );
-	cfg->Write( wxT( "/Colors/SbcPlanetFgColor" ), sbcPlanetFgColor );
-	cfg->Write( wxT( "/Colors/SbcTransitFgColor" ), sbcTransitFgColor );
-	cfg->Write( wxT( "/Colors/BgColor" ), bgColor );
-	cfg->Write( wxT( "/Colors/TableHeaderBgColor" ), tableHeaderBgColor );
-	cfg->Write( wxT( "/Colors/SbcMarkedFieldsBgColor" ), sbcMarkedFieldsBgColor );
-	cfg->Write( wxT( "/Colors/SbcBeneficBgColor" ), sbcBeneficBgColor );
-	cfg->Write( wxT( "/Colors/SbcMaleficBgColor" ), sbcMaleficBgColor );
-	
-	for( i = 0; i < 12; i++ )
-	{
-		s.Printf( wxT( "/Colors/SignFgColor/%02d" ), i );
-		cfg->Write( s, signFgColor[i] );
-		s.Printf( wxT( "/Colors/SignBgColor/%02d" ), i );
-		cfg->Write( s, signBgColor[i] );
-		s.Printf( wxT( "/Colors/HouseFgColor/%02d" ), i );
-		cfg->Write( s, houseFgColor[i] );
-		s.Printf( wxT( "/Colors/HouseBgColor/%02d" ), i );
-		cfg->Write( s, houseBgColor[i] );
-	}
-	for( i = 0; i < MAX_EPHEM_OBJECTS; i++ )
-	{
-		s.Printf( wxT( "/Colors/PlanetColor/%02d" ), i );
-		cfg->Write( s, planetFgColor[i] );
-	}
-	for( i = 0; i < MAX_DASA_COLORS; i++ )
-	{
-		s.Printf( wxT( "/Colors/Dasas/%02d" ), i );
-		cfg->Write( s, dasaColor[i] );
-	}
+  // Aspects
+  for( i = 0; i < MAX_ASPECT_TYPES; i++ )
+  {
+    s.Printf( wxT( "/Aspects/AspectTypes/%02d" ), i );
+    cfg->Write( s, wAspectConfig[i] );
+  }
 
 	// Window Sizes
 	cfg->Write( wxT( "/WindowSizes/xMasterWindow" ), xMasterWindow );
@@ -742,6 +751,9 @@ void Config::writeAll()
 	cfg->Write( wxT( "/WindowSizes/xConfigDialog" ), xConfigDialog );
 	cfg->Write( wxT( "/WindowSizes/yConfigDialog" ), yConfigDialog );
 
+	cfg->Write( wxT( "/WindowSizes/xSimpleConfigDialog" ), xSimpleConfigDialog );
+	cfg->Write( wxT( "/WindowSizes/ySimpleConfigDialog" ), ySimpleConfigDialog );
+
 	cfg->Write( wxT( "/WindowSizes/xYogaEditor" ), xYogaEditor );
 	cfg->Write( wxT( "/WindowSizes/yYogaEditor" ), yYogaEditor );
 
@@ -753,9 +765,10 @@ void Config::writeAll()
 
 	// Print
 	cfg->Write( wxT( "/Print/ShowPageFrame" ), printShowPageFrame );
-	cfg->Write( wxT( "/Print/UseColors" ), printUseColors );
+	cfg->Write( wxT( "/Print/SaveFileOption" ), printSaveFileOption );
 	cfg->Write( wxT( "/Print/ShowSelectDialog" ), printShowSelectDialog );
 	cfg->Write( wxT( "/Print/AskOnOverwrite" ), printAskOverwrite );
+	cfg->Write( wxT( "/Print/CompressPdf" ), printCompressPdf );
 	cfg->Write( wxT( "/Print/LaunchPdfViewer" ), printLaunchPdfViewer );
 	cfg->Write( wxT( "/Print/PdfViewerCommand" ), printPdfViewerCommand );
 	cfg->Write( wxT( "/Print/CustomFooter" ), printCustomFooter );
@@ -773,6 +786,29 @@ void Config::writeAll()
 	cfg->Write( wxT( "/Fonts/Symbol" ), fontSymbol );
 	cfg->Write( wxT( "/Fonts/SymbolFontSize" ), symbolFontSize );
 
+	cfg->Write( wxT( "/Fonts/PdfDefault" ), fontPdfDefault );
+	cfg->Write( wxT( "/Fonts/PdfHeader" ), fontPdfHeader );
+	cfg->Write( wxT( "/Fonts/PdfGraphic" ), fontPdfGraphic );
+	cfg->Write( wxT( "/Fonts/PdfSymbol" ), fontPdfSymbol );
+	cfg->Write( wxT( "/Fonts/PdfSymbolFontSize" ), pdfSymbolFontSize );
+
+	// Atlas
+	cfg->Write( wxT( "/Atlas/DatabaseFile" ), databaseFile );
+	cfg->Write( wxT( "/Atlas/AtlasFavouriteCountries" ), atlasFavouriteCountries );
+	cfg->Write( wxT( "/Atlas/AtlasFilterMode" ), atlasFilterMode );
+	cfg->Write( wxT( "/Atlas/AtlasLastSelectedCountry" ), atlasLastSelectedCountry );
+	cfg->Write( wxT( "/Atlas/AtlasGridColumnSizes" ), atlasGridColumnSizes );
+	cfg->Write( wxT( "/Atlas/AtlasFilterCaseSensitive" ), atlasFilterCaseSensitive );
+	cfg->Write( wxT( "/Atlas/AtlasCacheSize" ), atlasCacheSize );
+	cfg->Write( wxT( "/Atlas/xAtlasDialog" ), xAtlasDialog );
+	cfg->Write( wxT( "/Atlas/yAtlasDialog" ), yAtlasDialog );
+
+	// show warning dialogs
+	cfg->Write( wxT( "/WarningDialogs/DoNotShowAgainWarningDFonts" ), doNotShowAgainWarningDFonts );
+	cfg->Write( wxT( "/WarningDialogs/DoNotShowAgainWarningEphemFiles" ), doNotShowAgainWarningEphemFiles );
+	cfg->Write( wxT( "/WarningDialogs/DoNotShowAgainAfterLanguageChange" ), doNotShowAgainAfterLanguageChange );
+	cfg->Write( wxT( "/WarningDialogs/DoNotShowAgainEphemConfigChange" ), doNotShowAgainEphemConfigChange );
+
 	cfg->Flush();
 }
 
