@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/base/DataSet.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,11 +19,12 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "DataSet.h"
+#pragma implementation "DataSet.h"
 #endif
 
 #include "DataSet.h"
 
+#include <wx/datetime.h>
 #include <wx/log.h>
 #include <wx/string.h>
 #include <wx/tokenzr.h>
@@ -39,7 +40,7 @@ extern Config *config;
 
 /*****************************************************
 **
-**   JDate   ---   Constructor 
+**   JDate   ---   Constructor
 **
 ******************************************************/
 JDate::JDate()
@@ -49,7 +50,7 @@ JDate::JDate()
 
 /*****************************************************
 **
-**   JDate   ---   Constructor 
+**   JDate   ---   Constructor
 **
 ******************************************************/
 JDate::JDate( const double &jd )
@@ -59,7 +60,7 @@ JDate::JDate( const double &jd )
 
 /*****************************************************
 **
-**   JDate   ---   Constructor 
+**   JDate   ---   Constructor
 **
 ******************************************************/
 JDate::JDate( const JDate &org )
@@ -69,22 +70,32 @@ JDate::JDate( const JDate &org )
 
 /*****************************************************
 **
-**   JDate   ---   setActualDate 
+**   JDate   ---   getCurrentJD
 **
 ******************************************************/
-void JDate::setActualDate()
+// static
+double JDate::getCurrentJD()
 {
 	time_t tt;
 	time( &tt );
-	double mytime;
 
-	mytime = 2440587.5 + (double)tt / 86400.0; // 1.1.1970 + seconds elapsed / number of seconds per day
-	setDate( mytime );
+	// 1.1.1970 + seconds elapsed / number of seconds per day
+	return( 2440587.5 + (double)tt / 86400.0 );
 }
 
 /*****************************************************
 **
-**   JDate   ---   updateJD 
+**   JDate   ---   setActualDate
+**
+******************************************************/
+void JDate::setActualDate()
+{
+	setDate( getCurrentJD() );
+}
+
+/*****************************************************
+**
+**   JDate   ---   updateJD
 **
 ******************************************************/
 void JDate::updateJD( const int &day, const int &month, const int &year, const double &time )
@@ -94,7 +105,7 @@ void JDate::updateJD( const int &day, const int &month, const int &year, const d
 
 /*****************************************************
 **
-**   JDate   ---   setDate 
+**   JDate   ---   setDate
 **
 ******************************************************/
 void JDate::setDate( const double &jd )
@@ -104,7 +115,7 @@ void JDate::setDate( const double &jd )
 
 /*****************************************************
 **
-**   JDate   ---   setDate 
+**   JDate   ---   setDate
 **
 ******************************************************/
 void JDate::setDate( const int &day, const int &month, const int &year, const double &time )
@@ -124,19 +135,19 @@ void JDate::operator=( const JDate& _date )
 
 /*****************************************************
 **
-**   JDate   ---   dump 
+**   JDate   ---   dump
 **
 ******************************************************/
 void JDate::dump(wxString &o)
 {
 	Formatter *formatter = Formatter::get();
 	o << _( "Date" ) << wxT( " : " ) << formatter->getDateStringFromJD( jd ) << Endl
-		<< _( "Universal Time" ) << wxT( " : " ) << formatter->getTimeFormatted( getTimeFromJD(jd)) << Endl;
+	<< _( "Universal Time" ) << wxT( " : " ) << formatter->getTimeFormatted( getTimeFromJD(jd)) << Endl;
 }
 
 /*****************************************************
 **
-**   Location   ---   Constructor 
+**   Location   ---   Constructor
 **
 ******************************************************/
 Location::Location()
@@ -150,12 +161,12 @@ Location::Location()
 
 /*****************************************************
 **
-**   Location   ---   Constructor 
+**   Location   ---   Constructor
 **
 ******************************************************/
 Location::Location( const Location &l )
 {
-  setLocation( l.locName, l.longitude, l.latitude, l.tz, l.dst );	
+	setLocation( l.locName, l.longitude, l.latitude, l.tz, l.dst );
 }
 
 /**************************************************************
@@ -165,16 +176,16 @@ Location::Location( const Location &l )
 ***************************************************************/
 void Location::operator=( const Location& l )
 {
-  setLocation( l.locName, l.longitude, l.latitude, l.tz, l.dst );	
+	setLocation( l.locName, l.longitude, l.latitude, l.tz, l.dst );
 }
 
 /*****************************************************
 **
-**   Location   ---   setLocation 
+**   Location   ---   setLocation
 **
 ******************************************************/
 void Location::setLocation( const wxString &locname, const double &longitude, const double &latitude,
-	const double &tz, const double &dst )
+                            const double &tz, const double &dst )
 {
 	this->locName = locname;
 	this->longitude = longitude;
@@ -185,7 +196,7 @@ void Location::setLocation( const wxString &locname, const double &longitude, co
 
 /*****************************************************
 **
-**   Location   ---   dump 
+**   Location   ---   dump
 **
 ******************************************************/
 void Location::dump(wxString &o)
@@ -193,13 +204,13 @@ void Location::dump(wxString &o)
 	Formatter *formatter = Formatter::get();
 
 	o << _( "Location" ) << wxT( ": " ) << locName << Endl
-		<< _( "Longitude" ) << wxT( " : " ) << formatter->getLongitudeFormatted( longitude ) << Endl
-		<< _( "Latitude" ) << wxT( " : " ) << formatter->getLatitudeFormatted( latitude ) << Endl;
+	<< _( "Longitude" ) << wxT( " : " ) << formatter->getLongitudeFormatted( longitude ) << Endl
+	<< _( "Latitude" ) << wxT( " : " ) << formatter->getLatitudeFormatted( latitude ) << Endl;
 }
 
 /*****************************************************
 **
-**   DataSet   ---   Constructor 
+**   DataSet   ---   Constructor
 **
 ******************************************************/
 DataSet::DataSet()
@@ -211,7 +222,7 @@ DataSet::DataSet()
 
 /*****************************************************
 **
-**   DataSet   ---   Constructor 
+**   DataSet   ---   Constructor
 **
 ******************************************************/
 DataSet::DataSet( const Location &loc )
@@ -223,7 +234,7 @@ DataSet::DataSet( const Location &loc )
 
 /*****************************************************
 **
-**   DataSet   ---   Constructor 
+**   DataSet   ---   Constructor
 **
 ******************************************************/
 DataSet::DataSet( const DataSet &org )
@@ -236,7 +247,7 @@ DataSet::DataSet( const DataSet &org )
 
 /*****************************************************
 **
-**   DataSet   ---   Destructor 
+**   DataSet   ---   Destructor
 **
 ******************************************************/
 DataSet::~DataSet()
@@ -260,18 +271,18 @@ void DataSet::operator=( const DataSet &d )
 
 /*****************************************************
 **
-**   DataSet   ---   setLocation 
+**   DataSet   ---   setLocation
 **
 ******************************************************/
 void DataSet::setLocation( const wxString &locName, const double &longitude, const double &latitute,
-	const double &timeZone, const double &dst )
+                           const double &timeZone, const double &dst )
 {
 	location->setLocation( locName, longitude, latitute, timeZone, dst );
 }
 
 /*****************************************************
 **
-**   DataSet   ---   setLocation 
+**   DataSet   ---   setLocation
 **
 ******************************************************/
 void DataSet::setLocation( const Location &loc )
@@ -281,7 +292,7 @@ void DataSet::setLocation( const Location &loc )
 
 /*****************************************************
 **
-**   DataSet   ---   setName 
+**   DataSet   ---   setName
 **
 ******************************************************/
 void DataSet::setName( const wxString &name )
@@ -291,7 +302,7 @@ void DataSet::setName( const wxString &name )
 
 /*****************************************************
 **
-**   DataSet   ---   setRemark 
+**   DataSet   ---   setRemark
 **
 ******************************************************/
 void DataSet::setRemark( const wxString &remark )
@@ -301,7 +312,7 @@ void DataSet::setRemark( const wxString &remark )
 
 /*****************************************************
 **
-**   DataSet   ---   setDate 
+**   DataSet   ---   setDate
 **
 ******************************************************/
 void DataSet::setDate( const double &jd )
@@ -311,7 +322,7 @@ void DataSet::setDate( const double &jd )
 
 /*****************************************************
 **
-**   DataSet   ---   setDate 
+**   DataSet   ---   setDate
 **
 ******************************************************/
 void DataSet::setDate( const int &day, const int &month, const int &year, const double &time )
@@ -321,7 +332,7 @@ void DataSet::setDate( const int &day, const int &month, const int &year, const
 
 /*****************************************************
 **
-**   DataSet   ---   setActualDate 
+**   DataSet   ---   setActualDate
 **
 ******************************************************/
 void DataSet::setActualDate()
@@ -336,7 +347,7 @@ void DataSet::setActualDate()
 ******************************************************/
 int DataSet::getWeekDay()
 {
-  return (int)( getJD() + ( location->getTimeZone() + location->getDST() )/24.0 + 1.5 ) % 7;
+	return (int)( getJD() + ( location->getTimeZone() + location->getDST() )/24.0 + 1.5 ) % 7;
 }
 
 /*****************************************************
@@ -353,7 +364,7 @@ double DataSet::getLocalMeanTime()
 
 /*****************************************************
 **
-**   DataSet   ---   dump 
+**   DataSet   ---   dump
 **
 ******************************************************/
 void DataSet::dump(wxString &o)
@@ -366,15 +377,15 @@ void DataSet::dump(wxString &o)
 
 	time = a_red( getTimeFromJD(date->getJD()) + location->getTimeZone() + location->getDST(), 24 );
 	o << _( "Local Time" ) << wxT( " : " ) << f->getTimeFormatted( time ) << Endl
-		<< _( "Time Zone" ) << wxT( " : " ) << location->getTimeZone() << Endl
-		<< _( "DST" ) << wxT( " : " ) << location->getDST() << Endl;
+	<< _( "Time Zone" ) << wxT( " : " ) << location->getTimeZone() << Endl
+	<< _( "DST" ) << wxT( " : " ) << location->getDST() << Endl;
 
 	location->dump(o);
 }
 
 /*****************************************************
 **
-**   FileDataSet   ---   Constructor 
+**   FileDataSet   ---   Constructor
 **
 ******************************************************/
 FileDataSet::FileDataSet() : DataSet()
@@ -383,17 +394,17 @@ FileDataSet::FileDataSet() : DataSet()
 
 /*****************************************************
 **
-**   FileDataSet   ---   Constructor 
+**   FileDataSet   ---   Constructor
 **
 ******************************************************/
 FileDataSet::FileDataSet( const Location &loc )
-	: DataSet( loc )
+		: DataSet( loc )
 {
 }
 
 /*****************************************************
 **
-**   FileDataSet   ---   Destructor 
+**   FileDataSet   ---   Destructor
 **
 ******************************************************/
 FileDataSet::~FileDataSet()
@@ -402,7 +413,7 @@ FileDataSet::~FileDataSet()
 
 /*****************************************************
 **
-**   FileDataSet   ---   openFile 
+**   FileDataSet   ---   openFile
 **
 ******************************************************/
 bool FileDataSet::openFile( const wxString &fn, bool isAtxFile )
@@ -415,7 +426,7 @@ bool FileDataSet::openFile( const wxString &fn, bool isAtxFile )
 		wxLogError( wxT( "Cannot open file %s" ), (const wxChar*)filename );
 		return false;
 	}
-  wxString c = this->filename.Right( 3 );
+	wxString c = this->filename.Right( 3 );
 	if ( c == wxT( "jhd" )) ret = readJhdFile();
 	else if ( c == wxT( "atx" )) ret = readAtxFile();
 	else ret = readMtxFile();
@@ -425,14 +436,14 @@ bool FileDataSet::openFile( const wxString &fn, bool isAtxFile )
 
 /*****************************************************
 **
-**   FileDataSet   ---   readAtxFile 
+**   FileDataSet   ---   readAtxFile
 **
 ******************************************************/
 bool FileDataSet::readAtxFile()
 {
 	long day, month, year, hour, minute, second, ldeg, bdeg;
 	double timeZone, dmin, lmin, bmin, dst;
-	
+
 	wxString buf;
 	wxString s;
 
@@ -572,14 +583,14 @@ bool FileDataSet::readAtxFile()
 
 /*****************************************************
 **
-**   FileDataSet   ---   readMtxFile 
+**   FileDataSet   ---   readMtxFile
 **
 ******************************************************/
 bool FileDataSet::readMtxFile()
 {
 	long day, month, year;
 	double time, timeZone, longitude, latitude, dst;
-	
+
 	wxString buf;
 	wxString s;
 
@@ -676,7 +687,7 @@ bool FileDataSet::readMtxFile()
 	setLocation( s, longitude, latitude, timeZone, dst );
 
 	remark.Clear();
-	while( ! theFile.Eof())
+	while ( ! theFile.Eof())
 	{
 		buf = theFile.GetNextLine();
 		if ( remark.Length() > 0 ) remark += Endl;
@@ -687,7 +698,7 @@ bool FileDataSet::readMtxFile()
 
 /*****************************************************
 **
-**   FileDataSet   ---   readJhdFile 
+**   FileDataSet   ---   readJhdFile
 **
 ******************************************************/
 bool FileDataSet::readJhdFile()
@@ -725,7 +736,7 @@ bool FileDataSet::readJhdFile()
 	tz = myatof( buf );
 	timeZone = (int)tz + ( tz - (double)(int)tz ) * 5/3;
 	timeZone *= -1;
-	
+
 	time -= timeZone;
 	setDate( day, month, year, time );
 
@@ -758,7 +769,7 @@ bool FileDataSet::readJhdFile()
 
 /*****************************************************
 **
-**   FileDataSet   ---   saveAs 
+**   FileDataSet   ---   saveAs
 **
 ******************************************************/
 bool FileDataSet::saveAs( const wxString &fn )
@@ -774,7 +785,7 @@ bool FileDataSet::saveAs( const wxString &fn )
 
 /*****************************************************
 **
-**   FileDataSet   ---   save 
+**   FileDataSet   ---   save
 **
 ******************************************************/
 bool FileDataSet::save()
@@ -788,7 +799,7 @@ bool FileDataSet::save()
 		return false;
 	}
 
-  c = filename.Right( 3 );
+	c = filename.Right( 3 );
 	if ( c.CmpNoCase( wxT( "jhd" )) == 0 )
 	{
 		wxLogError( wxT( "Can't write jhd format!" ));
@@ -804,11 +815,11 @@ bool FileDataSet::save()
 	wxTextFile wfile( filename );
 	if ( wfile.Exists() )
 	{
-	 wxRemove( filename );
+		wxRemove( filename );
 	}
-	if ( ! wfile.Create( filename ) ) 
+	if ( ! wfile.Create( filename ) )
 	{
-		wxLogError( wxT( "Cannot create file!" )); 
+		wxLogError( wxT( "Cannot create file!" ));
 		return false;
 	}
 
@@ -836,7 +847,7 @@ bool FileDataSet::save()
 	c.Printf( wxT( "%2.8f" ), getLocation()->getDST() );
 	wfile.AddLine( c );
 	wfile.AddLine( getLocation()->getLocName() );
-	
+
 	double lon = getLocation()->getLongitude() + .000005;
 	double lat = getLocation()->getLatitude() + .000005;
 
@@ -845,7 +856,7 @@ bool FileDataSet::save()
 	c.Printf( wxT( "%2.8f" ), lat );
 	wfile.AddLine( c );
 	wxStringTokenizer t( getRemark(), Endl, wxTOKEN_RET_EMPTY );
-	while( t.HasMoreTokens() )
+	while ( t.HasMoreTokens() )
 	{
 		c = t.GetNextToken();
 		wfile.AddLine( c );
@@ -858,7 +869,7 @@ bool FileDataSet::save()
 
 /*****************************************************
 **
-**   LocationEntry   ---   Constructor 
+**   LocationEntry   ---   Constructor
 **
 ******************************************************/
 LocationEntry::LocationEntry()
@@ -872,7 +883,7 @@ LocationEntry::LocationEntry()
 
 /*****************************************************
 **
-**   LocationEntry   ---   Constructor 
+**   LocationEntry   ---   Constructor
 **
 ******************************************************/
 LocationEntry::LocationEntry( wxString n, wxString c, wxString s, const double &lon, const double &lat, const double &tz )
@@ -889,14 +900,14 @@ LocationEntry::LocationEntry( wxString n, wxString c, wxString s, const double &
 
 /*****************************************************
 **
-**   LocationEntry   ---   Constructor 
+**   LocationEntry   ---   Constructor
 **
 ******************************************************/
 LocationEntry::LocationEntry( const wxString &s )
 {
 	wxString buf;
 	wxStringTokenizer t( s, wxT( ";" ));
-	
+
 	name = t.GetNextToken();
 	country = t.GetNextToken();
 	state = t.GetNextToken();
@@ -909,7 +920,7 @@ LocationEntry::LocationEntry( const wxString &s )
 
 	buf = t.GetNextToken();
 	timeZone = myatof( buf );
-}  
+}
 
 /**************************************************************
 ***
@@ -935,7 +946,7 @@ void LocationEntry::operator=( const LocationEntry& e )
 	latitude = e.latitude;
 	timeZone = e.timeZone;
 }
-  
+
 /**************************************************************
 ***
 ** LocationEntry   ---   operator ==
@@ -952,6 +963,122 @@ bool LocationEntry::operator==( const LocationEntry& e )
 	return true;
 }
 
-  
+static bool abbreviate_tzname = false;
+
+/**************************************************************
+***
+**   TzUtil   ---   getDateFormatted
+***
+***************************************************************/
+TzFormattedDate TzUtil::getDateFormatted( const double &jd, const bool &isLocal )
+{
+	/*
+	* CAUTION
+	* format specifiers don't work on MinGW: %F %z
+	*/
+	Formatter *formatter = Formatter::get();
+
+	TzFormattedDate fd;
+	wxDateTime dt( jd );
+	wxDateTime::TimeZone tz = isLocal ? wxDateTime::Local : wxDateTime::UTC;
+
+	double tzoffset = 0;
+	if ( isLocal )
+	{
+		TzInfo tzinfo = calculateTzInfoForJD( jd );
+		tzoffset = tzinfo.tzhours + tzinfo.dsthours;
+	}	
+
+	fd.dateFormatted = formatter->getDateStringFromJD( jd + tzoffset / 24.0 );
+
+	double tzminutes = tzoffset - (int)tzoffset;
+	tzminutes *= 60;
+
+	fd.fullDateTimeTzFormatted = wxString::Format( wxT( "%s %c%d:%02d" ),
+		(const wxChar*)formatter->getFullDateStringFromJD( jd + tzoffset / 24.0 ), tzoffset < 0 ? '-' : '+', (int)fabs( tzoffset ), (int)tzminutes );
+
+	int year = dt.GetYear();
+
+	// ugly workaround: tz formatting crashes for dates before 1970
+	if ( year >= 1970 )
+	{
+		if ( ! abbreviate_tzname ) fd.timezoneFormatted = dt.Format( wxT( "%Z" ), tz );
+		if ( fd.timezoneFormatted.Length() > 5 ) abbreviate_tzname = true;
+		if ( abbreviate_tzname ) fd.timezoneFormatted = dt.Format( wxT( "%z" ), tz );
+	}
+	fd.timeFormatted = dt.Format( wxT( "%H:%M:%S" ), tz );
+
+	fd.weekDay = dt.Format( wxT( "%a" ), tz );
+	fd.dayOfMonth = dt.GetDay( tz );
+	return fd;
+}
+
+/**************************************************************
+***
+**   TzUtil   ---   getCurrentTzInfo
+***
+***************************************************************/
+TzInfo TzUtil::getCurrentTzInfo()
+{
+	time_t tt;
+	time( &tt );
+	return calculateTzInfo( tt );
+}
+
+/**************************************************************
+***
+**   TzUtil   ---   calculateTzInfoForJD
+***
+***************************************************************/
+TzInfo TzUtil::calculateTzInfoForJD( const double &jd )
+{
+	double d = 86400.0 * ( jd - 2440587.5 );
+	return calculateTzInfo( (time_t)d );
+}
+
+/**************************************************************
+***
+**   TzUtil   ---   calculateTzInfo
+***
+***************************************************************/
+TzInfo TzUtil::calculateTzInfo( const time_t &tt )
+{
+	TzInfo tzinfo;
+	wxDateTime dt( tt );
+
+	tzinfo.localTimeFormatted = dt.Format( wxT( "%H:%M:%S" ));
+
+	wxDateTime utdt = dt.ToUTC();
+	tzinfo.utcTimeFormatted = utdt.Format( wxT( "%H:%M:%S" ));
+
+	wxTimeSpan span = dt - utdt;
+	int tzminutes = span.GetMinutes();
+	tzinfo.tzhours = tzminutes;
+	tzinfo.tzhours /= 60;
+
+	tzminutes = (int)a_red( tzminutes, 60 );
+
+	tzinfo.isdst = dt.IsDST();
+	switch ( tzinfo.isdst )
+	{
+	case 0:
+		tzinfo.dstOffsetFormatted = wxT( "+00:00" );
+		tzinfo.dsthours = 0;
+		break;
+	case 1:
+		tzinfo.dstOffsetFormatted = wxT( "+01:00" );
+		tzinfo.tzhours -= 1;
+		tzinfo.dsthours = 1;
+		break;
+	default:
+		tzinfo.dstOffsetFormatted = wxT( "unknown" );
+		tzinfo.dsthours = 0;
+		break;
+	}
+	tzinfo.tzOffsetFormatted = wxString::Format( wxT( "%c%02d:%02d" ),
+		tzinfo.tzhours < 0 ? '-' : '+', abs( (int)tzinfo.tzhours ), tzminutes );
+	return tzinfo;
+}
+
 
 
