@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/base/Lang.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "Lang.h"
+#pragma implementation "Lang.h"
 #endif
 
 #include "Lang.h"
@@ -35,9 +35,6 @@
 #include "LanguageConfig.h"
 #include "mathbase.h"
 
-// round degrees, 5% of a second of arc
-#define ROUND_DEGREES .005 / 3600
-
 extern Config *config;
 
 IMPLEMENT_SINGLETON( Formatter )
@@ -72,22 +69,25 @@ const wxString Lang::getNakshatraName( const int &nak, const int &nsys, const in
 	assert(( nsys == N27 ) || ( nsys == N28 ));
 	assert(( nsys > 0 ) || ( nsys < ( nsys == N27 ? 27 : 28 ) ));
 
-	const wxChar *k_nakshatra_name_large[28]  = { _( "Aswini" ), _( "Bharani" ), _( "Krittika" ), _( "Rohini" ), 
+	const static wxChar *k_nakshatra_name_large[28]  = { _( "Aswini" ), _( "Bharani" ), _( "Krittika" ), _( "Rohini" ),
 		_( "Mrigasira" ), _( "Ardra" ), _( "Punarvasu" ), _( "Pushyami" ), _( "Aslesha" ), _( "Magha" ),
 		_( "Poorvaphalguni" ), _( "Uttaraphalguni" ), _( "Hasta" ), _( "Chitra" ), _( "Swati" ), _( "Visakha" ),
 		_( "Anuradha" ), _( "Jyeshta" ), _( "Moola" ), _( "Poorvashadha" ), _( "Uttarashadha" ), _( "Abhijit" ),
-		_( "Sravana" ), _( "Dhanista" ), _( "Satabhisha" ), _( "Poorvabhadra" ), _( "Uttarabhadra" ), _( "Revati" ) };
-	const wxChar *k_nakshatra_name_medium[28]  = {
+		_( "Sravana" ), _( "Dhanista" ), _( "Satabhisha" ), _( "Poorvabhadra" ), _( "Uttarabhadra" ), _( "Revati" )
+	};
+	const static wxChar *k_nakshatra_name_medium[28]  = {
 		_( "Aswini" ), _( "Bharani" ), _( "Krittika" ), _( "Rohini" ), _( "Mrigasira" ), _( "Ardra" ),
 		_( "Punarvasu" ), _( "Pushyami" ), _( "Aslesha" ), _( "Magha" ), _( "P.Phalguni" ), _( "U.Phalguni" ),
 		_( "Hasta" ), _( "Chitra" ), _( "Swati" ), _( "Visakha" ), _( "Anuradha" ), _( "Jyeshta" ),
 		_( "Moola" ), _( "P.Shadha" ), _( "U.Shadha" ), _( "Abhijit" ), _( "Sravana" ), _( "Dhanista" ),
-		_( "Satabisha" ), _( "P.Bhadra" ), _( "U.Bhadra" ), _( "Revati" ) };
-	const wxChar *k_nakshatra_name_short[28]  = { _( "Asw" ), _( "Bhr" ), _( "Kri" ), _( "Roh" ), 
+		_( "Satabisha" ), _( "P.Bhadra" ), _( "U.Bhadra" ), _( "Revati" )
+	};
+	const static wxChar *k_nakshatra_name_short[28]  = { _( "Asw" ), _( "Bhr" ), _( "Kri" ), _( "Roh" ),
 		_( "Mri" ), _( "Ard" ), _( "Pun" ), _( "Pus" ), _( "Asl" ), _( "Mak" ), _( "PPl" ),
 		_( "UPl" ), _( "Hst" ), _( "Cit" ), _( "Sva" ), _( "Vis" ), _( "Anu" ), _( "Jye" ),
 		_( "Mul" ), _( "PSa" ), _( "USa" ), _( "Abh" ), _( "Sra" ), _( "Dha" ), _( "Sat" ),
-		_( "PBa" ), _( "UBa" ), _( "Rev" ) };
+		_( "PBa" ), _( "UBa" ), _( "Rev" )
+	};
 	// shift index after  U. shadha for system of 27 nakshatras
 	const int index = (( nsys == N27 ) && ( nak > 20 )) ? nak+1 : nak;
 
@@ -146,13 +146,22 @@ const wxString Lang::getObjectName( const int &num, const int &format, const boo
 	}
 	else assert( false );
 
-	if ( format == TLARGE ) {	return ret; }
+	if ( format == TLARGE ) {
+		return ret;
+	}
 	else if ( format == TMEDIUM )
 	{
 		if ( num == OMERIDIAN ) return _( "MC" );
+		else if ( ! vedic )
+		{
+			if ( num == OTRUENODE || num == OMEANNODE ) ret = _( "NNo" );
+			if ( num == OTRUEDESCNODE || num == OMEANDESCNODE ) ret = _( "SNo" );
+		}
 		return ret.Left( 3 );
 	}
-	else { return ret.Left( 2 ); }
+	else {
+		return ret.Left( 2 );
+	}
 }
 
 /**************************************************************
@@ -163,10 +172,12 @@ const wxString Lang::getObjectName( const int &num, const int &format, const boo
 const wxString Lang::getYoniName( const int &index ) const
 {
 	assert( index >= 0 && index < 14 );
-	const wxChar *k_yoni_name_sk[14] = { wxT( "Ashwa"), wxT( "Mahish" ), wxT( "Simha" ), wxT( "Gaja" ), wxT( "Mesha" ), wxT( "Vanar" ),
-		wxT( "Nakul" ), wxT( "Sarpa" ), wxT( "Mriga" ), wxT( "Swan" ), wxT( "Marjar" ), wxT( "Mushak" ), wxT( "Vyaghra" ), wxT( "Gow" ) };
-	const wxChar *k_yoni_name[14] = { _( "Horse" ), _( "Buffalo" ), _( "Lion" ), _( "Elephant" ), _( "Goat" ), _( "Monkey" ),
-	_( "Mongoose" ), _( "Serpent" ), _( "Deer" ), _( "Dog" ), _( "Cat" ), _( "Rat" ), _( "Tiger" ), _( "Cow"  ) };
+	const wxChar *k_yoni_name_sk[14] = { wxT( "Ashwa"), wxT( "Mahish" ), wxT( "Simha" ), wxT( "Gaja" ),
+		wxT( "Mesha" ), wxT( "Vanar" ), wxT( "Nakul" ), wxT( "Sarpa" ), wxT( "Mriga" ), wxT( "Swan" ),
+		wxT( "Marjar" ), wxT( "Mushak" ), wxT( "Vyaghra" ), wxT( "Gow" ) };
+	const wxChar *k_yoni_name[14] = { _( "Horse" ), _( "Buffalo" ), _( "Lion" ), _( "Elephant" ),
+		_( "Goat" ), _( "Monkey" ), _( "Mongoose" ), _( "Serpent" ), _( "Deer" ), _( "Dog" ), _( "Cat" ),
+		_( "Rat" ), _( "Tiger" ), _( "Cow"  ) };
 	return cfg->useVedicPlanetNames ? k_yoni_name_sk[index] : k_yoni_name[index];
 }
 
@@ -189,9 +200,12 @@ const wxString Lang::getGanaName( const int &index ) const
 ***************************************************************/
 const wxString Lang::getTaraName( const int &index ) const
 {
-	assert( index >= 0 && index < 9 );
+	assert( index >= -1 && index < 9 );
+
+	if ( index == -1 ) return _( "none" );
+
 	const wxChar *k_tara_name[9] = { _( "Janma" ), _( "Sampat" ), _( "Vipat" ), _( "Kshema" ),
-    _( "Pratyak" ), _( "Sadhaka" ), _( "Vadha" ), _( "Mitra" ), _( "Parma Mitra" ) };
+		_( "Pratyak" ), _( "Sadhaka" ), _( "Vadha" ), _( "Mitra" ), _( "Parma Mitra" ) };
 
 	return k_tara_name[index];
 }
@@ -241,14 +255,12 @@ const wxString Lang::getDignityName( const int &index, const int& format ) const
 {
 	assert( index >= DIGNITY_EXALTED && index <= DIGNITY_ADHISATRU );
 
-	const wxChar *k_dignity_name_sk[9] = { wxT( "Uchcha" ), wxT( "Nicha" ), wxT( "Moolatrikona" ), wxT( "Svakshetra" ), wxT( "Adhimitra" ),
-		wxT( "Mitra" ), wxT( "Sama" ), wxT( "Satru" ), wxT( "Adhisatru" ) };
-
-	const wxChar *k_dignity_name[9] = { _( "Exalted" ), _( "Debilated" ), _( "Moolatrikona" ), _( "Own" ), _( "Great friend" ),
-	_( "Friend" ), _( "Neutral" ), _( "Enemy" ), _( "Great enemy" ) };
-
-	const wxChar *k_dignity_short[9] = { _( "Exal" ), _( "Deb" ), _( "Mool" ), _( "Own" ), _( "Gr.Fr" ),
-	_( "Frnd" ), _( "Neutr" ), _( "Enem" ), _( "Gr.En" ) };
+	const static wxChar *k_dignity_name_sk[9] = { _( "Uchcha" ), _( "Nicha" ), _( "Moolatrikona" ),
+		_( "Svakshetra" ), _( "Adhimitra" ), _( "Mitra" ), _( "Sama" ), _( "Satru" ), _( "Adhisatru" ) }; 
+	const static wxChar *k_dignity_name[9] = { _( "Exalted" ), _( "Debilated" ), _( "Moolatrikona" ),
+		_( "Own" ), _( "Great friend" ), _( "Friend" ), _( "Neutral" ), _( "Enemy" ), _( "Great enemy" ) }; 
+	const static wxChar *k_dignity_short[9] = { _( "Exal" ), _( "Deb" ), _( "Mool" ), _( "Own" ),
+		_( "Gr.Fr" ), _( "Frnd" ), _( "Neutr" ), _( "Enem" ), _( "Gr.En" ) };
 
 	if ( format == TSHORT )	return k_dignity_short[index];
 	else return cfg->useVedicPlanetNames ? k_dignity_name_sk[index] : k_dignity_name[index];
@@ -264,13 +276,10 @@ const wxString Lang::getVimsopakaDignityName( const int &index, const int &digni
 	assert( index >= 0 && index < 4 && dignity >= 0 );
 
 	const wxChar *shadvarga_dignity[5] = { _( "Kimsuka" ), _( "Vyanjana" ), _( "Chamara" ), _( "Chatra" ), _( "Kundala" ) };
-
-	const wxChar *saptavarga_dignity[6] = { _( "Kimsuka" ), _( "Vyanjana" ), _( "Chamara" ), _( "Chatra" ), _( "Kundala" ),
-		_( "Mukuta" ) };
-
+	const wxChar *saptavarga_dignity[6] = { _( "Kimsuka" ), _( "Vyanjana" ), _( "Chamara" ), _( "Chatra" ),
+		_( "Kundala" ), _( "Mukuta" ) };
 	const wxChar *dasavarga_dignity[9] = { _( "Parijata" ), _( "Uttama" ),	_( "Gopura" ), _( "Simhasana" ),
 		_( "Paravata" ), _( "Devaloka" ), _( "Brahmaloka" ), _( "Sakravahana" ), _( "Sridhama"  ) };
-
 	const wxChar *shodasavarga_dignity[15] = { _( "Bhedaka" ), _( "Kusumia"), _( "Nagapushpa" ), _( "Kanduka" ),
 		_( "Kerala" ), _( "Kalpa Vriksha" ), _( "Chandana Vana" ), _( "PoornaChandra" ), _( "Uchchaisrava" ),
 		_( "Dhanvantari" ), _( "Suryakanta" ), _( "Vidruma" ), _( "Chakra Simhasana" ), _( "Goloki" ), _( "Sri Vallabha" ) };
@@ -279,24 +288,24 @@ const wxString Lang::getVimsopakaDignityName( const int &index, const int &digni
 
 	switch ( index )
 	{
-		case 0:
-			assert( dignity <= 6 );
-			return shadvarga_dignity[dignity-2];
+	case 0:
+		assert( dignity <= 6 );
+		return shadvarga_dignity[dignity-2];
 		break;
-		case 1:
-			assert( dignity <= 7 );
-			return saptavarga_dignity[dignity-2];
+	case 1:
+		assert( dignity <= 7 );
+		return saptavarga_dignity[dignity-2];
 		break;
-		case 2:
-			assert( dignity <= 10 );
-			return dasavarga_dignity[dignity-2];
+	case 2:
+		assert( dignity <= 10 );
+		return dasavarga_dignity[dignity-2];
 		break;
-		case 3:
-			assert( dignity <= 16 );
-			return shodasavarga_dignity[dignity-2];
+	case 3:
+		assert( dignity <= 16 );
+		return shodasavarga_dignity[dignity-2];
 		break;
-		default:
-			assert( false );
+	default:
+		assert( false );
 		break;
 	}
 	return wxEmptyString;
@@ -311,36 +320,36 @@ const wxString Lang::getTithiName( const int &index ) const
 {
 	assert( index >= 0 && index < 30 );
 	const wxChar *k_tithi_name[30] = {
-	_( "Sukla - Pratipad" ),
-	_( "Sukla - Dviteeya" ),
-	_( "Sukla - Triteeya" ),
-	_( "Sukla - Chaturthi" ),
-	_( "Sukla - Panchami" ),
-	_( "Sukla - Shasti" ),
-	_( "Sukla - Saptami" ),
-	_( "Sukla - Asthami" ),
-	_( "Sukla - Navami" ),
-	_( "Sukla - Dasami" ),
-	_( "Sukla - Ekadasi" ),
-	_( "Sukla - Dvadasi" ),
-	_( "Sukla - Trayodasi" ),
-	_( "Sukla - Chaturdasi" ),
-	_( "Sukla - Purnima" ),
-	_( "Krishna - Pratipad" ),
-	_( "Krishna - Dviteeya" ),
-	_( "Krishna - Triteeya" ),
-	_( "Krishna - Chaturthi" ),
-	_( "Krishna - Panchami" ),
-	_( "Krishna - Shasti" ),
-	_( "Krishna - Saptami" ),
-	_( "Krishna - Asthami" ),
-	_( "Krishna - Navami" ),
-	_( "Krishna - Dasami" ),
-	_( "Krishna - Ekadasi" ),
-	_( "Krishna - Dvadasi" ),
-	_( "Krishna - Trayodasi" ),
-	_( "Krishna - Chaturdasi" ),
-	_( "Krishna - Amavasya" ),
+		_( "Sukla - Pratipad" ),
+		_( "Sukla - Dviteeya" ),
+		_( "Sukla - Triteeya" ),
+		_( "Sukla - Chaturthi" ),
+		_( "Sukla - Panchami" ),
+		_( "Sukla - Shasti" ),
+		_( "Sukla - Saptami" ),
+		_( "Sukla - Asthami" ),
+		_( "Sukla - Navami" ),
+		_( "Sukla - Dasami" ),
+		_( "Sukla - Ekadasi" ),
+		_( "Sukla - Dvadasi" ),
+		_( "Sukla - Trayodasi" ),
+		_( "Sukla - Chaturdasi" ),
+		_( "Sukla - Purnima" ),
+		_( "Krishna - Pratipad" ),
+		_( "Krishna - Dviteeya" ),
+		_( "Krishna - Triteeya" ),
+		_( "Krishna - Chaturthi" ),
+		_( "Krishna - Panchami" ),
+		_( "Krishna - Shasti" ),
+		_( "Krishna - Saptami" ),
+		_( "Krishna - Asthami" ),
+		_( "Krishna - Navami" ),
+		_( "Krishna - Dasami" ),
+		_( "Krishna - Ekadasi" ),
+		_( "Krishna - Dvadasi" ),
+		_( "Krishna - Trayodasi" ),
+		_( "Krishna - Chaturdasi" ),
+		_( "Krishna - Amavasya" ),
 	};
 	return k_tithi_name[index];
 }
@@ -353,7 +362,6 @@ const wxString Lang::getTithiName( const int &index ) const
 const wxString Lang::getWeekdayName( const int &day ) const
 {
 	assert(( day >= 0 ) || ( day < 7 ));
-
 	static const wxChar *wd[7] = { _("Sunday"), _("Monday"), _("Tuesday"), _("Wednesday"),
 		_("Thursday"), _("Friday"), _("Saturday") };
 	return wd[day];
@@ -383,13 +391,13 @@ const wxString Lang::getPlanetSymbolCode( const int &i ) const
 		wxT( "}" ) // Lilith
 	};
 
-	switch( i )
+	switch ( i )
 	{
-		case OURANUS:
-			return cfg->uranusSymbol ? wxT( "[" ) :   k_symbol[i];
+	case OURANUS:
+		return cfg->uranusSymbol ? wxT( "[" ) :   k_symbol[i];
 		break;
-		case OPLUTO:
-			return cfg->plutoSymbol ? wxT( "\\" ) :   k_symbol[i];
+	case OPLUTO:
+		return cfg->plutoSymbol ? wxT( "\\" ) :   k_symbol[i];
 		break;
 	}
 	return k_symbol[i];
@@ -402,17 +410,19 @@ const wxString Lang::getPlanetSymbolCode( const int &i ) const
 ***************************************************************/
 const wxChar Lang::getSignSymbolCode( const int &i, const int type ) const
 {
-	switch( type )
+	switch ( type )
 	{
-		case 0:
-			if ( i == CAPRICORN && cfg->capricornSymbol ) return 'Z';
-			else return (wxChar)( i + 'A' );
+	case 0:
+		if ( i == CAPRICORN && cfg->capricornSymbol ) return 'Z';
+		else return (wxChar)( i + 'A' );
 		break;
-		case 1:
-			return (wxChar)( i + '!' );
+	/*
+	case 1:
+		return (wxChar)( i + '!' );
 		break;
-		default:
-			return (wxChar)( i + 'A' );
+	*/
+	default:
+		return (wxChar)( i + 'A' );
 		break;
 	}
 }
@@ -429,14 +439,21 @@ const wxString Lang::getSignName( const int &i, const int format ) const
 
 	const wxChar *k_sign[12]={ _( "Ar" ), _( "Ta" ), _( "Ge" ), _( "Cn" ), _( "Le" ), _( "Vi" ),
 		_( "Li" ), _( "Sc" ), _( "Sa" ), _( "Cp" ), _( "Aq" ), _( "Pi" ) };
-	const wxChar *k_sign_full[12]={ _( "Aries" ), _( "Taurus" ), _( "Gemini" ), _( "Cancer" ), _( "Leo" ),
-		_( "Virgo" ), _( "Libra" ), _( "Scorpio" ), _( "Sagittarius" ), _( "Capricorn" ), _( "Aquarius" ), _( "Pisces" )};
-	const wxChar *k_sign_full_vedic[12]={ wxT( "Mesha" ), wxT( "Vrishabha" ), wxT( "Mithuna" ), wxT( "Kataka" ), wxT( "Simha" ),
-		wxT( "Kanya" ), wxT( "Thula" ), wxT( "Vrischika" ), wxT( "Dhanus" ), wxT( "Makara" ), wxT( "Kumbha" ), wxT( "Meena" )};
-	const wxChar *k_sign_medium_vedic[12]={ wxT( "Mes" ), wxT( "Vrs" ), wxT( "Mit" ), wxT( "Kat" ), wxT( "Sim" ),
-		wxT( "Kan" ), wxT( "Thu" ), wxT( "Vrc" ), wxT( "Dha" ), wxT( "Mak" ), wxT( "Kum" ), wxT( "Mee" )};
-	const wxChar *k_sign_vedic[12]={ wxT( "Ms" ), wxT( "Vs" ), wxT( "Mi" ), wxT( "Kt" ), wxT( "Si" ),
-		wxT( "Kn" ), wxT( "Th" ), wxT( "Vc" ), wxT( "Dh" ), wxT( "Mk" ), wxT( "Ku" ), wxT( "Mn" )};
+
+	const wxChar *k_sign_full[12]={ _( "Aries" ), _( "Taurus" ), _( "Gemini" ), _( "Cancer" ),
+		_( "Leo" ), _( "Virgo" ), _( "Libra" ), _( "Scorpio" ), _( "Sagittarius" ), _( "Capricorn" ),
+		_( "Aquarius" ), _( "Pisces" ) };
+
+	const wxChar *k_sign_full_vedic[12]={ wxT( "Mesha" ), wxT( "Vrishabha" ), wxT( "Mithuna" ),
+		wxT( "Kataka" ), wxT( "Simha" ), wxT( "Kanya" ), wxT( "Thula" ), wxT( "Vrischika" ),
+		wxT( "Dhanus" ), wxT( "Makara" ), wxT( "Kumbha" ), wxT( "Meena" ) };
+
+	const wxChar *k_sign_medium_vedic[12]={ wxT( "Mes" ), wxT( "Vrs" ), wxT( "Mit" ),
+		wxT( "Kat" ), wxT( "Sim" ), wxT( "Kan" ), wxT( "Thu" ), wxT( "Vrc" ), wxT( "Dha" ),
+		wxT( "Mak" ), wxT( "Kum" ), wxT( "Mee" ) };
+
+	const wxChar *k_sign_vedic[12]={ wxT( "Ms" ), wxT( "Vs" ), wxT( "Mi" ), wxT( "Kt" ),
+	wxT( "Si" ), wxT( "Kn" ), wxT( "Th" ), wxT( "Vc" ), wxT( "Dh" ), wxT( "Mk" ), wxT( "Ku" ), wxT( "Mn" ) };
 
 	if ( format == TLARGE )
 	{
@@ -464,7 +481,8 @@ const wxString Lang::getSignName( const int &i, const int format ) const
 const wxString Lang::getKarakaName( const int &i, const int format ) const
 {
 	const wxChar *chara_karaka_name_long[8] = { _( "Atma" ), _( "Amatya" ), _( "Bhratru" ),
-		_( "Matru" ), _( "Pitru" ), _( "Putra" ), _( "Gnati" ), _( "Dhanya"  ) };
+	        _( "Matru" ), _( "Pitru" ), _( "Putra" ), _( "Gnati" ), _( "Dhanya"  )
+	                                          };
 	assert( i >= 0 && i < 8 );
 	return chara_karaka_name_long[i];
 }
@@ -501,10 +519,8 @@ const wxString Lang::getShastiamsaName( const int &i ) const
 ***************************************************************/
 const wxString Lang::getBhavaName( const int &i ) const
 {
-	const wxChar *k_bhava_name[12] = { _( "Lagna" ), _( "Dhana" ), _( "Sahaja" ), _( "Bandhu" ),
-	_( "Putra" ), _( "Satru" ), _( "Yuvati" ), _( "Randhra" ), _( "Dharma" ), _( "Karma" ),
-	_( "Labha" ), _( "Vyaya" ) };
-	
+	const wxChar *k_bhava_name[12] = { _( "Lagna" ), _( "Dhana" ), _( "Sahaja" ), _( "Bandhu" ), _( "Putra" ),
+		_( "Satru" ), _( "Yuvati" ), _( "Randhra" ), _( "Dharma" ), _( "Karma" ), _( "Labha" ), _( "Vyaya" ) }; 
 	assert( i >= 0 && i < 12 );
 	return k_bhava_name[i];
 }
@@ -517,39 +533,38 @@ const wxString Lang::getBhavaName( const int &i ) const
 const wxString Lang::getMonthName( const int &i ) const
 {
 	assert( i >= 0 && i < 12 );
-	const wxChar *month_name[12] = { _( "January" ), _( "February" ), _( "March" ), _( "April" ),
-		_( "May" ), _( "June" ), _( "July" ),
-		_( "August" ), _( "September" ), _( "October" ), _( "November" ), _( "December" ) };
+	const wxChar *month_name[12] = { _( "January" ), _( "February" ), _( "March" ), _( "April" ), _( "May" ),
+		_( "June" ), _( "July" ), _( "August" ), _( "September" ), _( "October" ), _( "November" ), _( "December" ) };
 	return month_name[i];
 }
 
 /*****************************************************
 **
-**   Lang   ---   getAyanamsaName 
+**   Lang   ---   getAyanamsaName
 **
 ******************************************************/
 const wxString Lang::getAyanamsaName( const int &ayatype )
 {
 	wxString s;
-	switch( ayatype )
+	switch ( ayatype )
 	{
-		case 0:
-			s= _( "None" );
+	case 0:
+		s= _( "None" );
 		break;
-		case 1:
-			s = _( "Lahiri" );
+	case 1:
+		s = _( "Lahiri" );
 		break;
-		case 2:
-			s = _( "Raman" );
+	case 2:
+		s = _( "Raman" );
 		break;
-		case 3:
-			s = _( "Krishnamurti" );
+	case 3:
+		s = _( "Krishnamurti" );
 		break;
-		case 4:
-			s.Printf( wxT( "Custom (t0 %8.1f ayan_t0 %2.8f)" ), cfg->custom_t0, cfg->custom_ayan_t0 );
+	case 4:
+		s.Printf( wxT( "Custom (t0 %8.1f ayan_t0 %2.8f)" ), cfg->custom_t0, cfg->custom_ayan_t0 );
 		break;
-		default:
-			s= wxT( "Unknown" );
+	default:
+		s= wxT( "Unknown" );
 		break;
 	}
 	return s;
@@ -557,170 +572,191 @@ const wxString Lang::getAyanamsaName( const int &ayatype )
 
 /*****************************************************
 **
-**   Formatter   ---   getDegMinSecInts2 
+**   Formatter   ---   getDegMinSecInts2
 **
 ******************************************************/
 void Formatter::getDegMinSecInts2( const double &len, int &deg, int &min, double &sec )
 {
 	double mylen = fabs( len );
-  deg = (int)mylen;
-  double mind = (double)((mylen - (double)deg ) * 60);
-  min = (int)mind;
-  sec = ( mind - (double)min ) * 60;
+	deg = (int)mylen;
+	double mind = (double)((mylen - (double)deg ) * 60);
+	min = (int)mind;
+	sec = ( mind - (double)min ) * 60;
 	if ( len < 0 ) deg *= -1;
 }
 
 /*****************************************************
 **
-**   Formatter   ---   getDegMinSecInts 
+**   Formatter   ---   getDegMinSecInts
 **
 ******************************************************/
 void Formatter::getDegMinSecInts( const double &len, int &deg, int &min, int &sec )
 {
-	double mylen = fabs( len ) + ROUND_DEGREES;
-  deg = (int)mylen;
-  double mind = (double)((mylen - (double)deg ) * 60);
-  min = (int)mind;
-  sec = (int)(( mind - (double)min ) * 60 );
+	double mylen = fabs( len );
+	deg = (int)mylen;
+	double mind = (double)((mylen - (double)deg ) * 60);
+	min = (int)mind;
+	sec = (int)(( mind - (double)min ) * 60 );
 	if ( len < 0 ) deg *= -1;
 }
 
 /*****************************************************
 **
-**   Formatter   ---   getLenFormatted 
+**   Formatter   ---   getDegMinSecFormatted
 **
 ******************************************************/
-const wxString Formatter::getLenFormatted( const double &len, const int &format )
+const wxString Formatter::getDegMinSecFormatted( double t, const int &format, const double &minvalue, const double &maxvalue )
 {
-	int deg, min, sec;
-	double sec2;
-	wxString s;
+	double sec = 0;
+	int deg = 0;
+	int min = 0;
 
-	if (format == DEG_PRECISION_MORE )
+	if ( t < minvalue || t > maxvalue )
 	{
-		getDegMinSecInts2( len, deg, min, sec2 );
-		s.Printf( wxT( "%c%02d:%02d:%02.5f" ), ( len < 0 ? '-' : ' ' ), (int)fabs( (double)deg ), min, sec2 );
+		printf( "Formatter::getDegMinSecFormatted WARN invalid value %f, minvalue was %f maxvalue %f\n", t, minvalue, maxvalue );
+		t = a_red( t, maxvalue );
 	}
-	else if (format == DEG_PRECISION_SECOND )
+
+	getDegMinSecInts2( t, deg, min, sec );
+
+	if ( format == DEG_PRECISION_MORE )
 	{
-		getDegMinSecInts( len, deg, min, sec );
-		s.Printf( wxT( "%c%02d:%02d:%02d" ),  ( len < 0 ? '-' : ' ' ), (int)fabs( (double)deg ), min, sec );
+		return wxString::Format( wxT( "%02d:%02d:%02d.%02d" ), deg, min, (int)sec, (int)(.5 + 100 * ( sec - (int)sec )));
 	}
 	else
 	{
-		getDegMinSecInts( len, deg, min, sec );
-		s.Printf( wxT( "%c%02d:%02d" ),  ( len < 0 ? '-' : ' ' ), (int)fabs( (double)deg ), min );
+		sec = a_rund( sec );
+		if ( sec == 60 )
+		{
+			sec = 0;
+			min++;
+			if ( min == 60 )
+			{
+				min = 0;
+				deg++;
+			}
+		}
 	}
+	if ( format == DEG_PRECISION_SECOND || ( DEG_PRECISION_FLEXIBLE && sec != 0 ))
+	{
+		return wxString::Format( wxT( "%02d:%02d:%02d" ), deg, min, (int)sec );
+	}
+	else
+	{
+		if ( sec >= 30 ) min++;
+		if ( min == 60 )
+		{
+			min = 0;
+			deg++;
+		}
+		return wxString::Format( wxT( "%02d:%02d" ), deg, min );
+	}
+}
+
+/*****************************************************
+**
+**   Formatter   ---   getLenFormatted
+**
+******************************************************/
+const wxString Formatter::getLenFormatted( const double &len, const int format )
+{
+	wxString s;
+	if ( len < 0 ) s = wxT( "-" );
+	s <<  getDegMinSecFormatted( fabs( len ), format, 0.0, 360.0 );
 	return s;
 }
 
 /*****************************************************
 **
-**   Formatter   ---   getTimeFormatted 
+**   Formatter   ---   getLatitudeFormatted
 **
 ******************************************************/
-const wxString Formatter::getTimeFormatted( const double &t )
+const wxString Formatter::getLatitudeFormatted( const double &l, const int format )
 {
 	wxString s;
-	if (( t > 24 ) || ( t < 0 ))
-		wxLogError( wxT( "Warning: time value is %f" ), t );
+	s <<  getDegMinSecFormatted( fabs( l ), format, 0.0, 90.0 );
 
+	if ( l > 0 ) s << wxT( "N" );
+	else if ( l < 0 ) s << wxT( "S" );
+	return s;
+}
+
+/*****************************************************
+**
+**   Formatter   ---   getLongitudeFormatted
+**
+******************************************************/
+const wxString Formatter::getLongitudeFormatted( const double &l, const int format )
+{
+	wxString s;
+	s <<  getDegMinSecFormatted( fabs( l ), format, 0.0, 180.0 );
+	if ( l > 0 ) s << wxT( "E" );
+	else if ( l < 0 ) s << wxT( "W" );
+	return s;
+}
+
+#define PARSE_FORMAT_STRICT
+
+/*****************************************************
+**
+**   Formatter   ---   parseDegMinSecAndFormat
+**
+******************************************************/
+wxString Formatter::parseDegMinSecAndFormat( wxString s, const int format )
+{
 	int deg, min, sec;
-	double mytime = a_red( t, 24 );
-	getDegMinSecInts( mytime, deg, min, sec );
 
-  s.Printf( wxT( "%02d:%02d:%02d" ), deg, min, sec );
-	return s;
+	bool b = getDegreeIntsFromString( (const wxChar*)s, deg, min, sec );
+	if ( ! b )
+	{
+		printf( "Formatter::parseDegreesAndFormat WARN: invalid string %s\n", str2char( s ));
+		return s;
+	}
+	double value = sec + min * 60 + deg * 3600;
+	value /= 3600.0;
+	wxString ret = getDegreesFormatted( value, format );
+
+#ifdef PARSE_FORMAT_STRICT
+	int deg2, min2, sec2;
+
+	bool b2 = getDegreeIntsFromString( (const wxChar*)ret, deg2, min2, sec2 );
+	if ( ! b2 )
+	{
+		printf( "ERROR Formatter::parseDegMinSecAndFormat: output parsing failed for string %s\n", str2char( ret )); 
+	}
+	if ( deg != deg2 || min != min2 || sec != sec2 )
+	{
+		printf( "ERROR Formatter::parseDegMinSecAndFormat: wrong values (rounding errors?): old %d %d %d, new %d %d %d,\n", deg, min, sec, deg2, min2, sec2 );
+	}
+#endif
+
+	return ret;
 }
 
 /*****************************************************
 **
-**   Formatter   ---   getDegreesFormatted 
-**
-******************************************************/
-const wxString Formatter::getDegreesFormatted( const double &t, const int p )
-{
-	wxString s;
-	int deg, min;
-	double sec;
-	//double mytime = a_red( t, 180 );
-
-	if (( t > 360 ) || ( t < 0 )) wxLogError( wxT( "Warning: time value is %f" ), t );
-	getDegMinSecInts2( t, deg, min, sec );
-
-	if ( p == DEG_PRECISION_MORE )
-		s.Printf( wxT( "%02d:%02d:%02d.%02d" ), deg, min, (int)sec, (int)(100 * ( sec - (int)sec )));
-	else s.Printf( wxT( "%02d:%02d:%02d" ), deg, min, (int)sec );
-	return s;
-}
-
-/*****************************************************
-**
-**   Formatter   ---   getLatitudeFormatted 
-**
-******************************************************/
-const wxString Formatter::getLatitudeFormatted( const double &l, const int p )
-{
-	wxString s;
-	s << getDegreesFormatted( fabs( l ), p );
-
-	if ( l > 0 ) s << wxT( " " ) << _( "North" );
-	else if ( l < 0 ) s << wxT( " " ) << _( "South" );
-	return s;
-}
-
-/*****************************************************
-**
-**   Formatter   ---   getLongitudeFormatted 
-**
-******************************************************/
-const wxString Formatter::getLongitudeFormatted( const double &l, const int p )
-{
-	wxString s;
-	s.Printf( wxT( "%s %s" ), (const wxChar*)getDegreesFormatted( fabs( l ), p ), ( l >= 0 ? _( "East" ) : _( "West" ) ));
-	return s;
-}
-
-/*****************************************************
-**
-**   Formatter   ---   getLenString 
+**   Formatter   ---   getLenString
 **
 ******************************************************/
+// unused
 const wxString Formatter::getLenString( const double &len, const int format, const int dir )
 {
 	wxString s;
 	Lang lang;
-	int deg, min, sec, sign;
-	wxChar d = 'D';
-	
+
+	s << ( dir == 1 ? 'R' : 'D' );
+
 	double mylen = red_deg( len );
-  sign = (int)( mylen / 30 );
+	const int sign = (int)( mylen / 30 );
 	mylen -= 30 * sign;
-	
-	getDegMinSecInts( mylen, deg, min, sec );
 
-	if ( dir == 1 ) d = 'R';
-
-  if ( format & FORMAT_SHOW_DIR ) 
-	{
-		if ( format & FORMAT_SHOW_SECONDS )
-			s.Printf( wxT( "%c %02d %s %02d %02d" ), d, deg, (const wxChar*)lang.getSignName(sign), min, sec );
-		else
-			s.Printf( wxT( "%c %02d %s %02d" ), d, deg, (const wxChar*)lang.getSignName(sign), min );
-	}
-	else
-	{
-		if ( format & FORMAT_SHOW_SECONDS )
-			s.Printf( wxT( "%02d %s %02d %02d" ), deg, (const wxChar*)lang.getSignName(sign), min, sec );
-		else
-			s.Printf( wxT( "%02d %s %02d" ), deg, (const wxChar*)lang.getSignName(sign), min );
-	}
+	s << getDegMinSecFormatted( mylen, format, 0.0, 30.0 ) << lang.getSignName(sign);
 	return s;
 }
 
 /*****************************************************
 **
-**   Formatter   ---   getDateIntsFromJD 
+**   Formatter   ---   getDateIntsFromJD
 **
 ******************************************************/
 void Formatter::getDateIntsFromJD( const double &jd, int &day, int &month, int &year, const int format )
@@ -731,7 +767,7 @@ void Formatter::getDateIntsFromJD( const double &jd, int &day, int &month, int &
 
 /*****************************************************
 **
-**   Formatter   ---   getFullDateStringFromJD 
+**   Formatter   ---   getFullDateStringFromJD
 **
 ******************************************************/
 const wxString Formatter::getFullDateStringFromJD( const double &jd, const int format )
@@ -743,14 +779,14 @@ const wxString Formatter::getFullDateStringFromJD( const double &jd, const int f
 	int l = LanguageConfig::get()->getLocale()->GetLanguage();
 	if ( l == wxLANGUAGE_GERMAN ) // German
 		s.Printf( wxT( "%02d.%02d.%04d %s" ), day, month, year, (const wxChar*)getTimeFormatted( getTimeFromJD(jd)) );
-	else 
+	else
 		s.Printf( wxT( "%04d-%02d-%02d %s" ), year, month, day, (const wxChar*)getTimeFormatted( getTimeFromJD(jd)) );
 	return s;
 }
 
 /*****************************************************
 **
-**   Formatter   ---   getDateStringFromJD 
+**   Formatter   ---   getDateStringFromJD
 **
 ******************************************************/
 const wxString Formatter::getDateStringFromJD( const double &jd, const int format )
@@ -769,32 +805,38 @@ const wxString Formatter::getDateStringFromJD( const double &jd, const int forma
 
 /*****************************************************
 **
-**   Formatter   ---   getDateIntsFromString 
+**   Formatter   ---   getDateIntsFromString
 **
 ******************************************************/
 bool Formatter::getDateIntsFromString( const wxChar *value, int &day, int &month, int &year, int format )
 {
+	const int l = LanguageConfig::get()->getLocale()->GetLanguage();
 	if ( ! value )
 	{
 		wxLogError( wxT( "Error: date value is NULL" ));
 		return false;
 	}
 	wxChar *c = (wxChar*)value;
-	while( *c )
+	int sepTokenCount = 0;
+	const wxChar septoken = ( l == wxLANGUAGE_GERMAN ? '.' : '-' );
+
+	while ( *c )
 	{
+		if ( *c == septoken ) sepTokenCount++;
 		if ( ! ( isdigit( *c ) || *c == '.' || *c == '-' )) return false;
 		c++;
 	}
+	if ( sepTokenCount > 2 ) return false;
+
 	int ret;
-	int l = LanguageConfig::get()->getLocale()->GetLanguage();
 	if ( l == wxLANGUAGE_GERMAN ) // German
 		ret = wxSscanf( value, wxT( "%d.%d.%d" ), &day, &month, &year );
 	else
 		ret = wxSscanf( value, wxT( "%d-%d-%d" ), &year, &month, &day );
 	if ( ret != 3 ) return false;
 
-	if (( day < 0 ) || ( day > 31 )) return false;
-	if (( month < 0 ) || ( month > 12 )) return false;
+	if (( day < 1 ) || ( day > 31 )) return false;
+	if (( month < 1 ) || ( month > 12 )) return false;
 	if (( year < -3000 ) || ( year > 6000 )) return false;
 
 	return true;
@@ -802,7 +844,7 @@ bool Formatter::getDateIntsFromString( const wxChar *value, int &day, int &month
 
 /*****************************************************
 **
-**   Formatter   ---   getDegreeIntsFromString 
+**   Formatter   ---   getDegreeIntsFromString
 **
 ******************************************************/
 bool Formatter::getDegreeIntsFromString( const wxChar *value, int &deg, int &minute, int &second, int format )
@@ -813,13 +855,16 @@ bool Formatter::getDegreeIntsFromString( const wxChar *value, int &deg, int &min
 		return false;
 	}
 	wxChar *c = (wxChar*)value;
+	int sepTokenCount = 0;
 	// TODO
-	while( *c )
+	while ( *c )
 	{
+		if ( *c == ':' ) sepTokenCount++;
 		if ( ! ( isdigit( *c ) || *c == ':' )) return false;
 		c++;
 	}
-	
+	if ( sepTokenCount > 2 ) return false;
+
 	deg = 0;
 	minute = 0;
 	second = 0;
@@ -834,6 +879,17 @@ bool Formatter::getDegreeIntsFromString( const wxChar *value, int &deg, int &min
 	return true;
 }
 
+/*****************************************************
+**
+**   Formatter   ---   getIntOrEmtpy
+**
+******************************************************/
+wxString Formatter::getIntOrEmtpy( const int &a )
+{
+	if ( a ) return  wxString::Format( wxT( "%d" ), a );
+	else return wxEmptyString;
+}
+
 
 
 
