@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/dialogs/DataDialog.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "DataDialog.h"
+#pragma implementation "DataDialog.h"
 #endif
 
 #include "DataDialog.h"
@@ -36,37 +36,32 @@
 #include <wx/stattext.h>
 #include <wx/textctrl.h>
 
-#include "constants.h"
-#include "Calculator.h"
+#include "AtlasDialog.h"
+#include "CommonInputFields.h"
 #include "Conf.h"
+#include "constants.h"
 #include "DataSet.h"
-#include "Lang.h"
-#include "mathbase.h"
-
-#include "DateTimeCtrl.h"
-#include "DialogElements.h"
 #include "Document.h"
-#include "guibase.h"
 #include "IconProvider.h"
-#include "LocationDialog.h"
+#include "mathbase.h"
 
 extern Config *config;
 
-enum { DD_NOW = wxID_HIGHEST + 300, DD_DEFLOC, DD_SEARCHLOC, DD_LOCALTIME, DD_REVERT };
+enum { DD_NOW = wxID_HIGHEST + 300, DD_DEFLOC, DD_SEARCHLOC, DD_LOCALTIME, DD_DATE, DD_TIME,
+	DD_LONGITUDE, DD_LATITUDE, DD_TIMEZONE, DD_DST };
 
 IMPLEMENT_CLASS( DataDialog, wxDialog )
 
 /*****************************************************
 **
-**   DataDialog   ---   Constructor 
+**   DataDialog   ---   Constructor
 **
 ******************************************************/
 DataDialog::DataDialog( wxWindow* parent, Document *d )
- : wxDialog( parent, -1, wxEmptyString, wxDefaultPosition, wxDefaultSize, DEFAULT_DIALOG_STYLE )
+		: wxDialog( parent, -1, wxEmptyString, wxDefaultPosition, wxDefaultSize, DEFAULT_DIALOG_STYLE )
 {
 	doc = d;
 	writeDoc2Model();
-	//SetExtraStyle( wxWS_EX_VALIDATE_RECURSIVELY ); -> dialog doesnt close under linux if closed by right upper corner button
 
     // begin wxGlade: DataDialog::DataDialog
     sizer_datetime_staticbox = new wxStaticBox(this, -1, _("Date and Time"));
@@ -76,25 +71,25 @@ DataDialog::DataDialog( wxWindow* parent, Document *d )
     sizer_name_staticbox = new wxStaticBox(this, -1, _("Name"));
     text_name = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
     label_date = new wxStaticText(this, wxID_ANY, _("Date"));
-    text_date = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
+    text_date = new DateInputField(this, DD_DATE);
     label_datefmt = new wxStaticText(this, wxID_ANY, _("YYYY-MM-DD"));
     label_time = new wxStaticText(this, wxID_ANY, _("Time"));
-    text_time = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
-    label_timefmt = new wxStaticText(this, wxID_ANY, _("HH:MM:SS"));
+    text_time = new TimeInputField(this, DD_TIME);
+    label_timefmt = new wxStaticText(this, wxID_ANY, _("HH:MM[:SS]"));
     button_now = new wxButton(this, DD_NOW, _("Now"));
     label_locname = new wxStaticText(this, wxID_ANY, _("Name"));
     text_locname = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
     label_long = new wxStaticText(this, wxID_ANY, _("Longitude"));
-    text_longitude = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
-    label_long_fmt = new wxStaticText(this, wxID_ANY, _("DD:MM:SS"));
+    text_longitude = new LongitudeInputField(this, DD_LONGITUDE);
+    label_long_fmt = new wxStaticText(this, wxID_ANY, _("DD:MM[:SS]"));
     const wxString choice_ew_choices[] = {
         _("East"),
         _("West")
     };
     choice_ew = new wxChoice(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 2, choice_ew_choices, 0);
     label_lat = new wxStaticText(this, wxID_ANY, _("Latitude"));
-    text_latitude = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
-    label_lat_fmt = new wxStaticText(this, wxID_ANY, _("DD:MM:SS"));
+    text_latitude = new LatitudeInputField(this, DD_LATITUDE);
+    label_lat_fmt = new wxStaticText(this, wxID_ANY, _("DD:MM[:SS]"));
     const wxString choice_ns_choices[] = {
         _("North"),
         _("South")
@@ -103,22 +98,21 @@ DataDialog::DataDialog( wxWindow* parent, Document *d )
     button_defloc = new wxButton(this, DD_DEFLOC, _("Default Location"));
     button_searchloc = new wxButton(this, DD_SEARCHLOC, _("Search Location"));
     label_tz = new wxStaticText(this, wxID_ANY, _("Time Zone"));
-    text_tz = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
-    label_tzfmt = new wxStaticText(this, wxID_ANY, _("HH"));
+    text_tz = new TimeInputField(this, DD_TIMEZONE);
+    label_tzfmt = new wxStaticText(this, wxID_ANY, _("HH:MM[:SS]"));
     const wxString choice_tzew_choices[] = {
         _("East"),
         _("West")
     };
     choice_tzew = new wxChoice(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 2, choice_tzew_choices, 0);
     label_dst = new wxStaticText(this, wxID_ANY, _("DST"));
-    text_dst = new wxTextCtrl(this, wxID_ANY, wxEmptyString);
-    label_dstfmt = new wxStaticText(this, wxID_ANY, _("HH"));
+    text_dst = new TimeInputField(this, DD_DST);
+    label_dstfmt = new wxStaticText(this, wxID_ANY, _("HH:MM[:SS]"));
     button_localtime = new wxButton(this, DD_LOCALTIME, _("Local Time"));
     text_remark = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_WORDWRAP);
     button_ok = new wxButton(this, wxID_OK, _("OK"));
-    button_cancel = new wxButton(this, wxID_CANCEL, _("Cancel"));
     button_apply = new wxButton(this, wxID_APPLY, _("Apply"));
-    button_revert = new wxButton(this, DD_REVERT, _("Revert"));
+    button_cancel = new wxButton(this, wxID_CANCEL, _("Cancel"));
 
     set_properties();
     do_layout();
@@ -126,147 +120,116 @@ DataDialog::DataDialog( wxWindow* parent, Document *d )
 
 	writeModel2Gui();
 	SetIcon( IconProvider::getIcon( BITMAP_EDIT ));
-	moveWindow2Center( this );
 	text_name->SetFocus();
 
-  Connect( DD_NOW, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnActual ));
-  Connect( DD_DEFLOC, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnDefaultLocation ));
-  Connect( DD_SEARCHLOC, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnSearchLocation ));
-  Connect( DD_REVERT, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnRevert ));
-  Connect( wxID_APPLY, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnApply ));
-  Connect( wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnOK ));
-  Connect( wxID_CANCEL, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnCancel ));
-  Connect( DD_LOCALTIME, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnLocalTime ));
+	Connect( DD_NOW, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnActual ));
+	Connect( DD_DEFLOC, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnDefaultLocation ));
+	Connect( DD_SEARCHLOC, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnSearchLocation ));
+	Connect( wxID_APPLY, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnApply ));
+	Connect( wxID_OK, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnOK ));
+	Connect( wxID_CANCEL, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnCancel ));
+	Connect( DD_LOCALTIME, wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( DataDialog::OnLocalTime ));
+	Connect( wxID_ANY, wxEVT_COMMAND_TEXT_UPDATED, wxCommandEventHandler( DataDialog::OnTextChanged ));
 }
 
 /*****************************************************
 **
-**   DataDialog   ---   writeDoc2Model 
+**   DataDialog   ---   writeDoc2Model
 **
 ******************************************************/
 void DataDialog::writeDoc2Model()
 {
-	Formatter *formatter = Formatter::get();
-	Location *loc = doc->getLocation();
-	double tmpjd, time;
-
 	model.name = doc->getHName();
 
-	tmpjd = doc->getJD() + loc->getTimeZone() / 24 + loc->getDST() / 24;
-	model.date = formatter->getDateStringFromJD( floor( tmpjd + .5 ));
-
-	time = ( tmpjd - floor( tmpjd ) + .5 ) * 24;
-	if ( time > 24 ) time -= 24;
-	model.time = formatter->getTimeFormatted( time );
+	Location *loc = doc->getLocation();
+	const double tmpjd = doc->getJD() + ( loc->getTimeZone() + loc->getDST() ) / 24.0;
+	model.date = floor( tmpjd + .5 );
+	model.time = a_red(( tmpjd - floor( tmpjd ) + .5 ) * 24, 24 );
 
 	model.locname = loc->getLocName();
-  model.longitude = formatter->getDegreesFormatted( fabs( loc->getLongitude() ));
-  model.ew = ( loc->getLongitude() < 0 );
+	model.longitude = loc->getLongitude();
+	model.latitude = loc->getLatitude();
 
-  model.latitude = formatter->getDegreesFormatted( fabs( loc->getLatitude() ));
-  model.ns = ( loc->getLatitude() < 0 );
-
-	model.tz = printfDouble( ::fabs( loc->getTimeZone()));
-	model.tzew = ( loc->getTimeZone() < 0 );
-	model.dst = printfDouble( ::fabs( loc->getDST() ));
+	model.tz = loc->getTimeZone();
+	model.dst = loc->getDST();
 
 	model.remark = doc->getRemark();
 }
 
 /*****************************************************
 **
-**   DataDialog   ---   writeModel2Gui 
+**   DataDialog   ---   writeModel2Gui
 **
 ******************************************************/
 void DataDialog::writeModel2Gui()
 {
 	text_name->SetValue( model.name );
-	text_date->SetValue( model.date );
-	text_time->SetValue( model.time );
+	text_date->setDoubleValue( model.date );
+	text_time->setDoubleValue( model.time );
 
 	text_locname->SetValue( model.locname );
-	text_longitude->SetValue( model.longitude );
-	choice_ew->SetSelection( model.ew );
-	text_latitude->SetValue( model.latitude );
-	choice_ns->SetSelection( model.ns );
+	text_longitude->setDoubleValue( fabs( model.longitude ));
+	choice_ew->SetSelection( model.longitude < 0 );
+	text_latitude->setDoubleValue( fabs( model.latitude ));
+	choice_ns->SetSelection( model.latitude < 0 );
 
-	text_tz->SetValue( model.tz );
-	text_dst->SetValue( model.dst );
-	choice_tzew->SetSelection( model.tzew );
+	text_tz->setDoubleValue( fabs( model.tz ));
+	choice_tzew->SetSelection( model.tz < 0 );
+	text_dst->setDoubleValue( model.dst );
 
 	text_remark->SetValue( model.remark );
 }
 
-
 /*****************************************************
 **
-**   DataDialog   ---   writeGui2Model 
+**   DataDialog   ---   writeGui2Model
 **
 ******************************************************/
 void DataDialog::writeGui2Model()
 {
 	model.name = text_name->GetValue();
-	model.date = text_date->GetValue();
-	model.time = text_time->GetValue();
+	model.date = text_date->getDoubleValue();
+	model.time = text_time->getDoubleValue();
 
 	model.locname = text_locname->GetValue();
-	model.longitude = text_longitude->GetValue();
-	model.ew = choice_ew->GetSelection();
-	model.latitude = text_latitude->GetValue();
-	model.ns = choice_ns->GetSelection();
+	model.longitude = text_longitude->getDoubleValue();
+	if ( choice_ew->GetSelection() > 0 ) model.longitude *= -1;;
+	model.latitude = text_latitude->getDoubleValue();
+	if ( choice_ns->GetSelection() > 0 ) model.latitude *= -1;
 
-	model.tz = text_tz->GetValue();
-	model.tzew = choice_tzew->GetSelection();
-	model.dst = text_dst->GetValue();
+	model.tz = text_tz->getDoubleValue();
+	if ( choice_tzew->GetSelection() > 0 ) model.tz *= -1;
+	model.dst = text_dst->getDoubleValue();
 	model.remark = text_remark->GetValue();
 }
 
-
 /*****************************************************
 **
-**   DataDialog   ---   writeModel2Doc 
+**   DataDialog   ---   writeModel2Doc
 **
 ******************************************************/
 void DataDialog::writeModel2Doc()
 {
-	int day, month, year;
-	int hour, minute, second, deg;
-	double jd, time, tz, dst, longitude, latitude;
-  Formatter *formatter = Formatter::get();
-	Calculator *calculator = CalculatorFactory().getCalculator();
-
 	doc->setHName( (const wxChar*)model.name );
-
-  formatter->getDateIntsFromString( (const wxChar*)model.date, day, month, year );
-
-  formatter->getDegreeIntsFromString( (const wxChar*)model.time, hour, minute, second );
-  time = 3600 * hour + 60 * minute + second;
-  time /= 3600;
-  tz = myatof( model.tz );
-	if ( model.tzew ) tz *= -1;
-  dst = myatof( model.dst );
-	time -= tz + dst;
-	jd = calculator->calc_jd( year, month, day, time );
-	doc->setDate( jd );
-
-  formatter->getDegreeIntsFromString( (const wxChar*)model.longitude, deg, minute, second );
-  longitude = 3600 * deg + 60 * minute + second;
-  longitude /= 3600;
-	if ( model.ew ) longitude *= -1;
-
-  formatter->getDegreeIntsFromString( (const wxChar*)model.latitude, deg, minute, second );
-  latitude = 3600 * deg + 60 * minute + second;
-  latitude /= 3600;
-	if ( model.ns ) latitude *= -1;
-
-	doc->setLocation( (const wxChar*)model.locname, longitude, latitude, tz, dst );
-
+	doc->setDate( model.date + ( model.time - model.tz - model.dst ) / 24.0 );
+	doc->setLocation( (const wxChar*)model.locname, model.longitude, model.latitude, model.tz, model.dst );
 	doc->setRemark( (const wxChar*)model.remark );
 }
 
 /*****************************************************
 **
-**   DataDialog   ---   OnCancel 
+**   DataDialog   ---   OnTextChanged
+**
+******************************************************/
+void DataDialog::OnTextChanged( wxCommandEvent &event )
+{
+	Validate();
+	event.Skip();
+}
+
+/*****************************************************
+**
+**   DataDialog   ---   OnCancel
 **
 ******************************************************/
 void DataDialog::OnCancel( wxCommandEvent &event )
@@ -276,7 +239,7 @@ void DataDialog::OnCancel( wxCommandEvent &event )
 
 /*****************************************************
 **
-**   DataDialog   ---   OnOK 
+**   DataDialog   ---   OnOK
 **
 ******************************************************/
 void DataDialog::OnOK( wxCommandEvent &event )
@@ -289,41 +252,27 @@ void DataDialog::OnOK( wxCommandEvent &event )
 
 /*****************************************************
 **
-**   DataDialog   ---   OnRevert 
-**
-******************************************************/
-void DataDialog::OnRevert( wxCommandEvent &event )
-{
-	writeDoc2Model();
-	writeModel2Gui();
-}
-
-/*****************************************************
-**
-**   DataDialog   ---   OnActual 
+**   DataDialog   ---   OnActual
 **
 ******************************************************/
 void DataDialog::OnActual( wxCommandEvent &event )
 {
-	double jd, tmpjd, time;
-	Formatter *formatter = Formatter::get();
-	Location *loc = &config->defaultLocation;
+	TzUtil tzu;
+	TzInfo info = tzu.getCurrentTzInfo();
 
-	jd = JDate().getJD();
-	tmpjd = jd + loc->getTimeZone() / 24 + loc->getDST() / 24;
-
-	model.date = formatter->getDateStringFromJD( tmpjd );
-
-	time = ( tmpjd - floor( tmpjd ) + .5 ) * 24;
-	if ( time > 24 ) time -= 24;
-	model.time = formatter->getTimeFormatted( time );
+	model.tz = info.tzhours;
+	model.dst = info.dsthours;
+	double jd = JDate::getCurrentJD();
+	double tmpjd = jd + ( model.tz + model.dst ) / 24.0;
+	model.date = floor( tmpjd + .5 );
+	model.time = a_red(( tmpjd - floor( tmpjd ) + .5 ) * 24, 24 );
 
 	writeModel2Gui();
 }
 
 /*****************************************************
 **
-**   DataDialog   ---   doApply 
+**   DataDialog   ---   doApply
 **
 ******************************************************/
 bool DataDialog::doApply()
@@ -339,51 +288,46 @@ bool DataDialog::doApply()
 
 /*****************************************************
 **
-**   DataDialog   ---   OnDefaultLocation 
+**   DataDialog   ---   OnDefaultLocation
 **
 ******************************************************/
 void DataDialog::OnDefaultLocation( wxCommandEvent &event )
 {
-	Formatter *formatter = Formatter::get();
+	TzUtil tzu;
 
 	writeGui2Model();
-	Location *loc = &config->defaultLocation;
+	double jd = model.date + ( model.time - model.tz - model.dst ) / 24.0;
+	TzInfo info = tzu.calculateTzInfoForJD( jd );
+
+	const Location *loc = &config->defaultLocation;
 	model.locname = loc->getLocName();
 
-	model.longitude = formatter->getDegreesFormatted( fabs( loc->getLongitude() ));
-	model.ew = ( loc->getLongitude() < 0 );
-	model.latitude = formatter->getDegreesFormatted( fabs( loc->getLatitude() ));
-	model.ns = ( loc->getLatitude() < 0 );
-	model.tz = wxString::Format( wxT( "%1.1f" ), fabs( loc->getTimeZone() ));
-	model.tzew = ( loc->getTimeZone() < 0 );
-	model.dst = wxString::Format( wxT( "%1.1f" ), loc->getDST() );
+	model.longitude = loc->getLongitude();
+	model.latitude = loc->getLatitude();
+	model.tz = info.tzhours;
+	model.dst = info.dsthours;
 
 	writeModel2Gui();
 }
 
 /*****************************************************
 **
-**   DataDialog   ---   OnSearchLocation 
+**   DataDialog   ---   OnSearchLocation
 **
 ******************************************************/
 void DataDialog::OnSearchLocation( wxCommandEvent &event )
 {
-	Formatter *formatter = Formatter::get();
-
-  LocationDialog dialog( this, 1 );
-	if ( dialog.ShowModal() == wxID_OK )
+	AtlasDialogWrapper dialog( this, ATLASDIALOG_CALLER_DATADIALOG );
+	if ( dialog.run() == wxID_OK )
 	{
 		writeGui2Model();
+		AtlasEntry *e = dialog.getSelectedEntry();
+		model.locname = e->name;
 
-		Location *loc = dialog.getSelectedLocation();
-		model.locname = loc->getLocName();
-		model.longitude = formatter->getDegreesFormatted( fabs( loc->getLongitude() ));
-		model.ew = ( loc->getLongitude() < 0 );
-		model.latitude = formatter->getDegreesFormatted( fabs( loc->getLatitude() ));
-		model.ns = ( loc->getLatitude() < 0 );
-		model.tz = wxString::Format( wxT( "%1.1f" ), fabs( loc->getTimeZone() ));
-		model.tzew = ( loc->getTimeZone() < 0 );
-		model.dst = wxString::Format( wxT( "%1.1f" ), loc->getDST() );
+		model.longitude = e->longitude;
+		model.latitude = e->latitude;
+
+		model.tz = e->tzoffset;
 
 		writeModel2Gui();
 	}
@@ -391,81 +335,60 @@ void DataDialog::OnSearchLocation( wxCommandEvent &event )
 
 /*****************************************************
 **
-**   DataDialog   ---   OnLocalTime 
+**   DataDialog   ---   OnLocalTime
 **
 ******************************************************/
 void DataDialog::OnLocalTime( wxCommandEvent &event )
 {
-	Formatter *formatter = Formatter::get();
-	int deg, minute, second;
-	double longitude;
-
 	if ( !Validate() ) return;
 	writeGui2Model();
-  formatter->getDegreeIntsFromString( (const wxChar*)model.longitude, deg, minute, second );
-  longitude = 3600 * deg + 60 * minute + second;
-  longitude /= 3600;
-	if ( model.ew ) longitude *= -1;
-	model.tz = wxString::Format( wxT( "%1.6f" ), fabs( longitude/15 ));
-	model.tzew = ( longitude < 0 );
-	model.dst = wxString::Format( wxT( "%1.1f" ), 0.0 );
+	model.tz = model.longitude/15.0;
+	model.dst = 0;
 	writeModel2Gui();
 }
 
 /*****************************************************
 **
-**   DataDialog   ---   set_properties 
+**   DataDialog   ---   set_properties
 **
 ******************************************************/
 void DataDialog::set_properties()
 {
-		text_date->SetValidator( DateValidator( &model.date ));
-		//text_date->SetName( "Date" );
-		text_time->SetValidator( DegreeValidator( &model.time ));
-		//text_time->SetName( "Time" );
-		text_longitude->SetValidator( DegreeValidator( &model.longitude ));
-		//text_longitude->SetName( "Longitude" );
-		text_latitude->SetValidator( DegreeValidator( &model.latitude ));
-		//text_latitude->SetName( "Latitude" );
-		text_tz->SetValidator( DoubleValidator( &model.tz ));
-		//text_tz->SetName( "Time zone" );
-		text_dst->SetValidator( DoubleValidator( &model.dst ));
-		//text_dst->SetName( "DST" );
-
     // begin wxGlade: DataDialog::set_properties
     SetTitle(_("Birth Data"));
     text_name->SetMinSize(wxSize(300, 22));
     label_date->SetMinSize(wxSize(70, 20));
-    text_date->SetMinSize(wxSize(120, -1));
+    text_date->SetMinSize(wxSize(130,-1));
     label_time->SetMinSize(wxSize(70, 20));
-    text_time->SetMinSize(wxSize(120, -1));
+    text_time->SetMinSize(wxSize(130,-1));
     button_now->SetToolTip(_("Set current date"));
     label_locname->SetMinSize(wxSize(70, 20));
     label_long->SetMinSize(wxSize(70, 20));
+    text_longitude->SetMinSize(wxSize(130,-1));
     choice_ew->SetSelection(0);
     label_lat->SetMinSize(wxSize(70, 20));
+    text_latitude->SetMinSize(wxSize(130,-1));
     choice_ns->SetSelection(0);
     button_defloc->SetToolTip(_("Set default location"));
     button_searchloc->SetToolTip(_("Search location in database"));
     label_tz->SetMinSize(wxSize(70, 20));
-    text_tz->SetMinSize(wxSize(100, 22));
+    text_tz->SetMinSize(wxSize(100,-1));
     choice_tzew->SetSelection(0);
     label_dst->SetMinSize(wxSize(70, 20));
-    text_dst->SetMinSize(wxSize(100, 22));
+    text_dst->SetMinSize(wxSize(100, -1));
     button_localtime->SetToolTip(_("Set time zone for local time"));
     text_remark->SetMinSize(wxSize(300,80));
     button_ok->SetToolTip(_("Apply data and close dialog"));
     button_ok->SetDefault();
-    button_cancel->SetToolTip(_("Cancel changes and close dialog"));
     button_apply->SetToolTip(_("Apply changes and recalculate chart with new data"));
-    button_revert->SetToolTip(_("Revert changes"));
+    button_cancel->SetToolTip(_("Cancel changes and close dialog"));
     // end wxGlade
 }
 
 
 /*****************************************************
 **
-**   DataDialog   ---   do_layout 
+**   DataDialog   ---   do_layout
 **
 ******************************************************/
 void DataDialog::do_layout()
@@ -492,10 +415,10 @@ void DataDialog::do_layout()
     sizer_name->Add(grid_sizer_name, 1, wxEXPAND, 0);
     sizer_main->Add(sizer_name, 1, wxALL|wxEXPAND, 3);
     grid_datetime->Add(label_date, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
-    grid_datetime->Add(text_date, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
+    grid_datetime->Add(text_date, 1, wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL, 3);
     grid_datetime->Add(label_datefmt, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
     grid_datetime->Add(label_time, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
-    grid_datetime->Add(text_time, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
+    grid_datetime->Add(text_time, 1, wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL, 3);
     grid_datetime->Add(label_timefmt, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
     sizer_datetime->Add(grid_datetime, 1, wxALL|wxEXPAND, 3);
     grid_sizer_dt->Add(sizer_datetime, 1, wxALL|wxEXPAND, 3);
@@ -505,12 +428,12 @@ void DataDialog::do_layout()
     sizer_locname->AddGrowableCol(1);
     sizer_location->Add(sizer_locname, 1, wxALL|wxEXPAND, 3);
     sizer_long->Add(label_long, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
-    sizer_long->Add(text_longitude, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
+    sizer_long->Add(text_longitude, 1, wxALL|wxEXPAND, 3);
     sizer_long->Add(label_long_fmt, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
     sizer_long->Add(choice_ew, 0, wxALL, 3);
     sizer_location->Add(sizer_long, 1, wxALL|wxEXPAND, 3);
     sizer_lat->Add(label_lat, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
-    sizer_lat->Add(text_latitude, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
+    sizer_lat->Add(text_latitude, 1, wxALL|wxEXPAND, 3);
     sizer_lat->Add(label_lat_fmt, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
     sizer_lat->Add(choice_ns, 0, wxALL, 3);
     sizer_location->Add(sizer_lat, 1, wxALL|wxEXPAND, 3);
@@ -519,11 +442,11 @@ void DataDialog::do_layout()
     sizer_loc_action_copy->Add(button_searchloc, 0, wxALL|wxEXPAND|wxALIGN_CENTER_HORIZONTAL, 3);
     grid_sizer_dt->Add(sizer_loc_action_copy, 1, wxALIGN_CENTER_VERTICAL, 0);
     grid_tz->Add(label_tz, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
-    grid_tz->Add(text_tz, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
+    grid_tz->Add(text_tz, 1, wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL, 3);
     grid_tz->Add(label_tzfmt, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
     grid_tz->Add(choice_tzew, 0, wxALL, 3);
     grid_tz->Add(label_dst, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
-    grid_tz->Add(text_dst, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
+    grid_tz->Add(text_dst, 1, wxALL|wxEXPAND|wxALIGN_CENTER_VERTICAL, 3);
     grid_tz->Add(label_dstfmt, 0, wxALL|wxALIGN_CENTER_VERTICAL, 3);
     grid_tz->Add(20, 20, 0, 0, 0);
     grid_tz->AddGrowableCol(3);
@@ -538,9 +461,8 @@ void DataDialog::do_layout()
     sizer_remark->Add(grid_sizer_remark, 1, 0, 0);
     sizer_main->Add(sizer_remark, 1, wxALL|wxEXPAND, 3);
     sizer_buttons->Add(button_ok, 0, wxALL, 5);
-    sizer_buttons->Add(button_cancel, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
     sizer_buttons->Add(button_apply, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
-    sizer_buttons->Add(button_revert, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
+    sizer_buttons->Add(button_cancel, 0, wxALL|wxALIGN_CENTER_VERTICAL, 5);
     sizer_buttons->AddGrowableCol(0);
     sizer_buttons->AddGrowableCol(1);
     sizer_buttons->AddGrowableCol(2);
