@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/gui/DasaGraphicView.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "DasaGraphicView.h"
+#pragma implementation "DasaGraphicView.h"
 #endif
 
 #include "DasaGraphicView.h"
@@ -39,7 +39,6 @@
 #include "FontConfig.h"
 #include "Lang.h"
 #include "Painter.h"
-#include "Writer.h"
 
 extern Config *config;
 
@@ -48,11 +47,11 @@ IMPLEMENT_CLASS( DasaGraphicWidget, BasicWidget )
 
 /*****************************************************
 **
-**   DasaGraphicView   ---   Constructor 
+**   DasaGraphicView   ---   Constructor
 **
 ******************************************************/
 DasaGraphicView::DasaGraphicView( wxWindow *parent, ChildWindow *frame, Document *doc )
- : BasicView( parent, frame, doc, VIEW_GRAPHICALDASA, true, true )
+		: BasicView( parent, frame, doc, VIEW_GRAPHICALDASA, true, true )
 {
 	supportEWToggle = false;
 	setVedic( true );
@@ -62,12 +61,12 @@ DasaGraphicView::DasaGraphicView( wxWindow *parent, ChildWindow *frame, Document
 	initToolItems();
 	widget = swidget;
 
-  Connect( TBB_DASA, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( DasaGraphicView::OnDasas ));
-  Connect( TBB_NOW, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( DasaGraphicView::OnNow ));
-  Connect( TBB_GOTO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( DasaGraphicView::OnYear ));
+	Connect( TBB_DASA, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( DasaGraphicView::OnDasas ));
+	Connect( TBB_NOW, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( DasaGraphicView::OnNow ));
+	Connect( TBB_GOTO, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( DasaGraphicView::OnYear ));
 
-  Connect( TBS_ZOOM, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( DasaGraphicView::OnChoiceCommand ));
-  Connect( TBS_ANTARDASALEVEL, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( DasaGraphicView::OnChoiceAntardasa ));
+	Connect( TBS_ZOOM, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( DasaGraphicView::OnChoiceCommand ));
+	Connect( TBS_ANTARDASALEVEL, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( DasaGraphicView::OnChoiceAntardasa ));
 }
 
 /*****************************************************
@@ -114,7 +113,7 @@ void DasaGraphicView::postCreate()
 
 /*****************************************************
 **
-**   DasaGraphicView   ---   OnNow 
+**   DasaGraphicView   ---   OnNow
 **
 ******************************************************/
 void DasaGraphicView::OnNow( wxCommandEvent &event )
@@ -122,13 +121,13 @@ void DasaGraphicView::OnNow( wxCommandEvent &event )
 	Formatter *formatter = Formatter::get();
 	int month, day, year;
 
-	formatter->getDateIntsFromJD( JDate().getJD(), day, month, year );
+	formatter->getDateIntsFromJD( JDate::getCurrentJD(), day, month, year );
 	swidget->setYear( year );
 }
 
 /*****************************************************
 **
-**   DasaGraphicView   ---   dispatchCommand 
+**   DasaGraphicView   ---   dispatchCommand
 **
 ******************************************************/
 bool DasaGraphicView::dispatchCommand( const int &command )
@@ -143,7 +142,7 @@ bool DasaGraphicView::dispatchCommand( const int &command )
 
 /*****************************************************
 **
-**   DasaGraphicView   ---   OnDataChanged 
+**   DasaGraphicView   ---   OnDataChanged
 **
 ******************************************************/
 void DasaGraphicView::OnDataChanged()
@@ -154,7 +153,7 @@ void DasaGraphicView::OnDataChanged()
 
 /*****************************************************
 **
-**   DasaGraphicView   ---   OnYear 
+**   DasaGraphicView   ---   OnYear
 **
 ******************************************************/
 void DasaGraphicView::OnYear( wxCommandEvent &event )
@@ -171,9 +170,9 @@ void DasaGraphicView::OnYear( wxCommandEvent &event )
 
 	defvalue.Printf( wxT( "%d" ), year );
 
-	wxTextEntryDialog dialog( this, 
-		wxString::Format( _("Please enter a year between %d and %d"), birthyear, birthyear + totalyears ),
-		GUI_APP_NAME, defvalue );
+	wxTextEntryDialog dialog( this,
+	                          wxString::Format( _("Please enter a year between %d and %d"), birthyear, birthyear + totalyears ),
+	                          GUI_APP_NAME, defvalue );
 	if ( dialog.ShowModal() == wxID_OK )
 	{
 		wxString s = dialog.GetValue();
@@ -181,8 +180,8 @@ void DasaGraphicView::OnYear( wxCommandEvent &event )
 		if ( y < birthyear || y > birthyear + totalyears )
 		{
 			doMessageBox( this,
-				wxString::Format( _("The value \"%s\" is not valid."), (const wxChar*)s ),
-				wxOK | wxICON_ERROR | wxCENTRE );
+			              wxString::Format( _("The value \"%s\" is not valid."), (const wxChar*)s ),
+			              wxOK | wxICON_ERROR | wxCENTRE );
 			return;
 		}
 		swidget->setYear( y );
@@ -191,16 +190,16 @@ void DasaGraphicView::OnYear( wxCommandEvent &event )
 
 /*****************************************************
 **
-**   DasaGraphicView   ---   OnDasas 
+**   DasaGraphicView   ---   OnDasas
 **
 ******************************************************/
 void DasaGraphicView::OnDasas( wxCommandEvent &event )
 {
 	wxString l[DasaExpertFactory::MAX_DASAEXPERTS];
 
-	for( unsigned int i = 0; i < DasaExpertFactory::MAX_DASAEXPERTS; i++ )
+	for ( unsigned int i = 0; i < DasaExpertFactory::MAX_DASAEXPERTS; i++ )
 		l[i] = DasaExpertFactory::get()->getDasaExpert( i )->getName();
-		
+
 	wxMultiChoiceDialog dialog( this, _( "Choose Dasas" ), GUI_APP_NAME, DasaExpertFactory::MAX_DASAEXPERTS, l );
 
 	dialog.SetSelections( swidget->dasas );
@@ -215,7 +214,7 @@ void DasaGraphicView::OnDasas( wxCommandEvent &event )
 
 /*****************************************************
 **
-**   DasaGraphicView   ---   OnToolbarCommand 
+**   DasaGraphicView   ---   OnToolbarCommand
 **
 ******************************************************/
 void DasaGraphicView::OnToolbarCommand()
@@ -267,19 +266,21 @@ wxString DasaGraphicView::getWindowLabel( const bool shortname )
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   Constructor 
+**   DasaGraphicWidget   ---   Constructor
 **
 ******************************************************/
 DasaGraphicWidget::DasaGraphicWidget( wxWindow *parent, ChartProperties *chartprops, Horoscope *doc, wxWindowID id, const wxPoint& pos,
-	const wxSize& size, long style ) 
- : BasicWidget( parent, chartprops )
+                                      const wxSize& size, long style )
+		: BasicWidget( parent, chartprops )
 {
 	this->doc = doc;
 	updateOfTextRequired = false;
 	totalyears = 100;
 
-	for( unsigned int i = 0; i < DasaExpertFactory::MAX_DASAEXPERTS; i++ )
-		{	if ( ( config->vDefaultDasaList >> i ) % 2 ) dasas.Add( i ); }
+	for ( unsigned int i = 0; i < DasaExpertFactory::MAX_DASAEXPERTS; i++ )
+	{
+		if ( ( config->vDefaultDasaList >> i ) % 2 ) dasas.Add( i );
+	}
 
 	theText = 0;
 	mouse_x = mouse_y = 0;
@@ -291,37 +292,35 @@ DasaGraphicWidget::DasaGraphicWidget( wxWindow *parent, ChartProperties *chartpr
 
 	Formatter *formatter = Formatter::get();
 	int month, day, year;
-	
-	formatter->getDateIntsFromJD( JDate().getJD(), day, month, year );
-	writer = WriterFactory().getWriter( WRITER_TEXT );
 
-  Connect( wxEVT_IDLE, wxIdleEventHandler( DasaGraphicWidget::OnIdle ));
-  Connect( wxEVT_LEAVE_WINDOW, wxMouseEventHandler( DasaGraphicWidget::OnMouseLeave ));
-  Connect( wxEVT_ENTER_WINDOW, wxMouseEventHandler( DasaGraphicWidget::OnMouseEnter ));
-  Connect( wxEVT_MOTION, wxMouseEventHandler( DasaGraphicWidget::OnMouseMotion ));
+	formatter->getDateIntsFromJD( JDate::getCurrentJD(), day, month, year );
+
+	Connect( wxEVT_IDLE, wxIdleEventHandler( DasaGraphicWidget::OnIdle ));
+	Connect( wxEVT_LEAVE_WINDOW, wxMouseEventHandler( DasaGraphicWidget::OnMouseLeave ));
+	Connect( wxEVT_ENTER_WINDOW, wxMouseEventHandler( DasaGraphicWidget::OnMouseEnter ));
+	Connect( wxEVT_MOTION, wxMouseEventHandler( DasaGraphicWidget::OnMouseMotion ));
 }
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   Destructor 
+**   DasaGraphicWidget   ---   Destructor
 **
 ******************************************************/
 DasaGraphicWidget::~DasaGraphicWidget()
 {
 	config->vDefaultDasaList = 0;
-	for( unsigned int i = 0; i < dasas.size(); i++ ) config->vDefaultDasaList += 1 << dasas[i];
+	for ( unsigned int i = 0; i < dasas.size(); i++ ) config->vDefaultDasaList += 1 << dasas[i];
 	clearDasaItems();
-	delete writer;
 }
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   clearDasaItems 
+**   DasaGraphicWidget   ---   clearDasaItems
 **
 ******************************************************/
 void DasaGraphicWidget::clearDasaItems()
 {
-	for( unsigned int i = 0; i < dasaitems.size(); i++ ) delete dasaitems[i];
+	for ( unsigned int i = 0; i < dasaitems.size(); i++ ) delete dasaitems[i];
 	dasaitems.clear();
 }
 
@@ -374,12 +373,12 @@ void DasaGraphicWidget::HandleMouseWheelEvent( wxMouseEvent &event )
 
 	GetViewStart( &x, &y );      // scroll position
 	GetSize( &xs, &ys );         // size of visible widget only
-	GetVirtualSize( &x0, &y0 );  // complete size of widget 
+	GetVirtualSize( &x0, &y0 );  // complete size of widget
 
 	// Wheel scolls in vertical direction, but horizontal if nothing left to scroll
 	if ( event.GetWheelRotation() < 0 )
 	{
-		if ( y + ys >= y0 ) x = Min( x0, x + offset ); 
+		if ( y + ys >= y0 ) x = Min( x0, x + offset );
 		else y += offset;
 	}
 	else
@@ -392,45 +391,47 @@ void DasaGraphicWidget::HandleMouseWheelEvent( wxMouseEvent &event )
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   OnDataChanged 
+**   DasaGraphicWidget   ---   OnDataChanged
 **
 ******************************************************/
 void DasaGraphicWidget::OnDataChanged()
 {
-  unsigned int dasa, mahadasa, antardasa, pratyantardasa;
-  DasaExpert *expert;
-  GraphicalDasaItem *item, *depitem, *item2;
-  vector<Dasa*> w, w2;
+	unsigned int dasa, mahadasa, antardasa, pratyantardasa;
+	DasaExpert *expert;
+	GraphicalDasaItem *item, *depitem, *item2;
+	vector<Dasa*> w, w2;
 	Formatter *formatter = Formatter::get();
 	int day, month;
 
 	formatter->getDateIntsFromJD( doc->getJD(), day, month, birthyear );
 
-  clearDasaItems();
-  for( dasa = 0; dasa < dasas.Count(); dasa++ ) // First loop generates Dasas like Vimsottari, Yogini, Kalachakra
-  {
-    expert = DasaExpertFactory::get()->getDasaExpert( dasas.Item( dasa ));
-    item = new GraphicalDasaItem( expert, 0 );
-    vector<Dasa*> v = expert->getFirstLevel( doc, 0 );
-    if ( dasas.Item( dasa ) == 2 && ! v[0]->getExtraText().IsEmpty()) // Kalachakra
-    {
-      item->name.Printf( wxT( "%s (%s)" ), expert->getName(), (const wxChar*)(v[0]->getExtraText()) );
-    }
-    else { item->name = expert->getName(); }
+	clearDasaItems();
+	for ( dasa = 0; dasa < dasas.Count(); dasa++ ) // First loop generates Dasas like Vimsottari, Yogini, Kalachakra
+	{
+		expert = DasaExpertFactory::get()->getDasaExpert( dasas.Item( dasa ));
+		item = new GraphicalDasaItem( expert, 0 );
+		vector<Dasa*> v = expert->getFirstLevel( doc, 0 );
+		if ( dasas.Item( dasa ) == 2 && ! v[0]->getExtraText().IsEmpty()) // Kalachakra
+		{
+			item->name.Printf( wxT( "%s (%s)" ), expert->getName(), (const wxChar*)(v[0]->getExtraText()) );
+		}
+		else {
+			item->name = expert->getName();
+		}
 
-		for( mahadasa = 0; mahadasa < v.size(); mahadasa++ )
+		for ( mahadasa = 0; mahadasa < v.size(); mahadasa++ )
 		{
 			depitem = new GraphicalDasaItem( expert, v[mahadasa] );
 			if ( adlevel > 0 )
 			{
 				w = expert->getNextLevel( v[mahadasa] );
-				for( antardasa = 0; antardasa < w.size(); antardasa++ )
+				for ( antardasa = 0; antardasa < w.size(); antardasa++ )
 				{
 					item2 = new GraphicalDasaItem( expert, w[antardasa] );
 					if ( adlevel > 1 )
 					{
 						w2 = expert->getNextLevel( w[antardasa] );
-						for( pratyantardasa = 0; pratyantardasa < w2.size(); pratyantardasa++ )
+						for ( pratyantardasa = 0; pratyantardasa < w2.size(); pratyantardasa++ )
 						{
 							item2->depitems.push_back( new GraphicalDasaItem( expert, w2[pratyantardasa] ) );
 						}
@@ -441,14 +442,14 @@ void DasaGraphicWidget::OnDataChanged()
 				w.clear();
 			}
 			item->depitems.push_back( depitem );
-    }
-    dasaitems.push_back( item );
-  }
+		}
+		dasaitems.push_back( item );
+	}
 }
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   paintRuler 
+**   DasaGraphicWidget   ---   paintRuler
 **
 ******************************************************/
 void DasaGraphicWidget::paintRuler()
@@ -468,13 +469,13 @@ void DasaGraphicWidget::paintRuler()
 
 	painter->setTransparentBrush();
 
-	wxPen fgpen( config->colors->fgColor, 1, wxSOLID );
+	wxPen fgpen( config->colors->fgColor );
 	painter->setPen( fgpen );
 	painter->drawLine( (int)xleft, y, (int)xright, y );
 	painter->setTextColor( config->colors->fgColor );
 	painter->setGraphicFont();
 
-	for( i = 0; i <= totalyears; i++ )
+	for ( i = 0; i <= totalyears; i++ )
 	{
 		x = (int)(xleft + i * steplength);
 		painter->drawLine( x, (int)(ytop + 2 * hborder), x, y );
@@ -485,7 +486,7 @@ void DasaGraphicWidget::paintRuler()
 			drawTextFormatted( wxRect( x-xtext, y + ysizey + ytext, 2*xtext, 2*ytext ), s, Align::Center );
 		if ( i < totalyears )
 		{
-			for( j = 1; j < 12; j++ )
+			for ( j = 1; j < 12; j++ )
 			{
 				xm = (int)(x + j* xmonth);
 				ylength = ysizey/( j % 3 ? 3 : 2 );
@@ -510,7 +511,7 @@ void DasaGraphicWidget::paintRuler()
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   setYear 
+**   DasaGraphicWidget   ---   setYear
 **
 ******************************************************/
 void DasaGraphicWidget::setYear( int year )
@@ -518,7 +519,9 @@ void DasaGraphicWidget::setYear( int year )
 	Formatter *formatter = Formatter::get();
 	int day, month;
 	assert( doc );
-	if ( year == 0 ) { formatter->getDateIntsFromJD( JDate().getJD(), day, month, year ); }
+	if ( year == 0 ) {
+		formatter->getDateIntsFromJD( JDate::getCurrentJD(), day, month, year );
+	}
 	double portion = ( year - birthyear ) / (double)totalyears;
 	double y = portion * ( xright - xleft );
 	Scroll( (int)(y/scrolly), 0 );
@@ -538,7 +541,7 @@ void DasaGraphicWidget::setAntardasaLevel( const int &a )
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   setZoom 
+**   DasaGraphicWidget   ---   setZoom
 **
 ******************************************************/
 void DasaGraphicWidget::setZoom( const int z )
@@ -565,7 +568,7 @@ void DasaGraphicWidget::setZoom( const int z )
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   getX4Jd 
+**   DasaGraphicWidget   ---   getX4Jd
 **
 ******************************************************/
 int DasaGraphicWidget::getX4Jd( const double &jd )
@@ -576,7 +579,7 @@ int DasaGraphicWidget::getX4Jd( const double &jd )
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   showDasaBar 
+**   DasaGraphicWidget   ---   showDasaBar
 **
 ******************************************************/
 void DasaGraphicWidget::showDasaBar( Dasa *d, DasaExpert *expert, double y )
@@ -597,12 +600,13 @@ void DasaGraphicWidget::showDasaBar( Dasa *d, DasaExpert *expert, double y )
 	}
 
 	int objectid = config->colors->getDasaColorConfigId( d->getDasaLord(), expert->isRasiDasaExpert() );
-	painter->drawDasaBar( d->getDasaLord(), expert->isRasiDasaExpert(), &config->colors->dasa[objectid], x1, y, x2, (int)ybar, config->dasaCornerRadius );
+	GraphicalDasaPaintHelper( painter ).drawDasaBar( d->getDasaLord(), expert->isRasiDasaExpert(),
+		&config->colors->dasa[objectid], x1, y, x2, (int)ybar, config->dasaCornerRadius );
 }
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   paintDasa 
+**   DasaGraphicWidget   ---   paintDasa
 **
 ******************************************************/
 void DasaGraphicWidget::paintDasa( const int &whichdasa )
@@ -614,7 +618,7 @@ void DasaGraphicWidget::paintDasa( const int &whichdasa )
 
 	int x1 = getX4Jd( doc->getJD());
 	int x2;
-	for( i = 0; i < dasaitem->depitems.size(); i++ )  
+	for ( i = 0; i < dasaitem->depitems.size(); i++ )
 	{
 		painter->setGraphicFont();
 		//dc->SetTextBackground( *wxWHITE );
@@ -624,7 +628,7 @@ void DasaGraphicWidget::paintDasa( const int &whichdasa )
 		painter->getTextExtent( s, &w, &h );
 
 		x2 = x1;
-		while( x2 < xright - w )
+		while ( x2 < xright - w )
 		{
 			drawTextFormatted( wxRect( x2, (int)(ycursor-ybar), 100, (int)ybar ), s, Align::Bottom + Align::Left );
 			x2 += Max( 800, 2 * w );
@@ -635,12 +639,12 @@ void DasaGraphicWidget::paintDasa( const int &whichdasa )
 		{
 			showDasaBar( item2->dasa, item2->expert, ycursor );
 		}
-		for( j = 0; j < item2->depitems.size(); j++ )  
+		for ( j = 0; j < item2->depitems.size(); j++ )
 		{
 			item3 = item2->depitems[j];
 			if (( doc->getJD() < item3->dasa->getEndJD() && endjd > item3->dasa->getStartJD() ))
 				showDasaBar( item3->dasa, item3->expert, ycursor + ybar + 2 );
-			for( k = 0; k < item3->depitems.size(); k++ )  
+			for ( k = 0; k < item3->depitems.size(); k++ )
 			{
 				item4 = item3->depitems[k];
 				if (( doc->getJD() < item4->dasa->getEndJD() && endjd > item4->dasa->getStartJD() ))
@@ -653,7 +657,7 @@ void DasaGraphicWidget::paintDasa( const int &whichdasa )
 
 /*****************************************************
 **
-**   DasaGraphicWidget   ---   doPaint 
+**   DasaGraphicWidget   ---   doPaint
 **
 ******************************************************/
 void DasaGraphicWidget::doPaint( bool eraseBackground, const wxRect* rect )
@@ -675,17 +679,17 @@ void DasaGraphicWidget::doPaint( bool eraseBackground, const wxRect* rect )
 	endjd = d.getJD();
 
 	ycursor = (int)(ybar * 1.5);
-	
+
 	painter->setBrushColor( *wxLIGHT_GREY );
 	painter->drawRectangle( 0, 0, w, h );
 	painter->setGraphicFont();
 
-	painter->setBrush( wxBrush( config->colors->bgColor, wxSOLID ));
-	painter->setPen( wxPen( config->colors->fgColor, 1, wxSOLID ));
+	painter->setDefaultBrush();
+	painter->setPen( wxPen( config->colors->fgColor ));
 	painter->drawRectangle( hborder, vborder, (int)(xright + hborder), (int)(ybottom + vborder) );
 
 	paintRuler();
-	for( unsigned int i = 0; i < dasaitems.size(); i++ ) paintDasa( i );
+	for ( unsigned int i = 0; i < dasaitems.size(); i++ ) paintDasa( i );
 	//wxLongLong elapsed = wxGetLocalTimeMillis() - starttime;
 	//printf( "DasaGraphicWidget::doPaint time %ld millisec\n", elapsed.ToLong());
 }
@@ -715,42 +719,55 @@ wxString DasaGraphicWidget::getDasaText( const int &whichdasa, const int &level,
 	double startJd, endJd;
 	wxString startDate, endDate;
 	Formatter *formatter = Formatter::get();
+	Lang lang;
 
 	DasaExpert *expert = dasaitem->expert;
-	for( i = 0; i < dasaitem->depitems.size(); i++ )  
+	for ( i = 0; i < dasaitem->depitems.size(); i++ )
 	{
 		//ret =  dasaitem->name;
 		item2 = dasaitem->depitems[i];
 		if ( item2->dasa->getStartJD() < theJd && item2->dasa->getEndJD() > theJd )
 		{
 			if ( expert->isGrahaDasaExpert() )
-				{ mahalord = writer->getObjectName(item2->dasa->getDasaLord(), TMEDIUM, true ); }
-			else { mahalord = writer->getSignName(item2->dasa->getDasaLord(), TMEDIUM); }
+			{
+				mahalord = lang.getObjectName(item2->dasa->getDasaLord(), TMEDIUM, true );
+			}
+			else {
+				mahalord = lang.getSignName(item2->dasa->getDasaLord(), TMEDIUM);
+			}
 			startJd = item2->dasa->getStartJD();
 			endJd = item2->dasa->getEndJD();
 			if ( level > 0 )
 			{
-				for( j = 0; j < item2->depitems.size(); j++ )  
+				for ( j = 0; j < item2->depitems.size(); j++ )
 				{
 					item3 = item2->depitems[j];
 					if ( item3->dasa->getStartJD() < theJd && item3->dasa->getEndJD() > theJd )
 					{
 						if ( expert->isGrahaDasaExpert())
-							{ antarlord = writer->getObjectName(item3->dasa->getDasaLord(), TMEDIUM, true ); }
-						else { antarlord = writer->getSignName(item3->dasa->getDasaLord(), TMEDIUM); }
+						{
+							antarlord = lang.getObjectName(item3->dasa->getDasaLord(), TMEDIUM, true );
+						}
+						else {
+							antarlord = lang.getSignName(item3->dasa->getDasaLord(), TMEDIUM);
+						}
 
 						startJd = item3->dasa->getStartJD();
 						endJd = item3->dasa->getEndJD();
 						if ( level > 1 )
 						{
-							for( k = 0; k < item3->depitems.size(); k++ )  
+							for ( k = 0; k < item3->depitems.size(); k++ )
 							{
 								item4 = item3->depitems[k];
 								if ( item4->dasa->getStartJD() < theJd && item4->dasa->getEndJD() > theJd )
 								{
 									if ( expert->isGrahaDasaExpert() )
-										{ pratyantarlord = writer->getObjectName(item4->dasa->getDasaLord(), TMEDIUM, true ); }
-									else { pratyantarlord = writer->getSignName(item4->dasa->getDasaLord(), TMEDIUM); }
+									{
+										pratyantarlord = lang.getObjectName(item4->dasa->getDasaLord(), TMEDIUM, true );
+									}
+									else {
+										pratyantarlord = lang.getSignName(item4->dasa->getDasaLord(), TMEDIUM);
+									}
 									startJd = item4->dasa->getStartJD();
 									endJd = item4->dasa->getEndJD();
 									break;
@@ -818,8 +835,8 @@ wxString DasaGraphicWidget::getTextFieldContents()
 	theJd = date.getJD() + portionx * 36525;
 	theDate = formatter->getDateStringFromJD( theJd );
 	mycursor = 1.5 * ybar;
-  for( dasa = 0; dasa < dasas.Count(); dasa++ )
-  {
+	for ( dasa = 0; dasa < dasas.Count(); dasa++ )
+	{
 		if ( y >= (int)mycursor && y < (int)(mycursor + ybar ))
 		{
 			return getDasaText( dasa, 0, theJd );
@@ -859,7 +876,7 @@ GraphicalDasaItem::~GraphicalDasaItem()
 	unsigned int i;
 	delete dasa;
 	dasa = 0;
-	for( i = 0; i < depitems.size(); i++ )
+	for ( i = 0; i < depitems.size(); i++ )
 	{
 		delete depitems[i];
 	}
