@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/gui/EphemView.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "EphemView.h"
+#pragma implementation "EphemView.h"
 #endif
 
 #include "EphemView.h"
@@ -54,16 +54,16 @@ IMPLEMENT_CLASS( EphemWidget, BasicWidget )
 
 /*************************************************//**
 *
-*  
+*
 *
 ******************************************************/
-class EphemTextWidget : public TextWidget 
+class EphemTextWidget : public TextWidget
 {
 	DECLARE_CLASS( EphemTextWidget )
 
 public:
 	EphemTextWidget( wxWindow *parent, BasicView *v, EphemExpert *expert )
-		: TextWidget( parent, v )
+			: TextWidget( parent, v )
 	{
 		this->expert = expert;
 	}
@@ -73,23 +73,23 @@ public:
 		int ret = 0;
 		switch ( mode )
 		{
-			case 1:
-				assert( false );
+		case 1:
+			assert( false );
 			break;
-			case 2:
-				expert->writeDetails( writer );
+		case 2:
+			expert->writeDetails( writer );
 			break;
-			case 3:
-				ret = expert->writeIngress( writer );
+		case 3:
+			ret = expert->writeIngress( writer );
 			break;
-			case 4:
-				expert->writeLunar( writer );
+		case 4:
+			expert->writeLunar( writer );
 			break;
-			case 5:
-				expert->writeKp( writer, dasa );
+		case 5:
+			expert->writeKp( writer, dasa );
 			break;
-			default:
-				ret = expert->writeDefaultEphemeris( writer );
+		default:
+			ret = expert->writeDefaultEphemeris( writer );
 			break;
 		}
 		return ret;
@@ -105,50 +105,50 @@ IMPLEMENT_CLASS( EphemTextWidget, TextWidget )
 
 /*****************************************************
 **
-**   EphemView   ---   Constructor 
+**   EphemView   ---   Constructor
 **
 ******************************************************/
 EphemView::EphemView( wxWindow *parent, ChildWindow *frame, const bool &vedic )
- : BasicView( parent, frame, 0, VIEW_EPHEM, true, vedic )
- {
+		: BasicView( parent, frame, 0, VIEW_EPHEM, true, vedic )
+{
 	supportEWToggle = supportRestrictedObjectToggle = true;
-  int dummy;
-  Formatter::get()->getDateIntsFromJD( JDate().getJD(), dummy, month, year );
+	int dummy;
+	Formatter::get()->getDateIntsFromJD( JDate::getCurrentJD(), dummy, month, year );
 	expert = new EphemExpert( this );
 
-  lastmonth = month;
+	lastmonth = month;
 	dasa = 0;
-  planets = 1;
-  mode = config->ephemMode;
-  circleType = config->ephemCircleType;
+	planets = 1;
+	mode = config->ephemMode;
+	circleType = config->ephemCircleType;
 
-  gmode = ( mode == 1 );
+	gmode = ( mode == 1 );
 	theText = 0;
 	ewidget = 0;
-	tz = config->ephemTimezone;
-	
-	expert->prepareMonth( month, year, tz );
+	isLocaltime = config->ephemTimezone;
+
+	expert->prepareMonth( month, year, isLocaltime );
 
 	initToolItems();
 	initClientView();
 	updateView = true;
 
-  Connect( TBB_NOW, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( EphemView::OnNow ));
-  
-  Connect( TBS_YEAR, wxEVT_COMMAND_SPINCTRL_UPDATED, wxSpinEventHandler( EphemView::OnYearSpin ));
-  Connect( TBS_MONTH, wxEVT_COMMAND_SPINCTRL_UPDATED, wxSpinEventHandler( EphemView::OnMonthSpin ));
-  
-  Connect( TBS_EPHEMPLANETS, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceCommand ));
-  Connect( TBS_EPHEMTZ, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceTZ ));
-  Connect( TBS_DASA, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceCommand ));
-  Connect( TBS_EPHEMMODE, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceCommand ));
-  Connect( TBS_EPHEMDEG, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceCommand ));
-  Connect( wxEVT_IDLE, wxIdleEventHandler( EphemView::OnIdle ));
+	Connect( TBB_NOW, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler( EphemView::OnNow ));
+
+	Connect( TBS_YEAR, wxEVT_COMMAND_SPINCTRL_UPDATED, wxSpinEventHandler( EphemView::OnYearSpin ));
+	Connect( TBS_MONTH, wxEVT_COMMAND_SPINCTRL_UPDATED, wxSpinEventHandler( EphemView::OnMonthSpin ));
+
+	Connect( TBS_EPHEMPLANETS, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceCommand ));
+	Connect( TBS_TZ, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceTZ ));
+	Connect( TBS_DASA, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceCommand ));
+	Connect( TBS_EPHEMMODE, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceCommand ));
+	Connect( TBS_EPHEMDEG, wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler( EphemView::OnChoiceCommand ));
+	Connect( wxEVT_IDLE, wxIdleEventHandler( EphemView::OnIdle ));
 }
 
 /*****************************************************
 **
-**   EphemView   ---   Destructor 
+**   EphemView   ---   Destructor
 **
 ******************************************************/
 EphemView::~EphemView()
@@ -166,26 +166,29 @@ void EphemView::initToolItems()
 	if ( toolbar )
 	{
 		wxSpinCtrl *spin_year = (wxSpinCtrl*)toolbar->FindControl( TBS_YEAR );
-		if( spin_year ) spin_year->SetValue( year );
+		if ( spin_year ) spin_year->SetValue( year );
 
 		wxSpinCtrl *spin_month = (wxSpinCtrl*)toolbar->FindControl( TBS_MONTH );
-		if( spin_month ) spin_month->SetValue( month );
+		if ( spin_month ) spin_month->SetValue( month );
 
 		wxChoice *choice_ephem_mode = (wxChoice*)toolbar->FindControl( TBS_EPHEMMODE );
-		if( choice_ephem_mode ) choice_ephem_mode->SetSelection( mode );
+		if ( choice_ephem_mode ) choice_ephem_mode->SetSelection( mode );
 
-		wxChoice *choice_ephemtz = (wxChoice*)toolbar->FindControl( TBS_EPHEMTZ );
-		if( choice_ephemtz ) choice_ephemtz->SetSelection( tz );
+		wxChoice *choice_ephemtz = (wxChoice*)toolbar->FindControl( TBS_TZ );
+		if ( choice_ephemtz ) choice_ephemtz->SetSelection( isLocaltime );
 
 		wxChoice *choice_ephemdeg = (wxChoice*)toolbar->FindControl( TBS_EPHEMDEG );
 		if ( choice_ephemdeg )
 		{
 			choice_ephemdeg->SetSelection( circleType );
-			choice_ephemdeg->Enable( mode == 1 );
+			ADJUST_TOOLBAR_ITEM( choice_ephemdeg, mode == 1 );
 		}
 
 		wxChoice *choice_dasa = (wxChoice*)toolbar->FindControl( TBS_DASA );
-		if ( choice_dasa ) choice_dasa->Enable( mode == 5 );
+		if ( choice_dasa )
+		{
+			ADJUST_TOOLBAR_ITEM( choice_dasa, mode == 5 );
+		}
 	}
 }
 
@@ -243,13 +246,13 @@ void EphemView::OnDataChanged()
 {
 	BasicView::OnDataChanged();
 	OnToolbarCommand();
-	expert->prepareMonth( month, year, tz );
+	expert->prepareMonth( month, year, isLocaltime );
 	updateView = true;
 }
 
 /*****************************************************
 **
-**   EphemView   ---   OnChoiceTZ 
+**   EphemView   ---   OnChoiceTZ
 **
 ******************************************************/
 void EphemView::OnChoiceTZ( wxCommandEvent& )
@@ -259,7 +262,7 @@ void EphemView::OnChoiceTZ( wxCommandEvent& )
 
 /*****************************************************
 **
-**   EphemView   ---   OnYearSpin 
+**   EphemView   ---   OnYearSpin
 **
 ******************************************************/
 void EphemView::OnYearSpin( wxSpinEvent &event )
@@ -269,17 +272,17 @@ void EphemView::OnYearSpin( wxSpinEvent &event )
 
 /*****************************************************
 **
-**   EphemView   ---   OnMonthSpin 
+**   EphemView   ---   OnMonthSpin
 **
 ******************************************************/
 void EphemView::OnMonthSpin( wxSpinEvent &event )
 {
 	OnDataChanged();
 }
-      
+
 /*****************************************************
 **
-**   EphemView   ---   OnToolbarCommand 
+**   EphemView   ---   OnToolbarCommand
 **
 ******************************************************/
 void EphemView::OnToolbarCommand()
@@ -315,11 +318,11 @@ void EphemView::OnToolbarCommand()
 			month = spin_month->GetValue();
 		}
 
-		wxChoice *choice_ephemtz = (wxChoice*)toolbar->FindControl( TBS_EPHEMTZ );
+		wxChoice *choice_ephemtz = (wxChoice*)toolbar->FindControl( TBS_TZ );
 		if ( choice_ephemtz )
 		{
-			tz = choice_ephemtz->GetSelection();
-			config->ephemTimezone = tz;
+			isLocaltime = choice_ephemtz->GetSelection();
+			config->ephemTimezone = isLocaltime;
 		}
 
 		wxChoice *choice_ephemdeg = (wxChoice*)toolbar->FindControl( TBS_EPHEMDEG );
@@ -328,19 +331,19 @@ void EphemView::OnToolbarCommand()
 			circleType = choice_ephemdeg->GetSelection();
 			config->ephemCircleType = circleType;
 			setMaxDeg( circleType );
-			choice_ephemdeg->Enable( mode == 1 );
+			ADJUST_TOOLBAR_ITEM( choice_ephemdeg, mode == 1 );
 		}
 
 		wxChoice *choice_dasa = (wxChoice*)toolbar->FindControl( TBS_DASA );
 		if ( choice_dasa )
 		{
 			dasa = choice_dasa->GetSelection();
-			choice_dasa->Enable( mode == 5 );
+			ADJUST_TOOLBAR_ITEM( choice_dasa, mode == 5 );
 		}
 		updateView = true;
 	}
 }
-   
+
 /*****************************************************
 **
 **   EphemView   ---   setMaxDeg
@@ -349,25 +352,45 @@ void EphemView::OnToolbarCommand()
 void EphemView::setMaxDeg( const int &sel )
 {
 	double max_deg;
-	switch( sel )
+	switch ( sel )
 	{
-		case 0 : max_deg =   360; break;
-		case 1 : max_deg =   180; break;
-		case 2 : max_deg =    90; break;
-		case 3 : max_deg =    45; break;
-		case 4 : max_deg =    30; break;
-		case 5 : max_deg =  22.5; break;
-		case 6 : max_deg = 13.333333333; break;
-		case 7 : max_deg = 11.25; break;
-		case 8 : max_deg = 5.125; break;
-		default: max_deg =   360; break;
+	case 0 :
+		max_deg =   360;
+		break;
+	case 1 :
+		max_deg =   180;
+		break;
+	case 2 :
+		max_deg =    90;
+		break;
+	case 3 :
+		max_deg =    45;
+		break;
+	case 4 :
+		max_deg =    30;
+		break;
+	case 5 :
+		max_deg =  22.5;
+		break;
+	case 6 :
+		max_deg = 13.333333333;
+		break;
+	case 7 :
+		max_deg = 11.25;
+		break;
+	case 8 :
+		max_deg = 5.125;
+		break;
+	default:
+		max_deg =   360;
+		break;
 	}
 	if ( ewidget ) ewidget->setMaxDeg( max_deg );
 }
 
 /*****************************************************
 **
-**   EphemView   ---   OnNow 
+**   EphemView   ---   OnNow
 **
 ******************************************************/
 void EphemView::OnNow( wxCommandEvent &event )
@@ -379,19 +402,19 @@ void EphemView::OnNow( wxCommandEvent &event )
 		wxSpinCtrl *spin_month = (wxSpinCtrl*)toolbar->FindControl( TBS_MONTH );
 		if ( spin_year && spin_month )
 		{
-			Formatter::get()->getDateIntsFromJD( JDate().getJD(), dummy, month, year );
+			Formatter::get()->getDateIntsFromJD( JDate::getCurrentJD(), dummy, month, year );
 			spin_year->SetValue( year );
 			spin_month->SetValue( month );
 			lastmonth = month;
 		}
-		expert->prepareMonth( month, year, tz );
+		expert->prepareMonth( month, year, isLocaltime );
 	}
 	updateView = true;
-}   
+}
 
 /*****************************************************
 **
-**   EphemView   ---   dispatchCommand 
+**   EphemView   ---   dispatchCommand
 **
 ******************************************************/
 bool EphemView::dispatchCommand( const int &command )
@@ -434,7 +457,13 @@ void EphemView::OnIdle( wxIdleEvent &event )
 		theText->dasa = dasa;
 		ret = theText->writeContents();
 	}
-	if ( ret ) tellThatObjectsAreNotCalculated( this, ret );
+	if ( ret )
+	{
+		wxCommandEvent event( SHOW_EPHEM_FILE_WARNING );
+		event.SetEventObject( this );
+		wxPostEvent( GetParent(), event );
+	}
+	
 	updateView = false;
 }
 
@@ -455,7 +484,7 @@ wxString EphemView::getWindowLabel( const bool shortname )
 ******************************************************/
 bool EphemView::supportsTextExport() const
 {
-  return( theText != 0 );
+	return( theText != 0 );
 }
 
 /**************************************************************
@@ -464,14 +493,14 @@ bool EphemView::supportsTextExport() const
 ***
 ***************************************************************/
 EphemWidget::EphemWidget( wxWindow *parent, ChartProperties *chartprops, EphemExpert *e )
- : BasicWidget( parent, chartprops ),
-	 expert( e )
+		: BasicWidget( parent, chartprops ),
+		expert( e )
 {
 	max_deg = 360;
 	filter = 1;
 	writer = WriterFactory().getWriter( WRITER_TEXT );
 
-  Connect( wxEVT_SIZE, wxSizeEventHandler( EphemWidget::OnSize ));
+	Connect( wxEVT_SIZE, wxSizeEventHandler( EphemWidget::OnSize ));
 }
 
 /**************************************************************
@@ -481,30 +510,30 @@ EphemWidget::EphemWidget( wxWindow *parent, ChartProperties *chartprops, EphemEx
 ***************************************************************/
 EphemWidget::~EphemWidget()
 {
- 	delete writer;
+	delete writer;
 }
 
 /*****************************************************
 **
-**   EphemWidget   ---   setZoom 
+**   EphemWidget   ---   setZoom
 **
 ******************************************************/
 void EphemWidget::setZoom( const int z )
 {
-  if ( z ) zoom = z;
-  int width = 100 * zoom;
-  int height = 500;
-  SetVirtualSize( width, height );
+	if ( z ) zoom = z;
+	int width = 100 * zoom;
+	int height = 500;
+	SetVirtualSize( width, height );
 
-  xleft = 6 * hborder;
-  xright = width - xleft - 4 * hborder;
-  ytop = 2 * vborder;
-  ybottom = height - 4 * vborder;
+	xleft = 6 * hborder;
+	xright = width - xleft - 4 * hborder;
+	ytop = 2 * vborder;
+	ybottom = height - 4 * vborder;
 }
 
 /*****************************************************
 **
-**   EphemWidget   ---   OnSize 
+**   EphemWidget   ---   OnSize
 **
 ******************************************************/
 void EphemWidget::OnSize( wxSizeEvent &event )
@@ -530,17 +559,17 @@ void EphemWidget::paintRuler()
 	double ystep;
 
 	painter->setTransparentBrush();
-	
-	painter->setPenColor( config->colors->fgColor ); 
+
+	painter->setPenColor( config->colors->fgColor );
 	painter->drawRectangle( (int)xleft, (int)ybottom, (int)(xright - xleft), (int)(ytop - ybottom) );
 	int mlen = expert->getNumberOfDays();
 	double dstep = ( xright - xleft ) / ( mlen - 1 );
-	for( i = 0; i < mlen; i++ )
+	for ( i = 0; i < mlen; i++ )
 	{
-		painter->setPenColor( config->colors->fgColor ); 
+		painter->setPenColor( config->colors->fgColor );
 		x =  (int)(xleft+i*dstep);
 		painter->drawLine( x, (int)ybottom, x, (int)ybottom+5 );
-    s.Printf( wxT( "%d" ), i+1 );
+		s.Printf( wxT( "%d" ), i+1 );
 		drawTextFormatted( wxRect( x-xtext, y + ysizey + ytext, 2*xtext, 2*ytext ), s, Align::Center );
 		if ( ! ( ( i + 1 ) % 5 ) && i > 0 )
 		{
@@ -550,16 +579,16 @@ void EphemWidget::paintRuler()
 	}
 
 	ystep = (ybottom - ytop) / max_deg;
-	for( i = 1; i < max_deg; i++ )
+	for ( i = 1; i < max_deg; i++ )
 	{
-		painter->setPenColor( config->colors->fgColor ); 
+		painter->setPenColor( config->colors->fgColor );
 
 		if ( max_deg >= 180 && ( i % 30 ) ) continue;
 		if ( max_deg >= 90 && ( i % 10 ) ) continue;
 		if ( max_deg >= 22 && ( i % 5 ) ) continue;
 		y = (int)(ybottom - i * ystep);
 		painter->drawLine( (int)xleft-5, (int)y, (int)xleft, (int)y );
-    s.Printf( wxT( "%d" ), i );
+		s.Printf( wxT( "%d" ), i );
 		drawTextFormatted( wxRect( (int)xleft-xtext-5, y - ysizey, xtext, 2 * ysizey ), s, Align::Right+Align::VCenter );
 
 		painter->setPenColor( *wxLIGHT_GREY ); // TODO DASH
@@ -581,36 +610,30 @@ void EphemWidget::paintRuler()
 
 /*****************************************************
 **
-**   EphemWidget   ---   doPaint 
+**   EphemWidget   ---   doPaint
 **
 ******************************************************/
 void EphemWidget::doPaint( bool eraseBackground, const wxRect* rect )
 {
-  wxCoord w, h;
+	wxCoord w, h;
 	expert->calcMonth();
 
-  assert( painter ); 
-  GetVirtualSize( &w, &h );
+	assert( painter );
+	GetVirtualSize( &w, &h );
 
-	painter->setPenColor( *wxLIGHT_GREY );
-  //dc->SetBrush( *wxGREY_BRUSH );
-  painter->drawRectangle( 0, 0, w, h );
+	painter->setBrushColor( config->colors->bgColor );
+	painter->setPenColor( config->colors->bgColor );
+	painter->drawRectangle( hborder, vborder, (int)(w - 2 * hborder), (int)(h - 2 * vborder) );
 
-  // White page background
-  //dc->SetBrush( *wxWHITE_BRUSH );
-	painter->setBrushColor( *wxWHITE );
-  painter->setPenColor( *wxWHITE );
 	painter->setGraphicFont();
-  painter->drawRectangle( hborder, vborder, (int)(w - 2 * hborder), (int)(h - 2 * vborder) );
+	painter->setTextColor( config->colors->fgColor );
 
-  //SetVirtualSize( Max( w, 500 ), Max( h, 300 ));
+	xleft = 6 * hborder;
+	xright = w - xleft - 2 * hborder;
+	ytop = 2 * vborder;
+	ybottom = h - 4 * vborder;
 
-  xleft = 6 * hborder;
-  xright = w - xleft - 2 * hborder;
-  ytop = 2 * vborder;
-  ybottom = h - 4 * vborder;
-
-  paintRuler();
+	paintRuler();
 	paintPlanets();
 }
 
@@ -626,7 +649,6 @@ void EphemWidget::paintPlanets()
 	int xp;              // x position for jumps
 	//int i;
 	double l1, l2;       // length of planets
-	double s1, s2;       // daily speed
 	double yp, yp2, ydiff;      // length values for jumps
 	const int sshift = 60;
 	int lsymbolshift[sshift];
@@ -642,13 +664,13 @@ void EphemWidget::paintPlanets()
 	wxString s;
 	Lang lang;
 
-	for( int i = 0; i < sshift; i++ )
+	for ( int i = 0; i < sshift; i++ )
 	{
 		lsymbolshift[i] = 0;
 		rsymbolshift[i] = 0;
 	}
 
-	for( unsigned i1 = 0; i1 < expert->getPlanetdataSize(); i1++ )
+	for ( unsigned i1 = 0; i1 < expert->getPlanetdataSize(); i1++ )
 	{
 		p = expert->getPlanetId( i1 );
 
@@ -656,15 +678,13 @@ void EphemWidget::paintPlanets()
 		if ( max_deg < 14 &&  p == OMOON ) continue;
 
 		setLineStyle( p );
-		for( int day = 0; day < mlen; day++ )
+		for ( int day = 0; day < mlen; day++ )
 		{
 			x1 =  (int)(xleft + day * dstep);
 			x2 =  (int)(x1 + dstep);
 			l1 = a_red( expert->getPlanetLength( i1, day ), max_deg );
-			s1 = expert->getPlanetSpeed( i1, day );
 			y1 = (int)(ybottom - l1 * ytotal / max_deg);
 			l2 = a_red( expert->getPlanetLength( i1, day+1 ), max_deg );
-			s2 = expert->getPlanetSpeed( i1, day+1 );
 			y2 = (int)(ybottom - l2 * ytotal / max_deg);
 
 			if ( expert->getPlanetRetro( i1, day ))
@@ -698,7 +718,7 @@ void EphemWidget::paintPlanets()
 				else // that happens normally
 				{
 					painter->drawLine( x1, y1, x2, y2 );
-				} 
+				}
 			}
 		}
 		if ( config->usePlanetSymbols )
@@ -733,12 +753,8 @@ void EphemWidget::paintPlanets()
 ******************************************************/
 void EphemWidget::setLineStyle( const int &planet )
 {
-	const wxPen *pen = wxBLACK_PEN;
 	const int lw = 2;
-	pen = wxThePenList->FindOrCreatePen( config->colors->getPlanetColor( planet ), lw, wxSOLID );
-
-	painter->setPen( *pen ); 
-	painter->setTextColor( pen->GetColour() ); 
+	painter->setPen( wxPen( config->colors->getPlanetColor( planet ), lw, wxSOLID ));
 }
 
 /**************************************************************
