@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/gui/MenuProvider.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "MenuProvider.h"
+#pragma implementation "MenuProvider.h"
 #endif
 
 #include "MenuProvider.h"
@@ -45,8 +45,8 @@ IMPLEMENT_CLASS( MyMenu, wxMenu )
 ***
 ***************************************************************/
 MyMenu::MyMenu( const wxString title, const BasicView *view )
-	: wxMenu( title ),
-	view( view )
+		: wxMenu( title ),
+		view( view )
 {
 	handler = CommandHandler::get();
 }
@@ -65,7 +65,7 @@ void MyMenu::addItem( const int &id )
 		if ( command->getBitmapId() != 0 ) item->SetBitmap( IconProvider::getMenuBitmap( command->getBitmapId() ));
 		Append( item );
 	}
-else assert( 0 );
+	else assert( 0 );
 }
 
 /**************************************************************
@@ -102,7 +102,7 @@ void MyMenu::addCheckItem( const int &id, const wxString title, const bool check
 ***************************************************************/
 void MyMenu::addArray( const int a[] )
 {
-	while( *a != -1 )
+	while ( *a != -1 )
 	{
 		if ( *a ) addItem( *a );
 		else AppendSeparator();
@@ -121,11 +121,17 @@ void MyMenu::addWindowListMenu()
 	wlmenu->AppendRadioItem( APP_LIST_BIGICONS, _( "Big Icons" ) );
 	wlmenu->AppendRadioItem( APP_LIST_SMALLICONS, _( "Small Icons" ) );
 	wlmenu->AppendRadioItem( APP_LIST_TABLE, _( "Table" ) );
-	switch( config->listStyle )
+	switch ( config->listStyle )
 	{
-		case 1: wlmenu->Check( APP_LIST_SMALLICONS, true ); break;
-		case 2: wlmenu->Check( APP_LIST_TABLE, true ); break;
-		default: wlmenu->Check( APP_LIST_BIGICONS, true ); break;
+	case 1:
+		wlmenu->Check( APP_LIST_SMALLICONS, true );
+		break;
+	case 2:
+		wlmenu->Check( APP_LIST_TABLE, true );
+		break;
+	default:
+		wlmenu->Check( APP_LIST_BIGICONS, true );
+		break;
 	}
 	Append( APP_LISTCTRL, _("Window List"), wlmenu );
 }
@@ -138,141 +144,167 @@ void MyMenu::addWindowListMenu()
 void MyMenu::addVargaMenu()
 {
 	MyMenu *menu = new MyMenu( wxEmptyString, view );
-	for( int i = 0; i < NB_VARGAS; i++ ) menu->addItem( CMD_CHILD_NEW_RASI + i );
+	for ( int i = 0; i < NB_VARGAS; i++ ) menu->addItem( CMD_CHILD_NEW_RASI + i );
 	Append( -1, _( "Varga"), menu );
 }
 
 /*****************************************************
 **
-**   AppMenuBar   ---   Constructor 
+**   AppMenuBar   ---   Constructor
 **
 ******************************************************/
 AppMenuBar::AppMenuBar( int style )
- : wxMenuBar( style )
+		: wxMenuBar( style )
 {
-	const int docitems[] = { CMD_CHILD_EDITDATA, 0, CMD_CHILD_CLOSE_DOC,
-		CMD_CHILD_SAVE, CMD_CHILD_SAVEAS, 0, CMD_CHILD_PRINT, CMD_CHILD_QUICKPRINT, 0, -1	};
-	const int vedicitems[13] = { CMD_CHILD_NEW_VARGA, CMD_CHILD_NEW_SBC, CMD_CHILD_NEW_TEXT, 0,
+	const int docitems[] = { CMD_CHILD_EDITDATA, 0, CMD_CHILD_CLOSE_DOC, CMD_CHILD_SAVE, CMD_CHILD_SAVEAS, 0,
+		CMD_CHILD_PRINT, CMD_CHILD_QUICKPRINT, 0, -1 };
+	const int vedicitems[12] = { CMD_CHILD_NEW_VARGA, CMD_CHILD_NEW_SBC, CMD_CHILD_NEW_TEXT, 0,
 		CMD_CHILD_NEW_DASA, CMD_CHILD_NEW_GRAPHICALDASA, CMD_CHILD_NEW_TRANSIT, 0, CMD_CHILD_NEW_YOGA,
-		CMD_CHILD_NEW_ASHTAKAVARGA,
-		CMD_CHILD_NEW_SOLAR, -1 };
-	const int westernitems[8] = { CMD_CHILD_NEW_WCHART, CMD_CHILD_NEW_TEXT, 0,
-		CMD_CHILD_NEW_URANIAN, CMD_CHILD_NEW_TRANSIT, 0, CMD_CHILD_NEW_SOLAR, -1 };
-	const int extraitems[11] = { APP_SHOWEPHEM, APP_SHOWECLIPSE, APP_SHOWHORA, APP_NEW_PARTNER,
-		0, APP_CONFIGURE, APP_LOCATION, APP_YOGAEDITOR, APP_SAVECONFIG, 0, -1	};
+		CMD_CHILD_NEW_ASHTAKAVARGA, CMD_CHILD_NEW_SOLAR, -1 };
+	const int westernitems[8] = { CMD_CHILD_NEW_WCHART, CMD_CHILD_NEW_TEXT, 0, CMD_CHILD_NEW_URANIAN,
+		CMD_CHILD_NEW_TRANSIT, 0, CMD_CHILD_NEW_SOLAR, -1 };
+	const int extraitems[11] = { APP_SHOWEPHEM, APP_SHOWECLIPSE, APP_SHOWHORA, APP_NEW_PARTNER, 0,
+		APP_CONFIGURE, APP_LOCATION, APP_YOGAEDITOR, APP_SAVECONFIG, 0, -1 };
 
-  newmenu = new MyMenu( wxEmptyString, 0 );
-  vector<MultipleViewConfig*> v = MultipleViewConfigLoader::get()->getConfigs();
-	for( unsigned int i = 0; i < v.size(); i++ )
+	newmenu = new MyMenu( wxEmptyString, 0 );
+	vector<BaseXmlConfig*> v = MultipleViewConfigLoader::get()->getConfigs();
+	for ( unsigned int i = 0; i < v.size(); i++ )
 	{
 		newmenu->Append( APP_FILE_NEW_MULTIPLE + i, v[i]->name );
 	}
 
-  recentmenu = new MyMenu( wxEmptyString, 0 );
-  wxString rfiles = config->recentFileList;
-  int i = rfiles.Find( '\t' );
-  while( i != -1 )
-  {
-    recentfiles.Add( rfiles.Left( i ));
-    rfiles = rfiles.Right( rfiles.Len() - i - 1 );
-    i = rfiles.Find( '\t' );
-  }
-  if ( rfiles.Len() > 0 ) recentfiles.Add( rfiles );
-  for( size_t n = 0; n < recentfiles.GetCount(); n++ ) recentmenu->Append( APP_FILERECENT + n, recentfiles[n] );
+	recentmenu = new MyMenu( wxEmptyString, 0 );
+	wxString rfiles = config->recentFileList;
+	int i = rfiles.Find( '\t' );
+	while ( i != -1 )
+	{
+		recentfiles.Add( rfiles.Left( i ));
+		rfiles = rfiles.Right( rfiles.Len() - i - 1 );
+		i = rfiles.Find( '\t' );
+	}
+	if ( rfiles.Len() > 0 ) recentfiles.Add( rfiles );
+	for ( size_t n = 0; n < recentfiles.GetCount(); n++ ) recentmenu->Append( APP_FILERECENT + n, recentfiles[n] );
+
+	filemenu = new MyMenu( wxEmptyString, 0 );
+	filemenu->addItem( APP_FILENEW );
+
+	if ( ! config->useMultipleViews ) filemenu->addItem( APP_FILE_NEW_SIMPLE );
+	newChildSimpleChildCommandIncluded = ! config->useMultipleViews;
 
-  filemenu = new MyMenu( wxEmptyString, 0 );
-  filemenu->addItem( APP_FILENEW );
-  filemenu->addItem( APP_FILE_NEW_SIMPLE );
 	filemenu->Append( APP_FILE_NEW_MULTIPLE + 99, _( "New Multiple View" ), newmenu );
-  filemenu->addItem( APP_FILEOPEN );
+	filemenu->addItem( APP_FILEOPEN );
 	filemenu->Append( APP_FILERECENT, _("Open Recent"), recentmenu );
 	filemenu->AppendSeparator();
 	filemenu->addItem( APP_FILEQUIT );
-  Append( filemenu, _( "&File" ));
+	Append( filemenu, _( "&File" ));
 
 	docmenu = new MyMenu( wxEmptyString, 0 );
 	docmenu->addArray( docitems );
 	docmenu->AppendCheckItem( CMD_CHILD_ANIMATE, _("Animate" ));
-  Append( docmenu, _( "&Document" ));
+	Append( docmenu, _( "&Document" ));
 
-  vedicmenu = new MyMenu( wxEmptyString, 0 );
+	vedicmenu = new MyMenu( wxEmptyString, 0 );
 	vedicmenu->addVargaMenu();
 	vedicmenu->addArray( vedicitems );
-  Append( vedicmenu, _( "&Vedic"));
+	Append( vedicmenu, _( "&Vedic"));
 
-  westernmenu = new MyMenu( wxEmptyString, 0 );
+	westernmenu = new MyMenu( wxEmptyString, 0 );
 	westernmenu->addArray( westernitems );
-  Append( westernmenu, _( "&Western"));
+	Append( westernmenu, _( "&Western"));
 
-  extramenu = new MyMenu( wxEmptyString, 0 );
-	extramenu->addArray( extraitems );	
-  extramenu->addWindowListMenu();
-  extramenu->addCheckItem( APP_SHOWMAINTOOLBAR, config->showMainToolbar );
-  extramenu->addCheckItem( APP_SHOWSTATUSBAR, config->showStatusBar );
-  Append( extramenu, _( "&Extras"));
+	extramenu = new MyMenu( wxEmptyString, 0 );
+	extramenu->addArray( extraitems );
+	extramenu->addWindowListMenu();
+	extramenu->addCheckItem( APP_SHOWMAINTOOLBAR, config->showMainToolbar );
+	extramenu->addCheckItem( APP_SHOWSTATUSBAR, config->showStatusBar );
+	Append( extramenu, _( "&Extras"));
 
-  helpmenu = new MyMenu( wxEmptyString, 0 );
+	helpmenu = new MyMenu( wxEmptyString, 0 );
 	helpmenu->addItem( APP_HELP );
 	helpmenu->addItem( APP_ABOUT );
-  Append( helpmenu, _( "&Help"));
+	//helpmenu->addItem( wxID_ABOUT );
+	Append( helpmenu, _( "&Help"));
+
 }
 
 /*****************************************************
 **
-**   AppMenuBar   ---   updateMenus 
+**   AppMenuBar   ---   updateMenus
 **
 ******************************************************/
 void AppMenuBar::updateMenus( const bool &isDoc, const bool &animated )
 {
-  EnableTop( 1, isDoc );
-  EnableTop( 2, isDoc );
-  EnableTop( 3, isDoc );
+	EnableTop( 1, isDoc );
+	EnableTop( 2, isDoc );
+	EnableTop( 3, isDoc );
 
 	filemenu->Enable( APP_FILERECENT, recentfiles.GetCount() > 0  );
-  if ( isDoc ) 
+	if ( isDoc )
 	{
 		docmenu->Check( CMD_CHILD_ANIMATE, animated );
 	}
+	
+	if ( config->useMultipleViews )
+	{
+		if ( ! newChildSimpleChildCommandIncluded )
+		{
+			// Insert Item for new single view
+			MaitreyaCommand *command = CommandHandler::get()->getCommand( APP_FILE_NEW_SIMPLE );
+			assert( command );
+			wxMenuItem *item = new wxMenuItem( filemenu, APP_FILE_NEW_SIMPLE, command->getMenuRepresentation());
+			if ( command->getBitmapId() != 0 ) item->SetBitmap( IconProvider::getMenuBitmap( command->getBitmapId() ));
+			filemenu->Insert( 1, item );
+			newChildSimpleChildCommandIncluded  = true;
+		}
+	}
+	else
+	{
+		if ( newChildSimpleChildCommandIncluded )
+		{
+			filemenu->Delete( APP_FILE_NEW_SIMPLE );
+			newChildSimpleChildCommandIncluded  = false;
+		}
+	}
 }
 
 /*****************************************************
 **
-**   AppMenuBar   ---   addToRecentFiles 
+**   AppMenuBar   ---   addToRecentFiles
 **
 ******************************************************/
 void AppMenuBar::addToRecentFiles( const wxString &filename )
 {
-  unsigned int maxrecent = config->maxRecentFiles;
-  unsigned int recentcount = recentmenu->GetMenuItemCount();
-  size_t n;
-   
-  // clear recent file menu
-  for( n = 0; n < recentcount; n++ ) recentmenu->Delete( APP_FILERECENT + n );
-  
-  // Clear obsolete items ( if number of files has changed via ConfigDialog)
-  if ( recentfiles.GetCount() > maxrecent )
-  {
-    for( n = 0; n < recentfiles.GetCount() - maxrecent; n++ )
-      recentfiles.RemoveAt( maxrecent + n );
-  }
-   
-  // update file list
-  for( n = 0; n < recentfiles.GetCount(); n++ )
-  {
-    if ( recentfiles[n] == filename )
-    {
-      recentfiles.RemoveAt( n );
-      break;
-    }
-  }
-  if ( recentfiles.GetCount() == (unsigned int)config->maxRecentFiles ) recentfiles.RemoveAt( recentfiles.GetCount() - 1 );
-  recentfiles.Insert( filename, 0 );
-  for( n = 0; n < recentfiles.GetCount(); n++ ) recentmenu->Append( APP_FILERECENT + n, recentfiles[n] );
+	unsigned int maxrecent = config->maxRecentFiles;
+	unsigned int recentcount = recentmenu->GetMenuItemCount();
+	size_t n;
+
+	// clear recent file menu
+	for ( n = 0; n < recentcount; n++ ) recentmenu->Delete( APP_FILERECENT + n );
+
+	// Clear obsolete items ( if number of files has changed via ConfigDialog)
+	if ( recentfiles.GetCount() > maxrecent )
+	{
+		for ( n = 0; n < recentfiles.GetCount() - maxrecent; n++ )
+			recentfiles.RemoveAt( maxrecent + n );
+	}
+
+	// update file list
+	for ( n = 0; n < recentfiles.GetCount(); n++ )
+	{
+		if ( recentfiles[n] == filename )
+		{
+			recentfiles.RemoveAt( n );
+			break;
+		}
+	}
+	if ( recentfiles.GetCount() == (unsigned int)config->maxRecentFiles ) recentfiles.RemoveAt( recentfiles.GetCount() - 1 );
+	recentfiles.Insert( filename, 0 );
+	for ( n = 0; n < recentfiles.GetCount(); n++ ) recentmenu->Append( APP_FILERECENT + n, recentfiles[n] );
 }
 
 /*****************************************************
 **
-**   AppMenuBar   ---   getRecentFilesEntry 
+**   AppMenuBar   ---   getRecentFilesEntry
 **
 ******************************************************/
 wxString AppMenuBar::getRecentFilesEntry( const int &index )
@@ -282,19 +314,19 @@ wxString AppMenuBar::getRecentFilesEntry( const int &index )
 
 /*****************************************************
 **
-**   AppMenuBar   ---   saveRecentFiles 
+**   AppMenuBar   ---   saveRecentFiles
 **
 ******************************************************/
 void AppMenuBar::saveRecentFiles()
 {
 	wxString rfiles = wxT( "" );
-  size_t n;
-  
-  for( n = 0; n < recentfiles.GetCount(); n++ )
-  {
-    if ( n > 0 ) rfiles += wxT( "\t" );
-    rfiles += recentfiles[n];
-  }
+	size_t n;
+
+	for ( n = 0; n < recentfiles.GetCount(); n++ )
+	{
+		if ( n > 0 ) rfiles += wxT( "\t" );
+		rfiles += recentfiles[n];
+	}
 	config->recentFileList = rfiles;
 }
 
@@ -308,21 +340,21 @@ wxMenu *ContextMenuProvider::getContextMenu( ChildWindow *child, BasicView *basi
 	assert( basicview );
 	int count = 0;
 	view = basicview;
-  MyMenu *menu = new MyMenu( child->GetTitle(), view );
+	MyMenu *menu = new MyMenu( child->GetTitle(), view );
 
 	bool ismainview = view->isMainView();
-	
+
 	menu->Append( -1, _( "File" ), getChartMenu());
 	if ( view->getDoc()) menu->addItem( CMD_CHILD_EDITDATA );
 
 	if ( view->supportsEWToggle() )
-  {
-    menu->AppendSeparator();
+	{
+		menu->AppendSeparator();
 		menu->addCheckItem( CMD_CHILD_VEDICMODE, view->isVedic());
 		menu->addCheckItem( CMD_CHILD_WESTERNMODE, ! view->isVedic());
-  }
+	}
 
-  menu->AppendSeparator();
+	menu->AppendSeparator();
 	if ( view->supportsObjectToggle() || view->supportsRestrictedObjectToggle())
 	{
 		menu->AppendSubMenu( getObjectSubMenu(), _( "Objects"));
@@ -349,19 +381,19 @@ wxMenu *ContextMenuProvider::getContextMenu( ChildWindow *child, BasicView *basi
 		count++;
 	}
 	if ( count > 0 ) menu->AppendSeparator();
-	
+
 	menu->AppendSubMenu( getNewViewMenu(), _( "New View"));
 	if ( view->getDoc()) // TODO auch fr non doc
-  {
-    menu->AppendSeparator();
-    menu->Append( CMD_CHILD_ANIMATE, _( "Animate"), wxT( "" ), true );
+	{
+		menu->AppendSeparator();
+		menu->Append( CMD_CHILD_ANIMATE, _( "Animate"), wxT( "" ), true );
 		menu->Check( CMD_CHILD_ANIMATE, view->getDoc()->isAnimated());
 	}
 	if ( view->supportsGraphicExport() )
-  {
-    menu->AppendSeparator();
-    menu->addItem( CMD_CHILD_EXPORT );
-  }
+	{
+		menu->AppendSeparator();
+		menu->addItem( CMD_CHILD_EXPORT );
+	}
 	if ( view->supportsTextExport() )
 	{
 		menu->AppendSeparator();
@@ -379,7 +411,7 @@ wxMenu *ContextMenuProvider::getContextMenu( ChildWindow *child, BasicView *basi
 ***************************************************************/
 wxMenu *ContextMenuProvider::getMainViewColumnMenu()
 {
-  wxMenu *menu = new wxMenu( _( "Columns" ));
+	wxMenu *menu = new wxMenu( _( "Columns" ));
 	if ( view->isVedic())
 	{
 		menu->Append( CMD_CHILD_MAIN_SHOW_DECLINATION, _( "Declinations" ), wxT( "" ), true );
@@ -427,11 +459,11 @@ wxMenu *ContextMenuProvider::getMainViewColumnMenu()
 wxMenu *ContextMenuProvider::getSkinMenu()
 {
 	unsigned int i = 0;
-  MyMenu *menu = new MyMenu( _( "Skin" ), view );
+	MyMenu *menu = new MyMenu( _( "Skin" ), view );
 
 	if ( view->isVedic())
 	{
-		vector<VedicChartConfig*> list =  VedicChartConfigLoader::get()->getConfigs();
+		vector<BaseXmlConfig*> list =  VedicChartConfigLoader::get()->getConfigs();
 		for ( unsigned i = 0; i < list.size(); i++ )
 		{
 			menu->addCheckItem( CMD_CHILD_GRAPHIC_STYLE + i, list[i]->name, i == (unsigned int)view->getSkin());
@@ -439,7 +471,7 @@ wxMenu *ContextMenuProvider::getSkinMenu()
 	}
 	else
 	{
-		vector<WesternChartConfig*> list =  WesternChartConfigLoader::get()->getConfigs();
+		vector<BaseXmlConfig*> list =  WesternChartConfigLoader::get()->getConfigs();
 		for ( i = 0; i < list.size(); i++ )
 		{
 			menu->addCheckItem( CMD_CHILD_GRAPHIC_STYLE + i, list[i]->name, i == (unsigned int) view->getSkin());
@@ -455,7 +487,7 @@ wxMenu *ContextMenuProvider::getSkinMenu()
 ***************************************************************/
 wxMenu *ContextMenuProvider::getObjectSubMenu()
 {
-  MyMenu *menu = new MyMenu( _( "Objects" ), view );
+	MyMenu *menu = new MyMenu( _( "Objects" ), view );
 	menu->addCheckItem( CMD_CHILD_SHOWOUTER, _( "3 Outer Planets"), view->getObjectStyle() & OBJECTS_INCLUDE_OUTER );
 	menu->addCheckItem( CMD_CHILD_SHOWDRAGONHEAD, _( "Dragon Head"), view->getObjectStyle() & OBJECTS_INCLUDE_DRAGONHEAD );
 	menu->addCheckItem( CMD_CHILD_SHOWDRAGONTAIL, _( "Dragon Tail"), view->getObjectStyle() & OBJECTS_INCLUDE_DRAGONTAIL );
@@ -477,7 +509,7 @@ wxMenu *ContextMenuProvider::getObjectSubMenu()
 		menu->addCheckItem( CMD_CHILD_SHOWKALAVELAS, _( "Kalavelas"), view->getObjectStyle() & OBJECTS_INCLUDE_KALAVELAS );
 		menu->addCheckItem( CMD_CHILD_SHOWSPECIALLAGNAS, _( "3 Special Lagnas"), view->getObjectStyle() & OBJECTS_INCLUDE_SPECIALLAGNAS );
 		menu->addCheckItem( CMD_CHILD_SHOWD9LAGNA, _( "Sphuta Navamsa Lagna"), view->getObjectStyle() & OBJECTS_INCLUDE_D9_LAGNA );
-		
+
 		size_t size = ( view->isVedic() ? config->vArabicParts.Count() : config->wArabicParts.Count() );
 		if ( size > 0 )
 		{
@@ -499,38 +531,40 @@ wxMenu *ContextMenuProvider::getObjectSubMenu()
 ***************************************************************/
 wxMenu *ContextMenuProvider::getGraphicStyleMenu()
 {
-  wxMenu *menu = new MyMenu( _( "Graphic Options" ), view );
+	wxMenu *menu = new MyMenu( _( "Graphic Options" ), view );
 	if ( view->isVedic())
 	{
 		menu->Append( CMD_CHILD_VSHOWSOUTHINDIAN, _("South Indian Style"), wxT( "" ), true );
-		menu->Check( CMD_CHILD_VSHOWSOUTHINDIAN, view->getVedicGraphicStyle() & VGRAPHIC_SOUTH_INDIAN );
+		menu->Check( CMD_CHILD_VSHOWSOUTHINDIAN, view->getVedicGraphicStyle().indianChartType == VGRAPHIC_SOUTH_INDIAN );
 		menu->Append( CMD_CHILD_VSHOWNORTHINDIAN, _("North Indian Style"), wxT( "" ), true );
-		menu->Check( CMD_CHILD_VSHOWNORTHINDIAN, view->getVedicGraphicStyle() & VGRAPHIC_NORTH_INDIAN );
+		menu->Check( CMD_CHILD_VSHOWNORTHINDIAN, view->getVedicGraphicStyle().indianChartType == VGRAPHIC_NORTH_INDIAN );
 		menu->Append( CMD_CHILD_VSHOWEASTINDIAN, _("East Indian Style"), wxT( "" ), true );
-		menu->Check( CMD_CHILD_VSHOWEASTINDIAN, view->getVedicGraphicStyle() & VGRAPHIC_EAST_INDIAN );
+		menu->Check( CMD_CHILD_VSHOWEASTINDIAN, view->getVedicGraphicStyle().indianChartType == VGRAPHIC_EAST_INDIAN );
 
-		if ( view->getVedicGraphicStyle() & VGRAPHIC_NORTH_INDIAN )
+		if ( view->getVedicGraphicStyle().indianChartType == VGRAPHIC_NORTH_INDIAN )
 		{
 			wxMenu *northmenu = new wxMenu();
 			northmenu->Append( CMD_CHILD_VSHOWNORTHASC, _("Ascendant Only"), wxT( "" ), true );
-			northmenu->Check( CMD_CHILD_VSHOWNORTHASC, view->getVedicGraphicStyle() & VGRAPHIC_NORTH_INDIAN_ASC );
+			northmenu->Check( CMD_CHILD_VSHOWNORTHASC, view->getVedicGraphicStyle().northIndianSignDisplayType == VGRAPHIC_NORTH_INDIAN_ASC );
 			northmenu->Append( CMD_CHILD_VSHOWNORTHNUMBER, _("Number"), wxT( "" ), true );
-			northmenu->Check( CMD_CHILD_VSHOWNORTHNUMBER, view->getVedicGraphicStyle() & VGRAPHIC_NORTH_INDIAN_NUMBER );
+			northmenu->Check( CMD_CHILD_VSHOWNORTHNUMBER, view->getVedicGraphicStyle().northIndianSignDisplayType == VGRAPHIC_NORTH_INDIAN_NUMBER );
 			northmenu->Append( CMD_CHILD_VSHOWNORTHSHORT, _("Short String"), wxT( "" ), true );
-			northmenu->Check( CMD_CHILD_VSHOWNORTHSHORT, view->getVedicGraphicStyle() & VGRAPHIC_NORTH_INDIAN_SHORT );
+			northmenu->Check( CMD_CHILD_VSHOWNORTHSHORT, view->getVedicGraphicStyle().northIndianSignDisplayType == VGRAPHIC_NORTH_INDIAN_SHORT );
 			northmenu->Append( CMD_CHILD_VSHOWNORTHSYMBOL, _("Symbol"), wxT( "" ), true );
-			northmenu->Check( CMD_CHILD_VSHOWNORTHSYMBOL, view->getVedicGraphicStyle() & VGRAPHIC_NORTH_INDIAN_SYMBOL );
+			northmenu->Check( CMD_CHILD_VSHOWNORTHSYMBOL, view->getVedicGraphicStyle().northIndianSignDisplayType == VGRAPHIC_NORTH_INDIAN_SYMBOL );
 			menu->Append( -1, _( "Display Signs" ), northmenu );
 		}
 		if ( view->supportsFullGraphicStyleToggle())
 		{
 			MyMenu *centermenu = new MyMenu( _( "Chart Center" ), view );
 			centermenu->Append( CMD_CHILD_VSHOWCENTERNOTHING, _("Nothing"), wxT( "" ), true );
-			centermenu->Check( CMD_CHILD_VSHOWCENTERNOTHING, view->getVedicGraphicStyle() & VGRAPHIC_CHART_CENTER_NOTHING );
+			centermenu->Check( CMD_CHILD_VSHOWCENTERNOTHING, view->getVedicGraphicStyle().centerInfoType == VGRAPHIC_CHART_CENTER_NOTHING );
 			centermenu->Append( CMD_CHILD_VSHOWCENTERVARGA, _("Varga"), wxT( "" ), true );
-			centermenu->Check( CMD_CHILD_VSHOWCENTERVARGA, view->getVedicGraphicStyle() & VGRAPHIC_CHART_CENTER_VARGA );
+			centermenu->Check( CMD_CHILD_VSHOWCENTERVARGA, view->getVedicGraphicStyle().centerInfoType == VGRAPHIC_CHART_CENTER_VARGA );
 			centermenu->Append( CMD_CHILD_VSHOWCENTERDIVISION, _("Division"), wxT( "" ), true );
-			centermenu->Check( CMD_CHILD_VSHOWCENTERDIVISION, view->getVedicGraphicStyle() & VGRAPHIC_CHART_CENTER_DIVISION );
+			centermenu->Check( CMD_CHILD_VSHOWCENTERDIVISION, view->getVedicGraphicStyle().centerInfoType == VGRAPHIC_CHART_CENTER_DIVISION );
+			centermenu->Append( CMD_CHILD_VSHOWCENTERBOTH, _("Both"), wxT( "" ), true );
+			centermenu->Check( CMD_CHILD_VSHOWCENTERBOTH, view->getVedicGraphicStyle().centerInfoType == VGRAPHIC_CHART_CENTER_BOTH );
 			menu->Append( -1, _( "Chart Center" ), centermenu );
 		}
 		menu->AppendSeparator();
@@ -538,46 +572,30 @@ wxMenu *ContextMenuProvider::getGraphicStyleMenu()
 		if ( view->supportsFullGraphicStyleToggle())
 		{
 			menu->Append( CMD_CHILD_VSHOWRETRO, _( "Retrograde Planets"), wxT( "" ), true );
-			menu->Check( CMD_CHILD_VSHOWRETRO, view->getVedicGraphicStyle() & VGRAPHIC_SHOW_RETRO );
+			menu->Check( CMD_CHILD_VSHOWRETRO, view->getVedicGraphicStyle().showRetro );
 			menu->Append( CMD_CHILD_VSHOWARUDHA, _( "Arudhas"), wxT( "" ), true );
-			menu->Check( CMD_CHILD_VSHOWARUDHA, view->getVedicGraphicStyle() & VGRAPHIC_SHOW_ARUDHAS );
+			menu->Check( CMD_CHILD_VSHOWARUDHA, view->getVedicGraphicStyle().showArudhas );
 		}
 		menu->Append( CMD_CHILD_VSHOWPLANETCOLORS, _( "Colors for Planet Symbols"), wxT( "" ), true );
-		menu->Check( CMD_CHILD_VSHOWPLANETCOLORS, view->getVedicGraphicStyle() & VGRAPHIC_SHOW_PLANET_COLORS );
+		menu->Check( CMD_CHILD_VSHOWPLANETCOLORS, view->getVedicGraphicStyle().showPlanetColors );
 	}
 	else
 	{
 		menu->Append( CMD_CHILD_WSHOWRETRO, _( "Show Retrograde Planets"), wxT( "" ), true );
-		menu->Check( CMD_CHILD_WSHOWRETRO, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_RETRO );
+		menu->Check( CMD_CHILD_WSHOWRETRO, view->getWesternGraphicStyle().showRetro );
 		menu->Append( CMD_CHILD_WSHOWASPECTS, _( "Show Aspects"), wxT( "" ), true );
-		menu->Check( CMD_CHILD_WSHOWASPECTS, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_ASPECTS );
+		menu->Check( CMD_CHILD_WSHOWASPECTS, view->getWesternGraphicStyle().showAspects );
 		menu->Append( CMD_CHILD_WSHOWASPECTSYMBOLS, _( "Show Aspect Symbols"), wxT( "" ), true );
-		menu->Check( CMD_CHILD_WSHOWASPECTSYMBOLS, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_ASPECT_SYMBOLS );
+		menu->Check( CMD_CHILD_WSHOWASPECTSYMBOLS, view->getWesternGraphicStyle().showAspectSymbols );
 		menu->Append( CMD_CHILD_WSHOWHOUSES, _( "Show Houses"), wxT( "" ), true );
-		menu->Check( CMD_CHILD_WSHOWHOUSES, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_HOUSES );
+		menu->Check( CMD_CHILD_WSHOWHOUSES, view->getWesternGraphicStyle().showHouses );
 
-		MyMenu *leftsidemenu = new MyMenu( _( "Left Side" ), view );
-		leftsidemenu->Append( CMD_CHILD_WSHOWLEFTASC, _( "Ascendant"), wxT( "" ), true );
-		leftsidemenu->Check( CMD_CHILD_WSHOWLEFTASC, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_LEFT_ASC );
-		leftsidemenu->Append( CMD_CHILD_WSHOWLEFTARIES, _( "Aries"), wxT( "" ), true );
-		leftsidemenu->Check( CMD_CHILD_WSHOWLEFTARIES, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_LEFT_ARIES );
-		leftsidemenu->Append( CMD_CHILD_WSHOWLEFTCANCER, _( "Cancer"), wxT( "" ), true );
-		leftsidemenu->Check( CMD_CHILD_WSHOWLEFTCANCER, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_LEFT_CANCER );
-		leftsidemenu->Append( CMD_CHILD_WSHOWLEFTLIBRA, _( "Libra"), wxT( "" ), true );
-		leftsidemenu->Check( CMD_CHILD_WSHOWLEFTLIBRA, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_LEFT_LIBRA );
-		leftsidemenu->Append( CMD_CHILD_WSHOWLEFTCAPRICORN, _( "Capricorn"), wxT( "" ), true );
-		leftsidemenu->Check( CMD_CHILD_WSHOWLEFTCAPRICORN, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_LEFT_CAPRICORN );
-		menu->Append( -1, _( "Left Side" ), leftsidemenu  );
+		if ( view->getViewId() == VIEW_TRANSIT )
+		{
+			menu->Append( CMD_CHILD_WTRANSIT_STYLE, _( "Transits Inside"), wxT( "" ), true );
+			menu->Check( CMD_CHILD_WTRANSIT_STYLE, view->getWesternGraphicStyle().transitStyle );
+		}
 
-		MyMenu *colormenu = new MyMenu( wxEmptyString, view );
-		colormenu->Append( CMD_CHILD_WSHOWPLANETCOLORS, _( "Planets"), wxT( "" ), true );
-		colormenu->Check( CMD_CHILD_WSHOWPLANETCOLORS, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_PLANET_COLORS );
-		colormenu->Append( CMD_CHILD_WSHOWSIGNCOLORS, _( "Signs"), wxT( "" ), true );
-		colormenu->Check( CMD_CHILD_WSHOWSIGNCOLORS, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_SIGN_COLORS );
-		colormenu->Append( CMD_CHILD_WSHOWHOUSECOLORS, _( "Houses"), wxT( "" ), true );
-		colormenu->Check( CMD_CHILD_WSHOWHOUSECOLORS, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_HOUSE_COLORS );
-		colormenu->Append( CMD_CHILD_WSHOWASPECTCOLORS, _( "Aspects"), wxT( "" ), true );
-		colormenu->Check( CMD_CHILD_WSHOWASPECTCOLORS, view->getWesternGraphicStyle() & WGRAPHIC_SHOW_ASPECT_COLORS );
 	}
 	return menu;
 }
@@ -589,15 +607,17 @@ wxMenu *ContextMenuProvider::getGraphicStyleMenu()
 ***************************************************************/
 wxMenu *ContextMenuProvider::getSarvatobhadraMenu()
 {
-  MyMenu *menu = new MyMenu( _( "Sarvatobhadra" ), view );
+	MyMenu *menu = new MyMenu( _( "Sarvatobhadra" ), view );
 	menu->Append( CMD_CHILD_VSHOWRETRO, _( "Retrograde Planets"), wxT( "" ), true );
-	menu->Check( CMD_CHILD_VSHOWRETRO, view->getVedicGraphicStyle() & VGRAPHIC_SHOW_RETRO );
+	menu->Check( CMD_CHILD_VSHOWRETRO, view->getVedicGraphicStyle().showRetro );
 	menu->Append( CMD_CHILD_VSHOWPLANETCOLORS, _( "Colors for Planet Symbols"), wxT( "" ), true );
-	menu->Check( CMD_CHILD_VSHOWPLANETCOLORS, view->getVedicGraphicStyle() & VGRAPHIC_SHOW_PLANET_COLORS );
+	menu->Check( CMD_CHILD_VSHOWPLANETCOLORS, view->getVedicGraphicStyle().showPlanetColors );
 	menu->Append( CMD_CHILD_VSHOWSANSKRITSYMBOLS, _( "Sanskrit Symbols"), wxT( "" ), true );
-	menu->Check( CMD_CHILD_VSHOWSANSKRITSYMBOLS, view->getVedicGraphicStyle() & VGRAPHIC_SHOW_SANSKRIT_SYMBOLS );
+	menu->Check( CMD_CHILD_VSHOWSANSKRITSYMBOLS, view->getVedicGraphicStyle().showSbcSanskritSymbols );
 	menu->Append( CMD_CHILD_VSHOWAFFLICTIONS, _( "Afflictions"), wxT( "" ), true );
-	menu->Check( CMD_CHILD_VSHOWAFFLICTIONS, view->getVedicGraphicStyle() & VGRAPHIC_SHOW_AFFLICTIONS );
+	menu->Check( CMD_CHILD_VSHOWAFFLICTIONS, view->getVedicGraphicStyle().showSbcAfflictions );
+	menu->Append( CMD_CHILD_VSHOW_SBC_NAKSHATRA_QUALITY, _( "Benefic/Malefic Nakshatras"), wxT( "" ), true );
+	menu->Check( CMD_CHILD_VSHOW_SBC_NAKSHATRA_QUALITY, view->getVedicGraphicStyle().showSbcNakshatraQuality );
 	return menu;
 }
 
@@ -608,15 +628,15 @@ wxMenu *ContextMenuProvider::getSarvatobhadraMenu()
 ***************************************************************/
 wxMenu *ContextMenuProvider::getAppWindowContextMenu()
 {
-	const int appmenuitems[] = { APP_FILENEW, APP_FILEOPEN, 0, APP_SHOWEPHEM, APP_SHOWECLIPSE,
-		APP_SHOWHORA, APP_NEW_PARTNER, 0, -1 };
+	const int appmenuitems[] = { APP_FILENEW, APP_FILEOPEN, 0, APP_SHOWEPHEM,
+		APP_SHOWECLIPSE, APP_SHOWHORA, APP_NEW_PARTNER, 0, -1 };
 
-  MyMenu *menu = new MyMenu( _( "Main Window Menu" ), view );
-  menu->addArray( appmenuitems );
-  menu->addWindowListMenu();
+	MyMenu *menu = new MyMenu( _( "Main Window Menu" ), view );
+	menu->addArray( appmenuitems );
+	menu->addWindowListMenu();
 	menu->addItem( APP_CONFIGURE );
 	menu->addItem( APP_LOCATION );
-  menu->AppendSeparator();
+	menu->AppendSeparator();
 	menu->addItem( APP_FILEQUIT );
 	return menu;
 }
@@ -631,7 +651,7 @@ wxMenu *ContextMenuProvider::getChartMenu()
 	MyMenu *menu = new MyMenu( _( "File" ), view );
 	menu->addItem( CMD_CHILD_CLOSE );
 	if ( view->getDoc() )
-  {
+	{
 		menu->addItem( CMD_CHILD_CLOSE_DOC );
 		menu->addItem( CMD_CHILD_SAVE );
 		menu->addItem( CMD_CHILD_SAVEAS );
@@ -648,8 +668,8 @@ wxMenu *ContextMenuProvider::getChartMenu()
 ***************************************************************/
 wxMenu *ContextMenuProvider::getNewViewMenu()
 {
-	const int vedicitems[] = { 0, CMD_CHILD_NEW_DASA, CMD_CHILD_NEW_GRAPHICALDASA, CMD_CHILD_NEW_TRANSIT,
-		0, CMD_CHILD_NEW_YOGA, CMD_CHILD_NEW_ASHTAKAVARGA,
+	const int vedicitems[] = { 0, CMD_CHILD_NEW_DASA, CMD_CHILD_NEW_GRAPHICALDASA,
+		CMD_CHILD_NEW_TRANSIT, 0, CMD_CHILD_NEW_YOGA, CMD_CHILD_NEW_ASHTAKAVARGA,
 		CMD_CHILD_NEW_SOLAR, -1 };
 	const int westernitems[5] = { 0, CMD_CHILD_NEW_URANIAN, CMD_CHILD_NEW_TRANSIT, CMD_CHILD_NEW_SOLAR, -1 };
 	const int appitems[5] = { APP_SHOWEPHEM, APP_SHOWECLIPSE, APP_SHOWHORA, APP_NEW_PARTNER, -1 };
@@ -657,21 +677,21 @@ wxMenu *ContextMenuProvider::getNewViewMenu()
 	MyMenu *menu = new MyMenu( _( "New View" ), view );
 	bool vedicmode = view->isVedic();
 	if ( wxGetKeyState( WXK_SHIFT )) vedicmode = ! vedicmode;
-	
-	if ( view->getDoc())
-  {
-    // Varga submenu or western chart
-		if ( vedicmode ) menu->addVargaMenu();
-    else menu->addItem( CMD_CHILD_NEW_WCHART );
 
-    if ( vedicmode )
+	if ( view->getDoc())
+	{
+		// Varga submenu or western chart
+		if ( vedicmode ) menu->addVargaMenu();
+		else menu->addItem( CMD_CHILD_NEW_WCHART );
+
+		if ( vedicmode )
 		{
 			menu->addItem( CMD_CHILD_NEW_VARGA );
 			menu->addItem( CMD_CHILD_NEW_SBC );
 		}
 		menu->addItem( CMD_CHILD_NEW_TEXT );
 		menu->addArray( vedicmode ? vedicitems : westernitems );
-  }
+	}
 	else
 	{
 		menu->addArray( appitems );
