@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/gui/SarvatobhadraChart.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "SarvatobhadraChart.h"
+#pragma implementation "SarvatobhadraChart.h"
 #endif
 
 #include "SarvatobhadraChart.h"
@@ -68,7 +68,7 @@ const wxChar * sanskrit_li[2] = { wxT( "li" ), wxT( "\u090C" ) };
 
 // line 4
 const wxChar * sanskrit_j[2] = { wxT( "j" ), wxT( "\u091C" ) };
-const wxChar * sanskrit_ah[2] = { wxT( "ah" ), wxT( "\u0905\u0903" ) }; 
+const wxChar * sanskrit_ah[2] = { wxT( "ah" ), wxT( "\u0905\u0903" ) };
 const wxChar * sanskrit_o[2] = { wxT( "o" ), wxT( "\u0913" ) };
 const wxChar * sanskrit_v[2] = { wxT( "v" ), wxT( "\u0935" ) };
 
@@ -101,11 +101,11 @@ const wxChar * sanskrit_aa[2] = { wxT( "aa" ), wxT( "\u0906" ) };
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   Constructor 
+**   SarvatobhadraChart   ---   Constructor
 **
 ******************************************************/
 SarvatobhadraChart::SarvatobhadraChart( const Horoscope* h1,  const Horoscope* h2, const ChartProperties *chartprops )
- : BasicVedicChart( chartprops, 28, 2 )
+		: BasicVedicChart( chartprops, 28, 2 )
 {
 	nh1 = new NakshatraHoroscope( h1, N28 );
 	nh2 = new NakshatraHoroscope( h2, N28 );
@@ -114,12 +114,12 @@ SarvatobhadraChart::SarvatobhadraChart( const Horoscope* h1,  const Horoscope* h
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   Constructor 
+**   SarvatobhadraChart   ---   Constructor
 **
 ******************************************************/
 SarvatobhadraChart::SarvatobhadraChart( const Horoscope* h, const ChartProperties *chartprops )
- : BasicVedicChart( chartprops, 28, 1 ),
- nh2( 0 )
+		: BasicVedicChart( chartprops, 28, 1 ),
+		nh2( 0 )
 {
 	nh1 = new NakshatraHoroscope( h, N28 );
 	init();
@@ -127,33 +127,33 @@ SarvatobhadraChart::SarvatobhadraChart( const Horoscope* h, const ChartPropertie
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   Constructor 
+**   SarvatobhadraChart   ---   Constructor
 **
 ******************************************************/
 SarvatobhadraChart::SarvatobhadraChart( const NakshatraHoroscope* h, const ChartProperties *chartprops )
- : BasicVedicChart( chartprops, 28, 1 ),
- nh1( h ),
- nh2( 0 )
+		: BasicVedicChart( chartprops, 28, 1 ),
+		nh1( h ),
+		nh2( 0 )
 {
 	init();
 }
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   Constructor 
+**   SarvatobhadraChart   ---   Constructor
 **
 ******************************************************/
 SarvatobhadraChart::SarvatobhadraChart( const NakshatraHoroscope* h1, const NakshatraHoroscope *h2, const ChartProperties *chartprops )
- : BasicVedicChart( chartprops, 28, 2 ),
- nh1( h1 ),
- nh2( h2 )
+		: BasicVedicChart( chartprops, 28, 2 ),
+		nh1( h1 ),
+		nh2( h2 )
 {
 	init();
 }
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   Destructor 
+**   SarvatobhadraChart   ---   Destructor
 **
 ******************************************************/
 SarvatobhadraChart::~SarvatobhadraChart()
@@ -170,8 +170,8 @@ SarvatobhadraChart::~SarvatobhadraChart()
 void SarvatobhadraChart::init()
 {
 	int x, y;
-	for( int i = 0; i < 9; i++ )
-		for( int j = 0; j < 9; j++ )
+	for ( int i = 0; i < 9; i++ )
+		for ( int j = 0; j < 9; j++ )
 			mark[i][j] = 0;
 
 	if ( chartprops->isBlank() ) return;
@@ -184,6 +184,9 @@ void SarvatobhadraChart::init()
 	mark[x][y] = 1;
 	getWeekdayIndex( nh1->getHoroscope()->getDataSet()->getWeekDay(), x, y );
 	mark[x][y] = 1;
+
+	NakshatraExpert expert;
+	expert.getSbcNakshatraProperties( getNakshatra( nh1->getHoroscope()->getVedicLength( OMOON ), N27 ), nakshatraProps );
 }
 
 /*****************************************************
@@ -196,81 +199,95 @@ void SarvatobhadraChart::paintChart()
 	Lang lang;
 	int i, j;
 	double w, h;
-	wxRect rect, fieldrect;
+	MyRect rect, fieldrect;
 	wxString s;
 
+
+	// SBC has smaller fonts
+	const double sbcsclaingfactor = .6;
+	textzoom *= sbcsclaingfactor;
+	symbolzoom *= sbcsclaingfactor;
+	symbol_height *= sbcsclaingfactor;
+	text_width *= sbcsclaingfactor;
+	text_height *= sbcsclaingfactor;
+
 	xstep = xr / 4.5;
 	ystep = yr / 4.5;
 
 	const int text_border = xr / 100;
 
-  //if ( ! vconf->bgImage.IsEmpty()) painter->setBackgroundImage( vconf->bgImage );
-  if ( vconf->outerLineWidth > 0 )
-  {
-    painter->setPenWidth( vconf->outerLineWidth );
-    painter->drawRectangle( (int)( xcenter - xr ), (int)( ycenter - yr ), (int)( 2 * xr ), (int)( 2 * yr ) );
-    if ( vconf->doubleOuterLine )
-    {
-      double xtol = Max( (int)( RTOL_DOUBLE_FRAME * xr + xr ) - xr, 1 );
-      double ytol = Max( (int)( RTOL_DOUBLE_FRAME * yr + yr ) - yr, 1 );
-			painter->drawRectangle( (int)( xcenter - xr - xtol ), (int)( ycenter - yr - ytol ), (int)( 2 * ( xr + xtol)), (int)( 2 * ( yr + ytol )));
-    }
-  }
-	for( i = 1; i <= 8; i++ )
+	paintOuterRectangle();
+
+	painter->setPen( defaultPen );
+	painter->setTransparentBrush();
+
+	for ( i = 1; i <= 8; i++ )
 	{
-		painter->drawLine( (int)( xcenter - xr + i * xstep ), (int)( ycenter - yr ), (int)( xcenter - xr + i * xstep), (int)( ycenter + yr ));
-		painter->drawLine( (int)( xcenter - xr ), (int)( ycenter - yr + i * ystep ), (int)( xcenter + xr ), (int)( ycenter - yr + i * ystep ));
+		painter->drawLine( xcenter - xr + i * xstep, ycenter - yr, xcenter - xr + i * xstep, ycenter + yr );
+		painter->drawLine( xcenter - xr , ycenter - yr + i * ystep , xcenter + xr, ycenter - yr + i * ystep );
 	}
-	if ( vconf->useSignColors )
+
+  GcZodiacalSigns *signs = &vconf->signs;
+	if ( signs->needsBgPaint() )
 	{
-		for( i = 0; i < 28; i++ )
+		for ( i = 0; i < 28; i++ )
 		{
-			painter->setPenColor( vconf->getSignBgColor( i ));
-			painter->setBrushColor( vconf->getSignBgColor( i ) );
+			painter->setDefaultPen();
+			painter->setBrush(  signs->getBrush( i ));
 			painter->drawRectangle( getNakshatraCoord( i ));
 		}
 		painter->setTransparentBrush();
 	}
 
-	// mark special fields
-	//painter->setPenColor( vconf->markedFieldColor );
-	painter->setBrush( wxBrush( vconf->markedFieldColor, convertInt2WxBrush( vconf->markedFieldBrush )));
-	for( i = 0; i < 9; i++ )
+	painter->setPen( defaultPen );
+	if ( vconf->markedFields.brush.IsOk() )
 	{
-		for( j = 0; j < 9; j++ ) if ( mark[i][j] ) painter->drawRectangle( getFieldCoord( i, j ));
+		painter->setBrush( vconf->markedFields.brush );
 	}
-	painter->setPenColor( vconf->fgColor );
+	for ( i = 0; i < 9; i++ )
+	{
+		for ( j = 0; j < 9; j++ ) if ( mark[i][j] ) painter->drawRectangle( getFieldCoord( i, j ));
+	}
+	painter->setPen( defaultPen );
 	painter->setTransparentBrush();
 
-	if ( chartprops->getVedicGraphicStyle() & VGRAPHIC_SHOW_AFFLICTIONS ) drawAspects();
+	if ( chartprops->getVedicGraphicStyle().showSbcAfflictions ) drawAspects();
 
 	// Draw Nakshatra Names
-	painter->setGraphicFont();
-	painter->setTextColor( vconf->fgColor );
-	for( i = 0; i < 28; i++ )
+	painter->setGraphicFont( textzoom );
+	setDefaultTextColor();
+	for ( i = 0; i < 28; i++ )
 	{
-		// use short form if string is too wide
-		s = lang.getNakshatraName( i, N28, TMEDIUM );
+		int quality = nakshatraProps[i].taraQuality;
+		wxString qq;
+
+		if ( chartprops->getVedicGraphicStyle().showSbcNakshatraQuality )
+		{
+			if ( quality == TARA_PROP_BENEFIC ) qq = wxT( " B" );
+			else if ( quality == TARA_PROP_MALEFIC ) qq = wxT( " M" );
+		}
+		s = lang.getNakshatraName( i, N28, TMEDIUM ) + qq;
 		painter->getTextExtent( s, &w, &h );
-		if ( w > .9 * xstep ) s = lang.getNakshatraName( i, N28, TSHORT );
-		drawTextFormatted( getNakshatraCoord( i ), s, Align::HCenter + Align::Bottom );
+
+		if ( w > .9 * xstep ) s = lang.getNakshatraName( i, N28, TSHORT ) + qq;
+		painter->drawTextFormatted( getNakshatraCoord( i ), s, Align::HCenter + Align::Bottom );
 	}
 
 	// Static text portions
-	const int symbolindex = chartprops->getVedicGraphicStyle() & VGRAPHIC_SHOW_SANSKRIT_SYMBOLS ? 1 : 0;
+	const int symbolindex = chartprops->getVedicGraphicStyle().showSbcSanskritSymbols ? 1 : 0;
 
 #define DRAW_STATIC_FIELD( a, b, t ) \
 	fieldrect = getFieldCoord( a, b ); \
-	if ( refreshRect && refreshRect->Intersects( fieldrect )) \
+	if ( ! refreshRect || ( refreshRect && refreshRect->Intersects( fieldrect.toWxRect() ))) \
 	{ \
-		drawTextFormatted( fieldrect, t, Align::HCenter + Align::Bottom ); \
-	} 
+		painter->drawTextFormatted( fieldrect, t, Align::HCenter + Align::Bottom ); \
+	}
 
 #define DRAW_STATIC_FIELD_SANSKRIT( a, b, t ) \
 	fieldrect = getFieldCoord( a, b ); \
-	if ( refreshRect && refreshRect->Intersects( fieldrect )) \
+	if ( ! refreshRect || ( refreshRect && refreshRect->Intersects( fieldrect.toWxRect() ))) \
 	{ \
-		drawTextFormatted( fieldrect, t[symbolindex], Align::HCenter + Align::Bottom ); \
+		painter->drawTextFormatted( fieldrect, t[symbolindex], Align::HCenter + Align::Bottom ); \
 	}
 
 	DRAW_STATIC_FIELD_SANSKRIT( 0, 0, sanskrit_ee )
@@ -284,7 +301,7 @@ void SarvatobhadraChart::paintChart()
 	DRAW_STATIC_FIELD_SANSKRIT( 5, 1, sanskrit_ch )
 	DRAW_STATIC_FIELD_SANSKRIT( 6, 1, sanskrit_l )
 	DRAW_STATIC_FIELD_SANSKRIT( 7, 1, sanskrit_u )
-	
+
 	// Line 3
 	DRAW_STATIC_FIELD_SANSKRIT( 1, 2, sanskrit_kh )
 	DRAW_STATIC_FIELD_SANSKRIT( 2, 2, sanskrit_ai )
@@ -330,16 +347,16 @@ void SarvatobhadraChart::paintChart()
 	// Line 9
 	DRAW_STATIC_FIELD_SANSKRIT( 0, 8, sanskrit_i )
 	DRAW_STATIC_FIELD_SANSKRIT( 8, 8, sanskrit_aa )
-#undef DRAW_STATIC_FIELD 
-#undef DRAW_STATIC_FIELD_SANSKRIT 
+#undef DRAW_STATIC_FIELD
+#undef DRAW_STATIC_FIELD_SANSKRIT
 
 	// Signs
 	if ( config->useSignSymbols ) painter->setSymbolFont( symbolzoom );
 #define DRAW_SIGN_SYMBOL( x, y, index ) \
-	drawTextFormatted( getFieldCoord( x, y ), \
+	painter->drawTextFormatted( getFieldCoord( x, y ), \
 	config->useSignSymbols ? (wxString)lang.getSignSymbolCode( index ) : lang.getSignName( index, TMEDIUM ), \
 		Align::HCenter + Align::Bottom );
-	
+
 	DRAW_SIGN_SYMBOL( 5, 2, ARIES )
 	DRAW_SIGN_SYMBOL( 6, 3, TAURUS )
 	DRAW_SIGN_SYMBOL( 6, 4, GEMINI )
@@ -353,31 +370,30 @@ void SarvatobhadraChart::paintChart()
 	DRAW_SIGN_SYMBOL( 3, 2, AQUARIUS )
 	DRAW_SIGN_SYMBOL( 4, 2, PISCES )
 #undef DRAW_SIGN_SYMBOL
-	painter->setGraphicFont();
+	painter->setGraphicFont( textzoom );
 
 	// Draw planets
-	painter->setTextColor( vconf->fgColor);
-	for( i = 0; i < 28; i++ )
+	setDefaultTextColor();
+
+	for ( i = 0; i < 28; i++ )
 	{
 		rect = getNakshatraCoord( i );
+
+		// radix planet in upper area
+		rect.height /= 3.0;
+		if ( ! refreshRect || (refreshRect && refreshRect->Intersects( rect.toWxRect() )))
+		{
+			//printf( "SBC drawFieldText rect.x %f, rect.y %f rect.height %f rect.width %f\n", rect.x, rect.y, rect.height, rect.width );
+			drawFieldText( rect, i, Align::Center, 0 );
+		}
 		if ( chart_count > 1 )
 		{
-			rect.SetY( rect.GetY() + rect.GetHeight() / 3 );
-			rect.SetHeight( rect.GetHeight() / 3 );
-		}
-		else
-		{
-			rect.SetHeight( 2 * rect.GetHeight() / 3 );
-		}
-		if ( refreshRect && refreshRect->Intersects( rect )) drawFieldText( rect, i, Align::Center, 0 );
-	}
-	if ( chart_count > 1 )
-	{
-		for( i = 0; i < 28; i++ )
-		{
-			rect = getNakshatraCoord( i );
-			rect.SetHeight( rect.GetHeight() / 3 );
-			if ( refreshRect && refreshRect->Intersects( rect )) drawFieldText( rect, i, Align::Center, 1, text_border );
+			// transit planet vertically centered
+			rect.y += rect.height;
+			if ( ! refreshRect || ( refreshRect && refreshRect->Intersects( rect.toWxRect() )))
+			{
+				drawFieldText(  rect, i, Align::Center, 1, text_border );
+			}
 		}
 	}
 }
@@ -439,7 +455,7 @@ bool SarvatobhadraChart::handleMouseLeftEvent( int x, int y, wxRect &rect )
 	if ( getFieldIndex( &x, &y ))
 	{
 		mark[x][y] = ! mark[x][y];
-		rect = getFieldCoord( x, y );
+		rect = getFieldCoord( x, y ).toWxRect();
 		return true;
 	}
 	return false;
@@ -447,7 +463,7 @@ bool SarvatobhadraChart::handleMouseLeftEvent( int x, int y, wxRect &rect )
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   markField 
+**   SarvatobhadraChart   ---   markField
 **
 ******************************************************/
 void SarvatobhadraChart::markField( int x, int y )
@@ -460,7 +476,7 @@ void SarvatobhadraChart::markField( int x, int y )
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   getFieldIndex 
+**   SarvatobhadraChart   ---   getFieldIndex
 **
 ******************************************************/
 bool SarvatobhadraChart::getFieldIndex( int *x, int *y )
@@ -477,40 +493,55 @@ bool SarvatobhadraChart::getFieldIndex( int *x, int *y )
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   getFieldCoord 
+**   SarvatobhadraChart   ---   getFieldCoord
 **
 ******************************************************/
-wxRect SarvatobhadraChart::getFieldCoord( const int &x, const int &y )
+MyRect SarvatobhadraChart::getFieldCoord( const int &x, const int &y )
 {
-	return wxRect( (int)(xcenter - xr + x * xstep ), (int)( ycenter - yr + y * ystep ), (int)xstep, (int)ystep );
+	return MyRect( xcenter - xr + x * xstep, ycenter - yr + y * ystep, xstep, ystep );
 }
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   getWeekdayIndex 
+**   SarvatobhadraChart   ---   getWeekdayIndex
 **
 ******************************************************/
 void SarvatobhadraChart::getWeekdayIndex( const int &w, int &x, int &y )
 {
 	assert( w >= 0 && w < 7 );
-	switch( w )
+	switch ( w )
 	{
-		case 0:
-		case 2: x = 5; y = 4; break;
-		case 1:
-		case 3: x = 4; y = 5; break;
-		case 4: x = 3; y = 4; break;
-		case 5: x = 4; y = 3; break;
-		default: x = 4; y = 4; break;
+	case 0:
+	case 2:
+		x = 5;
+		y = 4;
+		break;
+	case 1:
+	case 3:
+		x = 4;
+		y = 5;
+		break;
+	case 4:
+		x = 3;
+		y = 4;
+		break;
+	case 5:
+		x = 4;
+		y = 3;
+		break;
+	default:
+		x = 4;
+		y = 4;
+		break;
 	}
 }
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   getWeekdayCoord 
+**   SarvatobhadraChart   ---   getWeekdayCoord
 **
 ******************************************************/
-wxRect SarvatobhadraChart::getWeekdayCoord( const int &w )
+MyRect SarvatobhadraChart::getWeekdayCoord( const int &w )
 {
 	int x, y;
 	getWeekdayIndex( w, x, y );
@@ -519,36 +550,72 @@ wxRect SarvatobhadraChart::getWeekdayCoord( const int &w )
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   getRasiIndex 
+**   SarvatobhadraChart   ---   getRasiIndex
 **
 ******************************************************/
 void SarvatobhadraChart::getRasiIndex( const int &r, int &x, int &y )
 {
 	assert( r >= 0 && r < 12 );
-	switch( r )
+	switch ( r )
 	{
-		case ARIES: x = 5; y = 2; break;
-		case TAURUS: x = 6; y = 3; break;
-		case GEMINI: x = 6; y = 4; break;
-		case CANCER: x = 6; y = 5; break;
-		case LEO: x = 5; y = 6; break;
-		case VIRGO: x = 4; y = 6; break;
-		case LIBRA: x = 3; y = 6; break;
-		case SCORPIO: x = 2; y = 5; break;
-		case SAGITTARIUS: x = 2; y = 4; break;
-		case CAPRICORN: x = 2; y = 3; break;
-		case AQUARIUS: x = 3; y = 2; break;
-		case PISCES:
-		default: x = 4; y = 2; break;
+	case ARIES:
+		x = 5;
+		y = 2;
+		break;
+	case TAURUS:
+		x = 6;
+		y = 3;
+		break;
+	case GEMINI:
+		x = 6;
+		y = 4;
+		break;
+	case CANCER:
+		x = 6;
+		y = 5;
+		break;
+	case LEO:
+		x = 5;
+		y = 6;
+		break;
+	case VIRGO:
+		x = 4;
+		y = 6;
+		break;
+	case LIBRA:
+		x = 3;
+		y = 6;
+		break;
+	case SCORPIO:
+		x = 2;
+		y = 5;
+		break;
+	case SAGITTARIUS:
+		x = 2;
+		y = 4;
+		break;
+	case CAPRICORN:
+		x = 2;
+		y = 3;
+		break;
+	case AQUARIUS:
+		x = 3;
+		y = 2;
+		break;
+	case PISCES:
+	default:
+		x = 4;
+		y = 2;
+		break;
 	}
 }
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   getRasiCoord 
+**   SarvatobhadraChart   ---   getRasiCoord
 **
 ******************************************************/
-wxRect SarvatobhadraChart::getRasiCoord( const int &r )
+MyRect SarvatobhadraChart::getRasiCoord( const int &r )
 {
 	int x, y;
 	getRasiIndex( r, x, y );
@@ -557,62 +624,73 @@ wxRect SarvatobhadraChart::getRasiCoord( const int &r )
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   getNakshatraIndex 
+**   SarvatobhadraChart   ---   getNakshatraIndex
 **
 ******************************************************/
 void SarvatobhadraChart::getNakshatraIndex( const int &i, int &x, int &y )
 {
 	assert( i >= 0 && i <= 27 );
 	if ( i < 2 )
-		{ x = i + 6; y = 0; }
+	{
+		x = i + 6;
+		y = 0;
+	}
 	else if ( i >= 2 && i <= 8 )
-		{ x = 8; y = i-1; }
+	{
+		x = 8;
+		y = i-1;
+	}
 	else if ( i >= 9 && i <= 15 )
-		{ x = 16 - i; y = 8; }
+	{
+		x = 16 - i;
+		y = 8;
+	}
 	else if ( i >= 16 && i <= 22 )
-		{ x = 0; y =  8 - ( i - 15 ); }
+	{
+		x = 0;
+		y =  8 - ( i - 15 );
+	}
 	else
-		{ x = i - 22; y = 0; }
+	{
+		x = i - 22;
+		y = 0;
+	}
 }
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   getNakshatraCoord 
+**   SarvatobhadraChart   ---   getNakshatraCoord
 **
 ******************************************************/
-wxRect SarvatobhadraChart::getNakshatraCoord( const int &i )
+MyRect SarvatobhadraChart::getNakshatraCoord( const int &i )
 {
 	int x, y;
 	assert( i >= 0 && i <= 27 );
 	getNakshatraIndex( i, x, y );
-	return wxRect( (int)(xcenter - xr + ( x * xstep )), (int)( ycenter - yr + ( y * ystep )), (int)xstep, (int)ystep );
+	return MyRect( xcenter - xr + ( x * xstep ), ycenter - yr + ( y * ystep ), xstep, ystep );
 }
 
 /*****************************************************
 **
-**   SarvatobhadraChart   ---   drawAspects 
+**   SarvatobhadraChart   ---   drawAspects
 **
 ******************************************************/
 void SarvatobhadraChart::drawAspects()
 {
 	int i, j, xr1, yr1, xr2, yr2, sw, p;
 	unsigned int k;
-	wxRect r1, r2;
+	MyRect r1, r2;
 	const Horoscope *h1 = nh1->getHoroscope();
 
-	// TODO falsch
-	//const static bool isbenefic[10] = { false, true, false, true, true, true, false, false, false, true };
-
 	int nb_lines[28];
-	for( i = 0; i < 28; i++ ) nb_lines[i] = 0;
+	for ( i = 0; i < 28; i++ ) nb_lines[i] = 0;
 
-	painter->setPenWidth( 1 );
+	painter->setDefaultPen();
 
-	// TODO unterscheiden
-	vector<ChartContents> &c = getChartContents( 0 );
-	for( i = 0; i < 28; i++ )
+	vector<ChartContents> &c = getChartContents( chart_count > 1 ? 1 : 0 );
+	for ( i = 0; i < 28; i++ )
 	{
-		for( k = 0; k < c[i].planets.size(); k++ )
+		for ( k = 0; k < c[i].planets.size(); k++ )
 		{
 			r1 = getNakshatraCoord( i );
 			xr1 = r1.x + r1.width/2;
@@ -621,7 +699,7 @@ void SarvatobhadraChart::drawAspects()
 
 			painter->setPenColor( ( p < 10 && h1->isBenefic( p ) ? config->colors->sbcBeneficBgColor : config->colors->sbcMaleficBgColor ));
 
-			for( j = 1; j <= 3; j++ )
+			for ( j = 1; j <= 3; j++ )
 			{
 				r2 = getNakshatraCoord( nak_aspects[i][j] );
 				xr2 = r2.x + r2.width/2;
@@ -634,7 +712,7 @@ void SarvatobhadraChart::drawAspects()
 			}
 		}
 	}
-	painter->setPenColor( vconf->fgColor );
+	painter->setPen( defaultPen );
 }
 
 
