@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/jyotish/Ashtakavarga.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "Ashtakavarga.h"
+#pragma implementation "Ashtakavarga.h"
 #endif
 
 #include "Ashtakavarga.h"
@@ -40,18 +40,18 @@ extern Config *config;
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   update 
+**   AshtakavargaExpert   ---   update
 **
 ******************************************************/
 void AshtakavargaExpert::update()
 {
 	int i, j;
 
-	for( i = 0; i < 8; i++ )
-		for( j = ARIES; j <= PISCES; j++ )
+	for ( i = 0; i < 8; i++ )
+		for ( j = ARIES; j <= PISCES; j++ )
 			rekha[i][j] = trikona[i][j] = ekadhi[i][j] = 0;
 
-	for( i = ARIES; i <= PISCES; i++ ) planetNumber[i] = 0;
+	for ( i = ARIES; i <= PISCES; i++ ) planetNumber[i] = 0;
 
 	calcRekha();
 	calcTrikonaShodana();
@@ -62,25 +62,25 @@ void AshtakavargaExpert::update()
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   getPinda 
+**   AshtakavargaExpert   ---   getPinda
 **
 ******************************************************/
 int AshtakavargaExpert::getPinda( const int &type, const int &rasi ) const
 {
-	switch( type )
+	switch ( type )
 	{
-		case GRAHAPINDA:
-			return grahaPinda[rasi];
+	case GRAHAPINDA:
+		return grahaPinda[rasi];
 		break;
-		case RASIPINDA:
-			return rasiPinda[rasi];
+	case RASIPINDA:
+		return rasiPinda[rasi];
 		break;
-		case YOGAPINDA:
-			return yogaPinda[rasi];
+	case YOGAPINDA:
+		return yogaPinda[rasi];
 		break;
-		default:
-			wxLogError( wxT( "Error  AshtakavargaExpert::getPinda, type is %d" ), type );
-			exit(1);
+	default:
+		wxLogError( wxT( "Error  AshtakavargaExpert::getPinda, type is %d" ), type );
+		exit(1);
 		break;
 	}
 	return 0;
@@ -88,27 +88,27 @@ int AshtakavargaExpert::getPinda( const int &type, const int &rasi ) const
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   getItem 
+**   AshtakavargaExpert   ---   getItem
 **
 ******************************************************/
 int AshtakavargaExpert::getItem( const int &type, const int &p, const int &rasi )
 {
 	assert(( p >= OSUN && p <= OSATURN ) || p == OASCENDANT );
 	int planet = ( p <= OSATURN ? p : 7 );
-	switch( type )
+	switch ( type )
 	{
-		case REKHA:
-			return rekha[planet][rasi];
+	case REKHA:
+		return rekha[planet][rasi];
 		break;
-		case TRIKONA:
-			return trikona[planet][rasi];
+	case TRIKONA:
+		return trikona[planet][rasi];
 		break;
-		case EKADHI:
-			return ekadhi[planet][rasi];
+	case EKADHI:
+		return ekadhi[planet][rasi];
 		break;
-		default:
-			wxLogError( wxT( "Error  AshtakavargaExpert::getItem, type is %d" ), type );
-			exit(1);
+	default:
+		wxLogError( wxT( "Error  AshtakavargaExpert::getItem, type is %d" ), type );
+		exit(1);
 		break;
 	}
 	return 0;
@@ -116,26 +116,26 @@ int AshtakavargaExpert::getItem( const int &type, const int &p, const int &rasi
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   getSarva 
+**   AshtakavargaExpert   ---   getSarva
 **
 ******************************************************/
 int AshtakavargaExpert::getSarva( const int &type, const int &rasi ) const
 {
 	assert( rasi >= ARIES && rasi <= PISCES );
-	switch( type )
+	switch ( type )
 	{
-		case REKHA:
-			return sarvaRekha[rasi];
+	case REKHA:
+		return sarvaRekha[rasi];
 		break;
-		case TRIKONA:
-			return sarvaTrikona[rasi];
+	case TRIKONA:
+		return sarvaTrikona[rasi];
 		break;
-		case EKADHI:
-			return sarvaEkadhi[rasi];
+	case EKADHI:
+		return sarvaEkadhi[rasi];
 		break;
-		default:
-			wxLogError( wxT( "Error  AshtakavargaExpert::getSarva, type is %d" ), type );
-			exit(1);
+	default:
+		wxLogError( wxT( "Error  AshtakavargaExpert::getSarva, type is %d" ), type );
+		exit(1);
 		break;
 	}
 	return 0;
@@ -143,27 +143,27 @@ int AshtakavargaExpert::getSarva( const int &type, const int &rasi ) const
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   getPlanetSarva 
+**   AshtakavargaExpert   ---   getPlanetSarva
 **
 ******************************************************/
 int AshtakavargaExpert::getPlanetSarva( const int &type, const int &p ) const
 {
 	assert(( p >= OSUN && p <= OSATURN ) || p == OASCENDANT );
 	int planet = ( p <= OSATURN ? p : 7 );
-	switch( type )
+	switch ( type )
 	{
-		case REKHA:
-			return psarvaRekha[planet];
+	case REKHA:
+		return psarvaRekha[planet];
 		break;
-		case TRIKONA:
-			return psarvaTrikona[planet];
+	case TRIKONA:
+		return psarvaTrikona[planet];
 		break;
-		case EKADHI:
-			return psarvaEkadhi[planet];
+	case EKADHI:
+		return psarvaEkadhi[planet];
 		break;
-		default:
-			wxLogError( wxT( "Error  AshtakavargaExpert::getPlanetSarva, type is %d" ), type );
-			exit(1);
+	default:
+		wxLogError( wxT( "Error  AshtakavargaExpert::getPlanetSarva, type is %d" ), type );
+		exit(1);
 		break;
 	}
 	return 0;
@@ -179,36 +179,35 @@ void AshtakavargaExpert::write( Writer *writer, const bool show_header )
 	VargaExpert expert;
 	wxString s;
 	int i;
-	int d = chart->getDivision();
 
 	if ( show_header )
 	{
 		writer->writeHeader1( _( "Ashtakavarga" ));
-		s << _T( "Varga" ) << wxT( ": " ) << expert.getVargaName(d) << wxT( " D" ) << expert.getVargaDivision(d);
-		writer->writeLine( s );
+		//s << _T( "Varga" ) << wxT( ": " ) << expert.getVargaName(d) << wxT( " D" ) << expert.getVargaDivision(d);
+		//writer->writeLine( s );
 	}
 
-  writer->writeHeader2( _( "Rekha" ));
-  writeSingleAv( writer, REKHA );
-  writer->writeHeader2( _( "Trikona Shodana" ));
-  writeSingleAv( writer, TRIKONA );
-  writer->writeHeader2( _( "Ekadhipatya Shodana" ));
-  writeSingleAv( writer, EKADHI );
+	writer->writeHeader2( _( "Rekha" ));
+	writeSingleAv( writer, REKHA );
+	writer->writeHeader2( _( "Trikona Shodana" ));
+	writeSingleAv( writer, TRIKONA );
+	writer->writeHeader2( _( "Ekadhipatya Shodana" ));
+	writeSingleAv( writer, EKADHI );
 
 	Table table( 4, 8 );
-  writer->writeHeader2( _( "Pinda" ));
-  table.setHeader( 0,  _( "Planet" ));
-  table.setHeader( 1,  _( "Rasi" ));
-  table.setHeader( 2,  _( "Graha" ));
-  table.setHeader( 3,  _( "Sodhya" ));
+	writer->writeHeader2( _( "Pinda" ));
+	table.setHeader( 0,  _( "Planet" ));
+	table.setHeader( 1,  _( "Rasi" ));
+	table.setHeader( 2,  _( "Graha" ));
+	table.setHeader( 3,  _( "Sodhya" ));
 
 	int line = 1;
 	vector<int> obs = PlanetList().getDefaultVedicObjectList();
-  for( unsigned int p = 0; p < obs.size(); p++ )
-  {
+	for ( unsigned int p = 0; p < obs.size(); p++ )
+	{
 		i = obs[p];
-    if ( i > OSATURN && i != OASCENDANT ) continue;
-    table.setEntry( 0, line, writer->getObjectName( i, TLARGE ), config->usePlanetSymbols );
+		if ( i > OSATURN && i != OASCENDANT ) continue;
+		table.setEntry( 0, line, writer->getObjectName( i, TLARGE ), config->usePlanetSymbols );
 		s.Printf( wxT( "%3d" ), getPinda( RASIPINDA, i ));
 		table.setEntry( 1, line, s );
 		s.Printf( wxT( "%3d" ), getPinda( GRAHAPINDA, i ));
@@ -216,8 +215,8 @@ void AshtakavargaExpert::write( Writer *writer, const bool show_header )
 		s.Printf( wxT( "%3d" ), getPinda( YOGAPINDA, i ));
 		table.setEntry( 3, line, s );
 		line++;
-  }
-	writer->writeTable( table ); 
+	}
+	writer->writeTable( table );
 }
 
 /*****************************************************
@@ -231,35 +230,35 @@ void AshtakavargaExpert::writeSingleAv( Writer *writer, const int&w )
 	wxString s1;
 	Table table( 13, 10 );
 	table.setHeader( 0, wxEmptyString);
-	for( i = ARIES; i <= PISCES; i++ )
+	for ( i = ARIES; i <= PISCES; i++ )
 	{
 		table.setHeader( i+1, writer->getSignName( i, TMEDIUM), config->usePlanetSymbols );
 	}
 
 	vector<int> obs = PlanetList().getDefaultVedicObjectList();
 	obs.push_back( OASCENDANT );
-  for( unsigned int p = 0; p < obs.size(); p++ )
-  {
+	for ( unsigned int p = 0; p < obs.size(); p++ )
+	{
 		i = obs[p];
-    if ( i > OSATURN && i != OASCENDANT ) continue;
-    table.setEntry( 0, line, writer->getObjectName( i, TLARGE ), config->usePlanetSymbols );
-    for( j = ARIES; j <= PISCES; j++ )
-    {
-      //s1.Printf( wxT( "%02d "), getItem( w, i, j ));
+		if ( i > OSATURN && i != OASCENDANT ) continue;
+		table.setEntry( 0, line, writer->getObjectName( i, TLARGE ), config->usePlanetSymbols );
+		for ( j = ARIES; j <= PISCES; j++ )
+		{
+			//s1.Printf( wxT( "%02d "), getItem( w, i, j ));
 			s1.Printf( wxT( "%2d "), getItem( w, i, j ));
 			table.setEntry( j+1, line, s1 );
-    }
+		}
 		line++;
-  } 
-  table.setEntry( 0, 9, _( "Sarva" ));
-  for( j = ARIES; j <= PISCES; j++ )
-  { 
-    //s1.Printf( wxT( "%02d " ), getSarva( w, j ));
+	}
+	table.setEntry( 0, 9, _( "Sarva" ));
+	for ( j = ARIES; j <= PISCES; j++ )
+	{
+		//s1.Printf( wxT( "%02d " ), getSarva( w, j ));
 		s1.Printf( wxT( "%2d " ), getSarva( w, j ));
 		table.setEntry( j+1, 9, s1 );
-  } 
+	}
 
-	writer->writeTable( table ); 
+	writer->writeTable( table );
 }
 
 /*****************************************************
@@ -377,7 +376,7 @@ int AshtakavargaExpert::getSingleRekha( const int &i, const int &j, const int &k
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   calcRekha 
+**   AshtakavargaExpert   ---   calcRekha
 **
 ******************************************************/
 void AshtakavargaExpert::calcRekha()
@@ -401,19 +400,19 @@ void AshtakavargaExpert::calcRekha()
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   calcSarva 
+**   AshtakavargaExpert   ---   calcSarva
 **
 ******************************************************/
 void AshtakavargaExpert::calcSarva()
 {
 	int i, j, jmax;
-	for( i = 0; i < 12; i++ ) sarvaRekha[i] = sarvaTrikona[i] = sarvaEkadhi[i] = 0;
-	for( i = 0; i < 8; i++ ) psarvaRekha[i] = psarvaTrikona[i] = psarvaEkadhi[i] = 0;
+	for ( i = 0; i < 12; i++ ) sarvaRekha[i] = sarvaTrikona[i] = sarvaEkadhi[i] = 0;
+	for ( i = 0; i < 8; i++ ) psarvaRekha[i] = psarvaTrikona[i] = psarvaEkadhi[i] = 0;
 
 	jmax = config->vAshtakavargaSarva ? 8 : 7;
-	for( i = 0; i < 12; i++ )
+	for ( i = 0; i < 12; i++ )
 	{
-		for( j = 0; j < jmax; j++ )
+		for ( j = 0; j < jmax; j++ )
 		{
 			sarvaRekha[i] += rekha[j][i];
 			sarvaTrikona[i] += trikona[j][i];
@@ -428,7 +427,7 @@ void AshtakavargaExpert::calcSarva()
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   calcPinda 
+**   AshtakavargaExpert   ---   calcPinda
 **
 ******************************************************/
 void AshtakavargaExpert::calcPinda()
@@ -438,26 +437,26 @@ void AshtakavargaExpert::calcPinda()
 	// Numbers for virgo and Capricorn are not clear: some say 5,  others 6, others vi 6 and cp 5
 	const int k_rasimana[12] = { 7, 10, 8, 4, 10, 6, 7, 8, 9, 5, 11, 12 };
 	//const int k_rasimana[12] = { 7, 10, 8, 4, 10, 5, 7, 8, 9, 5, 11, 12 };
-	
+
 	// Grahamana
 	// so=5 mo=5 ma=8 me=5 ju=10 ve=7 sa=5
 	const int k_grahamana[7] = { 5, 5, 5, 7, 8, 10, 5 };
-	
-	for( int i = ARIES; i <= PISCES; i++ )
+
+	for ( int i = ARIES; i <= PISCES; i++ )
 		rasiPinda[i] = grahaPinda[i] = yogaPinda[i] = 0;
 
-	for( planet = 0; planet <= 8; planet++ )
+	for ( planet = 0; planet <= 8; planet++ )
 	{
-		for( rasi = ARIES; rasi <= PISCES; rasi++ )
+		for ( rasi = ARIES; rasi <= PISCES; rasi++ )
 		{
 			rasiPinda[planet] += ekadhi[planet][rasi] * k_rasimana[rasi];
 
 			// Sodhya Pinda: Mantreswar
-			if ( config->vAshtakavargaMantreswarMode ) yogaPinda[planet] += ekadhi[planet][rasi]; 
+			if ( config->vAshtakavargaMantreswarMode ) yogaPinda[planet] += ekadhi[planet][rasi];
 		}
-		for( p = 0; p < 7; p++ )
-		{ 
-			grahaPinda[planet] += ekadhi[planet][chart->getRasi(p)] * k_grahamana[p]; 
+		for ( p = 0; p < 7; p++ )
+		{
+			grahaPinda[planet] += ekadhi[planet][chart->getRasi(p)] * k_grahamana[p];
 		}
 
 		// Sodhya Pinda: Parasara
@@ -468,34 +467,34 @@ void AshtakavargaExpert::calcPinda()
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   calcTrikonaShodana 
+**   AshtakavargaExpert   ---   calcTrikonaShodana
 **
 ******************************************************/
 void AshtakavargaExpert::calcTrikonaShodana()
 {
 	wxChar minrekha;
-	for( int i = 0; i < 8; i++ )
+	for ( int i = 0; i < 8; i++ )
 	{
-		for( int j = 0; j < 4; j++ )
+		for ( int j = 0; j < 4; j++ )
 		{
 			minrekha = Min( rekha[i][j], rekha[i][j+4] );
 			minrekha = Min( rekha[i][j+8], minrekha );
 
-			for( int k = 0; k < 3; k++ ) trikona[i][j+4*k] = rekha[i][j+4*k] - minrekha;
+			for ( int k = 0; k < 3; k++ ) trikona[i][j+4*k] = rekha[i][j+4*k] - minrekha;
 		}
 	}
 }
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   calcEkadhipatyaPair 
+**   AshtakavargaExpert   ---   calcEkadhipatyaPair
 **
 ******************************************************/
 void AshtakavargaExpert::calcEkadhipatyaPair( const int &rasi1, const int &rasi2 )
 {
 	int p;
 
-	for( p = 0; p < 12; p++ )
+	for ( p = 0; p < 12; p++ )
 	{
 		/*********************************************************
 		*** (0)
@@ -516,7 +515,7 @@ void AshtakavargaExpert::calcEkadhipatyaPair( const int &rasi1, const int &rasi2
 		***  BUG FIX 5.0:
 		***  -> No test for Trikona different, so condition (5) was never reached
 		*********************************************************/
-		else if(( ! planetNumber[rasi1] ) && ( ! planetNumber[rasi2] ) && ( trikona[p][rasi1] != trikona[p][rasi2]) )
+		else if (( ! planetNumber[rasi1] ) && ( ! planetNumber[rasi2] ) && ( trikona[p][rasi1] != trikona[p][rasi2]) )
 		{
 			ekadhi[p][rasi1] = ekadhi[p][rasi2] = Min( trikona[p][rasi1], trikona[p][rasi2] );
 		}
@@ -526,7 +525,7 @@ void AshtakavargaExpert::calcEkadhipatyaPair( const int &rasi1, const int &rasi2
 		** both rasis with planets -> no shodana
 		***
 		*********************************************************/
-		else if(( planetNumber[rasi1] ) && ( planetNumber[rasi2] ))
+		else if (( planetNumber[rasi1] ) && ( planetNumber[rasi2] ))
 		{
 		}
 
@@ -538,14 +537,14 @@ void AshtakavargaExpert::calcEkadhipatyaPair( const int &rasi1, const int &rasi2
 		**  and the number of the rasi with planet should be kept unchanged
 		***
 		*********************************************************/
-		else if(( planetNumber[rasi1] ) && ( ! planetNumber[rasi2] )
-			&& ( trikona[p][rasi1] < trikona[p][rasi2]))
+		else if (( planetNumber[rasi1] ) && ( ! planetNumber[rasi2] )
+		         && ( trikona[p][rasi1] < trikona[p][rasi2]))
 		{
 			ekadhi[p][rasi2] -= trikona[p][rasi1];
 		}
 
-		else if(( planetNumber[rasi2] ) && ( ! planetNumber[rasi1] )
-			&& ( trikona[p][rasi2] < trikona[p][rasi1]))
+		else if (( planetNumber[rasi2] ) && ( ! planetNumber[rasi1] )
+		         && ( trikona[p][rasi2] < trikona[p][rasi1]))
 		{
 			ekadhi[p][rasi1] -= trikona[p][rasi2];
 		}
@@ -558,13 +557,13 @@ void AshtakavargaExpert::calcEkadhipatyaPair( const int &rasi1, const int &rasi2
 		** and the number of the rasi with planet schould be kept unchaged
 		***
 		*********************************************************/
-		else if(( planetNumber[rasi1] ) && ( ! planetNumber[rasi2] )
-			&& ( trikona[p][rasi1] > trikona[p][rasi2]))
+		else if (( planetNumber[rasi1] ) && ( ! planetNumber[rasi2] )
+		         && ( trikona[p][rasi1] > trikona[p][rasi2]))
 		{
 			ekadhi[p][rasi2] = 0;
 		}
-		else if(( planetNumber[rasi2] ) && ( ! planetNumber[rasi1] )
-			&& ( trikona[p][rasi2] > trikona[p][rasi1]))
+		else if (( planetNumber[rasi2] ) && ( ! planetNumber[rasi1] )
+		         && ( trikona[p][rasi2] > trikona[p][rasi1]))
 		{
 			ekadhi[p][rasi1] = 0;
 		}
@@ -576,8 +575,8 @@ void AshtakavargaExpert::calcEkadhipatyaPair( const int &rasi1, const int &rasi2
 		**  -> shodana with both the numbers
 		***
 		*********************************************************/
-		else if(( ! planetNumber[rasi1] ) && ( ! planetNumber[rasi2] )
-			&& ( trikona[p][rasi1] == trikona[p][rasi2]))
+		else if (( ! planetNumber[rasi1] ) && ( ! planetNumber[rasi2] )
+		         && ( trikona[p][rasi1] == trikona[p][rasi2]))
 		{
 			ekadhi[p][rasi1] = ekadhi[p][rasi2] = 0;
 		}
@@ -589,11 +588,11 @@ void AshtakavargaExpert::calcEkadhipatyaPair( const int &rasi1, const int &rasi2
 		** -> the number of the latter should be reduced to zero
 		***
 		*********************************************************/
-		else if(( planetNumber[rasi1] ) && ( ! planetNumber[rasi2] ))
+		else if (( planetNumber[rasi1] ) && ( ! planetNumber[rasi2] ))
 		{
 			ekadhi[p][rasi2] = 0;
 		}
-		else if(( planetNumber[rasi2] ) && ( ! planetNumber[rasi1] ))
+		else if (( planetNumber[rasi2] ) && ( ! planetNumber[rasi1] ))
 		{
 			ekadhi[p][rasi1] = 0;
 		}
@@ -602,18 +601,18 @@ void AshtakavargaExpert::calcEkadhipatyaPair( const int &rasi1, const int &rasi2
 
 /*****************************************************
 **
-**   AshtakavargaExpert   ---   calcEkadhipatyaShodana 
+**   AshtakavargaExpert   ---   calcEkadhipatyaShodana
 **
 ******************************************************/
 void AshtakavargaExpert::calcEkadhipatyaShodana()
 {
 	int i, j;
-	
-	// calculate number of planets in rasi
-	for( i = OSUN; i < OSATURN; i++ ) planetNumber[chart->getRasi(i)]++;
 
-	for( i = 0; i < 8; i++ ) 
-		for( j = 0; j < 12; j++ ) 
+	// calculate number of planets in rasi
+	for ( i = OSUN; i < OSATURN; i++ ) planetNumber[chart->getRasi(i)]++;
+
+	for ( i = 0; i < 8; i++ )
+		for ( j = 0; j < 12; j++ )
 			ekadhi[i][j] = trikona[i][j];
 
 	calcEkadhipatyaPair( ARIES, SCORPIO );
