@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/jyotish/Ephemeris.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "Ephemeris.h"
+#pragma implementation "Ephemeris.h"
 #endif
 
 #include "Ephemeris.h"
@@ -44,10 +44,10 @@ extern Config *config;
 class IngressEventSorter
 {
 public:
-  bool operator()( const IngressEvent &e1, const IngressEvent &e2 )
-  {
-    return( e1.jd < e2.jd );
-  }
+	bool operator()( const IngressEvent &e1, const IngressEvent &e2 )
+	{
+		return( e1.jd < e2.jd );
+	}
 };
 
 /*************************************************//**
@@ -58,19 +58,19 @@ public:
 class LunarEventSort
 {
 public:
-  bool operator()( const LunarEvent &e1, const LunarEvent &e2 )
-  {
-    return( e1.jd < e2.jd );
-  }
+	bool operator()( const LunarEvent &e1, const LunarEvent &e2 )
+	{
+		return( e1.jd < e2.jd );
+	}
 };
 
 /*****************************************************
 **
-**   EphemExpert   ---   Constructor 
+**   EphemExpert   ---   Constructor
 **
 ******************************************************/
 EphemExpert::EphemExpert( const ChartProperties *props )
-: chartprops( props )
+		: chartprops( props )
 {
 	d = new DataSet();
 	d->setLocation( config->defaultLocation );
@@ -80,7 +80,7 @@ EphemExpert::EphemExpert( const ChartProperties *props )
 
 /*****************************************************
 **
-**   EphemExpert   ---   Destructor 
+**   EphemExpert   ---   Destructor
 **
 ******************************************************/
 EphemExpert::~EphemExpert()
@@ -90,7 +90,7 @@ EphemExpert::~EphemExpert()
 
 /*****************************************************
 **
-**   EphemExpert   ---   init 
+**   EphemExpert   ---   init
 **
 ******************************************************/
 void EphemExpert::init()
@@ -98,9 +98,10 @@ void EphemExpert::init()
 	int i;
 	clen = cingress = ckp = cdetails = clunar = false;
 	mydasa = -1;
+	dstchange = false;
 
 	// reset variables
-	for( i = 0; i <= 31; i++ )
+	for ( i = 0; i <= 31; i++ )
 	{
 		weekday[i] = tithi[i] = 0;
 		jd[i] = st[i] =  sunrise[i] = sunset[i] = 0;
@@ -110,62 +111,88 @@ void EphemExpert::init()
 
 /*****************************************************
 **
-**   EphemExpert   ---   prepareMonth 
+**   EphemExpert   ---   prepareMonth
 **
 ******************************************************/
-void EphemExpert::prepareMonth( const int &m, const int &y, const bool &isLocaltime )
+void EphemExpert::prepareMonth( const int &m, const int &y, const bool &isLocal )
 {
 	int dummy;
+	double lastdst = 0;
 	Formatter *formatter = Formatter::get();
+	TzUtil tzu;
+	TzInfo tzinfo;
+
 	year = y;
 	month = m;
-	localtime = isLocaltime;
-	tz = localtime ? config->defaultLocation.getTimeZone() + config->defaultLocation.getDST() : 0;
+	this->isLocaltime = isLocal;
+
 	assert( month >= 0 && month <= 12 );
 	assert( year >= -2000 && year <= 4000 );
 
 	init();
 
-	switch( month )
+	switch ( month )
 	{
-		case 0:
-			formatter->getDateIntsFromJD( d->getJD(), dummy, month, year );		break;
+	case 0:
+		formatter->getDateIntsFromJD( d->getJD(), dummy, month, year );
 		break;
-		case 1:
-		case 3:
-		case 5:
-		case 7:
-		case 8:
-		case 10:
-		case 12:
-			nb_days = 31;
 		break;
-		case 2: // Feb
-			nb_days = 28;
-			if ( year % 4 == 0 )
-			{
-				nb_days = 29;
-				if ( year % 100 == 0 ) nb_days = 28; // 1900, 1800 etc. have 28 days
-				if ( year % 400 == 0 ) nb_days = 29; // BUT Feb 2000 has 29 days
-			}
+	case 1:
+	case 3:
+	case 5:
+	case 7:
+	case 8:
+	case 10:
+	case 12:
+		nb_days = 31;
 		break;
-		default:
-			nb_days = 30;
+	case 2: // Feb
+		nb_days = 28;
+		if ( year % 4 == 0 )
+		{
+			nb_days = 29;
+			if ( year % 100 == 0 ) nb_days = 28; // 1900, 1800 etc. have 28 days
+			if ( year % 400 == 0 ) nb_days = 29; // BUT Feb 2000 has 29 days
+		}
+		break;
+	default:
+		nb_days = 30;
 		break;
 	}
 
+	Formatter::get()->getDateIntsFromJD( JDate::getCurrentJD(), current_day, current_month, current_year );
+
 	// set jds and week days
-	for( int i = 0; i <= nb_days; i++ ) // calc JDs for this month and year
+	for ( int i = 0; i <= nb_days; i++ ) // calc JDs for this month and year
 	{
-		d->setDate( i+1, month, year, - tz );
+		d->setDate( i+1, month, year, 0 );
 		jd[i] = d->getJD();
-    weekday[i] = ((int)(jd[i] + 2 )) % 7;
+		weekday[i] = ((int)(jd[i] + 2 )) % 7;
+
+		if ( isLocaltime )
+		{
+			tzinfo = tzu.calculateTzInfoForJD( jd[i] );
+			double tzdelta = tzinfo.tzhours + tzinfo.dsthours;
+			jd[i] -= tzdelta / 24.0;
+
+			// test for dst change during month
+			if ( i > 0 )
+			{
+				if ( lastdst != tzinfo.dsthours )
+				{
+					//printf( "DST change day %d last is %f current %f\n", i+1, lastdst, tzinfo.dsthours );
+					dstchange = true;
+				}
+			}
+			lastdst = tzinfo.dsthours;
+			//printf( "Ephem: tzdelta %f day %d\n", tzdelta, i+1 );
+		}
 	}
 }
 
 /*****************************************************
 **
-**   EphemExpert   ---   calcLength 
+**   EphemExpert   ---   calcLength
 **
 ******************************************************/
 int EphemExpert::calcLength()
@@ -174,18 +201,17 @@ int EphemExpert::calcLength()
 	unsigned int i1;
 	double dummy;
 	Calculator *calculator = CalculatorFactory().getCalculator();
-	
-	Formatter::get()->getDateIntsFromJD( JDate().getJD(), current_day, current_month, current_year );
 
 	planetdata.clear();
-	for( i1 = 0; i1 < chartprops->getPlanetList().size(); i1++ )
+	for ( i1 = 0; i1 < chartprops->getPlanetList().size(); i1++ )
 	{
 		p = chartprops->getPlanetList()[i1];
 		if ( p == OASCENDANT || p == OMERIDIAN || p > MAX_EPHEM_OBJECTS ) continue;
 		EphemPlanetData pdata( p );
-		for( i = 0; i <= nb_days; i++ ) // calc length and retrogression
+		for ( i = 0; i <= nb_days; i++ ) // calc length and retrogression
 		{
-			d->setDate( i+1, month, year, -tz );
+			//d->setDate( i+1, month, year, -tz );
+			d->setDate( jd[i] );
 			calculator->calcPositionSpeed( d, p, pdata.len[i], dummy, pdata.speed[i], true, chartprops->isVedic() );
 			pdata.retro[i] = ( pdata.speed[i] < 0 );
 			pdata.rasi[i] = getRasi( pdata.len[i] );
@@ -202,7 +228,7 @@ int EphemExpert::calcLength()
 
 /*****************************************************
 **
-**   EphemExpert   ---   calcKP 
+**   EphemExpert   ---   calcKP
 **
 ******************************************************/
 void EphemExpert::calcKP( const int &dasa )
@@ -217,7 +243,7 @@ void EphemExpert::calcKP( const int &dasa )
 	if ( !clen  ) calcLength();
 	kpevents = expert->getKPEventList( planetdata[OMOON].len[0], planetdata[OMOON].len[nb_days], jd[0] ); // calculates all lord/sublord events
 
-	for( iter = kpevents.begin(); iter != kpevents.end(); iter++ )
+	for ( iter = kpevents.begin(); iter != kpevents.end(); iter++ )
 	{
 		e = (KPEvent*)(&(*iter));
 		d->setDate( e->jd );
@@ -238,7 +264,8 @@ void EphemExpert::writeKp( Writer *writer, const int &dasa )
 	KPEvent *e;
 	int d, m, y, lord, wd;
 	vector<KPEvent>::iterator iter;
-	wxString s;
+	TzUtil tzu;
+	wxString s, thetime;
 	vector<wxString> v;
 	Table *table = 0;
 	DasaExpert *expert = DasaExpertFactory::get()->getDasaExpert( dasa );
@@ -258,13 +285,13 @@ void EphemExpert::writeKp( Writer *writer, const int &dasa )
 	if ( ! ckp || mydasa != dasa ) calcKP( dasa );
 	writeHeaderInfo( writer );
 
-	s.Printf( _( "%s Dasa" ), expert->getName());
-	writer->writeParagraph( s );
+	//s.Printf( _( "%s Dasa" ), expert->getName());
+	//writer->writeParagraph( s );
 
 	lord = -1;
 	int line = 1;
 	int index = -1;
-	for( iter = kpevents.begin(); iter != kpevents.end(); iter++ )
+	for ( iter = kpevents.begin(); iter != kpevents.end(); iter++ )
 	{
 		e = (KPEvent*)(&(*iter));
 		if ( e->dasaindex != index )
@@ -287,13 +314,18 @@ void EphemExpert::writeKp( Writer *writer, const int &dasa )
 			line = 1;
 		}
 		assert( table );
-		formatter->getDateIntsFromJD( e->jd  + tz/24, d, m, y );
-    wd = ((int)(e->jd + tz/24 + 1.5 )) % 7;
+		formatter->getDateIntsFromJD( e->jd, d, m, y );
+		wd = ((int)(e->jd + 1.5 )) % 7;
 		assert( wd >= 0 && wd < 7 );
 		s.Printf( wxT( "%02d %s" ), d, (const wxChar*)wxString( (const wxChar*)lang.getWeekdayName( wd )).Left(2) );
 		table->setEntry( 0, line, s );
 
-		table->setEntry( 1, line, formatter->getTimeFormatted( getTimeFromJD( e->jd  + tz/24 )) );
+		TzFormattedDate fd = tzu.getDateFormatted( e->jd, isLocaltime );
+		thetime = fd.timeFormatted;
+		//if ( dstchange ) thetime << wxT( " " ) << fd.timezoneFormatted;
+		table->setEntry( 1, line, thetime );
+		//table->setEntry( 1, line, formatter->getTimeFormatted( getTimeFromJD( e->jd  + tz/24 )) );
+
 		if ( expert->isRasiDasaExpert() )
 		{
 			table->setEntry( 2, line, writer->getSignName( e->lord, TLARGE ));
@@ -321,7 +353,7 @@ void EphemExpert::writeKp( Writer *writer, const int &dasa )
 
 /*****************************************************
 **
-**   EphemExpert   ---   calcIngress 
+**   EphemExpert   ---   calcIngress
 **
 ******************************************************/
 int EphemExpert::calcIngress()
@@ -331,17 +363,40 @@ int EphemExpert::calcIngress()
 	if ( !clen  ) ret = calcLength();
 	ingressEvents.clear();
 
-	for( int i = 0; i < nb_days; i++ )
+	// Achtung: das geht nicht immer für den Mond, weil der mehr als ein Event am Tag haben kann
+	for ( int i = 0; i < nb_days; i++ )
 	{
-		d->setDate( i+1, month, year, -tz + 12 );
-		for( unsigned int j = 0; j < planetdata.size(); j++ )
+		d->setDate( i+1, month, year, 12 );
+		for ( unsigned int j = 0; j < planetdata.size(); j++ )
 		{
 			p = planetdata[j].pindex;
 			if ( p == OARIES || p == OASCENDANT || p == OMERIDIAN ) continue;
-			testIngressEvent( planetdata[j].rasi[i], planetdata[j].rasi[i+1], p, 0, planetdata[j].retro[i] );
+
+			if( planetdata[j].rasi[i] != planetdata[j].rasi[i+1] )
+			{
+				testIngressEvent( planetdata[j].rasi[i], planetdata[j].rasi[i+1], p, 0, planetdata[j].retro[i] );
+			}
+
 			if ( chartprops->isVedic() )
 			{
-				testIngressEvent( planetdata[j].nakshatra[i], planetdata[j].nakshatra[i+1], p, 1, planetdata[j].retro[i] );
+				// Nakshatra changes
+				if ( planetdata[j].nakshatra[i] !=  planetdata[j].nakshatra[i+1] )
+				{
+					const int base_nak = planetdata[j].nakshatra[i];
+					const int diff_nak = red27( planetdata[j].nakshatra[i+1] -  base_nak );
+					switch( diff_nak )
+					{
+						case 1:
+							testIngressEvent( base_nak, red27( base_nak + 1 ), p, 1, planetdata[j].retro[i] );
+						break;
+						case 2:
+							testIngressEvent( base_nak, red27( base_nak + 1 ), p, 1, planetdata[j].retro[i] );
+							testIngressEvent( red27( base_nak + 1 ), red27( base_nak + 2 ), p, 1, planetdata[j].retro[i+1] );
+						break;
+						default:
+						break;
+					}
+				}
 			}
 		}
 	}
@@ -352,7 +407,7 @@ int EphemExpert::calcIngress()
 
 /*****************************************************
 **
-**   EphemExpert   ---   testIngressEvent 
+**   EphemExpert   ---   testIngressEvent
 **
 ******************************************************/
 void EphemExpert::testIngressEvent( const int& t1, const int &t2, const int &planet, const int &type, const bool &r )
@@ -384,18 +439,17 @@ void EphemExpert::writeHeaderInfo( Writer *writer )
 
 /*****************************************************
 **
-**   EphemExpert   ---   writeIngress 
+**   EphemExpert   ---   writeIngress
 **
 ******************************************************/
 int EphemExpert::writeIngress( Writer *writer )
 {
 	int ret = 0;
-	Formatter *formatter = Formatter::get();
 	Lang lang;
 	IngressEvent *e;
-	int d, m, y, wd;
 	list<IngressEvent>::iterator iter;
-	wxString s, buf, pname;
+	wxString s, buf, pname, thetime;
+	TzUtil tzu;
 
 	if ( ! cingress ) ret = calcIngress();
 	writeHeaderInfo( writer );
@@ -407,7 +461,7 @@ int EphemExpert::writeIngress( Writer *writer )
 	table.setHeader( 3, chartprops->isVedic() ? _( "Sign/Nakshatra" ) : _( "Sign" ));
 
 	int line = 1;
-	for( iter = ingressEvents.begin(); iter != ingressEvents.end(); iter++ )
+	for ( iter = ingressEvents.begin(); iter != ingressEvents.end(); iter++ )
 	{
 		e = (IngressEvent*)(&(*iter));
 		if ( e->type == 0 )
@@ -417,15 +471,15 @@ int EphemExpert::writeIngress( Writer *writer )
 
 		pname = writer->getObjectName( e->planet, TLARGE, chartprops->isVedic() );
 
-		formatter->getDateIntsFromJD( e->jd  + tz/24, d, m, y );
-    wd = ((int)(e->jd + tz/24+ 1.5 )) % 7;
-		assert( wd >= 0 && wd < 7 );
+		TzFormattedDate fd = tzu.getDateFormatted( e->jd, isLocaltime );
+		thetime = fd.timeFormatted;
+		//if ( dstchange ) thetime << wxT( " " ) << fd.timezoneFormatted;
 
-		s.Printf( wxT( "%02d %s" ), d, (const wxChar*)lang.getWeekdayName( wd ).Left(2) );
-		if ( testDayIndexForCurrent( d )) table.setHeaderEntry( 0, line, s );
-		else	table.setEntry( 0, line, s );	
+		s.Printf( wxT( "%02d %s" ), fd.dayOfMonth, fd.weekDay.c_str());
+		if ( testDayIndexForCurrent( fd.dayOfMonth )) table.setHeaderEntry( 0, line, s );
+		else	table.setEntry( 0, line, s );
 
-		table.setEntry( 1, line, formatter->getTimeFormatted( getTimeFromJD( e->jd  + tz/24 )) );
+		table.setEntry( 1, line, thetime );
 		table.setEntry( 2, line, pname );
 		table.setEntry( 3, line, buf );
 		line++;
@@ -449,8 +503,8 @@ void EphemExpert::addLunarEvent( const int &stdate, const double& target, const
 	cjd = calculator->calcSunMoonEvent( d, target, slen, mlen );
 	double aya = calculator->calcAyanamsa( cjd, config->vAyanamsa );
 	lunarEvents.push_back( LunarEvent( target, cjd,
-		chartprops->isVedic() ? red_deg( slen - aya ) : slen,
-		chartprops->isVedic() ? red_deg( mlen - aya ) : mlen ));
+	                                   chartprops->isVedic() ? red_deg( slen - aya ) : slen,
+	                                   chartprops->isVedic() ? red_deg( mlen - aya ) : mlen ));
 }
 
 
@@ -479,7 +533,7 @@ void EphemExpert::calcLunar()
 
 	diff = red_deg( planetdata[OMOON].len[0] - planetdata[OSUN].len[0] );
 	t = (int)(diff/12);
-	for( i = 1; i <= nb_days; i++ )
+	for ( i = 1; i <= nb_days; i++ )
 	{
 		diff2 = red_deg( planetdata[OMOON].len[i] - planetdata[OSUN].len[i] );
 		t2 = (int)(diff2/12);
@@ -520,12 +574,11 @@ void EphemExpert::calcLunar()
 void EphemExpert::writeLunar( Writer *writer )
 {
 	if ( ! clunar ) calcLunar();
-	Formatter *formatter = Formatter::get();
 	Lang lang;
-	int d, m, y, wd;
-	wxString s;
+	wxString s, thetime;
 	list<LunarEvent>::iterator iter;
 	LunarEvent *e;
+	TzUtil tzu;
 
 	writeHeaderInfo( writer );
 	Table table( 7, lunarEvents.size()+1 );
@@ -538,19 +591,18 @@ void EphemExpert::writeLunar( Writer *writer )
 	table.setHeader( 6, _( "Western" ));
 
 	int line = 1;
-	for( iter = lunarEvents.begin(); iter != lunarEvents.end(); iter++ )
+	for ( iter = lunarEvents.begin(); iter != lunarEvents.end(); iter++ )
 	{
 		e = (LunarEvent*)(&(*iter));
+		TzFormattedDate fd = tzu.getDateFormatted( e->jd, isLocaltime );
+		thetime = fd.timeFormatted;
+		//if ( dstchange ) thetime << wxT( " " ) << fd.timezoneFormatted;
 
-		formatter->getDateIntsFromJD( e->jd  + tz/24, d, m, y );
-    wd = ((int)(e->jd +tz/24 + 1.5 )) % 7;
-		assert( wd >= 0 && wd < 7 );
+		s.Printf( wxT( "%02d %s" ), fd.dayOfMonth, fd.weekDay.c_str());
+		if ( testDayIndexForCurrent( fd.dayOfMonth )) table.setHeaderEntry( 0, line, s );
+		else	table.setEntry( 0, line, s );
 
-		s.Printf( wxT( "%02d %s" ), d, (const wxChar*)lang.getWeekdayName( wd ).Left(2) );
-		if ( testDayIndexForCurrent( d )) table.setHeaderEntry( 0, line, s );
-		else	table.setEntry( 0, line, s );	
-
-		table.setEntry( 1, line, formatter->getTimeFormatted( getTimeFromJD( e->jd  + tz/24 )) );
+		table.setEntry( 1, line, thetime );
 		table.setEntry( 2, line, writer->getPosFormatted( e->slen, false ) );
 		table.setEntry( 3, line, writer->getPosFormatted( e->mlen, false ) );
 
@@ -580,7 +632,7 @@ void EphemExpert::writeLunar( Writer *writer )
 
 /*****************************************************
 **
-**   EphemExpert   ---   calcDetails 
+**   EphemExpert   ---   calcDetails
 **
 ******************************************************/
 void EphemExpert::calcDetails()
@@ -589,7 +641,7 @@ void EphemExpert::calcDetails()
 	Calculator *calculator = CalculatorFactory().getCalculator();
 	double mlen, slen, dummy, diff;
 
-	for( i = 0; i < nb_days; i++ )
+	for ( i = 0; i < nb_days; i++ )
 	{
 		d->setDate( i+1, month, year, 0 ); // Must be 0, correct offset will follow in formatting
 
@@ -603,34 +655,33 @@ void EphemExpert::calcDetails()
 		diff = red_deg( mlen - slen );
 		tithi[i] = (int)(diff/12);
 	}
-	
+
 	cdetails = true;
 }
 
 /*****************************************************
 **
-**   EphemExpert   ---   writeDetails 
+**   EphemExpert   ---   writeDetails
 **
 ******************************************************/
 void EphemExpert::writeDetails( Writer *writer )
 {
 	int i;
-	double tzdelta;
-	wxString s, tz_str;
+	wxString s, tz_str, thetime;
 	Lang lang;
 	Formatter *formatter = Formatter::get();
+	TzUtil tzu;
 
 	if ( ! cdetails ) calcDetails();
-	tzdelta = ( tz ? tz/24 : 0 );
 
 	s.Printf( wxT ( "%s %d" ), (const wxChar*)lang.getMonthName( month-1 ), year );
 	writer->writeHeader1( s );
 
 	int line = 1;
 	int nb_leaps = 0;
-  for( i = 0; i < nb_days; i++ )
-  {
-    if ( i > 0 && weekday[i] == 0 ) nb_leaps++;
+	for ( i = 0; i < nb_days; i++ )
+	{
+		if ( i > 0 && weekday[i] == 0 ) nb_leaps++;
 	}
 	Table table( 5, nb_days+1+nb_leaps );
 	table.setHeader( 0, _( "Day" ));
@@ -640,22 +691,28 @@ void EphemExpert::writeDetails( Writer *writer )
 	table.setHeader( 4, _( "Tithi (Sunrise)" ));
 
 	line = 1;
-  for( i = 0; i < nb_days; i++ )
-  {
-    // blank line on weekend
-    if ( i > 0 && weekday[i] == 0 )
+	for ( i = 0; i < nb_days; i++ )
+	{
+		// blank line on weekend
+		if ( i > 0 && weekday[i] == 0 )
 		{
-			for( int j = 0; j < 5; j++ ) table.setHeaderEntry( j, line, wxEmptyString );
+			for ( int j = 0; j < 5; j++ ) table.setHeaderEntry( j, line, wxEmptyString );
 			line++;
 		}
 
 		s.Printf( wxT( "%02d %s" ), i+1, (const wxChar*)lang.getWeekdayName( weekday[i] ).Left(2) );
 		if ( testDayIndexForCurrent( i+1 )) table.setHeaderEntry( 0, line, s );
-		else	table.setEntry( 0, line, s );	
+		else	table.setEntry( 0, line, s );
 
 		table.setEntry( 1, line, formatter->getTimeFormatted( st[i] ) );
-		table.setEntry( 2, line, formatter->getTimeFormatted( getTimeFromJD( sunrise[i] + tzdelta )));
-		table.setEntry( 3, line, formatter->getTimeFormatted( getTimeFromJD( sunset[i] + tzdelta )));
+
+		TzFormattedDate fd = tzu.getDateFormatted( sunrise[i], isLocaltime );
+		thetime = fd.timeFormatted;
+		//if ( dstchange ) thetime << wxT( " " ) << fd.timezoneFormatted;
+		table.setEntry( 2, line, thetime );
+
+		fd = tzu.getDateFormatted( sunset[i], isLocaltime );
+		table.setEntry( 3, line, fd.timeFormatted );
 
 		table.setEntry( 4, line, lang.getTithiName( tithi[i] ));
 		line++;
@@ -675,25 +732,25 @@ double EphemExpert::getLength( const int &i, const int &day )
 
 /*****************************************************
 **
-**   EphemExpert   ---   getSpeed 
+**   EphemExpert   ---   getSpeed
 **
 ******************************************************/
 double EphemExpert::getSpeed( const int &i, const int &day )
 {
 	assert( i >= 0 && i < (int)planetdata.size());
-  return planetdata[i].speed[day];
-}   
-    
+	return planetdata[i].speed[day];
+}
+
 /*****************************************************
-**  
+**
 **   EphemExpert   ---   getRetro
 **
 ******************************************************/
 bool EphemExpert::getRetro( const int &i, const int &day )
-{ 
+{
 	assert( i >= 0 && i < (int)planetdata.size());
-  return planetdata[i].retro[day];
-} 
+	return planetdata[i].retro[day];
+}
 
 /*****************************************************
 **
@@ -748,36 +805,46 @@ int EphemExpert::writeDefaultEphemeris( Writer *writer )
 
 	int line = 1;
 	int nb_leaps = 0;
-  for( i = 0; i < nb_days; i++ )
-  {
-    if ( i > 0 && weekday[i] == 0 ) nb_leaps++;
+	for ( i = 0; i < nb_days; i++ )
+	{
+		if ( i > 0 && weekday[i] == 0 ) nb_leaps++;
 	}
 	Table table( numcols, nb_days+nb_leaps+2 );
 	table.setHeader( 0, _( "Day" ));
 
 	// Header
 	int col = 1;
-	for( i1 = 0; i1 < planetdata.size(); i1++ )
+	for ( i1 = 0; i1 < planetdata.size(); i1++ )
 	{
 		table.setHeader( col, writer->getObjectName( planetdata[i1].pindex, TLARGE, chartprops->isVedic() ));
 		col++;
 	}
 	line = 1;
-  for( i = 0; i < nb_days; i++ )
-  {
-    //if ( i > 0 && weekday[i] == 0 ) o << Endl; // blank line on weekend
-    if ( i > 0 && weekday[i] == 0 )
+	for ( i = 0; i < nb_days; i++ )
+	{
+		//if ( i > 0 && weekday[i] == 0 ) o << Endl; // blank line on weekend
+		if ( i > 0 && weekday[i] == 0 )
 		{
-			for( j = 0; j < numcols; j++ ) table.setHeaderEntry( j, line, wxEmptyString );
+			for ( j = 0; j < numcols; j++ ) table.setHeaderEntry( j, line, wxEmptyString );
 			line++;
 		}
 
 		s.Printf( wxT( "%02d %s" ), i+1, (const wxChar*)lang.getWeekdayName( weekday[i] ).Left(2) );
+
+		// write TZ if change during month
+		/*
+		if ( dstchange )
+		{
+			TzFormattedDate fd = tzutil.getDateFormatted( jd[i], isLocaltime );
+			s << wxT( " " ) << fd.timezoneFormatted;
+		}
+		*/
+
 		if ( testDayIndexForCurrent( i+1 )) table.setHeaderEntry( 0, line, s );
-		else	table.setEntry( 0, line, s );	
+		else	table.setEntry( 0, line, s );
 
 		col = 1;
-		for( i1 = 0; i1 < planetdata.size(); i1++ )
+		for ( i1 = 0; i1 < planetdata.size(); i1++ )
 		{
 			rasilen = getRasiLen( planetdata[i1].len[i] ) + .008333333;
 			deg = (int)rasilen;
@@ -785,10 +852,16 @@ int EphemExpert::writeDefaultEphemeris( Writer *writer )
 
 			// symbol for retrogression
 			d = wxT( " " );
-			if ( planetdata[i1].retro[i] ) { d = writer->getRetroSymbol(); }
+			if ( planetdata[i1].retro[i] ) {
+				d = writer->getRetroSymbol();
+			}
 			else if ( i > 1 && planetdata[i1].retro[i] != planetdata[i1].retro[i-1] )
-				{ planetdata[i1].retro[i] ? d = writer->getRetroSymbol(): d = writer->getRetroSymbol( 1 ); }
-			else if ( i == nb_days - 1 && planetdata[i1].retro[i] ) { d = writer->getRetroSymbol(); }
+			{
+				planetdata[i1].retro[i] ? d = writer->getRetroSymbol(): d = writer->getRetroSymbol( 1 );
+			}
+			else if ( i == nb_days - 1 && planetdata[i1].retro[i] ) {
+				d = writer->getRetroSymbol();
+			}
 
 			showrasi = shownak = true;
 			rasi_str = showrasi ? writer->getSignName( planetdata[i1].rasi[i] ) : wxT( "  " );
@@ -818,7 +891,7 @@ int EphemExpert::writeDefaultEphemeris( Writer *writer )
 				}
 			}
 			table.setEntry( col, line, s );
-			
+
 			col++;
 		}
 
@@ -827,7 +900,7 @@ int EphemExpert::writeDefaultEphemeris( Writer *writer )
 	// Header
 	table.setHeaderEntry( 0, line, _( "Day" ));
 	col = 1;
-	for( i1 = 0; i1 < planetdata.size(); i1++ )
+	for ( i1 = 0; i1 < planetdata.size(); i1++ )
 	{
 		table.setHeaderEntry( col, line, writer->getObjectName( planetdata[i1].pindex, TLARGE, chartprops->isVedic() ));
 		col++;
