@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/jyotish/GenericDasa.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "GenericDasa.h"
+#pragma implementation "GenericDasa.h"
 #endif
 
 #include "GenericDasa.h"
@@ -34,7 +34,7 @@
 
 /*****************************************************
 **
-**   GenericNakshatraDasaExpert   ---   getPortion 
+**   GenericNakshatraDasaExpert   ---   getPortion
 **
 ******************************************************/
 double GenericNakshatraDasaExpert::getPortion( const Horoscope *h, int &nak )
@@ -54,13 +54,13 @@ double GenericNakshatraDasaExpert::getPortion( const Horoscope *h, int &nak )
 vector<Dasa*> GenericNakshatraDasaExpert::getNextLevel( Dasa *dasa )
 {
 	vector<Dasa*> ret;
-	
+
 	double start_jd = dasa->getStartJD();
 	double end_jd = dasa->getEndJD();
 	double dasa_len = end_jd - start_jd;
 	double antar_len = 0;
 	int lord = -1;
-	for( int i = 0; i < getNbLords(); i++ )
+	for ( int i = 0; i < getNbLords(); i++ )
 	{
 		if ( getDasaLordByIndex( i ) == dasa->getDasaLord() )
 		{
@@ -69,14 +69,14 @@ vector<Dasa*> GenericNakshatraDasaExpert::getNextLevel( Dasa *dasa )
 		}
 	}
 	assert( lord != -1 );
-	
-	for( int i = 0; i < getNbLords(); i++ )
+
+	for ( int i = 0; i < getNbLords(); i++ )
 	{
 		antar_len = ( dasa_len * getDasaDuration( lord )) / getTotalLength();
 		end_jd = start_jd + antar_len;
 
-		ret.push_back( new DasaImpl( this, getDasaLordByIndex( lord ), start_jd, end_jd, 
-			getExtraText( lord ), ( dasa->getParent() == dasa ? 0 : dasa ) ));
+		ret.push_back( new DasaImpl( this, getDasaLordByIndex( lord ), start_jd, end_jd,
+		                             getExtraText( lord ), ( dasa->getParent() == dasa ? 0 : dasa ) ));
 
 		lord = ( lord + 1 ) % getNbLords();
 		start_jd = end_jd;
@@ -102,12 +102,12 @@ vector<Dasa*> GenericNakshatraDasaExpert::getFirstLevel( const Horoscope *h, con
 	double start_jd = h->getJD() - nportion * getDasaDuration( pindex ) * getYearLength( true );
 	double end_jd = start_jd + getTotalLength() * getYearLength( true );
 
-	while( total_length < 100 )
+	while ( total_length < 100 )
 	{
 		DasaImpl d( this, pl, start_jd, end_jd, 0, 0 );
 		d.setParent( &d );
 		tret = getNextLevel( &d );
-		for( unsigned j = 0; j < tret.size(); j++ )
+		for ( unsigned j = 0; j < tret.size(); j++ )
 		{
 			ret.push_back( tret[j] );
 		}
@@ -121,61 +121,102 @@ vector<Dasa*> GenericNakshatraDasaExpert::getFirstLevel( const Horoscope *h, con
 
 /*****************************************************
 **
-**   GenericNakshatraDasaExpert   ---   getKPLord 
+**   GenericNakshatraDasaExpert   ---   getKPLordRecursive
 **
 ******************************************************/
-void GenericNakshatraDasaExpert::getKPLord( const double &len, int &lord, int &sublord )
+void GenericNakshatraDasaExpert::getKPLordRecursive( int &lord_index, double &elapsed, double &total_len )
 {
-	const int max_loop = 10;
 	int count = 0;
+	const int max_loop = 10; // maximum number of antardasas in cycle
+	double antar_len; // in years
 
-	int nak = getNakshatra( len, N27 );
-	int lord_index = getDasaIndex4Nakshatra( nak );
-	lord = getDasaLordByIndex( lord_index );
-	double nlen = getNakshatraLength( len, N27 );
-	double elapsed = nlen * getDasaDuration( lord_index ) / NAKSHATRA_LEN;
-	int sublord_index = lord_index;
+	if ( elapsed >= total_len ) printf( "ERROR GenericNakshatraDasaExpert::getKPLordRecursive: elapsed time too big, %f total was %f\n", elapsed, total_len );
+	assert ( elapsed < total_len );
 
-	double antar_len = (double)getDasaDuration( lord_index )
-		* (double)getDasaDuration( sublord_index ) / getTotalLength();
+	antar_len = getDasaDuration( lord_index ) * total_len;
+	antar_len /= getTotalLength();
+	//printf( "Elapsed 1 %f antar_len %f total_len %f dur1 %d dur2 %d\n", elapsed, antar_len, total_len, getDasaDuration( lord_index ), getDasaDuration( lord_index ) );
 
 	// count so many antardasas that elapsed time is more than antarlen
-	while(( elapsed > antar_len ) && ( count++ < max_loop  ))
+	while (( elapsed > antar_len ) && ( count++ < max_loop  ))
 	{
-		sublord_index++;
-		if ( sublord_index >= getNbLords() ) sublord_index -= getNbLords();
+		lord_index++;
+		if ( lord_index >= getNbLords() ) lord_index -= getNbLords();
 		elapsed -= antar_len;
-		antar_len = (double)getDasaDuration( lord_index ) * (double)getDasaDuration( sublord_index ) / getTotalLength();
+
+		antar_len = getDasaDuration( lord_index ) * total_len;
+		antar_len /= getTotalLength();
+		//printf( "Elapsed 2 %f antar_len %f\n", elapsed, antar_len );
 	}
+	if ( count >= max_loop ) printf( "ERROR GenericNakshatraDasaExpert::getKPLordRecursive: count %d max_loop %d\n", count, max_loop );
 	assert( count < max_loop );
-	sublord = getDasaLordByIndex( sublord_index );
+
+	total_len = antar_len;
 }
 
 /*****************************************************
 **
-**   GenericNakshatraDasaExpert   ---   getKPEventList 
+**   GenericNakshatraDasaExpert   ---   getKPLords
+**
+******************************************************/
+KpData GenericNakshatraDasaExpert::getKPLords( const double &len )
+{
+	KpData kp;
+
+	// 0...27 - ASWINI = 0 etc
+	int nak = getNakshatra( len, N27 );
+
+	// index of the first lord for that Nakshatra in the sequence of lords, e.g. Vimsottari has Sun, Moon, Mars, Rahu, Jupiter, Saturn, Mercury, Ketu, Venus
+	int lord_index = getDasaIndex4Nakshatra( nak );
+
+	// elapsed portion in the Nakshatra (0...1)
+	double nakshatraPortion = getNakshatraLength( len, N27 ) / NAKSHATRA_LEN;
+
+	// elapsed time in years
+	double elapsed = nakshatraPortion * getDasaDuration( lord_index );
+
+	// total length in years (e.g. 120 for Vimsottari)
+	double total_len = getTotalLength();
+
+	//printf( "START ------  CAll 1 KP len %f lord_index %d, portion %f elapsed %f total_len %f\n", len, lord_index, nakshatraPortion, elapsed, total_len );
+	getKPLordRecursive( lord_index, elapsed, total_len );
+	kp.lord = getDasaLordByIndex( lord_index );
+	//printf( "CAll 2 KP len %f lord_index %d KPLORD %d, elapsed %f total_len %f\n", len, lord_index, kp.lord, elapsed, total_len );
+
+	getKPLordRecursive( lord_index, elapsed, total_len );
+	kp.sublord = getDasaLordByIndex( lord_index );
+	//printf( "CAll 3 KP len %f lord_index %d KPLORD %d, elapsed %f total_len %f\n\n", len, lord_index,  kp.sublord, elapsed, total_len );
+
+	getKPLordRecursive( lord_index, elapsed, total_len );
+	kp.subsublord = getDasaLordByIndex( lord_index );
+	//printf( "FINISHED   ------  CAll 4 KP len %f lord_index %d KPLORD %d, elapsed %f total_len %f\n\n", len, lord_index,  kp.subsublord, elapsed, total_len );
+
+	return kp;
+}
+
+/*****************************************************
+**
+**   GenericNakshatraDasaExpert   ---   getKPEventList
 **
 ******************************************************/
 vector<KPEvent> GenericNakshatraDasaExpert::getKPEventList( const double &startlen, const double &endlen,
-	const double &startjd )
+        const double &startjd )
 {
 	vector<KPEvent> events;
-	int currentnak, currentlord, i, j, antarlord, dasatime, antardasatime;
-	double cursorlen, dasastartlen, antarstartlen, dist;
+	int currentnak, currentlord, i, j, antarlord, antardasatime;
+	double dasastartlen, antarstartlen, dist;
 	bool append;
 
 	int startnak = getNakshatra( startlen, N27 );
 
-	for( i = 0; i < 32; i++ )
+	for ( i = 0; i < 32; i++ )
 	{
 		currentnak = (startnak + i ) % 27;
 		currentlord = getDasaIndex4Nakshatra( currentnak );
-		cursorlen = startlen + i * 13;
 		dasastartlen = red_deg( ( startnak + i ) * 13.33333333 );
-		dasatime = getDasaDuration( currentlord ); // years
 		antarstartlen = dasastartlen;
 
-		for( j = 0; j < getNbLords(); j++ )
+		for ( j = 0; j < getNbLords(); j++ )
 		{
 			append = true;
 			antarlord = ( currentlord  + j ) % getNbLords();
@@ -195,8 +236,8 @@ vector<KPEvent> GenericNakshatraDasaExpert::getKPEventList( const double &startl
 				if ( dist > 0 ) append = false;     // could also be return ?
 			}
 			if ( append )
-			events.push_back( KPEvent( getDasaLordByIndex( currentlord ), getDasaLordByIndex( antarlord ),
-				antarstartlen + .0000001, startjd + i + (double)j/getNbLords(), i ));
+				events.push_back( KPEvent( getDasaLordByIndex( currentlord ), getDasaLordByIndex( antarlord ),
+				                           antarstartlen + .0000001, startjd + i + (double)j/getNbLords(), i ));
 			antarstartlen += 13.333333333 * (double)antardasatime / getTotalLength();
 		}
 	}
@@ -206,7 +247,7 @@ vector<KPEvent> GenericNakshatraDasaExpert::getKPEventList( const double &startl
 
 /*****************************************************
 **
-**   YoginiDasaExpert   ---   getDasaLordByIndex 
+**   YoginiDasaExpert   ---   getDasaLordByIndex
 **
 ******************************************************/
 int YoginiDasaExpert::getDasaLordByIndex( const int &index )
@@ -218,7 +259,7 @@ int YoginiDasaExpert::getDasaLordByIndex( const int &index )
 
 /*****************************************************
 **
-**   YoginiDasaExpert   ---   getDasaIndex4Nakshatra 
+**   YoginiDasaExpert   ---   getDasaIndex4Nakshatra
 **
 ******************************************************/
 int YoginiDasaExpert::getDasaIndex4Nakshatra( const int &nak )
@@ -228,7 +269,7 @@ int YoginiDasaExpert::getDasaIndex4Nakshatra( const int &nak )
 
 /*****************************************************
 **
-**   YoginiDasaExpert   ---   getDasaDuration 
+**   YoginiDasaExpert   ---   getDasaDuration
 **
 ******************************************************/
 int YoginiDasaExpert::getDasaDuration( const int &index )
@@ -245,14 +286,38 @@ int YoginiDasaExpert::getDasaDuration( const int &index )
 const wxChar *YoginiDasaExpert::getExtraText( const int &i )
 {
 	const wxChar* k_yogini[8] = { _( "Mangala" ), _( "Pingala" ), _( "Dhanya" ), _( "Bhramari" ),
-		_( "Bhadrika" ), _( "Ulka" ), _( "Siddha" ), _( "Sankata" )};
+	                              _( "Bhadrika" ), _( "Ulka" ), _( "Siddha" ), _( "Sankata" )
+	                            };
 	assert( i >= 0 && i < 8 );
 	return k_yogini[i];
 }
 
 /*****************************************************
 **
-**   ShodshottariDasaExpert   ---   getDasaLordByIndex 
+**   YoginiDasaExpert   ---   getDasaLordName
+**
+******************************************************/
+wxString YoginiDasaExpert::getDasaLordName( Writer*, const int &lord, const int format )
+{
+	wxString s;
+
+	for( int i = 0; i < 8; i++ )
+	{
+		if ( getDasaLordByIndex( i ) == lord )
+		{
+			s = getExtraText( i );
+			if ( format == TSHORT ) return s.Left( 2 );
+			else if ( format == TMEDIUM ) return s.Left( 3 );
+			else return s;
+		}
+	}
+	printf( "Error YoginiDasaExpert::getDasaLordName lord %d\n", lord );
+	return wxT( "Error" );
+}
+
+/*****************************************************
+**
+**   ShodshottariDasaExpert   ---   getDasaLordByIndex
 **
 ******************************************************/
 int ShodshottariDasaExpert::getDasaLordByIndex( const int &index )
@@ -264,7 +329,7 @@ int ShodshottariDasaExpert::getDasaLordByIndex( const int &index )
 
 /*****************************************************
 **
-**   ShodshottariDasaExpert   ---   getDasaIndex4Nakshatra 
+**   ShodshottariDasaExpert   ---   getDasaIndex4Nakshatra
 **
 ******************************************************/
 int ShodshottariDasaExpert::getDasaIndex4Nakshatra( const int &nak )
@@ -275,7 +340,7 @@ int ShodshottariDasaExpert::getDasaIndex4Nakshatra( const int &nak )
 
 /*****************************************************
 **
-**   ShodshottariDasaExpert   ---   getDasaDuration 
+**   ShodshottariDasaExpert   ---   getDasaDuration
 **
 ******************************************************/
 int ShodshottariDasaExpert::getDasaDuration( const int &index )
@@ -287,7 +352,7 @@ int ShodshottariDasaExpert::getDasaDuration( const int &index )
 
 /*****************************************************
 **
-**   DvadashottariDasaExpert   ---   getDasaLordByIndex 
+**   DvadashottariDasaExpert   ---   getDasaLordByIndex
 **
 ******************************************************/
 int DvadashottariDasaExpert::getDasaLordByIndex( const int &index )
@@ -299,7 +364,7 @@ int DvadashottariDasaExpert::getDasaLordByIndex( const int &index )
 
 /*****************************************************
 **
-**   DvadashottariDasaExpert   ---   getDasaIndex4Nakshatra 
+**   DvadashottariDasaExpert   ---   getDasaIndex4Nakshatra
 **
 ******************************************************/
 int DvadashottariDasaExpert::getDasaIndex4Nakshatra( const int &nak )
@@ -310,7 +375,7 @@ int DvadashottariDasaExpert::getDasaIndex4Nakshatra( const int &nak )
 
 /*****************************************************
 **
-**   DvadashottariDasaExpert   ---   getDasaDuration 
+**   DvadashottariDasaExpert   ---   getDasaDuration
 **
 ******************************************************/
 int DvadashottariDasaExpert::getDasaDuration( const int &index )
@@ -322,7 +387,7 @@ int DvadashottariDasaExpert::getDasaDuration( const int &index )
 
 /*****************************************************
 **
-**   PanchottariDasaExpert   ---   getDasaLordByIndex 
+**   PanchottariDasaExpert   ---   getDasaLordByIndex
 **
 ******************************************************/
 int PanchottariDasaExpert::getDasaLordByIndex( const int &index )
@@ -334,7 +399,7 @@ int PanchottariDasaExpert::getDasaLordByIndex( const int &index )
 
 /*****************************************************
 **
-**   PanchottariDasaExpert   ---   getDasaIndex4Nakshatra 
+**   PanchottariDasaExpert   ---   getDasaIndex4Nakshatra
 **
 ******************************************************/
 int PanchottariDasaExpert::getDasaIndex4Nakshatra( const int &nak )
@@ -345,7 +410,7 @@ int PanchottariDasaExpert::getDasaIndex4Nakshatra( const int &nak )
 
 /*****************************************************
 **
-**   PanchottariDasaExpert   ---   getDasaDuration 
+**   PanchottariDasaExpert   ---   getDasaDuration
 **
 ******************************************************/
 int PanchottariDasaExpert::getDasaDuration( const int &index )
@@ -357,7 +422,7 @@ int PanchottariDasaExpert::getDasaDuration( const int &index )
 
 /*****************************************************
 **
-**   ShatabdikaDasaExpert   ---   getDasaLordByIndex 
+**   ShatabdikaDasaExpert   ---   getDasaLordByIndex
 **
 ******************************************************/
 int ShatabdikaDasaExpert::getDasaLordByIndex( const int &index )
@@ -369,7 +434,7 @@ int ShatabdikaDasaExpert::getDasaLordByIndex( const int &index )
 
 /*****************************************************
 **
-**   ShatabdikaDasaExpert   ---   getDasaIndex4Nakshatra 
+**   ShatabdikaDasaExpert   ---   getDasaIndex4Nakshatra
 **
 ******************************************************/
 int ShatabdikaDasaExpert::getDasaIndex4Nakshatra( const int &nak )
@@ -380,7 +445,7 @@ int ShatabdikaDasaExpert::getDasaIndex4Nakshatra( const int &nak )
 
 /*****************************************************
 **
-**   ShatabdikaDasaExpert   ---   getDasaDuration 
+**   ShatabdikaDasaExpert   ---   getDasaDuration
 **
 ******************************************************/
 int ShatabdikaDasaExpert::getDasaDuration( const int &index )
@@ -392,7 +457,7 @@ int ShatabdikaDasaExpert::getDasaDuration( const int &index )
 
 /*****************************************************
 **
-**   ChaturashitiSamaDasaExpert   ---   getDasaLordByIndex 
+**   ChaturashitiSamaDasaExpert   ---   getDasaLordByIndex
 **
 ******************************************************/
 int ChaturashitiSamaDasaExpert::getDasaLordByIndex( const int &index )
@@ -404,7 +469,7 @@ int ChaturashitiSamaDasaExpert::getDasaLordByIndex( const int &index )
 
 /*****************************************************
 **
-**   ChaturashitiSamaDasaExpert   ---   getDasaIndex4Nakshatra 
+**   ChaturashitiSamaDasaExpert   ---   getDasaIndex4Nakshatra
 **
 ******************************************************/
 int ChaturashitiSamaDasaExpert::getDasaIndex4Nakshatra( const int &nak )
@@ -415,7 +480,7 @@ int ChaturashitiSamaDasaExpert::getDasaIndex4Nakshatra( const int &nak )
 
 /*****************************************************
 **
-**   ChaturashitiSamaDasaExpert   ---   getDasaDuration 
+**   ChaturashitiSamaDasaExpert   ---   getDasaDuration
 **
 ******************************************************/
 int ChaturashitiSamaDasaExpert::getDasaDuration( const int &index )
@@ -425,7 +490,7 @@ int ChaturashitiSamaDasaExpert::getDasaDuration( const int &index )
 
 /*****************************************************
 **
-**   DvisaptatiSamaDasaExpert   ---   getDasaLordByIndex 
+**   DvisaptatiSamaDasaExpert   ---   getDasaLordByIndex
 **
 ******************************************************/
 int DvisaptatiSamaDasaExpert::getDasaLordByIndex( const int &index )
@@ -437,7 +502,7 @@ int DvisaptatiSamaDasaExpert::getDasaLordByIndex( const int &index )
 
 /*****************************************************
 **
-**   DvisaptatiSamaDasaExpert   ---   getDasaIndex4Nakshatra 
+**   DvisaptatiSamaDasaExpert   ---   getDasaIndex4Nakshatra
 **
 ******************************************************/
 int DvisaptatiSamaDasaExpert::getDasaIndex4Nakshatra( const int &nak )
@@ -448,7 +513,7 @@ int DvisaptatiSamaDasaExpert::getDasaIndex4Nakshatra( const int &nak )
 
 /*****************************************************
 **
-**   DvisaptatiSamaDasaExpert   ---   getDasaDuration 
+**   DvisaptatiSamaDasaExpert   ---   getDasaDuration
 **
 ******************************************************/
 int DvisaptatiSamaDasaExpert::getDasaDuration( const int &index )
@@ -459,7 +524,7 @@ int DvisaptatiSamaDasaExpert::getDasaDuration( const int &index )
 
 /*****************************************************
 **
-**   ShatTrimshatSamaDasaExpert   ---   getDasaLordByIndex 
+**   ShatTrimshatSamaDasaExpert   ---   getDasaLordByIndex
 **
 ******************************************************/
 int ShatTrimshatSamaDasaExpert::getDasaLordByIndex( const int &index )
@@ -471,7 +536,7 @@ int ShatTrimshatSamaDasaExpert::getDasaLordByIndex( const int &index )
 
 /*****************************************************
 **
-**   ShatTrimshatSamaDasaExpert   ---   getDasaIndex4Nakshatra 
+**   ShatTrimshatSamaDasaExpert   ---   getDasaIndex4Nakshatra
 **
 ******************************************************/
 int ShatTrimshatSamaDasaExpert::getDasaIndex4Nakshatra( const int &nak )
@@ -482,7 +547,7 @@ int ShatTrimshatSamaDasaExpert::getDasaIndex4Nakshatra( const int &nak )
 
 /*****************************************************
 **
-**   ShatTrimshatSamaDasaExpert   ---   getDasaDuration 
+**   ShatTrimshatSamaDasaExpert   ---   getDasaDuration
 **
 ******************************************************/
 int ShatTrimshatSamaDasaExpert::getDasaDuration( const int &index )
@@ -497,13 +562,29 @@ int ShatTrimshatSamaDasaExpert::getDasaDuration( const int &index )
 **   DasaExpertFactory   ---   create
 **
 ******************************************************/
-DasaExpert *DasaExpertFactory::createYoginiDasaExpert() {	return new YoginiDasaExpert();}
-DasaExpert *DasaExpertFactory::createShodshottariDasaExpert() { return new ShodshottariDasaExpert(); }
-DasaExpert *DasaExpertFactory::createDvadashottariDasaExpert() { return new DvadashottariDasaExpert(); }
-DasaExpert *DasaExpertFactory::createPanchottariDasaExpert() { return new PanchottariDasaExpert(); }
-DasaExpert *DasaExpertFactory::createShatabdikaDasaExpert() { return new ShatabdikaDasaExpert(); }
-DasaExpert *DasaExpertFactory::createChaturashitiSamaDasaExpert() { return new ChaturashitiSamaDasaExpert(); }
-DasaExpert *DasaExpertFactory::createDvisaptatiSamaDasaExpert() { return new DvisaptatiSamaDasaExpert(); }
-DasaExpert *DasaExpertFactory::createShatTrimshatSamaDasaExpert() { return new ShatTrimshatSamaDasaExpert(); }
+DasaExpert *DasaExpertFactory::createYoginiDasaExpert() {
+	return new YoginiDasaExpert();
+}
+DasaExpert *DasaExpertFactory::createShodshottariDasaExpert() {
+	return new ShodshottariDasaExpert();
+}
+DasaExpert *DasaExpertFactory::createDvadashottariDasaExpert() {
+	return new DvadashottariDasaExpert();
+}
+DasaExpert *DasaExpertFactory::createPanchottariDasaExpert() {
+	return new PanchottariDasaExpert();
+}
+DasaExpert *DasaExpertFactory::createShatabdikaDasaExpert() {
+	return new ShatabdikaDasaExpert();
+}
+DasaExpert *DasaExpertFactory::createChaturashitiSamaDasaExpert() {
+	return new ChaturashitiSamaDasaExpert();
+}
+DasaExpert *DasaExpertFactory::createDvisaptatiSamaDasaExpert() {
+	return new DvisaptatiSamaDasaExpert();
+}
+DasaExpert *DasaExpertFactory::createShatTrimshatSamaDasaExpert() {
+	return new ShatTrimshatSamaDasaExpert();
+}
 
 
