@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/jyotish/astrobase.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -19,7 +19,7 @@
 ****************************************************************************/
 
 #ifdef __GNUG__
-	#pragma implementation "astrobase.h"
+#pragma implementation "astrobase.h"
 #endif
 
 #include "astrobase.h"
@@ -34,7 +34,7 @@ extern Config *config;
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 double getYearLength( const bool vedic )
@@ -42,28 +42,28 @@ double getYearLength( const bool vedic )
 	int yl = ( vedic ? config->vYearLength : config->wYearLength );
 	double custom_yl = ( vedic ? config->vCustomYearLength : config->wCustomYearLength );
 
-	switch( yl )
-  {
-    case 0:
-      return 365.25;
-    break;
-    case 1:
-      return 365.2422;
-    break;
-    case 2:
-      return 365;
-    break;
-    case 3:
-      return 360;
-    break;
-    case 4:
-      return custom_yl;
-    break;
-    default:
-			printf( "Warning: Wrong year length index %d taking default 365.25", yl );
-      return 365.25;
-    break;
-  }
+	switch ( yl )
+	{
+	case 0:
+		return 365.25;
+		break;
+	case 1:
+		return 365.2422;
+		break;
+	case 2:
+		return 365;
+		break;
+	case 3:
+		return 360;
+		break;
+	case 4:
+		return custom_yl;
+		break;
+	default:
+		printf( "Warning: Wrong year length index %d taking default 365.25", yl );
+		return 365.25;
+		break;
+	}
 }
 
 /*****************************************************
@@ -73,35 +73,35 @@ double getYearLength( const bool vedic )
 ******************************************************/
 double calcYearLength( const int sel, const bool vedic )
 {
-  double custom_yl = ( vedic ? config->vCustomYearLength : config->wCustomYearLength );
-  
-  switch( sel )
-  {
-    case 0:
-      return 365.25;
-    break;
-    case 1:
-      return 365.2422;
-    break;
-    case 2:
-      return 365;
-    break;
-    case 3:
-      return 360;
-    break;
-    case 4:
-      return custom_yl;
-    break;
-    default:
-      printf( "Warning: Wrong year length index %d taking default 365.25", sel );
-      return 365.25;
-    break;
-  }
+	double custom_yl = ( vedic ? config->vCustomYearLength : config->wCustomYearLength );
+
+	switch ( sel )
+	{
+	case 0:
+		return 365.25;
+		break;
+	case 1:
+		return 365.2422;
+		break;
+	case 2:
+		return 365;
+		break;
+	case 3:
+		return 360;
+		break;
+	case 4:
+		return custom_yl;
+		break;
+	default:
+		printf( "Warning: Wrong year length index %d taking default 365.25", sel );
+		return 365.25;
+		break;
+	}
 }
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 double getNavamsaPortion( const double &len, const int &pindex, const bool &retro )
@@ -128,7 +128,7 @@ double getNavamsaPortion( const double &len, const int &pindex, const bool &retr
 	double portion;
 	navamsavastha = 0;
 
-	for( int i = 0; i <= inav_portion; i++ )
+	for ( int i = 0; i <= inav_portion; i++ )
 	{
 		if ( i < inav_portion ) portion = 1;
 		else portion = nav_portion - (int)nav_portion;
@@ -169,7 +169,7 @@ double getNavamsaPortion( const double &len, const int &pindex, const bool &retr
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 void getDasaDurationInts( int &year, int &month, int &day, double length )
@@ -187,7 +187,7 @@ void getDasaDurationInts( int &year, int &month, int &day, double length )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 void getDasaDuration( wxString &buf, double length )
@@ -236,7 +236,7 @@ void getDasaDuration( wxString &buf, double length )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 double getOpposition( const double &v )
@@ -246,7 +246,7 @@ double getOpposition( const double &v )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int getRasi( const double &len )
@@ -264,7 +264,7 @@ int getRasi( const double &len )
 #define SRAVANA_LEN 12.444444444 // = 12:26:40
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 double getCorrectedNakshatraLength( const double &len )
@@ -289,7 +289,7 @@ double getCorrectedNakshatraLength( const double &len )
 }
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int getNakshatra( const double &len, const int nsys )
@@ -300,7 +300,7 @@ int getNakshatra( const double &len, const int nsys )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 double getNakshatraLength( const double &len, const int nsys )
@@ -311,7 +311,7 @@ double getNakshatraLength( const double &len, const int nsys )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int getPada( const double &len )
@@ -321,7 +321,7 @@ int getPada( const double &len )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int isOddFootedRasi( const int &rasi )
@@ -329,12 +329,12 @@ int isOddFootedRasi( const int &rasi )
 	assert(( rasi >= 0 ) && ( rasi < 12 ));
 
 	if (( rasi == ARIES )
-		|| ( rasi == TAURUS )
-		|| ( rasi == GEMINI )
-		|| ( rasi == LIBRA )
-		|| ( rasi == SCORPIO )
-		|| ( rasi == SAGITTARIUS )
-	)
+	        || ( rasi == TAURUS )
+	        || ( rasi == GEMINI )
+	        || ( rasi == LIBRA )
+	        || ( rasi == SCORPIO )
+	        || ( rasi == SAGITTARIUS )
+	   )
 		return true;
 
 	return false;
@@ -350,21 +350,21 @@ int getVarna( const double &len )
 	assert( len >= 0 && len <= 360 );
 	switch ( getRasi( len ) % 4 )
 	{
-		case 0:
-			return 2; // Kshattriya
+	case 0:
+		return 2; // Kshattriya
 		break;
-		case 1:
-			return 1; // Vaishya
+	case 1:
+		return 1; // Vaishya
 		break;
-		case 2:
-			return 0; // Shudra
+	case 2:
+		return 0; // Shudra
 		break;
-		case 3:
-			return 3; // Brahmin
+	case 3:
+		return 3; // Brahmin
 		break;
-		default:
-			assert( 0 );
-			return -1;
+	default:
+		assert( 0 );
+		return -1;
 		break;
 	}
 	return -1;
@@ -372,7 +372,7 @@ int getVarna( const double &len )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 bool hasJaiminiAspect( const int &pos1, const int &pos2 )
@@ -401,7 +401,7 @@ bool hasJaiminiAspect( const int &pos1, const int &pos2 )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int isOddRasi( const double &len )
@@ -411,7 +411,7 @@ int isOddRasi( const double &len )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int isEvenRasi( const double &len )
@@ -453,7 +453,7 @@ bool isVishamapadaRasi( const int &rasi )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 double getRasiLen( const double &len, const bool &reverse )
@@ -464,7 +464,7 @@ double getRasiLen( const double &len, const bool &reverse )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int between( const double &len, const double &min, const double &max )
@@ -474,7 +474,7 @@ int between( const double &len, const double &min, const double &max )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 double planetDistance( const double &len1, const double &len2 )
@@ -486,19 +486,19 @@ double planetDistance( const double &len1, const double &len2 )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int inAscendant( const double &len, const double &asc )
 {
 	return( a_red( getRasi( len ) - getRasi( asc ), 12 ) == 0 );
-	
+
 	return 0;
 }
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int inTrikona( const double &len, const double &asc )
@@ -511,7 +511,7 @@ int inTrikona( const double &len, const double &asc )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int inKendra( const double &len, const double &asc )
@@ -525,17 +525,17 @@ int inKendra( const double &len, const double &asc )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
-int inMovableSign( const double &len ) 
+int inMovableSign( const double &len )
 {
 	return( getRasi( len ) % 3 ) == 0;
 }
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int inFixedSign( const double &len )
@@ -545,7 +545,7 @@ int inFixedSign( const double &len )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int inDualSign( const double &len )
@@ -561,31 +561,34 @@ int inDualSign( const double &len )
 const wxChar *getAvasthaName( const int &i , const int &avastha )
 {
 	const wxChar *sayanadi[12] = { wxT( "Sayana" ), wxT( "Upavesana" ), wxT( "Netrapani" ),
-	wxT( "Prakasana" ), wxT( "Gamana" ), wxT( "Aagamana" ), wxT( "Sabha" ), wxT( "Agama" ),
-	wxT( "Bhojama" ), wxT( "Nrityalipsa" ), wxT( "Kautuka" ), wxT( "Nidra" ) };
+	                               wxT( "Prakasana" ), wxT( "Gamana" ), wxT( "Aagamana" ), wxT( "Sabha" ), wxT( "Agama" ),
+	                               wxT( "Bhojama" ), wxT( "Nrityalipsa" ), wxT( "Kautuka" ), wxT( "Nidra" )
+	                             };
 
-if ( i > 1 ) return sayanadi[i - 1];
-return sayanadi[11];
+	if ( i > 1 ) return sayanadi[i - 1];
+	return sayanadi[11];
 }
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int getLord( const int &sign, const bool vedic )
 {
 	assert(( sign >= ARIES ) && ( sign <= PISCES ));
 	const int k_vlord[12] = { OMARS, OVENUS, OMERCURY, OMOON, OSUN, OMERCURY,
-		OVENUS, OMARS, OJUPITER, OSATURN, OSATURN, OJUPITER };
+	                          OVENUS, OMARS, OJUPITER, OSATURN, OSATURN, OJUPITER
+	                        };
 	const int k_wlord[12] = { OMARS, OVENUS, OMERCURY, OMOON, OSUN, OMERCURY,
-		OPLUTO, OMARS, OJUPITER, OSATURN, OURANUS, ONEPTUNE };
+	                          OPLUTO, OMARS, OJUPITER, OSATURN, OURANUS, ONEPTUNE
+	                        };
 	return vedic ? k_vlord[sign] : k_wlord[sign];
 }
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int isFriend( const int &planet1, const int &planet2 )
@@ -593,18 +596,18 @@ int isFriend( const int &planet1, const int &planet2 )
 	assert(( planet1 >= OSUN ) && ( planet1 <= OSATURN ));
 	assert(( planet2 >= OSUN ) && ( planet2 <= OSATURN ));
 
-/********************************************************
-       Friends         Enemies        Neutrals
----------------------------------------------------------
- Sun   Moo, Mar, Jup   Ve, Sat        Mer
- Moo   Sun, Mer        -              Mar, Jup, Ven, Sat
- Mar   Sun, Moo, Jup   Mer            Ven, Sat
- Mer   Sun, Ven        Moo            Mar, Jup, Sat
- Jup   Sun, Moo, Mar   Mer, Ven       Sat
- Ven   Mer, Sat        Moo, Sun       Mar, Jup
- Sat   Mer, Ven        Sun, Moo, Mar  Jup
-*********************************************************/
-	
+	/********************************************************
+	       Friends         Enemies        Neutrals
+	---------------------------------------------------------
+	 Sun   Moo, Mar, Jup   Ve, Sat        Mer
+	 Moo   Sun, Mer        -              Mar, Jup, Ven, Sat
+	 Mar   Sun, Moo, Jup   Mer            Ven, Sat
+	 Mer   Sun, Ven        Moo            Mar, Jup, Sat
+	 Jup   Sun, Moo, Mar   Mer, Ven       Sat
+	 Ven   Mer, Sat        Moo, Sun       Mar, Jup
+	 Sat   Mer, Ven        Sun, Moo, Mar  Jup
+	*********************************************************/
+
 	const int k_friend[7][7] = {
 		{ FRIEND, FRIEND, NEUTRAL, ENEMY, FRIEND, FRIEND, ENEMY },      // Sun
 		{ FRIEND, FRIEND, FRIEND, NEUTRAL, NEUTRAL, NEUTRAL, NEUTRAL }, // Moon
@@ -619,7 +622,7 @@ int isFriend( const int &planet1, const int &planet2 )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int isRasiPosTempFriend( const int &rasi1, const int &rasi2 )
@@ -633,7 +636,7 @@ int isRasiPosTempFriend( const int &rasi1, const int &rasi2 )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 bool isInMulatrikona( const int &planet, const double &len )
@@ -678,12 +681,12 @@ bool isInMulatrikona( const int &planet, const double &len )
 		printf( "ERROR isInMulatrikona planet " );
 		exit(1);
 	}
-return false;
+	return false;
 }
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int getMulatrikonaRasi( const int &planet )
@@ -700,7 +703,7 @@ int getMulatrikonaRasi( const int &planet )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int getVashya( const double &len )
@@ -709,36 +712,36 @@ int getVashya( const double &len )
 	int rasi = ::getRasi( len );
 	double rasilen = ::getRasiLen( len );
 	//enum { VASHYA_QUADRUPED, VASHYA_HUMAN, VASHYA_JALACHARA, VASHYA_LEO, VASHYA_SCORPIO };
-	switch( rasi )
+	switch ( rasi )
 	{
-		case ARIES:
-		case TAURUS:
-			vashya = VASHYA_QUADRUPED;
+	case ARIES:
+	case TAURUS:
+		vashya = VASHYA_QUADRUPED;
 		break;
-		case GEMINI:
-		case VIRGO:
-		case LIBRA:
-		case AQUARIUS:
-			vashya = VASHYA_HUMAN;
+	case GEMINI:
+	case VIRGO:
+	case LIBRA:
+	case AQUARIUS:
+		vashya = VASHYA_HUMAN;
 		break;
-		case CANCER:
-		case PISCES:
-			vashya = VASHYA_JALACHARA;
+	case CANCER:
+	case PISCES:
+		vashya = VASHYA_JALACHARA;
 		break;
-		case LEO:
-			vashya = VASHYA_LEO;
+	case LEO:
+		vashya = VASHYA_LEO;
 		break;
-		case SCORPIO:
-			vashya = VASHYA_SCORPIO;
+	case SCORPIO:
+		vashya = VASHYA_SCORPIO;
 		break;
-		case SAGITTARIUS:
-			vashya = rasilen < 15 ? VASHYA_HUMAN : VASHYA_QUADRUPED;
+	case SAGITTARIUS:
+		vashya = rasilen < 15 ? VASHYA_HUMAN : VASHYA_QUADRUPED;
 		break;
-		case CAPRICORN:
-			vashya = rasilen < 15 ? VASHYA_QUADRUPED : VASHYA_JALACHARA;
+	case CAPRICORN:
+		vashya = rasilen < 15 ? VASHYA_QUADRUPED : VASHYA_JALACHARA;
 		break;
-		default:
-			assert( 0 );
+	default:
+		assert( 0 );
 		break;
 	}
 	assert( vashya != VASHYA_NONE );
@@ -747,7 +750,7 @@ int getVashya( const double &len )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 bool hasMulatrikonaRasi( const int &planet, const int &rasi )
@@ -758,29 +761,29 @@ bool hasMulatrikonaRasi( const int &planet, const int &rasi )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 int getExaltationRasi( const int &planet )
 {
 	const static int exalt_rasi[9] = { ARIES, TAURUS, VIRGO, PISCES, CAPRICORN, CANCER, LIBRA, GEMINI, SAGITTARIUS };
 	int ret = 0;
-	assert(  (( planet >= OSUN ) && ( planet <= OSATURN )) 
-		|| ( planet == OMEANNODE ) || ( planet == OTRUENODE )
-		|| ( planet == OMEANDESCNODE ) || ( planet == OTRUEDESCNODE ));
-	
-	switch( planet )
+	assert(  (( planet >= OSUN ) && ( planet <= OSATURN ))
+	         || ( planet == OMEANNODE ) || ( planet == OTRUENODE )
+	         || ( planet == OMEANDESCNODE ) || ( planet == OTRUEDESCNODE ));
+
+	switch ( planet )
 	{
-		case OMEANNODE:
-		case OTRUENODE:
-			ret = 7;
+	case OMEANNODE:
+	case OTRUENODE:
+		ret = 7;
 		break;
-		case OMEANDESCNODE:
-		case OTRUEDESCNODE:
-			ret = 8;
+	case OMEANDESCNODE:
+	case OTRUEDESCNODE:
+		ret = 8;
 		break;
-		default:
-			ret = planet;
+	default:
+		ret = planet;
 		break;
 	}
 	return exalt_rasi[ret];
@@ -788,7 +791,7 @@ int getExaltationRasi( const int &planet )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 bool hasExaltationRasi( const int &planet, const int &rasi )
@@ -798,7 +801,7 @@ bool hasExaltationRasi( const int &planet, const int &rasi )
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 bool hasDebilationRasi( const int &planet, const int &rasi )
@@ -833,20 +836,36 @@ double getAspectValue( const int &aspectingPlanet, const int &aspectedPlanet, co
 	if ( aspectingPlanet == OSATURN ) fsaturn = 1;
 
 	// Default 180
-	if ( len >= 150 && len < 180 ) { ret += rp; }
-	else if ( len >= 180 && len < 210 ) { ret += ap; }
+	if ( len >= 150 && len < 180 ) {
+		ret += rp;
+	}
+	else if ( len >= 180 && len < 210 ) {
+		ret += ap;
+	}
 
 	// Mars 90 + 210
-	if (( len >= 60 && len < 90 ) || ( len >= 180 && len < 210 )) { ret += fmars * rp; }
-	else if (( len >= 90 && len < 120 ) || ( len >= 210 && len < 240 )) { ret += fmars * ap; }
+	if (( len >= 60 && len < 90 ) || ( len >= 180 && len < 210 )) {
+		ret += fmars * rp;
+	}
+	else if (( len >= 90 && len < 120 ) || ( len >= 210 && len < 240 )) {
+		ret += fmars * ap;
+	}
 
 	// jupiter 120 + 240
-	if (( len >= 90 && len < 120 ) || ( len >= 210 && len < 240 )) { ret += fjupiter * rp; }
-	else if (( len >= 120 && len < 150 ) || ( len >= 240 && len < 270 )) { ret += fjupiter * ap; }
+	if (( len >= 90 && len < 120 ) || ( len >= 210 && len < 240 )) {
+		ret += fjupiter * rp;
+	}
+	else if (( len >= 120 && len < 150 ) || ( len >= 240 && len < 270 )) {
+		ret += fjupiter * ap;
+	}
 
 	// Saturn 60 + 270
-	if (( len >= 30 && len < 60 ) || ( len >= 240 && len < 270 )) { ret += fsaturn * rp; }
-	else if (( len >= 60 && len < 90 ) || ( len >= 270 && len < 300 )) { ret += fsaturn * ap; }
+	if (( len >= 30 && len < 60 ) || ( len >= 240 && len < 270 )) {
+		ret += fsaturn * rp;
+	}
+	else if (( len >= 60 && len < 90 ) || ( len >= 270 && len < 300 )) {
+		ret += fsaturn * ap;
+	}
 
 	assert( ret >= 0 && ret <= 60 );
 	return ret;
@@ -854,7 +873,7 @@ double getAspectValue( const int &aspectingPlanet, const int &aspectedPlanet, co
 
 /**************************************************************
 ***
-** 
+**
 ***
 ***************************************************************/
 double getAspectValue2( const int &planet1, const double &len1, const int &planet2, const double &len2 )
@@ -947,8 +966,8 @@ double getAspectValue2( const int &planet1, const double &len1, const int &plane
 ***************************************************************/
 double calcGeometricalMidpoint( const double &len1, const double &len2 )
 {
-		double dist = red_deg( len2 - len1 );
-		return( dist < 180 ? red_deg( len1 + dist/2 ) : red_deg( len2 + ( 360 - dist ) /2 ) );
+	double dist = red_deg( len2 - len1 );
+	return( dist < 180 ? red_deg( len1 + dist/2 ) : red_deg( len2 + ( 360 - dist ) /2 ) );
 }
 
 /**************************************************************
@@ -959,31 +978,31 @@ double calcGeometricalMidpoint( const double &len1, const double &len2 )
 double getDegrees4CircleType( const int &circle )
 {
 	assert( circle >= CIRCLE_TYPE_360 && circle <= CIRCLE_TYPE_5625 );
-	switch( circle )
+	switch ( circle )
 	{
-		case CIRCLE_TYPE_360:
-			return 360;
+	case CIRCLE_TYPE_360:
+		return 360;
 		break;
-		case CIRCLE_TYPE_180:
-			return 180;
+	case CIRCLE_TYPE_180:
+		return 180;
 		break;
-		case CIRCLE_TYPE_90:
-			return 90;
+	case CIRCLE_TYPE_90:
+		return 90;
 		break;
-		case CIRCLE_TYPE_45:
-			return 45;
+	case CIRCLE_TYPE_45:
+		return 45;
 		break;
-		case CIRCLE_TYPE_225:
-			return 22.5;
+	case CIRCLE_TYPE_225:
+		return 22.5;
 		break;
-		case CIRCLE_TYPE_1125:
-			return 11.25;
+	case CIRCLE_TYPE_1125:
+		return 11.25;
 		break;
-		case CIRCLE_TYPE_5625:
-			return 5.625;
+	case CIRCLE_TYPE_5625:
+		return 5.625;
 		break;
-		default:
-			assert( 0 );
+	default:
+		assert( 0 );
 		break;
 	}
 	return 0;
@@ -996,46 +1015,46 @@ double getDegrees4CircleType( const int &circle )
 **************************************************************************/
 int calcMonthLength( int monat, int jahr )
 {
-  if ( monat == 1 || monat == 3 || monat == 5 || monat == 7 || monat == 8 || monat == 10 || monat == 12 ) return 31;
-  if ( monat == 4 || monat == 6 || monat == 9 || monat == 11 ) return 30;
-  assert( monat == 2 );
-  if ( jahr % 4 )  return 28;
-  else
-  {
-    if (( jahr % 100 ) != 0 ) return 29;
-    if (( jahr % 100 ) == 0 && ( jahr % 400 ) != 0 ) return 28;
-    return 29;
-  }
+	if ( monat == 1 || monat == 3 || monat == 5 || monat == 7 || monat == 8 || monat == 10 || monat == 12 ) return 31;
+	if ( monat == 4 || monat == 6 || monat == 9 || monat == 11 ) return 30;
+	assert( monat == 2 );
+	if ( jahr % 4 )  return 28;
+	else
+	{
+		if (( jahr % 100 ) != 0 ) return 29;
+		if (( jahr % 100 ) == 0 && ( jahr % 400 ) != 0 ) return 28;
+		return 29;
+	}
 }
 
 /*****************************************************
 **
-**   getDegMinSecInts2 
+**   getDegMinSecInts2
 **
 ******************************************************/
 void getDegMinSecInts2( const double &len, int &deg, int &min, double &sec )
 {
 	double mylen = fabs( len );
-  deg = (int)mylen;
-  double mind = (double)((mylen - (double)deg ) * 60);
-  min = (int)mind;
-  sec = ( mind - (double)min ) * 60;
+	deg = (int)mylen;
+	double mind = (double)((mylen - (double)deg ) * 60);
+	min = (int)mind;
+	sec = ( mind - (double)min ) * 60;
 	if ( len < 0 ) deg *= -1;
 }
 
 /*****************************************************
 **
-**   getDegMinSecInts 
+**   getDegMinSecInts
 **
 ******************************************************/
 void getDegMinSecInts( const double &len, int &deg, int &min, int &sec )
 {
 #define ROUND_DEGREES .005 / 3600
 	double mylen = fabs( len ) + ROUND_DEGREES;
-  deg = (int)mylen;
-  double mind = (double)((mylen - (double)deg ) * 60);
-  min = (int)mind;
-  sec = (int)(( mind - (double)min ) * 60 );
+	deg = (int)mylen;
+	double mind = (double)((mylen - (double)deg ) * 60);
+	min = (int)mind;
+	sec = (int)(( mind - (double)min ) * 60 );
 	if ( len < 0 ) deg *= -1;
 }
 
