@@ -3,9 +3,9 @@
  *  for Vedic and western astrology.
 
  *  File       src/jyotish/textclient.cpp
- *  Release    6.0
- *  Author     M. Pettau
- *  Copyright  2003-2010 by the author
+ *  Release    7.0
+ *  Author     Martin Pettau
+ *  Copyright  2003-2012 by the author
 
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -50,12 +50,12 @@ Config *config;
 
 // wx2.9 workaround
 #ifdef  _WX29
-	#define wxT1( x ) x
-	#define wxEmptyString1 ""
+#define wxT1( x ) x
+#define wxEmptyString1 ""
 #else
-	#define wxT1( x ) wxT( x )
-	#define wxEmptyString1 wxEmptyString
-#endif 
+#define wxT1( x ) wxT( x )
+#define wxEmptyString1 wxEmptyString
+#endif
 
 static const wxCmdLineEntryDesc cmdLineDesc[] =
 {
@@ -98,10 +98,10 @@ static const wxCmdLineEntryDesc cmdLineDesc[] =
 	{ wxCMD_LINE_SWITCH, wxEmptyString1, wxT1("astronomical"), wxT1("Show astronomical data") },
 	{ wxCMD_LINE_SWITCH, wxEmptyString1, wxT1("arabic"), wxT1("Show Arabic parts") },
 	{ wxCMD_LINE_OPTION, wxEmptyString1, wxT1("printsummary"), wxT1("Show printout summary nb #n (header of PDF output)"), wxCMD_LINE_VAL_NUMBER },
-// start shadbalahook
-	{ wxCMD_LINE_SWITCH, wxEmptyString1, wxT1("shadbala"),   wxT1("Show Shad Bala") },
-// end shadbalahook
+
+
 	{ wxCMD_LINE_OPTION, wxEmptyString1, wxT1("dasas"), wxT1("Show Dasa nb #n (short form)"), wxCMD_LINE_VAL_NUMBER },
+	{ wxCMD_LINE_OPTION, wxEmptyString1, wxT1("dasas-compact"), wxT1("Show Dasa nb #n (compact form)"), wxCMD_LINE_VAL_NUMBER },
 	{ wxCMD_LINE_OPTION, wxEmptyString1, wxT1("dasas-long"), wxT1("Show Dasa nb #n (short form)"), wxCMD_LINE_VAL_NUMBER },
 	{ wxCMD_LINE_SWITCH, wxEmptyString1, wxT1("aspects"),   wxT1("Show aspects\n") },
 
@@ -215,22 +215,22 @@ protected:
 		formatter->getDateIntsFromJD( dataset.getJD(), day, month, year );
 		e.prepareMonth( month, year, 0 );
 
-		switch( type )
+		switch ( type )
 		{
-			case 0:
-				e.writeDefaultEphemeris( writer );
+		case 0:
+			e.writeDefaultEphemeris( writer );
 			break;
-			case 1:
-				e.writeDetails( writer );
+		case 1:
+			e.writeDetails( writer );
 			break;
-			case 2:
-				e.writeIngress( writer );
+		case 2:
+			e.writeIngress( writer );
 			break;
-			case 3:
-				e.writeLunar( writer );
+		case 3:
+			e.writeLunar( writer );
 			break;
-			case 4:
-				e.writeKp( writer, dasa );
+		case 4:
+			e.writeKp( writer, dasa );
 			break;
 		}
 		writecount++;
@@ -261,6 +261,7 @@ protected:
 		writecount++;
 	}
 
+
 	/**************************************************************
 	***
 	**    MaitreyaTextclient   ---   showTajaka
@@ -308,7 +309,7 @@ protected:
 		TransitExpert expert( h, chartprops );
 		expert.setTransitMode( transitmode );
 		expert.setTransitDate( jd );
-		expert.update();	
+		expert.update();
 		expert.writeTransitData( writer, false );
 
 		AspectExpert aexpert( h );
@@ -327,13 +328,13 @@ protected:
 		UranianConfig cfg;
 		UranianExpert expert( chartprops, writer, cfg );
 
-		switch( type )
+		switch ( type )
 		{
-			case 1:
-				expert.writeYearlyPreviewAnalysis( h, year );
+		case 1:
+			expert.writeYearlyPreviewAnalysis( h, year );
 			break;
-			default:
-				expert.writeRadixAnalysis( h );
+		default:
+			expert.writeRadixAnalysis( h );
 			break;
 		}
 		writecount++;
@@ -382,14 +383,14 @@ protected:
 		{
 			year = month = day = hour = minute = second = 0;
 			wxStringTokenizer t( s, wxT( " -:" ));
-			
+
 			if ( t.HasMoreTokens()) year = getIntFromString( t.GetNextToken());
 			if ( t.HasMoreTokens()) month = getIntFromString( t.GetNextToken());
 			if ( t.HasMoreTokens()) day = getIntFromString( t.GetNextToken());
 			if ( t.HasMoreTokens()) hour = getIntFromString( t.GetNextToken());
 			if ( t.HasMoreTokens()) minute = getIntFromString( t.GetNextToken());
 			if ( t.HasMoreTokens()) second = getIntFromString( t.GetNextToken());
-			
+
 			time = second + 60 * minute + 3600 * hour;
 			time /= 3600;
 
@@ -411,14 +412,14 @@ protected:
 		ldeg = lmin = bdeg = bmin = dst = 0;
 		lon = lat = tz = 0;
 		wxStringTokenizer t( s, wxT( " :" ));
-		
+
 		if ( t.HasMoreTokens()) name = t.GetNextToken();
 		if ( t.HasMoreTokens()) ldeg = getIntFromString( t.GetNextToken());
 		if ( t.HasMoreTokens()) lmin = getIntFromString( t.GetNextToken());
 		if ( t.HasMoreTokens()) bdeg = getIntFromString( t.GetNextToken());
 		if ( t.HasMoreTokens()) bmin = getIntFromString( t.GetNextToken());
 		if ( t.HasMoreTokens()) tz = myatof( t.GetNextToken());
-		
+
 		lon = lmin + 60 * ldeg;
 		lon /= 60;
 		lat = bmin + 60 * bdeg;
@@ -490,7 +491,7 @@ void MaitreyaTextclient::run( int argc, wxChar **argv )
 	h->setLocation( *dataset.getLocation());
 
 	texthelper = new TextHelper( h, chartprops, writer );
-	if ( parser.Found( wxT( "f" ), &s  ))
+	if ( parser.Found( wxT( "file" ), &s  ))
 	{
 		if ( ! h->openFile( s ))
 		{
@@ -500,10 +501,10 @@ void MaitreyaTextclient::run( int argc, wxChar **argv )
 	}
 	h->update();
 
-	if ( parser.Found( wxT( "p" )))               showPlanets();
+	if ( parser.Found( wxT( "planet-list" )))     showPlanets();
 	if ( parser.Found( wxT( "base-data" )))       showTextAnalysis( TEXT_MODE_BASE );
-	if ( parser.Found( wxT( "w" )))               showTextAnalysis( TEXT_MODE_WESTERN_PLANETS );
-	if ( parser.Found( wxT( "v" )))               showTextAnalysis( TEXT_MODE_VEDIC_PLANETS );
+	if ( parser.Found( wxT( "westernplanets" )))  showTextAnalysis( TEXT_MODE_WESTERN_PLANETS );
+	if ( parser.Found( wxT( "vedicplanets" )))    showTextAnalysis( TEXT_MODE_VEDIC_PLANETS );
 	if ( parser.Found( wxT( "bhavas" )))          showTextAnalysis( TEXT_MODE_BHAVA );
 	if ( parser.Found( wxT( "kp" )))              showTextAnalysis( TEXT_MODE_KP );
 	if ( parser.Found( wxT( "nakshatras" )))      showTextAnalysis( TEXT_MODE_NAKSHATRA );
@@ -513,13 +514,14 @@ void MaitreyaTextclient::run( int argc, wxChar **argv )
 	if ( parser.Found( wxT( "arabic" )))          showTextAnalysis( TEXT_MODE_ARABICPARTS );
 	if ( parser.Found( wxT( "astronomical" )))    showTextAnalysis( TEXT_MODE_ASTRONOMICAL );
 	if ( parser.Found( wxT( "dasas" ), &a ))      showTextAnalysis( TEXT_MODE_DASA, 0, (int)a );
+	if ( parser.Found( wxT( "dasas-compact" ), &a )) showTextAnalysis( TEXT_MODE_DASACOMPACT, 0, (int)a );
 	if ( parser.Found( wxT( "dasas-long" ), &a )) showTextAnalysis( TEXT_MODE_DASALONG, 0, (int)a );
-	if ( parser.Found( wxT( "aspects" )))         showTextAnalysis( 
-		chartprops->isVedic() ? TEXT_MODE_VEDIC_ASPECTARIUM : TEXT_MODE_WESTERN_ASPECTARIUM );
+	if ( parser.Found( wxT( "aspects" )))         showTextAnalysis(
+		    chartprops->isVedic() ? TEXT_MODE_VEDIC_ASPECTARIUM : TEXT_MODE_WESTERN_ASPECTARIUM );
 
 	if ( parser.Found( wxT( "printsummary" ), &a )) showPrintoutSummary( (int)a );
-	if ( parser.Found( wxT( "yogas" ))) showYogas();
 
+	if ( parser.Found( wxT( "yogas" )))        showYogas();
 	if ( parser.Found( wxT( "transits" )))     showTransits( TRANSIT_MODE_TRANSIT, tjd );
 	if ( parser.Found( wxT( "solar-arc" )))    showTransits( TRANSIT_MODE_SOLAR_ARC, tjd );
 	if ( parser.Found( wxT( "directions" )))   showTransits( TRANSIT_MODE_DIRECTION, tjd );
