@@ -103,11 +103,12 @@ wxPdfDocument::CalculatePageSize(wxPaperSize format)
   {
     paperType = printPaperDatabase->FindPaperType(wxPAPER_A4);
   }
+  wxSize paperSize = paperType->GetSize();
   if (deletePaperDatabase)
   {
     delete printPaperDatabase;
   }
-  return paperType->GetSize();
+  return paperSize;
 }
 
 wxSize
@@ -123,6 +124,7 @@ wxPdfDocument::Initialize(int orientation)
 {
   // Allocate arrays
   m_currentFont = NULL;
+  m_buffer = new wxMemoryOutputStream();
 
   m_page       = 0;
   m_n          = 2;
@@ -247,6 +249,8 @@ wxPdfDocument::Initialize(int orientation)
   m_currentParser = NULL;
   m_currentSource = wxEmptyString;
 
+  m_translate = false;
+
   m_zapfdingbats = 0;
 }
 
@@ -459,6 +463,11 @@ wxPdfDocument::~wxPdfDocument()
   {
     delete m_encryptor;
   }
+
+  if (m_buffer != NULL)
+  {
+    delete m_buffer;
+  }
 }
 
 // --- Public methods
@@ -783,6 +792,12 @@ wxPdfDocument::SetFont(const wxFont& font)
 void
 wxPdfDocument::SetFontSize(double size)
 {
+  if (m_currentFont == NULL)
+  {
+    wxLogError(wxString(wxT("wxPdfDocument::SetFontSize: ")) +
+               wxString(_("No font selected.")));
+    return;
+  }
   // Set font size in points
   if (m_fontSizePt == size)
   {
@@ -797,9 +812,28 @@ wxPdfDocument::SetFontSize(double size)
   }
 }
 
+wxPdfFont
+wxPdfDocument::GetCurrentFont() const
+{
+  if (m_currentFont == NULL)
+  {
+    wxLogError(wxString(wxT("wxPdfDocument::GetCurrentFont: ")) +
+               wxString(_("No font selected.")));
+    return wxPdfFont();
+  }
+  return m_currentFont->GetUserFont();
+}
+
 const wxPdfFontDescription&
 wxPdfDocument::GetFontDescription() const
 {
+  if (m_currentFont == NULL)
+  {
+    wxLogError(wxString(wxT("wxPdfDocument::SetFontSize: ")) +
+               wxString(_("No font selected.")));
+    static wxPdfFontDescription dummy;
+    return dummy;
+  }
   return m_currentFont->GetDescription();
 }
 
@@ -844,13 +878,20 @@ wxPdfDocument::GetFontStyles() const
 }
 
 double
-wxPdfDocument::GetFontSize()
+wxPdfDocument::GetFontSize() const
 {
   return m_fontSizePt;
 }
 
 double
 wxPdfDocument::GetStringWidth(const wxString& s)
+{
+  wxString voText = ApplyVisualOrdering(s);
+  return DoGetStringWidth(voText);
+}
+
+double
+wxPdfDocument::DoGetStringWidth(const wxString& s)
 {
   double w = 0;
   if (m_currentFont != 0)
@@ -864,6 +905,8 @@ void
 wxPdfDocument::Text(double x, double y, const wxString& txt)
 {
   // Output a string
+  wxString voText = ApplyVisualOrdering(txt);
+
   if (m_colourFlag)
   {
     Out("q ", false);
@@ -883,13 +926,13 @@ wxPdfDocument::Text(double x, double y, const wxString& txt)
              wxPdfUtility::Double2String(y*m_k,2) + wxString(wxT(" Td ")), false);
   }
   OutAscii(wxString::Format(wxT("%d Tr "), m_textRenderMode), false);
-  ShowText(txt);
+  ShowText(voText);
   Out("ET", false);
 
-  if ((m_decoration & wxPDF_FONTSTYLE_DECORATION_MASK) && txt.Length() > 0)
+  if ((m_decoration & wxPDF_FONTSTYLE_DECORATION_MASK) && voText.Length() > 0)
   {
     Out(" ", false);
-    OutAscii(DoDecoration(x, y, txt), false);
+    OutAscii(DoDecoration(x, y, voText), false);
   }
 
   if (m_colourFlag)
@@ -903,10 +946,17 @@ void
 wxPdfDocument::RotatedText(double x, double y, const wxString& txt, double angle)
 {
   // Text rotated around its origin
-  StartTransform();
-  Rotate(angle, x, y);
-  Text(x, y, txt);
-  StopTransform();
+  if (angle == 0)
+  {
+    Text(x, y, txt);
+  }
+  else
+  {
+    StartTransform();
+    Rotate(angle, x, y);
+    Text(x, y, txt);
+    StopTransform();
+  }
 }
 
 bool
@@ -918,6 +968,13 @@ wxPdfDocument::AcceptPageBreak()
 
 void
 wxPdfDocument::Cell(double w, double h, const wxString& txt, int border, int ln, int align, int fill, const wxPdfLink& link)
+{
+  wxString voText = ApplyVisualOrdering(txt);
+  DoCell(w, h, voText, border, ln, align, fill, link);
+}
+
+void
+wxPdfDocument::DoCell(double w, double h, const wxString& txt, int border, int ln, int align, int fill, const wxPdfLink& link)
 {
   // Output a cell
   double x, y;
@@ -1010,7 +1067,7 @@ wxPdfDocument::Cell(double w, double h, const wxString& txt, int border, int ln,
   
   if (txt.Length() > 0)
   {
-    double width = GetStringWidth(txt);
+    double width = DoGetStringWidth(txt);
     double dx;
     if (align == wxPDF_ALIGN_RIGHT)
     {
@@ -1092,7 +1149,7 @@ wxPdfDocument::MultiCell(double w, double h, const wxString& txt, int border, in
   }
 
   double wmax = (w - 2 * m_cMargin);
-  wxString s = txt;
+  wxString s = ApplyVisualOrdering(txt);
   s.Replace(wxT("\r"),wxT("")); // remove carriage returns
   int nb = (int) s.Length();
   if (nb > 0 && s[nb-1] == wxT('\n'))
@@ -1143,7 +1200,7 @@ wxPdfDocument::MultiCell(double w, double h, const wxString& txt, int border, in
         m_ws = 0;
         Out("0 Tw");
       }
-      Cell(w,h,s.SubString(j,i-1),b,2,align,fill);
+      DoCell(w,h,s.SubString(j,i-1),b,2,align,fill);
       i++;
       sep = -1;
       j = i;
@@ -1166,7 +1223,7 @@ wxPdfDocument::MultiCell(double w, double h, const wxString& txt, int border, in
       ls = len;
       ns++;
     }
-    len = GetStringWidth(s.SubString(j, i));
+    len = DoGetStringWidth(s.SubString(j, i));
 
     if (len > wmax)
     {
@@ -1182,7 +1239,7 @@ wxPdfDocument::MultiCell(double w, double h, const wxString& txt, int border, in
           m_ws=0;
           Out("0 Tw");
         }
-        Cell(w,h,s.SubString(j,i-1),b,2,align,fill);
+        DoCell(w,h,s.SubString(j,i-1),b,2,align,fill);
       }
       else
       {
@@ -1191,7 +1248,7 @@ wxPdfDocument::MultiCell(double w, double h, const wxString& txt, int border, in
           m_ws = (ns > 1) ? (wmax - ls)/(ns-1) : 0;
           OutAscii(wxPdfUtility::Double2String(m_ws*m_k,3)+wxString(wxT(" Tw")));
         }
-        Cell(w,h,s.SubString(j,sep-1),b,2,align,fill);
+        DoCell(w,h,s.SubString(j,sep-1),b,2,align,fill);
         i = sep + 1;
       }
       sep = -1;
@@ -1223,7 +1280,7 @@ wxPdfDocument::MultiCell(double w, double h, const wxString& txt, int border, in
   {
     b = b | wxPDF_BORDER_BOTTOM;
   }
-  Cell(w,h,s.SubString(j,i-1),b,2,align,fill);
+  DoCell(w,h,s.SubString(j,i-1),b,2,align,fill);
   m_x = m_lMargin;
   return i;
 }
@@ -1270,7 +1327,7 @@ wxPdfDocument::LineCount(double w, const wxString& txt)
     {
       sep = i;
     }
-    len = GetStringWidth(s.SubString(j, i));
+    len = DoGetStringWidth(s.SubString(j, i));
 
     if (len > wmax)
     {
@@ -1350,14 +1407,15 @@ void
 wxPdfDocument::WriteCell(double h, const wxString& txt, int border, int fill, const wxPdfLink& link)
 {
   // Output text in flowing mode
-  wxString s = txt;
+  wxString s = ApplyVisualOrdering(txt);
+
   s.Replace(wxT("\r"),wxT("")); // remove carriage returns
   int nb = (int) s.Length();
 
   // handle single space character
   if ((nb == 1) && s[0] == wxT(' '))
   {
-    m_x += GetStringWidth(s);
+    m_x += DoGetStringWidth(s);
     return;
   }
 
@@ -1380,7 +1438,7 @@ wxPdfDocument::WriteCell(double h, const wxString& txt, int border, int fill, co
     if (c == wxT('\n'))
     {
       // Explicit line break
-      Cell(w, h, s.SubString(j,i-1), border, 2, wxPDF_ALIGN_LEFT, fill, link);
+      DoCell(w, h, s.SubString(j,i-1), border, 2, wxPDF_ALIGN_LEFT, fill, link);
       i++;
       sep = -1;
       j = i;
@@ -1398,7 +1456,7 @@ wxPdfDocument::WriteCell(double h, const wxString& txt, int border, int fill, co
     {
       sep = i;
     }
-    len = GetStringWidth(s.SubString(j, i));
+    len = DoGetStringWidth(s.SubString(j, i));
     if (len > wmax)
     {
       // Automatic line break
@@ -1426,11 +1484,11 @@ wxPdfDocument::WriteCell(double h, const wxString& txt, int border, int fill, co
         {
           i++;
         }
-        Cell(w, h,s.SubString(j, i-1), border, 2, wxPDF_ALIGN_LEFT, fill, link);
+        DoCell(w, h,s.SubString(j, i-1), border, 2, wxPDF_ALIGN_LEFT, fill, link);
       }
       else
       {
-        Cell(w, h, s.SubString(j, sep-1), border, 2, wxPDF_ALIGN_LEFT, fill, link);
+        DoCell(w, h, s.SubString(j, sep-1), border, 2, wxPDF_ALIGN_LEFT, fill, link);
         i = sep + 1;
       }
       sep = -1;
@@ -1452,7 +1510,7 @@ wxPdfDocument::WriteCell(double h, const wxString& txt, int border, int fill, co
   // Last chunk
   if (i != j)
   {
-    Cell(len, h, s.SubString(j,i-1), border, 0, wxPDF_ALIGN_LEFT, fill, link);
+    DoCell(len, h, s.SubString(j,i-1), border, 0, wxPDF_ALIGN_LEFT, fill, link);
   }
 
   // Following statement was in PHP code, but seems to be in error.
@@ -1598,6 +1656,14 @@ wxPdfDocument::Image(const wxString& name, const wxImage& img, double x, double
           return false;
         }
       }
+      else if (tempImage.HasMask() && maskImage <= 0)
+      {
+        // Extract the mask
+        wxImage mask = tempImage.ConvertToMono(tempImage.GetMaskRed(), tempImage.GetMaskGreen(), tempImage.GetMaskBlue());
+        // Invert the mask
+        mask = mask.ConvertToMono(0, 0, 0);
+        maskImage = ImageMask(name+wxString(wxT(".mask")), mask);
+      }
       // First use of image, get info
       tempImage.SetMask(false);
       int i = (int) (*m_images).size() + 1;
@@ -1745,8 +1811,8 @@ wxPdfDocument::ImageMask(const wxString& name, const wxImage& img)
       }
       else
       {
-        tempImage = img.Copy();
-        tempImage.SetOption(wxIMAGE_OPTION_PNG_FORMAT, wxPNG_TYPE_GREY);
+        tempImage = img.ConvertToGreyscale();
+        tempImage.SetOption(wxIMAGE_OPTION_PNG_FORMAT, wxPNG_TYPE_GREY_RED);
       }
       tempImage.SetMask(false);
       // First use of image, get info
@@ -1853,20 +1919,31 @@ void
 wxPdfDocument::SaveAsFile(const wxString& name)
 {
   wxString fileName = name;
-  // Finish document if necessary
-  if (m_state < 3)
-  {
-    Close();
-  }
   // Normalize parameters
   if(fileName.Length() == 0)
   {
     fileName = wxT("doc.pdf");
   }
-  // Save to local file
+
   wxFileOutputStream outfile(fileName);
-  wxMemoryInputStream tmp(m_buffer);
-  outfile.Write(tmp);
+
+  // Finish document if necessary
+  if (m_state < 3)
+  {
+    if (m_buffer != NULL)
+    {
+      delete m_buffer;
+    }
+    m_buffer = &outfile;
+    Close();
+    m_buffer = NULL;
+  }
+  else
+  {
+    // Save to local file
+    wxMemoryInputStream tmp(*((wxMemoryOutputStream*) m_buffer));
+    outfile.Write(tmp);
+  }
   outfile.Close();
 }
 
@@ -1878,7 +1955,7 @@ wxPdfDocument::CloseAndGetBuffer()
     Close();
   }
   
-  return m_buffer;
+  return *((wxMemoryOutputStream*) m_buffer);
 }
 
 void
