@@ -28,9 +28,11 @@
 #include <wx/filefn.h>
 #include <wx/filename.h>
 #include <wx/filesys.h>
+#include <wx/font.h>
 #include <wx/thread.h>
 #include <wx/xml/xml.h>
 
+#include "wx/pdfencoding.h"
 #include "wx/pdffontmanager.h"
 #include "wx/pdffontdata.h"
 #include "wx/pdffontdatacore.h"
@@ -47,7 +49,16 @@
 #elif defined(__WXGTK20__)
 // TODO: Would testing for __WXGTK__ be sufficient?
   #include <fontconfig/fontconfig.h>
+
+  // Define some FontConfig symbols if they are missing
+  #ifndef FC_WEIGHT_BOOK
+    #define FC_WEIGHT_BOOK 75
+  #endif
+  #ifndef FC_FULLNAME
+    #define FC_FULLNAME "fullname"
+  #endif
 #elif defined(__WXMAC__)
+  #include "wx/pdffontmacosx.h"
 #else
 #endif
 
@@ -56,6 +67,9 @@
 // Include core font data
 #include "pdfcorefontdata.inc"
 
+// Include CJK font data
+#include "pdfcjkfontdata.inc"
+
 // wxPdfFontManager is a singleton.
 // Critical sections are used to make access to it thread safe if necessary.
 #if wxUSE_THREADS
@@ -92,6 +106,8 @@ wxPdfFontData::IncrementRefCount()
   return ++m_refCount;
 }
 
+// --- Font List
+
 int
 wxPdfFontData::DecrementRefCount()
 {
@@ -164,6 +180,101 @@ private:
   wxPdfFontData* m_fontData;
 };
 
+// --- Encoding checker
+
+// Include codepage data
+#include "pdfcodepagedata.inc"
+
+// Class representing a font encoding checker
+class WXDLLIMPEXP_PDFDOC wxPdfCodepageChecker : public wxPdfEncodingChecker
+{
+public:
+  /// Default constructor
+  wxPdfCodepageChecker(const wxString& encoding, int tableSize, const wxUniRangeDesc* cpTable)
+  {
+    m_encoding = encoding;
+    m_tableSize = tableSize;
+    m_cpTable = cpTable;
+  }
+
+  /// Destructor
+  virtual ~wxPdfCodepageChecker()
+  {
+  }
+
+  /// Get the name of the encoding
+  /**
+  * \return the name of the encoding
+  */
+  virtual bool IsIncluded(wxUint32 unicode) const
+  {
+    bool isIncluded = false;
+    if (unicode < 0x00010000)
+    {
+      unsigned short code = unicode & 0x0000ffff;
+      int lb = 0;
+      int hb = m_tableSize - 1;
+      int mid = (lb + hb) / 2;
+      while (mid != lb)
+      {
+        if (code >= m_cpTable[mid].uni1)
+        {
+          lb = mid;
+        }
+        else
+        {
+          hb = mid;
+        }
+        mid = (lb + hb) / 2;
+      }
+      isIncluded = (code <= m_cpTable[mid].uni2);
+    }
+    return isIncluded;
+  }
+
+private:
+  int                   m_tableSize;        ///< Size of the mapping table
+  const wxUniRangeDesc* m_cpTable;          ///< Table for code page
+};
+
+// Class representing a font encoding checker
+class WXDLLIMPEXP_PDFDOC wxPdfCjkChecker : public wxPdfEncodingChecker
+{
+public:
+  /// Default constructor
+  wxPdfCjkChecker(const wxString& encoding, const unsigned char* cjkTable)
+  {
+    m_encoding = encoding;
+    m_cjkTable = cjkTable;
+  }
+
+  /// Destructor
+  virtual ~wxPdfCjkChecker()
+  {
+  }
+
+  /// Check whether a given Unicode character is included in the encoding
+  /**
+  * \return TRUE if the Unicode character is included, FALSE otherwise
+  */
+  virtual bool IsIncluded(wxUint32 unicode) const
+  {
+    bool isIncluded = false;
+    if (unicode < 0x00010000)
+  {
+    int charPos = unicode / 8;
+    unsigned char bitPos = 1 << (7 - (unicode % 8));
+    isIncluded = ((m_cjkTable[charPos] & bitPos) != 0);
+  }
+  return isIncluded;
+  }
+
+private:
+  const unsigned char* m_cjkTable;         ///< Table for CJK encodings
+};
+
+// --- Font Manager Base
+
 WX_DEFINE_ARRAY_PTR(wxPdfFontListEntry*, wxPdfFontList);
 
 /// Hashmap class for mapping font families
@@ -175,6 +286,12 @@ WX_DECLARE_STRING_HASH_MAP(int, wxPdfFontNameMap);
 /// Hashmap class for mapping alias names to family names
 WX_DECLARE_STRING_HASH_MAP(wxString, wxPdfFontAliasMap);
 
+/// Hashmap class for mapping encodings
+WX_DECLARE_STRING_HASH_MAP(wxPdfEncoding*, wxPdfEncodingMap);
+
+/// Hashmap class for mapping encoding checkers
+WX_DECLARE_STRING_HASH_MAP(wxPdfEncodingChecker*, wxPdfEncodingCheckerMap);
+
 class wxPdfFontManagerBase
 {
 public:
@@ -222,6 +339,10 @@ public:
 
   bool InitializeFontData(const wxPdfFont& font);
 
+  bool RegisterEncoding(const wxPdfEncoding& encoding);
+
+  const wxPdfEncoding* GetEncoding(const wxString& encodingName);
+
   bool FindFile(const wxString& fileName, wxString& fullFileName) const;
 
   static wxString ConvertStyleToString(int fontStyle);
@@ -229,6 +350,16 @@ public:
 private:
   void InitializeCoreFonts();
 
+#if wxUSE_UNICODE
+  void InitializeCjkFonts();
+#endif
+
+  void InitializeEncodingChecker();
+
+  bool RegisterEncoding(const wxString& encoding);
+
+  void SetFontBaseEncoding(wxPdfFontData* fontData);
+
   bool RegisterFontCJK(const wxString& fontFileName, const wxString& fontStyle, const wxString& alias);
 
   wxPdfFontData* LoadFontFromXML(const wxString& fontFileName);
@@ -248,6 +379,9 @@ private:
 
   bool               m_defaultEmbed;
   bool               m_defaultSubset;
+  
+  wxPdfEncodingMap*        m_encodingMap;
+  wxPdfEncodingCheckerMap* m_encodingCheckerMap;
 };
 
 #include "wxmemdbg.h"
@@ -266,7 +400,13 @@ wxPdfFontManagerBase::wxPdfFontManagerBase()
     m_searchPaths.Add(wxT("fonts"));
     m_searchPaths.AddEnvList(wxT("WXPDF_FONTPATH"));
   }
+  m_encodingMap = new wxPdfEncodingMap();
+  m_encodingCheckerMap = new wxPdfEncodingCheckerMap();
+  InitializeEncodingChecker();
   InitializeCoreFonts();
+#if wxUSE_UNICODE
+  InitializeCjkFonts();
+#endif
 }
   
 wxPdfFontManagerBase::~wxPdfFontManagerBase()
@@ -284,6 +424,23 @@ wxPdfFontManagerBase::~wxPdfFontManagerBase()
     delete m_fontList[j];
   }
   m_fontList.clear();
+  
+  wxPdfEncodingMap::iterator encoding;
+  for (encoding = m_encodingMap->begin();
+       encoding != m_encodingMap->end(); ++encoding)
+  {
+    wxPdfEncoding* foundEncoding = encoding->second;
+    delete foundEncoding;
+  }
+  delete m_encodingMap;
+
+  wxPdfEncodingCheckerMap::iterator checker;
+  for (checker = m_encodingCheckerMap->begin();
+       checker != m_encodingCheckerMap->end(); ++checker)
+  {
+    delete checker->second;
+  }
+  delete m_encodingCheckerMap;
 }
 
 void
@@ -412,8 +569,8 @@ wxPdfFontManagerBase::RegisterFont(const wxString& fontFileName, const wxString&
         if (!AddFont(fontData, font))
         {
           delete fontData;
-          wxLogWarning(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
-                       wxString::Format(_("Font file '%s' already registered."), fontFileName.c_str()));
+          wxLogDebug(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
+                     wxString::Format(_("Font file '%s' already registered."), fontFileName.c_str()));
         }
       }
 #else
@@ -421,7 +578,7 @@ wxPdfFontManagerBase::RegisterFont(const wxString& fontFileName, const wxString&
                  wxString::Format(_("Format of font file '%s' not supported."), fontFileName.c_str()));
 #endif
     }
-    else if (/* ext.IsSameAs(wxT("pfa")) || */ ext.IsSameAs(wxT("pfb")))
+    else if (/* ext.IsSameAs(wxT("pfa")) || */ ext.IsSameAs(wxT("pfb")) || ext.IsEmpty())
     {
       // TODO: allow Type1 fonts in PFA format (this requires encoding the binary section)
 #if wxUSE_UNICODE
@@ -431,11 +588,12 @@ wxPdfFontManagerBase::RegisterFont(const wxString& fontFileName, const wxString&
       if (fontData != NULL)
       {
         fontData->SetAlias(aliasName);
+        SetFontBaseEncoding(fontData);
         if (!AddFont(fontData, font))
         {
           delete fontData;
-          wxLogWarning(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
-                       wxString::Format(_("Font file '%s' already registered."), fontFileName.c_str()));
+          wxLogDebug(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
+                     wxString::Format(_("Font file '%s' already registered."), fontFileName.c_str()));
         }
       }
 #else
@@ -450,11 +608,12 @@ wxPdfFontManagerBase::RegisterFont(const wxString& fontFileName, const wxString&
       if (fontData != NULL)
       {
         fontData->SetAlias(aliasName);
+        SetFontBaseEncoding(fontData);
         if (!AddFont(fontData, font))
         {
           delete fontData;
-          wxLogWarning(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
-                       wxString::Format(_("Font file '%s' already registered."), fontFileName.c_str()));
+          wxLogDebug(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
+                     wxString::Format(_("Font file '%s' already registered."), fontFileName.c_str()));
         }
       }
     }
@@ -485,8 +644,9 @@ wxPdfFontManagerBase::RegisterFont(const wxFont& font, const wxString& aliasName
     fontData->SetAlias(aliasName);
     if (!AddFont(fontData, regFont))
     {
-      wxLogWarning(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
-                   wxString::Format(_("wxFont '%s' already registered."), font.GetFaceName().c_str()));
+      delete fontData;
+      wxLogDebug(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
+                 wxString::Format(_("wxFont '%s' already registered."), font.GetFaceName().c_str()));
     }
   }
 #elif defined(__WXGTK20__)
@@ -609,8 +769,59 @@ wxPdfFontManagerBase::RegisterFont(const wxFont& font, const wxString& aliasName
                  wxString::Format(_("Font file name not found for wxFont '%s'."), fontDesc.c_str()));
   }
 #elif defined(__WXMAC__)
+#if wxPDFMACOSX_HAS_CORE_TEXT
+  wxPdfFontParserTrueType fontParser;
+  wxPdfFontData* fontData = fontParser.IdentifyFont(font);
+  if (fontData != NULL)
+  {
+    fontData->SetAlias(aliasName);
+    if (!AddFont(fontData, regFont))
+    {
+      delete fontData;
+      wxLogDebug(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
+                 wxString::Format(_("wxFont '%s' already registered."), font.GetFaceName().c_str()));
+    }
+  }
+#elif wxPDFMACOSX_HAS_ATSU_TEXT
+  wxString fontFileName = wxEmptyString;
+
+#if wxCHECK_VERSION(2,9,0)
+  // wxWidgets 2.9.x or higher
+#if wxOSX_USE_ATSU_TEXT
+  wxUint32 atsuFontID = font.MacGetATSUFontID();
+#endif
+#else // wxWidgets 2.8.x
+#ifdef __WXMAC_CLASSIC__
+  wxUint32 atsuFontID = font.GetMacATSUFontID();
+#else
+  wxUint32 atsuFontID = font.MacGetATSUFontID();
+#endif
+#endif
+
+  FSSpec fileSpecification;
+  FSRef fileReference;
+  if (::ATSFontGetFileSpecification(::FMGetATSFontRefFromFont(atsuFontID), &fileSpecification) == noErr)
+  {
+    if (::FSpMakeFSRef(&fileSpecification, &fileReference) == noErr)
+    {
+      fontFileName = wxMacFSRefToPath(&fileReference);
+    }
+  }
+
+  if (!fontFileName.IsEmpty())
+  {
+    regFont = RegisterFont(fontFileName, aliasName, 0);
+  }
+  else
+  {
+    wxString fontDesc = font.GetNativeFontInfoUserDesc();
+    wxLogWarning(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
+                 wxString::Format(_("Font file name not found for wxFont '%s'."), fontDesc.c_str()));
+  }
+#else
   wxLogError(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
-             wxString(_("Method 'RegisterFont' for wxFont instances is not yet available for platform WXMAC.")));
+             wxString(_("Method 'RegisterFont' for wxFont instances is not available for platform WXMAC.")));
+#endif
 #else
   wxLogError(wxString(wxT("wxPdfFontManagerBase::RegisterFont: ")) +
              wxString(_("Method 'RegisterFont' for wxFont instances is not available for your platform.")));
@@ -659,24 +870,34 @@ bool
 wxPdfFontManagerBase::RegisterFontCJK(const wxString& family)
 {
   bool ok = false;
-  wxString fontFileName = family.Lower() + wxString(wxT(".xml"));
-  wxString fullFontFileName;
-  if (FindFile(fontFileName, fullFontFileName))
+  wxPdfFontFamilyMap::const_iterator familyIter = m_fontFamilyMap.find(family.Lower());
+  if (familyIter == m_fontFamilyMap.end())
   {
-    ok = RegisterFontCJK(fullFontFileName, wxT(""), family);
-    if (ok)
+    wxString fontFileName = family.Lower() + wxString(wxT(".xml"));
+    wxString fullFontFileName;
+    if (FindFile(fontFileName, fullFontFileName))
     {
-      // Add all available styles (bold, italic and bold-italic)
-      // For all styles the same font metric file is used.
-      RegisterFontCJK(fullFontFileName, wxT(",Bold"), family);
-      RegisterFontCJK(fullFontFileName, wxT(",Italic"), family);
-      RegisterFontCJK(fullFontFileName, wxT(",BoldItalic"), family);
+      ok = RegisterFontCJK(fullFontFileName, wxT(""), family);
+      if (ok)
+      {
+        // Add all available styles (bold, italic and bold-italic)
+        // For all styles the same font metric file is used.
+        RegisterFontCJK(fullFontFileName, wxT(",Bold"), family);
+        RegisterFontCJK(fullFontFileName, wxT(",Italic"), family);
+        RegisterFontCJK(fullFontFileName, wxT(",BoldItalic"), family);
+      }
+    }
+    else
+    {
+      wxLogError(wxString(wxT("wxPdfFontManagerBase::RegisterFontCJK: ")) +
+                 wxString::Format(_("CJK Font file '%s' for CJK family '%s' does not exist or is not readable."), fontFileName.c_str(), family.c_str()));
     }
   }
   else
   {
-    wxLogError(wxString(wxT("wxPdfFontManagerBase::RegisterFontCJK: ")) +
-               wxString::Format(_("CJK Font file '%s' for CJK family '%s' does not exist or is not readable."), fontFileName.c_str(), family.c_str()));
+    wxLogDebug(wxString(wxT("wxPdfFontManagerBase::RegisterFontCJK: ")) +
+               wxString::Format(_("CJK family '%s' already registered."), family.c_str()));
+    ok = true;
   }
   return ok;
 }
@@ -700,9 +921,9 @@ wxPdfFontManagerBase::RegisterSystemFonts()
   wxRegKey* fontRegKey = new wxRegKey(strFont);
   fontRegKey->Open(wxRegKey::Read);
   // Retrieve the number of SubKeys and enumerate them
-  size_t nSubKeys;
-  size_t nValues;
-  fontRegKey->GetKeyInfo(&nSubKeys, NULL, &nValues, NULL);
+  size_t nSubKeys = 0, nMaxKeyLen = 0;
+  size_t nValues = 0, nMaxValueLen = 0;
+  fontRegKey->GetKeyInfo(&nSubKeys, &nMaxKeyLen, &nValues, &nMaxValueLen);
   wxString fontDirectory = wxGetOSDirectory();
   if (!wxEndsWithPathSeparator(fontDirectory))
   {
@@ -751,8 +972,8 @@ wxPdfFontManagerBase::RegisterSystemFonts()
   }
 #endif
   FcPattern* pat = FcPatternBuild(NULL,
-	                                FC_OUTLINE, FcTypeBool, 1,
-	                                FC_SCALABLE, FcTypeBool, 1,
+                                  FC_OUTLINE, FcTypeBool, 1,
+                                  FC_SCALABLE, FcTypeBool, 1,
                                   NULL);
   FcObjectSet* os = FcObjectSetBuild (FC_FAMILY, 
                                       FC_STYLE, 
@@ -789,11 +1010,11 @@ wxPdfFontManagerBase::RegisterSystemFonts()
     FcFontSetDestroy(fs);
   }
 #elif defined(__WXMAC__)
-  wxLogError(wxString(wxT("wxPdfFontManagerBase::RegisterSystemFonts: ")) +
-             wxString(_("Method is not yet implemented for platform WXMAC.")));
+  count += RegisterFontDirectory(wxT("/System/Library/Fonts"));
+  count += RegisterFontDirectory(wxT("/Library/Fonts"));
 #else
-  wxLogError(wxString(wxT("wxPdfFontManagerBase::RegisterSystemFonts: ")) +
-             wxString(_("Method is not available for your platform.")));
+  wxLogWarning(wxString(wxT("wxPdfFontManagerBase::RegisterSystemFonts: ")) +
+               wxString(_("Method is not available for your platform.")));
 #endif
   return count;
 }
@@ -918,7 +1139,7 @@ wxPdfFontManagerBase::GetFont(const wxString& fontName, int fontStyle) const
     else
     {
       wxString style = ConvertStyleToString(searchStyle);
-      wxLogError(wxString(wxT("wxPdfFontManagerBase::GetFont: ")) +
+      wxLogDebug(wxString(wxT("wxPdfFontManagerBase::GetFont: ")) +
                  wxString::Format(_("Font '%s' with style '%s' not found."), fontName.c_str(), style.c_str()));
     }
   }
@@ -933,7 +1154,7 @@ wxPdfFontManagerBase::GetFont(const wxString& fontName, const wxString& fontStyl
 {
   int style = wxPDF_FONTSTYLE_REGULAR;
   wxString localStyle = fontStyle.Lower();
-  if (localStyle.Len() > 2)
+  if (localStyle.length() > 2)
   {
     if (localStyle.Find(wxT("bold")) != wxNOT_FOUND)
     {
@@ -999,6 +1220,77 @@ wxPdfFontManagerBase::InitializeFontData(const wxPdfFont& font)
   return ok;
 }
 
+bool
+wxPdfFontManagerBase::RegisterEncoding(const wxPdfEncoding& encoding)
+{
+  bool ok = true;
+  wxString encodingName = encoding.GetEncodingName().Lower();
+  if (m_encodingMap->find(encodingName) == m_encodingMap->end())
+  {
+#if wxUSE_THREADS
+    wxCriticalSectionLocker locker(gs_csFontManager);
+#endif
+    wxPdfEncoding* addedEncoding = new wxPdfEncoding(encoding);
+    if (addedEncoding->IsOk())
+    {
+      addedEncoding->InitializeEncodingMap();
+      (*m_encodingMap)[encodingName] = addedEncoding;
+    }
+    else
+    {
+      ok = false;
+    }
+  }
+  return ok;
+}
+
+void
+wxPdfFontManagerBase::SetFontBaseEncoding(wxPdfFontData* fontData)
+{
+  if (fontData != NULL)
+  {
+    wxString fontType = fontData->GetType();
+    wxString encoding = fontData->GetEncoding();
+    if (encoding.IsEmpty())
+    {
+      encoding = wxT("iso-8859-1");
+    }
+    if (fontType.IsSameAs(wxT("TrueType")) || fontType.IsSameAs(wxT("Type1")))
+    {
+      if (RegisterEncoding(encoding))
+      {
+        wxPdfEncodingMap::const_iterator beIter = m_encodingMap->find(encoding);
+        wxPdfEncoding* baseEncoding = (beIter != m_encodingMap->end()) ? beIter->second : NULL;
+        fontData->SetEncoding(baseEncoding);
+      }
+    }
+    else if (fontType.IsSameAs(wxT("Type0")))
+    {
+      wxPdfEncodingCheckerMap::const_iterator ecIter = m_encodingCheckerMap->find(encoding);
+      wxPdfEncodingChecker* encodingChecker = (ecIter != m_encodingCheckerMap->end()) ? ecIter->second : NULL;
+      fontData->SetEncodingChecker(encodingChecker);
+    }
+  }
+}
+
+const wxPdfEncoding*
+wxPdfFontManagerBase::GetEncoding(const wxString& encodingName)
+{
+#if wxUSE_THREADS
+  wxCriticalSectionLocker locker(gs_csFontManager);
+#endif
+  wxPdfEncoding* foundEncoding = NULL;
+  if (RegisterEncoding(encodingName))
+  {
+    wxPdfEncodingMap::const_iterator encoding = m_encodingMap->find(encodingName.Lower());
+    if (encoding != m_encodingMap->end())
+    {
+      foundEncoding = encoding->second;
+    }
+  }
+  return foundEncoding;
+}
+
 wxString
 wxPdfFontManagerBase::ConvertStyleToString(int fontStyle)
 {
@@ -1028,10 +1320,23 @@ void
 wxPdfFontManagerBase::InitializeCoreFonts()
 {
   wxPdfFontDataCore* coreFontData;
+  if (!RegisterEncoding(wxT("winansi")) ||
+      !RegisterEncoding(wxT("iso-8859-1")))
+  {
+    wxLogDebug(wxString(wxT("wxPdfFontManagerBase::InitializeCoreFonts: ")) +
+               wxString::Format(_("Registering encodings for core fonts failed.")));
+  }
   int j;
-  for (j = 0; gs_coreFontTable[j].name != wxEmptyString; j++)
+  for (j = 0; gs_coreFontTable[j].name != wxEmptyString; ++j)
   {
     const wxPdfCoreFontDesc& coreFontDesc = gs_coreFontTable[j];
+    wxString family(coreFontDesc.family);
+    bool isWinAnsi = !(family.IsSameAs(wxT("Symbol")) || family.IsSameAs(wxT("ZapfDingbats")));
+    wxString encoding = (isWinAnsi) ? wxT("winansi") : wxT("iso-8859-1");
+
+    wxPdfEncodingMap::const_iterator beIter = m_encodingMap->find(encoding);
+    wxPdfEncoding* baseEncoding = (beIter != m_encodingMap->end()) ? beIter->second : NULL;
+
     coreFontData = new wxPdfFontDataCore(coreFontDesc.family, coreFontDesc.alias, coreFontDesc.name, 
                                          coreFontDesc.cwArray, coreFontDesc.kpArray,
                                          wxPdfFontDescription(coreFontDesc.ascent, coreFontDesc.descent,
@@ -1040,10 +1345,103 @@ wxPdfFontManagerBase::InitializeCoreFonts()
                                                               coreFontDesc.stemV, coreFontDesc.missingWidth,
                                                               coreFontDesc.xHeight, coreFontDesc.underlinePosition,
                                                               coreFontDesc.underlineThickness));
+    coreFontData->SetEncoding(baseEncoding);
     AddFont(coreFontData);
   }
 }
 
+#if wxUSE_UNICODE
+void
+wxPdfFontManagerBase::InitializeCjkFonts()
+{
+  const wxChar* fontStyles[4] = { wxT(""), wxT(",Bold"), wxT(",Italic"), wxT(",BoldItalic") };
+  wxString fontName;
+  wxString fontAlias;
+  wxPdfFontDataType0* cjkFontData;
+  bool ok;
+  int j, k;
+  for (j = 0; gs_cjkFontTable[j].name != wxEmptyString; ++j)
+  {
+    const wxPdfCjkFontDesc& cjkFontDesc = gs_cjkFontTable[j];
+    wxPdfEncodingCheckerMap::const_iterator ecIter = m_encodingCheckerMap->find(cjkFontDesc.encoding);
+    wxPdfEncodingChecker* encodingChecker = (ecIter != m_encodingCheckerMap->end()) ? ecIter->second : NULL;
+    for (k = 0; k < 4; ++k)
+    {
+      cjkFontData = new wxPdfFontDataType0(cjkFontDesc.family, cjkFontDesc.name, 
+                                           cjkFontDesc.encoding, cjkFontDesc.ordering, 
+                                           cjkFontDesc.supplement, cjkFontDesc.cmap, 
+                                           cjkFontDesc.cwArray,
+                                           wxPdfFontDescription(cjkFontDesc.ascent, cjkFontDesc.descent,
+                                                                cjkFontDesc.capHeight, cjkFontDesc.flags,
+                                                                cjkFontDesc.bbox, cjkFontDesc.italicAngle,
+                                                                cjkFontDesc.stemV, cjkFontDesc.missingWidth,
+                                                                cjkFontDesc.xHeight, cjkFontDesc.underlinePosition,
+                                                                cjkFontDesc.underlineThickness));
+      fontName = cjkFontDesc.name;
+      fontName += fontStyles[k];
+      cjkFontData->SetName(fontName);
+      fontAlias = cjkFontDesc.family;
+      cjkFontData->SetFamily(fontAlias);
+      cjkFontData->SetAlias(fontAlias);
+      cjkFontData->SetStyleFromName();
+      cjkFontData->SetEncodingChecker(encodingChecker);
+      ok = AddFont(cjkFontData);
+      if (!ok)
+      {
+        delete cjkFontData;
+      }
+    }
+  }
+}
+#endif
+
+void
+wxPdfFontManagerBase::InitializeEncodingChecker()
+{
+  int j;
+  for (j = 0; gs_encodingTableData[j].m_encodingName != NULL; ++j)
+  {
+    wxString encodingName(gs_encodingTableData[j].m_encodingName);
+    wxPdfEncodingChecker* encodingChecker;
+    if (gs_encodingTableData[j].m_encodingTable != NULL)
+    {
+      encodingChecker = new wxPdfCodepageChecker(gs_encodingTableData[j].m_encodingName,
+                                                 gs_encodingTableData[j].m_encodingTableSize,
+                                                 gs_encodingTableData[j].m_encodingTable);
+    }
+    else
+    {
+      encodingChecker = new wxPdfCjkChecker(gs_encodingTableData[j].m_encodingName,
+                                            gs_encodingTableData[j].m_encodingBase);
+    }
+    (*m_encodingCheckerMap)[encodingName] = encodingChecker;
+  }
+}
+
+bool
+wxPdfFontManagerBase::RegisterEncoding(const wxString& encoding)
+{
+  bool ok = true;
+  wxString encodingName = encoding.Lower();
+  if (m_encodingMap->find(encodingName) == m_encodingMap->end())
+  {
+    wxPdfEncoding* addedEncoding = new wxPdfEncoding();
+    if (addedEncoding->SetEncoding(encoding))
+    {
+      addedEncoding->InitializeEncodingMap();
+      (*m_encodingMap)[encodingName] = addedEncoding;
+    }
+    else
+    {
+      wxLogDebug(wxString(wxT("wxPdfFontManagerBase::RegisterEncoding: ")) +
+                 wxString::Format(_("Encoding '%s' is unknown."), encoding.c_str()));
+      delete addedEncoding;
+      ok = false;
+    }
+  }
+  return ok;
+}
+
 bool
 wxPdfFontManagerBase::RegisterFontCJK(const wxString& fontFileName, const wxString& fontStyle, const wxString& alias)
 {
@@ -1058,11 +1456,12 @@ wxPdfFontManagerBase::RegisterFontCJK(const wxString& fontFileName, const wxStri
     fontData->SetFamily(alias);
     fontData->SetAlias(alias);
     fontData->SetStyleFromName();
+    SetFontBaseEncoding(fontData);
     ok = AddFont(fontData);
     if (!ok)
     {
-      wxLogWarning(wxString(wxT("wxPdfFontManagerBase::RegisterFontCJK: ")) +
-                   wxString::Format(_("CJK font '%s' already registered."), fontName.c_str()));
+      wxLogDebug(wxString(wxT("wxPdfFontManagerBase::RegisterFontCJK: ")) +
+                 wxString::Format(_("CJK font '%s' already registered."), fontName.c_str()));
     }
   }
   return ok;
@@ -1076,7 +1475,7 @@ wxPdfFontManagerBase::LoadFontFromXML(const wxString& fontFileName)
   wxFileSystem fs;
 
   // Open font metrics XML file
-  wxFSFile* xmlFontMetrics = fs.OpenFile(fileName.GetFullPath());
+  wxFSFile* xmlFontMetrics = fs.OpenFile(wxFileSystem::FileNameToURL(fileName));
   if (xmlFontMetrics != NULL)
   {
     // Load the XML file
@@ -1191,14 +1590,14 @@ wxPdfFontManagerBase::AddFont(wxPdfFontData* fontData, wxPdfFont& font)
   wxCriticalSectionLocker locker(gs_csFontManager);
 #endif
   wxString fontName = fontData->GetName().Lower();
+  wxString family = fontData->GetFamily().Lower();
+  wxString alias = fontData->GetAlias().Lower();
 
   wxPdfFontNameMap::const_iterator fontIter = m_fontNameMap.find(fontName.Lower());
   if (fontIter == m_fontNameMap.end())
   {
     // Font not yet registered
-    wxString family = fontData->GetFamily().Lower();
     wxArrayString fullNames = fontData->GetFullNames();
-    wxString alias = fontData->GetAlias().Lower();
     size_t pos = m_fontList.GetCount();
     wxPdfFontListEntry* fontEntry = new wxPdfFontListEntry(fontData);
     m_fontList.Add(fontEntry);
@@ -1224,32 +1623,32 @@ wxPdfFontManagerBase::AddFont(wxPdfFontData* fontData, wxPdfFont& font)
     {
       m_fontFamilyMap[alias].Add(pos);
     }
-
-    // Register family alias
-    if (!alias.IsEmpty() && !alias.IsSameAs(family))
-    {
-      // Check whether the alias is already assigned and - if so - to the same family
-      wxPdfFontAliasMap::const_iterator aliasIter = m_fontAliasMap.find(alias);
-      if (aliasIter != m_fontAliasMap.end())
-      {
-        if (!aliasIter->second.IsSameAs(family))
-        {
-          wxLogError(wxString(wxT("wxPdfFontManagerBase::AddFont: ")) +
-                     wxString::Format(_("Family alias '%s' for family '%s' already assigned to family '%s'."), 
-                     alias.c_str(), family.c_str(), aliasIter->second.c_str()));
-        }
-      }
-      else
-      {
-        // alias not previously assigned, remember assignment
-        m_fontAliasMap[alias] = family;
-      }
-    }
   }
   else
   {
     font = wxPdfFont(m_fontList[fontIter->second]->GetFontData());
   }
+
+  // Register family alias
+  if (!alias.IsEmpty() && !alias.IsSameAs(family))
+  {
+    // Check whether the alias is already assigned and - if so - to the same family
+    wxPdfFontAliasMap::const_iterator aliasIter = m_fontAliasMap.find(alias);
+    if (aliasIter != m_fontAliasMap.end())
+    {
+      if (!aliasIter->second.IsSameAs(family))
+      {
+        wxLogError(wxString(wxT("wxPdfFontManagerBase::AddFont: ")) +
+                   wxString::Format(_("Family alias '%s' for family '%s' already assigned to family '%s'."), 
+                   alias.c_str(), family.c_str(), aliasIter->second.c_str()));
+      }
+    }
+    else
+    {
+      // alias not previously assigned, remember assignment
+      m_fontAliasMap[alias] = family;
+    }
+  }
   return ok;
 }
 
@@ -1402,6 +1801,18 @@ wxPdfFontManager::InitializeFontData(const wxPdfFont& font)
   return ok;
 }
 
+bool
+wxPdfFontManager::RegisterEncoding(const wxPdfEncoding& encoding)
+{
+  return m_fontManagerBase->RegisterEncoding(encoding);
+}
+
+const wxPdfEncoding*
+wxPdfFontManager::GetEncoding(const wxString& encodingName)
+{
+  return m_fontManagerBase->GetEncoding(encodingName);
+}
+
 // A module to allow initialization/cleanup of wxPdfDocument
 // singletons without calling these functions from app.cpp.
 
