@@ -191,6 +191,7 @@ ExtractFontData(HDC hdc, DWORD& fontDataSize, BYTE*& fontData)
     delete [] fontHeader;
     fontDataSize = bufferSize;
     fontData     = buffer;
+    ok = true;
   }
   else
   {
@@ -398,6 +399,8 @@ wxPdfFontParserTrueType::wxPdfFontParserTrueType()
   m_cmap31 = NULL;
   m_cmapExt = NULL;
   m_kp = NULL;
+  m_isMacCoreText = false;
+  m_savedStream = NULL;
 }
 
 wxPdfFontParserTrueType::~wxPdfFontParserTrueType()
@@ -455,6 +458,73 @@ wxPdfFontParserTrueType::ClearTableDirectory()
   }
 }
 
+static const wxChar* tableNamesDefault[] = {
+  wxT("cvt "), wxT("fpgm"), wxT("glyf"), wxT("head"),
+  wxT("hhea"), wxT("hmtx"), wxT("loca"), wxT("maxp"), wxT("prep"),
+  NULL
+};
+
+void
+wxPdfFontParserTrueType::LockTable(const wxString& tableName)
+{
+#if defined(__WXMAC__)
+#if wxPDFMACOSX_HAS_CORE_TEXT
+  if (m_isMacCoreText)
+  {
+    wxCharBuffer asciiTableName = tableName.ToAscii();
+    const char* localTableName = asciiTableName;
+    CTFontTableTag tableTag = localTableName[0] << 24 |
+                              localTableName[1] << 16 |
+                              localTableName[2] << 8 |
+                              localTableName[3];
+    m_tableRef.reset(CTFontCopyTable(m_fontRef, tableTag, 0));
+    const UInt8* tableData = CFDataGetBytePtr(m_tableRef);
+    CFIndex      tableLen  = CFDataGetLength(m_tableRef);
+    m_savedStream = m_inFont;
+    m_inFont = new wxMemoryInputStream((const char*) tableData, (size_t) tableLen);
+  }
+#else
+  wxUnusedVar(tableName);
+#endif
+#else
+  wxUnusedVar(tableName);
+#endif
+}
+
+void
+wxPdfFontParserTrueType::ReleaseTable()
+{
+#if defined(__WXMAC__)
+#if wxPDFMACOSX_HAS_CORE_TEXT
+  if (m_isMacCoreText)
+  {
+    delete m_inFont;
+    m_inFont = m_savedStream;
+  }
+#endif
+#endif
+}
+
+int
+wxPdfFontParserTrueType::CalculateChecksum(const char* b, size_t length)
+{
+  size_t len = length / 4;
+  int d0 = 0;
+  int d1 = 0;
+  int d2 = 0;
+  int d3 = 0;
+  size_t ptr = 0;
+  size_t k;
+  for (k = 0; k < len; ++k)
+  {
+    d3 += (int)b[ptr++] & 0xff;
+    d2 += (int)b[ptr++] & 0xff;
+    d1 += (int)b[ptr++] & 0xff;
+    d0 += (int)b[ptr++] & 0xff;
+  }
+  return d0 + (d1 << 8) + (d2 << 16) + (d3 << 24);
+}
+
 int
 wxPdfFontParserTrueType::GetCollectionFontCount(const wxString& fontFileName)
 {
@@ -462,7 +532,7 @@ wxPdfFontParserTrueType::GetCollectionFontCount(const wxString& fontFileName)
   wxFileName fileName(fontFileName);
   wxFileSystem fs;
 
-  wxFSFile* fontFile = fs.OpenFile(fileName.GetFullPath());
+  wxFSFile* fontFile = fs.OpenFile(wxFileSystem::FileNameToURL(fileName));
   if (fontFile != NULL)
   {
     m_inFont = fontFile->GetStream();
@@ -493,7 +563,7 @@ wxPdfFontParserTrueType::IdentifyFont(const wxString& fontFileName, int fontInde
   wxFileSystem fs;
 
   // Open font file
-  wxFSFile* fontFile = fs.OpenFile(fileName.GetFullPath());
+  wxFSFile* fontFile = fs.OpenFile(wxFileSystem::FileNameToURL(fileName));
   if (fontFile != NULL)
   {
     m_inFont = fontFile->GetStream();
@@ -608,6 +678,42 @@ wxPdfFontParserTrueType::IdentifyFont(const wxFont& font)
 }
 #endif
 
+#if defined(__WXMAC__)
+wxPdfFontData*
+wxPdfFontParserTrueType::IdentifyFont(const wxFont& font)
+{
+  wxPdfFontData* fontData = NULL;
+#if wxPDFMACOSX_HAS_CORE_TEXT
+#if wxCHECK_VERSION(2,9,0)
+// wxWidgets 2.9.x or higher
+  m_fontRef = font.OSXGetCTFont();
+#else // wxWidgets 2.8.x
+  m_fontRef = (const void*) font.MacGetCTFont();
+#endif
+
+  m_isMacCoreText = true;
+  m_fileName = wxEmptyString;
+
+  m_inFont = NULL;
+  m_directoryOffset = 0;
+
+  // Identify single font
+  fontData = IdentifyFont();
+  if (fontData != NULL)
+  {
+    fontData->SetFont(font);
+    fontData->SetFontIndex(0);
+  }
+  else
+  {
+    wxLogError(wxString(wxT("wxPdfFontParserTrueType::IdentifyFont: ")) +
+               wxString::Format(_("Reading of font directory failed for font file '%s'."), font.GetFaceName().c_str()));
+  }
+#endif
+  return fontData;
+}
+#endif
+
 wxPdfFontData*
 wxPdfFontParserTrueType::IdentifyFont()
 {
@@ -615,28 +721,32 @@ wxPdfFontParserTrueType::IdentifyFont()
 #if wxUSE_UNICODE
   if (ReadTableDirectory())
   {
-    CheckCff();
-    if (m_cff)
+    if (CheckTables())
     {
-      wxPdfFontDataOpenTypeUnicode* otfFontData =  new wxPdfFontDataOpenTypeUnicode();
-      otfFontData->SetCffOffset(m_cffOffset);
-      otfFontData->SetCffLength(m_cffLength);
-      fontData = otfFontData;
-    }
-    else
-    {
-      fontData = new wxPdfFontDataTrueTypeUnicode();
-    }
+      CheckCff();
+      if (m_cff)
+      {
+        wxPdfFontDataOpenTypeUnicode* otfFontData =  new wxPdfFontDataOpenTypeUnicode();
+        otfFontData->SetCffOffset(m_cffOffset);
+        otfFontData->SetCffLength(m_cffLength);
+        fontData = otfFontData;
+      }
+      else
+      {
+        fontData = new wxPdfFontDataTrueTypeUnicode();
+      }
 
-    // Get PostScript name, font family, font names and font style
-    fontData->SetName(GetBaseFont());
-    fontData->SetFamily(GetEnglishName(1));
-    fontData->SetFullNames(GetUniqueNames(4));
-    fontData->SetStyle(GetEnglishName(2));
+      // Get PostScript name, font family, font names and font style
+      fontData->SetName(GetBaseFont());
+      fontData->SetFamily(GetEnglishName(1));
+      fontData->SetFullNames(GetUniqueNames(4));
+      fontData->SetStyle(GetEnglishName(2));
+      m_fontName = fontData->GetName();
 
-    CheckRestrictions();
-    fontData->SetEmbedSupported(m_embedAllowed);
-    fontData->SetSubsetSupported(m_subsetAllowed);
+      CheckRestrictions();
+      fontData->SetEmbedSupported(m_embedAllowed);
+      fontData->SetSubsetSupported(m_subsetAllowed);
+    }
   }
 #endif
   return fontData;
@@ -653,11 +763,12 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
     m_inFont = NULL;
     int fontIndex = fontData->GetFontIndex();
     m_fileName = fontData->GetFontFileName();
+    m_fontName = fontData->GetName();
     if (!m_fileName.IsEmpty())
     {
       wxFileName fileName(m_fileName);
       wxFileSystem fs;
-      fontFile = fs.OpenFile(fileName.GetFullPath());
+      fontFile = fs.OpenFile(wxFileSystem::FileNameToURL(fileName));
       if (fontFile != NULL)
       {
         m_inFont = fontFile->GetStream();
@@ -665,7 +776,7 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
     }
     else
     {
-#ifdef __WXMSW__
+#if defined(__WXMSW__)
       wxFont font = fontData->GetFont();
       if (font.IsOk())
       {
@@ -675,6 +786,22 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
           m_inFont = fontStream;
         }
       }
+#elif defined(__WXMAC__)
+#if wxPDFMACOSX_HAS_CORE_TEXT
+      wxFont font = fontData->GetFont();
+      if (font.IsOk())
+      {
+#if wxCHECK_VERSION(2,9,0)
+        // wxWidgets 2.9.x or higher
+        m_fontRef = font.OSXGetCTFont();
+#else // wxWidgets 2.8.x
+        m_fontRef = (const void*) font.MacGetCTFont();
+#endif
+        m_isMacCoreText = true;
+        fontStream = new wxMemoryInputStream("dummy", 5);
+        m_inFont = fontStream;
+      }
+#endif
 #endif
     }
     if (m_inFont != NULL)
@@ -723,23 +850,31 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
       {
         if (ReadTableDirectory())
         {
-          CheckCff();
-          if (m_cff)
+          if (CheckTables())
           {
-            ok = fontData->GetType().IsSameAs(wxT("OpenTypeUnicode"));
-          }
-          else
-          {
-            ok = fontData->GetType().IsSameAs(wxT("TrueTypeUnicode"));
-          }
-          if (ok)
-          {
-            PrepareFontData(fontData);
+            CheckCff();
+            if (m_cff)
+            {
+              ok = fontData->GetType().IsSameAs(wxT("OpenTypeUnicode"));
+            }
+            else
+            {
+              ok = fontData->GetType().IsSameAs(wxT("TrueTypeUnicode"));
+            }
+            if (ok)
+            {
+              ok = PrepareFontData(fontData);
+            }
+            else
+            {
+              wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: ")) +
+                         wxString::Format(_("Wrong font data type '%s' for font file '%s'."), fontData->GetType().c_str(), m_fileName.c_str()));
+            }
           }
           else
           {
             wxLogError(wxString(wxT("wxPdfFontParserTrueType::LoadFontData: ")) +
-                       wxString::Format(_("Wrong font data type '%s' for font file '%s'."), fontData->GetType().c_str(), m_fileName.c_str()));
+                       wxString::Format(_("Missing font tables for font file '%s'."), m_fileName.c_str()));
           }
         }
         else
@@ -771,41 +906,43 @@ wxPdfFontParserTrueType::LoadFontData(wxPdfFontData* fontData)
   return ok;
 }
 
-void
+bool
 wxPdfFontParserTrueType::PrepareFontData(wxPdfFontData* fontData)
 {
-  wxPdfGlyphWidthMap* widths = new wxPdfGlyphWidthMap();
-  wxPdfChar2GlyphMap* glyphs = new wxPdfChar2GlyphMap();
-
-  ReadMaps();
-
-  wxPdfCMap* cMap;
-  if (m_cmapExt != NULL)
-    cMap = m_cmapExt;
-  else if (!m_fontSpecific && m_cmap31 != NULL)
-    cMap = m_cmap31;
-  else if (m_fontSpecific && m_cmap10 != NULL)
-    cMap = m_cmap10;
-  else if (m_cmap31 != NULL)
-    cMap = m_cmap31;
-  else 
-    cMap = m_cmap10;
-  wxPdfCMap::iterator cMapIter;
-  int cc;
-  wxPdfCMapEntry* cMapEntry;
-  for (cMapIter = cMap->begin(); cMapIter != cMap->end(); cMapIter++)
+  bool ok = ReadMaps();
+  if (ok)
   {
-    cc = cMapIter->first;
-    cMapEntry = cMapIter->second;
-    (*widths)[cc] = cMapEntry->m_width;
-    (*glyphs)[cc] = cMapEntry->m_glyph;
-  }
+    wxPdfGlyphWidthMap* widths = new wxPdfGlyphWidthMap();
+    wxPdfChar2GlyphMap* glyphs = new wxPdfChar2GlyphMap();
 
-  fontData->SetGlyphWidthMap(widths);
-  fontData->SetChar2GlyphMap(glyphs);
-  fontData->SetGlyphWidths(m_glyphWidths);
-  fontData->SetKernPairMap(m_kp);
-  fontData->SetDescription(m_fd);
+    wxPdfCMap* cMap;
+    if (m_cmapExt != NULL)
+      cMap = m_cmapExt;
+    else if (!m_fontSpecific && m_cmap31 != NULL)
+      cMap = m_cmap31;
+    else if (m_fontSpecific && m_cmap10 != NULL)
+      cMap = m_cmap10;
+    else if (m_cmap31 != NULL)
+      cMap = m_cmap31;
+    else 
+      cMap = m_cmap10;
+    wxPdfCMap::iterator cMapIter;
+    int cc;
+    wxPdfCMapEntry* cMapEntry;
+    for (cMapIter = cMap->begin(); cMapIter != cMap->end(); cMapIter++)
+    {
+      cc = cMapIter->first;
+      cMapEntry = cMapIter->second;
+      (*widths)[cc] = cMapEntry->m_width;
+      (*glyphs)[cc] = cMapEntry->m_glyph;
+    }
+
+    fontData->SetGlyphWidthMap(widths);
+    fontData->SetChar2GlyphMap(glyphs);
+    fontData->SetGlyphWidths(m_glyphWidths);
+    fontData->SetKernPairMap(m_kp);
+    fontData->SetDescription(m_fd);
+  }
 
   m_inFont->SeekI(0);
   size_t len = (m_cff) ? m_cffLength : m_inFont->GetSize();
@@ -813,6 +950,7 @@ wxPdfFontParserTrueType::PrepareFontData(wxPdfFontData* fontData)
 #if wxUSE_UNICODE
   fontData->CreateDefaultEncodingConv();
 #endif
+  return ok;
 }
 
 bool
@@ -820,35 +958,96 @@ wxPdfFontParserTrueType::ReadTableDirectory()
 {
   ClearTableDirectory();
   bool ok = true;
-  m_inFont->SeekI(m_directoryOffset);
-  int id = ReadInt();
-  if (id == 0x00010000 || id == 0x4F54544F)
+  if (!m_isMacCoreText)
   {
-    int num_tables = ReadUShort();
-    SkipBytes(6);
-    int k;
-    for (k = 0; k < num_tables; ++k)
+    m_inFont->SeekI(m_directoryOffset);
+    int id = ReadInt();
+    //        TrueType            OpenType            Mac TrueType
+    if (id == 0x00010000 || id == 0x4F54544F || id == 0x74727565)
     {
-      wxString tag = ReadString(4);
-      wxPdfTableDirectoryEntry* tableLocation = new wxPdfTableDirectoryEntry();
-      tableLocation->m_checksum = ReadInt();
-      tableLocation->m_offset = ReadInt();
-      tableLocation->m_length = ReadInt();
-      (*m_tableDirectory)[tag] = tableLocation;
+      int num_tables = ReadUShort();
+      SkipBytes(6);
+      int k;
+      for (k = 0; k < num_tables; ++k)
+      {
+        wxString tag = ReadString(4);
+        wxPdfTableDirectoryEntry* tableLocation = new wxPdfTableDirectoryEntry();
+        tableLocation->m_checksum = ReadInt();
+        tableLocation->m_offset = ReadInt();
+        tableLocation->m_length = ReadInt();
+        (*m_tableDirectory)[tag] = tableLocation;
+      }
+    }
+    else
+    {
+      if (!m_fileName.IsEmpty())
+      {
+        wxLogError(wxString(wxT("wxPdfFontParserTrueType::ReadTableDirectory: '")) + 
+                   wxString::Format(_("Font file '%s' not a valid TrueType (TTF) or OpenType (OTF) file."), m_fileName.c_str()));
+      }
+      ok = false;
     }
   }
   else
   {
-    if (!m_fileName.IsEmpty())
+#if defined(__WXMAC__)
+#if wxPDFMACOSX_HAS_CORE_TEXT
+    CFArrayRef tables = CTFontCopyAvailableTables(m_fontRef, 0);
+    CFIndex nTables = CFArrayGetCount(tables);
+    CFIndex n;
+    for (n = 0; n < nTables; ++n)
     {
-      wxLogError(wxString(wxT("wxPdfFontParserTrueType::ReadTableDirectory: '")) + 
-                 wxString::Format(_("Font file '%s' not a valid TrueType (TTF) or OpenType (OTF) file."), m_fileName.c_str()));
+      CTFontTableTag tag = (CTFontTableTag)(uintptr_t) CFArrayGetValueAtIndex(tables, n);
+      CFDataRef    tableRef  = CTFontCopyTable(m_fontRef, tag, 0);
+      const UInt8* tableData = CFDataGetBytePtr(tableRef);
+      CFIndex      tableLen  = CFDataGetLength(tableRef);
+      int checksum = CalculateChecksum((const char*) tableData, (size_t) tableLen);
+      CFRelease(tableRef);
+      char asciiTag[5];
+      asciiTag[0] = (tag >> 24) & 0xff;
+      asciiTag[1] = (tag >> 16) & 0xff;
+      asciiTag[2] = (tag >>  8) & 0xff;
+      asciiTag[3] =  tag        & 0xff;
+      asciiTag[4] = 0;
+      wxString tableTag = wxString::FromAscii(asciiTag);
+      wxPdfTableDirectoryEntry* tableLocation = new wxPdfTableDirectoryEntry();
+      tableLocation->m_checksum = checksum;
+      tableLocation->m_offset = 0;
+      tableLocation->m_length = tableLen;
+      (*m_tableDirectory)[tableTag] = tableLocation;
     }
+#else
     ok = false;
+#endif
+#endif
   }
   return ok;
 }
-    
+
+static const wxChar* checkTableNames[] = {
+  wxT("cmap"), wxT("head"), wxT("hhea"), wxT("hmtx"), wxT("name"),
+  wxT("post"), 
+  wxT("glyf"), wxT("loca"),
+  NULL
+};
+
+bool
+wxPdfFontParserTrueType::CheckTables()
+{
+  bool ok = true;
+  int maxTableCount = (m_tableDirectory->find(wxT("CFF ")) == m_tableDirectory->end()) ? 8 : 6;
+  int tableCount = 0;
+  while (ok && tableCount < maxTableCount && checkTableNames[tableCount] != NULL)
+  {
+    if (m_tableDirectory->find(checkTableNames[tableCount]) == m_tableDirectory->end())
+    {
+      ok = false;
+    }
+    ++tableCount;
+  }
+  return ok;
+}
+
 void
 wxPdfFontParserTrueType::CheckCff()
 {
@@ -877,6 +1076,7 @@ wxPdfFontParserTrueType::CheckRestrictions()
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
+    LockTable(wxT("OS/2"));
     m_inFont->SeekI(tableLocation->m_offset+8);
     short fsType = ReadShort();
     bool rl = (fsType & 0x0002) != 0; // restricted license
@@ -886,6 +1086,7 @@ wxPdfFontParserTrueType::CheckRestrictions()
     bool b  = (fsType & 0x0200) != 0; // bitmap embedding only
     m_embedAllowed = !((rl && !pp && !e) || b);
     m_subsetAllowed = !ns;
+    ReleaseTable();
   }
   else
   {
@@ -903,6 +1104,7 @@ wxPdfFontParserTrueType::GetBaseFont()
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
+    LockTable(wxT("name"));
     m_inFont->SeekI(tableLocation->m_offset+2);
     int numRecords = ReadUShort();
     int startOfStorage = ReadUShort();
@@ -934,6 +1136,7 @@ wxPdfFontParserTrueType::GetBaseFont()
       wxFileName::SplitPath(m_fileName, NULL, &fontName, NULL);
       fontName.Replace(wxT(" "), wxT("-"));
     }
+    ReleaseTable();
   }
   else
   {
@@ -959,7 +1162,7 @@ wxPdfFontParserTrueType::ReadMaps()
     return false;
   }
   tableLocation = entry->second;
-
+  LockTable(wxT("head"));
   m_inFont->SeekI(tableLocation->m_offset+16);
   head.m_flags = ReadUShort();
   head.m_unitsPerEm = ReadUShort();
@@ -969,7 +1172,8 @@ wxPdfFontParserTrueType::ReadMaps()
   head.m_xMax = ReadShort();
   head.m_yMax = ReadShort();
   head.m_macStyle = ReadUShort();
-        
+  ReleaseTable();
+
   entry = m_tableDirectory->find(wxT("hhea"));
   if (entry == m_tableDirectory->end())
   {
@@ -978,6 +1182,7 @@ wxPdfFontParserTrueType::ReadMaps()
     return false;
   }
   tableLocation = entry->second;
+  LockTable(wxT("hhea"));
   m_inFont->SeekI(tableLocation->m_offset+4);
   hhea.m_ascender = ReadShort();
   hhea.m_descender = ReadShort();
@@ -990,63 +1195,86 @@ wxPdfFontParserTrueType::ReadMaps()
   hhea.m_caretSlopeRun = ReadShort();
   SkipBytes(12);
   hhea.m_numberOfHMetrics = ReadUShort();
-        
+  ReleaseTable();
+
   entry = m_tableDirectory->find(wxT("OS/2"));
-  if (entry == m_tableDirectory->end())
+  if (entry != m_tableDirectory->end())
   {
-    wxLogError(wxString(wxT("wxPdfFontParser::ReadMaps: ")) +
-               wxString::Format(_("Table 'OS/2' does not exist in '%s,%s'."), m_fileName.c_str(), m_style.c_str()));
-    return false;
-  }
-  tableLocation = entry->second;
-  m_inFont->SeekI(tableLocation->m_offset);
-  int version = ReadUShort();
-  os_2.m_xAvgCharWidth = ReadShort();
-  os_2.m_usWeightClass = ReadUShort();
-  os_2.m_usWidthClass = ReadUShort();
-  os_2.m_fsType = ReadShort();
-  os_2.m_ySubscriptXSize = ReadShort();
-  os_2.m_ySubscriptYSize = ReadShort();
-  os_2.m_ySubscriptXOffset = ReadShort();
-  os_2.m_ySubscriptYOffset = ReadShort();
-  os_2.m_ySuperscriptXSize = ReadShort();
-  os_2.m_ySuperscriptYSize = ReadShort();
-  os_2.m_ySuperscriptXOffset = ReadShort();
-  os_2.m_ySuperscriptYOffset = ReadShort();
-  os_2.m_yStrikeoutSize = ReadShort();
-  os_2.m_yStrikeoutPosition = ReadShort();
-  os_2.m_sFamilyClass = ReadShort();
-  m_inFont->Read(os_2.m_panose, 10);
-  SkipBytes(16);
-  m_inFont->Read(os_2.m_achVendID, 4);
-  os_2.m_fsSelection = ReadUShort();
-  os_2.m_usFirstCharIndex = ReadUShort();
-  os_2.m_usLastCharIndex = ReadUShort();
-  os_2.m_sTypoAscender = ReadShort();
-  os_2.m_sTypoDescender = ReadShort();
-  if (os_2.m_sTypoDescender > 0)
-  {
-    os_2.m_sTypoDescender = (short)(-os_2.m_sTypoDescender);
-  }
-  os_2.m_sTypoLineGap = ReadShort();
-  os_2.m_usWinAscent = ReadUShort();
-  os_2.m_usWinDescent = ReadUShort();
-  os_2.m_ulCodePageRange1 = 0;
-  os_2.m_ulCodePageRange2 = 0;
-  if (version > 0)
-  {
-    os_2.m_ulCodePageRange1 = ReadInt();
-    os_2.m_ulCodePageRange2 = ReadInt();
-  }
-  if (version > 1)
-  {
-    os_2.m_sxHeight = ReadShort();
-    os_2.m_sCapHeight = ReadShort();
+    tableLocation = entry->second;
+    LockTable(wxT("OS/2"));
+    m_inFont->SeekI(tableLocation->m_offset);
+    int version = ReadUShort();
+    os_2.m_xAvgCharWidth = ReadShort();
+    os_2.m_usWeightClass = ReadUShort();
+    os_2.m_usWidthClass = ReadUShort();
+    os_2.m_fsType = ReadShort();
+    os_2.m_ySubscriptXSize = ReadShort();
+    os_2.m_ySubscriptYSize = ReadShort();
+    os_2.m_ySubscriptXOffset = ReadShort();
+    os_2.m_ySubscriptYOffset = ReadShort();
+    os_2.m_ySuperscriptXSize = ReadShort();
+    os_2.m_ySuperscriptYSize = ReadShort();
+    os_2.m_ySuperscriptXOffset = ReadShort();
+    os_2.m_ySuperscriptYOffset = ReadShort();
+    os_2.m_yStrikeoutSize = ReadShort();
+    os_2.m_yStrikeoutPosition = ReadShort();
+    os_2.m_sFamilyClass = ReadShort();
+    m_inFont->Read(os_2.m_panose, 10);
+    if (version == 0)
+    {
+      SkipBytes(4);
+    }
+    else
+    {
+      SkipBytes(16);
+    }
+    m_inFont->Read(os_2.m_achVendID, 4);
+    os_2.m_fsSelection = ReadUShort();
+    os_2.m_usFirstCharIndex = ReadUShort();
+    os_2.m_usLastCharIndex = ReadUShort();
+    os_2.m_sTypoAscender = ReadShort();
+    os_2.m_sTypoDescender = ReadShort();
+    if (os_2.m_sTypoDescender > 0)
+    {
+      os_2.m_sTypoDescender = (short)(-os_2.m_sTypoDescender);
+    }
+    os_2.m_sTypoLineGap = ReadShort();
+    os_2.m_usWinAscent = ReadUShort();
+    os_2.m_usWinDescent = ReadUShort();
+    os_2.m_ulCodePageRange1 = 0;
+    os_2.m_ulCodePageRange2 = 0;
+    if (version > 0)
+    {
+      os_2.m_ulCodePageRange1 = ReadInt();
+      os_2.m_ulCodePageRange2 = ReadInt();
+    }
+    if (version > 1)
+    {
+      os_2.m_sxHeight = ReadShort();
+      os_2.m_sCapHeight = ReadShort();
+    }
+    else
+    {
+      os_2.m_sxHeight = 0;
+      os_2.m_sCapHeight = (int)(0.7 * head.m_unitsPerEm);
+    }
+    ReleaseTable();
   }
   else
   {
+    // TODO: check definition of hhea values, convert if necessary
+    os_2.m_sTypoAscender = hhea.m_ascender;
+    os_2.m_sTypoDescender = hhea.m_descender;
+    if (os_2.m_sTypoDescender > 0)
+    {
+      os_2.m_sTypoDescender = (short)(-os_2.m_sTypoDescender);
+    }
     os_2.m_sxHeight = 0;
     os_2.m_sCapHeight = (int)(0.7 * head.m_unitsPerEm);
+    if (os_2.m_sCapHeight > os_2.m_sTypoAscender)
+    {
+      os_2.m_sCapHeight = os_2.m_sTypoAscender;
+    }
   }
 
   int underlinePosition = -100;
@@ -1064,6 +1292,7 @@ wxPdfFontParserTrueType::ReadMaps()
   else
   {
     tableLocation = entry->second;
+    LockTable(wxT("post"));
     m_inFont->SeekI(tableLocation->m_offset+4);
     short mantissa = ReadShort();
     int fraction = ReadUShort();
@@ -1071,6 +1300,7 @@ wxPdfFontParserTrueType::ReadMaps()
     underlinePosition = ReadShort();
     underlineThickness = ReadShort();
     m_isFixedPitch = ReadInt() != 0;
+    ReleaseTable();
   }
 
   ReadGlyphWidths(hhea.m_numberOfHMetrics, head.m_unitsPerEm);
@@ -1120,7 +1350,7 @@ wxPdfFontParserTrueType::ReadMaps()
     return false;
   }
   tableLocation = entry->second;
-
+  LockTable(wxT("cmap"));
   m_inFont->SeekI(tableLocation->m_offset);
   SkipBytes(2);
   int num_tables = ReadUShort();
@@ -1152,6 +1382,18 @@ wxPdfFontParserTrueType::ReadMaps()
     {
       map10 = offset;
     }
+    else if (platId == 0) // Apple Unicode
+    {
+      // Formats 0, 2, 4, 6, 8, 10, 12
+      if (platSpecId < 4)
+      {
+        map31 = offset;
+      }
+      else if (platSpecId == 4) // UCS32
+      {
+        mapExt = offset;
+      }
+    }
   }
   if (map10 > 0 && map30 <= 0)
   {
@@ -1208,12 +1450,20 @@ wxPdfFontParserTrueType::ReadMaps()
         break;
     }
   }
+  ReleaseTable();
 
   flags = m_fd.GetFlags();
   flags |= m_fontSpecific ? 4 : 32;
   m_fd.SetFlags(flags);
 
-  return true;
+  bool ok = (m_cmap10 != NULL) || (m_cmap31 != NULL) || (m_cmapExt != NULL);
+  if (!ok)
+  {
+    wxLogError(wxString(wxT("wxPdfFontParserTrueType::ReadMaps: ")) +
+               wxString::Format(_("No valid 'cmap' table found for font '%s'."),  m_fontName.c_str()));
+  }
+
+  return ok;
 }
 
 bool
@@ -1228,7 +1478,7 @@ wxPdfFontParserTrueType::ReadGlyphWidths(int numberOfHMetrics, int unitsPerEm)
     return false;
   }
   tableLocation = entry->second;
-
+  LockTable(wxT("hmtx"));
   m_inFont->SeekI(tableLocation->m_offset);
 
   m_glyphWidths.SetCount(numberOfHMetrics);
@@ -1238,6 +1488,7 @@ wxPdfFontParserTrueType::ReadGlyphWidths(int numberOfHMetrics, int unitsPerEm)
     m_glyphWidths[k] = (ReadUShort() * 1000) / unitsPerEm;
     ReadUShort();
   }
+  ReleaseTable();
   return true;
 }
 
@@ -1317,7 +1568,7 @@ wxPdfFontParserTrueType::ReadFormat4()
       r->m_width = GetGlyphWidth(r->m_glyph);
       int idx = m_fontSpecific ? ((j & 0xff00) == 0xf000 ? j & 0xff : j) : j;
       (*h)[idx] = r;
-//	  wxLogMessage(wxT("C %ld G %ld"), idx, glyph);
+//      wxLogMessage(wxT("C %ld G %ld"), idx, glyph);
     }
   }
 
@@ -1383,6 +1634,7 @@ wxPdfFontParserTrueType::ReadKerning(int unitsPerEm)
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
+    LockTable(wxT("kern"));
     m_kp = new wxPdfKernPairMap();
     wxPdfKernPairMap::iterator kp;
     wxPdfKernWidthMap* kwMap = NULL;
@@ -1429,6 +1681,7 @@ wxPdfFontParserTrueType::ReadKerning(int unitsPerEm)
         }
       }
     }
+    ReleaseTable();
   }
 }
     
@@ -1467,6 +1720,7 @@ wxPdfFontParserTrueType::GetNames(int id, bool namesOnly)
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
+    LockTable(wxT("name"));
     m_inFont->SeekI(tableLocation->m_offset+2);
     int numRecords = ReadUShort();
     int startOfStorage = ReadUShort();
@@ -1502,6 +1756,7 @@ wxPdfFontParserTrueType::GetNames(int id, bool namesOnly)
         m_inFont->SeekI(pos);
       }
     }
+    ReleaseTable();
   }
   else
   {
@@ -1520,6 +1775,7 @@ wxPdfFontParserTrueType::GetEnglishName(int id)
   if (entry != m_tableDirectory->end())
   {
     tableLocation = entry->second;
+    LockTable(wxT("name"));
     m_inFont->SeekI(tableLocation->m_offset+2);
     int numRecords = ReadUShort();
     int startOfStorage = ReadUShort();
@@ -1561,6 +1817,7 @@ wxPdfFontParserTrueType::GetEnglishName(int id)
         m_inFont->SeekI(pos);
       }
     }
+    ReleaseTable();
   }
   else
   {
