@@ -178,7 +178,7 @@ wxPdfImage::Parse()
 
   if (m_imageStream)
   {
-    if ((m_type.StartsWith(wxT("image/")) && m_type.EndsWith(wxT("png"))) ||
+    if ((m_type.StartsWith(wxT("image/")) && m_type.EndsWith(wxT("png"))) || 
         m_type == wxT("png"))
     {
       isValid = ParsePNG(m_imageStream);
@@ -270,9 +270,9 @@ wxPdfImage::ParsePNG(wxInputStream* imageStream)
   }
   else
   {
-    // Alpha channel not supported
+    // Unknown colour type
     wxLogDebug(wxString(wxT("wxPdfImage::ParsePNG: ")) +
-               wxString::Format(_("Alpha channel not supported: '%s'."), m_name.c_str()));
+               wxString::Format(_("Unknown colour type: '%s'."), m_name.c_str()));
     return false;
   }
   
@@ -300,7 +300,7 @@ wxPdfImage::ParsePNG(wxInputStream* imageStream)
   }
 
   imageStream->Read(buffer,4);
-  m_parms = wxString::Format(wxT("/DecodeParms <</Predictor 15 /Colors %d /BitsPerComponent %d /Columns %d>>"), (ct==2 ? 3 : 1), bpc, w);
+  m_parms = wxString::Format(wxT("/DecodeParms <</Predictor 15 /Colors %d /BitsPerComponent %d /Columns %d>>"), (ct==2 ? 3 : 1), (int) bpc, w);
 
   // Scan chunks looking for palette, transparency and image data
   m_palSize  = 0;
@@ -1021,11 +1021,11 @@ wxPdfImage::ParseWMF(wxInputStream* imageStream)
 
             if (k < numpoints)
             {
-              data += wxString::Format(wxT("%d %d l\n"), px, py);
+              data += wxString::Format(wxT("%d %d l\n"), (int) px, (int) py);
             }
             else
             {
-              data += wxString::Format(wxT("%d %d m\n"), px, py);
+              data += wxString::Format(wxT("%d %d m\n"), (int) px, (int) py);
             }
           }
 
@@ -1090,11 +1090,11 @@ wxPdfImage::ParseWMF(wxInputStream* imageStream)
 
               if (k == numpoints)
               {
-                data += wxString::Format(wxT("%d %d m\n"), px, py);
+                data += wxString::Format(wxT("%d %d m\n"), (int) px, (int) py);
               }
               else
               {
-                data += wxString::Format(wxT("%d %d m\n"), px, py);
+                data += wxString::Format(wxT("%d %d m\n"), (int) px, (int) py);
               }
             }
 
