@@ -43,10 +43,94 @@
 
 #include "pdfcorefontdata.inc"
 
+class wxPdfGraphicState
+{
+public:
+  wxString          m_fontFamily;
+  int               m_fontStyle;
+  double            m_fontSizePt;
+  wxPdfFontDetails* m_fontDetails;
+
+  wxPdfColour       m_drawColour;
+  wxPdfColour       m_fillColour;
+  wxPdfColour       m_textColour;
+  bool              m_colourFlag;
+
+  double            m_lineWidth;
+  wxPdfLineStyle    m_lineStyle;
+  int               m_extGState;
+};
+
+void
+wxPdfDocument::SaveGraphicState()
+{
+  wxPdfGraphicState* state = new wxPdfGraphicState();
+  // Save font state
+  state->m_fontFamily = m_fontFamily;
+  state->m_fontStyle = m_fontStyle;
+  state->m_fontSizePt = m_fontSizePt;
+  state->m_fontDetails = m_currentFont;
+  // Save colour state
+  state->m_drawColour = m_drawColour;
+  state->m_fillColour = m_fillColour;
+  state->m_textColour = m_textColour;
+  state->m_colourFlag = m_colourFlag;
+  // Save graphics state
+  state->m_lineWidth = m_lineWidth;
+  state->m_lineStyle = m_lineStyle;
+  state->m_extGState = m_currentExtGState;
+  m_graphicStates.Add(state);
+}
+
+void
+wxPdfDocument::RestoreGraphicState()
+{
+  wxPdfGraphicState* state = NULL;
+  if (!m_graphicStates.IsEmpty())
+  {
+    state = (wxPdfGraphicState*) m_graphicStates.Last();
+    m_graphicStates.RemoveAt(m_graphicStates.GetCount()-1);
+  }
+  if (state != NULL)
+  {
+    // Restore font state
+    m_fontFamily = state->m_fontFamily;
+    m_fontStyle = state->m_fontStyle;
+    m_fontSizePt = state->m_fontSizePt;
+    m_fontSize    = m_fontSizePt / m_k;
+    m_currentFont = state->m_fontDetails;
+    // Restore colour state
+    m_drawColour = state->m_drawColour;
+    m_fillColour = state->m_fillColour;
+    m_textColour = state->m_textColour;
+    m_colourFlag = state->m_colourFlag;
+    // Restore graphics state
+    m_lineWidth = state->m_lineWidth;
+    m_lineStyle = state->m_lineStyle;
+    m_currentExtGState = state->m_extGState;
+    delete state;
+  }
+}
+
+void
+wxPdfDocument::ClearGraphicState()
+{
+  wxPdfGraphicState* state = NULL;
+  size_t n = m_graphicStates.GetCount();
+  size_t j;
+  for (j = 0; j < n; ++j)
+  {
+    state = (wxPdfGraphicState*) m_graphicStates[j];
+    delete state;
+  }
+  m_graphicStates.Clear();
+}
+
 // ----------------------------------------------------------------------------
 // wxPdfDocument: class representing a PDF document
 // ----------------------------------------------------------------------------
 
+
 bool
 wxPdfDocument::SelectFont(const wxString& family, const wxString& style, double size, bool setFont)
 {
@@ -78,7 +162,7 @@ wxPdfDocument::SelectFont(const wxString& family, const wxString& style, double
 bool
 wxPdfDocument::SelectFont(const wxString& family, int style, double size, bool setFont)
 {
-  wxString fontFamily = (!family.IsEmpty()) ? family : m_currentFont->GetFontFamily();
+  wxString fontFamily = (!family.IsEmpty()) ? family : (m_currentFont != NULL) ? m_currentFont->GetFontFamily() : wxString();
   bool selected = false;
   wxPdfFont regFont = wxPdfFontManager::GetFontManager()->GetFont(fontFamily, style);
   if (regFont.IsValid())
@@ -272,6 +356,29 @@ wxPdfDocument::ForceCurrentFont()
   }
 }
 
+wxString
+wxPdfDocument::ApplyVisualOrdering(const wxString& txt)
+{
+  wxString s;
+  if (m_currentFont != NULL)
+  {
+    wxPdfFontExtended font = m_currentFont->GetFont();
+    if (font.HasVoltData())
+    {
+      s = font.ApplyVoltData(txt);
+    }
+    else
+    {
+      s = txt;
+    }
+  }
+  else
+  {
+    s = txt;
+  }
+  return s;
+}
+
 void
 wxPdfDocument::EndDoc()
 {
@@ -311,7 +418,7 @@ wxPdfDocument::EndDoc()
   Out("endobj");
   
   // Cross-Reference
-  int o = m_buffer.TellO();
+  int o = m_buffer->TellO();
   Out("xref");
   OutAscii(wxString(wxT("0 ")) + wxString::Format(wxT("%d"),(m_n+1)));
   Out("0000000000 65535 f ");
@@ -338,9 +445,6 @@ wxPdfDocument::BeginPage(int orientation, wxSize pageSize)
   m_page++;
   (*m_pages)[m_page] = new wxMemoryOutputStream();
   m_state = 2;
-  m_x = m_lMargin;
-  m_y = (m_yAxisOriginTop) ? m_tMargin : m_h - m_tMargin;
-  m_fontFamily = wxT("");
 
   // Page orientation
   if (orientation < 0)
@@ -349,30 +453,37 @@ wxPdfDocument::BeginPage(int orientation, wxSize pageSize)
   }
   if (orientation != m_defOrientation || pageSize != m_defPageSize)
   {
-    (*m_orientationChanges)[m_page] = true;
-    (*m_pageSizes)[m_page] = pageSize;
+    (*m_orientationChanges)[m_page] = orientation != m_defOrientation;
+    if (orientation == wxPORTRAIT)
+    {
+      (*m_pageSizes)[m_page] = pageSize;
+    }
+    else
+    {
+      (*m_pageSizes)[m_page] = wxSize(pageSize.GetHeight(), pageSize.GetWidth());
+    }
   }
   if (orientation != m_curOrientation || pageSize != m_curPageSize)
   {
-    m_fwPt = pageSize.GetWidth() / 254. * 72.;
-    m_fhPt = pageSize.GetHeight() / 254. * 72.;
-    m_fw = m_fwPt / m_k;
-    m_fh = m_fhPt / m_k;
+    double fwPt = pageSize.GetWidth() / 254. * 72.;
+    double fhPt = pageSize.GetHeight() / 254. * 72.;
+    double fw = fwPt / m_k;
+    double fh = fhPt / m_k;
 
     // Change orientation
     if (orientation == wxPORTRAIT)
     {
-      m_wPt = m_fwPt;
-      m_hPt = m_fhPt;
-      m_w = m_fw;
-      m_h = m_fh;
+      m_wPt = fwPt;
+      m_hPt = fhPt;
+      m_w = fw;
+      m_h = fh;
     }
     else
     {
-      m_wPt = m_fhPt;
-      m_hPt = m_fwPt;
-      m_w = m_fh;
-      m_h = m_fw;
+      m_wPt = fhPt;
+      m_hPt = fwPt;
+      m_w = fh;
+      m_h = fw;
     }
     m_pageBreakTrigger = (m_yAxisOriginTop) ? m_h - m_bMargin : m_bMargin;
     m_curOrientation = orientation;
@@ -382,6 +493,9 @@ wxPdfDocument::BeginPage(int orientation, wxSize pageSize)
   {
     Transform(1.0, 0.0, 0.0, -1.0, 0.0, m_h*m_k);
   }
+  m_x = m_lMargin;
+  m_y = (m_yAxisOriginTop) ? m_tMargin : m_h - m_tMargin;
+  m_fontFamily = wxT("");
 }
 
 void
@@ -393,6 +507,7 @@ wxPdfDocument::EndPage()
     StopTransform();
   }
   m_state = 1;
+  ClearGraphicState();
 }
 
 void
@@ -432,7 +547,7 @@ wxPdfDocument::NewObj(int objId)
 {
   // Begin a new object
   int id = (objId > 0) ? objId : GetNewObjId();
-  (*m_offsets)[id-1] = m_buffer.TellO();
+  (*m_offsets)[id-1] = m_buffer->TellO();
   OutAscii(wxString::Format(wxT("%d"),id) + wxString(wxT(" 0 obj")));
 }
 
@@ -655,9 +770,9 @@ wxPdfDocument::ReplaceNbPagesAlias()
 
 #if wxUSE_UNICODE
   wxMBConvUTF16BE conv;
-  size_t lenUni = conv.WC2MB(NULL, m_aliasNbPages, 0);
+  size_t lenUni = conv.FromWChar(NULL, 0, m_aliasNbPages.wc_str(), lenAsc);
   char* nbUni = new char[lenUni+3];
-  lenUni = conv.WC2MB(nbUni, m_aliasNbPages, lenUni+3);
+  lenUni = conv.FromWChar(nbUni, lenUni+3, m_aliasNbPages.wc_str(), lenAsc);
   size_t* fUni = makeFail(nbUni,lenUni);
 #endif
 
@@ -666,9 +781,9 @@ wxPdfDocument::ReplaceNbPagesAlias()
 #if wxUSE_UNICODE
   wxCharBuffer wpg(pg.ToAscii());
   const char* pgAsc = (const char*) wpg;
-  size_t lenPgUni = conv.WC2MB(NULL, pg, 0);
+  size_t lenPgUni = conv.FromWChar(NULL, 0, pg.wc_str(), lenPgAsc);
   char* pgUni = new char[lenPgUni+3];
-  lenPgUni = conv.WC2MB(pgUni, pg, lenPgUni+3);
+  lenPgUni = conv.FromWChar(pgUni, lenPgUni+3, pg.wc_str(), lenPgAsc);
 #else
   const char* pgAsc = pg.c_str();
 #endif
@@ -811,9 +926,12 @@ wxPdfDocument::PutPages()
     wxPdfBoolHashMap::iterator oChange = (*m_orientationChanges).find(n);
     if (oChange != (*m_orientationChanges).end())
     {
+      wxSize pageSize = (*m_pageSizes)[n];
+      double pageWidth = pageSize.GetWidth() / 254. * 72.;
+      double pageHeight = pageSize.GetHeight() / 254. * 72.;
       OutAscii(wxString(wxT("/MediaBox [0 0 ")) +
-               wxPdfUtility::Double2String(hPt,2) + wxString(wxT(" ")) +
-               wxPdfUtility::Double2String(wPt,2) + wxString(wxT("]")));
+               wxPdfUtility::Double2String(pageWidth,3) + wxString(wxT(" ")) +
+               wxPdfUtility::Double2String(pageHeight,3) + wxString(wxT("]")));
     }
 
     Out("/Resources 2 0 R");
@@ -849,7 +967,12 @@ wxPdfDocument::PutPages()
           double y = link->GetPosition()*m_k; 
           if (m_yAxisOriginTop)
           {
-            double h = (oChange != (*m_orientationChanges).end()) ? wPt : hPt;
+            double h = hPt;
+            if (oChange != (*m_orientationChanges).end())
+            {
+              wxSize pageSize = (*m_pageSizes)[link->GetPage()];
+              h = pageSize.GetHeight() / 254. * 72.;
+            }
             y = h - y;
           }
           OutAscii(wxString::Format(wxT("/Dest [%d 0 R /XYZ 0 "),m_firstPageId+2*(link->GetPage()-1)) +
@@ -891,7 +1014,11 @@ wxPdfDocument::PutPages()
       }
     }
     Out("]");
-
+    // TODO: not sure whether writing the group dictionary is necessary
+    if (m_PDFVersion > wxT("1.3"))
+    {
+      Out("/Group <</Type /Group /S /Transparency /CS /DeviceRGB>>");
+    }
     OutAscii(wxString::Format(wxT("/Contents %d 0 R>>"), m_n+1));
     Out("endobj");
     
@@ -911,7 +1038,7 @@ wxPdfDocument::PutPages()
 
     NewObj();
     OutAscii(wxString(wxT("<<")) + filter + wxString(wxT("/Length ")) + 
-             wxString::Format(wxT("%ld"), CalculateStreamLength(p->TellO())) + wxString(wxT(">>")));
+             wxString::Format(wxT("%lu"), (unsigned long) CalculateStreamLength(p->TellO())) + wxString(wxT(">>")));
     PutStream(*p);
     Out("endobj");
     if (m_compress)
@@ -920,7 +1047,7 @@ wxPdfDocument::PutPages()
     }
   }
   // Pages root
-  (*m_offsets)[0] = m_buffer.TellO();
+  (*m_offsets)[0] = m_buffer->TellO();
   Out("1 0 obj");
   Out("<</Type /Pages");
   wxString kids = wxT("/Kids [");
@@ -932,8 +1059,8 @@ wxPdfDocument::PutPages()
   OutAscii(kids + wxString(wxT("]")));
   OutAscii(wxString(wxT("/Count ")) + wxString::Format(wxT("%d"),nb));
   OutAscii(wxString(wxT("/MediaBox [0 0 ")) +
-           wxPdfUtility::Double2String(wPt,2) + wxString(wxT(" ")) +
-           wxPdfUtility::Double2String(hPt,2) + wxString(wxT("]")));
+           wxPdfUtility::Double2String(wPt,3) + wxString(wxT(" ")) +
+           wxPdfUtility::Double2String(hPt,3) + wxString(wxT("]")));
   Out(">>");
   Out("endobj");
 }
@@ -1079,7 +1206,7 @@ wxPdfDocument::PutShaders()
         Out("/Decode[0 1 0 1 0 1 0 1 0 1]");
         Out("/BitsPerFlag 8");
         wxMemoryOutputStream* p = grad->GetBuffer();
-        OutAscii(wxString::Format(wxT("/Length %ld"), CalculateStreamLength(p->TellO())));
+        OutAscii(wxString::Format(wxT("/Length %lu"), (unsigned long) CalculateStreamLength(p->TellO())));
         Out(">>");
         PutStream(*p);
         Out("endobj");
@@ -1132,7 +1259,7 @@ wxPdfDocument::PutFonts()
       size_t fontSize1 = font->WriteFontData(p);
   
       size_t fontLen = CalculateStreamLength(p->TellO());
-      OutAscii(wxString::Format(wxT("<</Length %d"), fontLen));
+      OutAscii(wxString::Format(wxT("<</Length %lu"), (unsigned long) fontLen));
       if (compressed)
       {
         Out("/Filter /FlateDecode");
@@ -1143,10 +1270,10 @@ wxPdfDocument::PutFonts()
       }
       else
       {
-        OutAscii(wxString::Format(wxT("/Length1 %d"), fontSize1));
+        OutAscii(wxString::Format(wxT("/Length1 %lu"), (unsigned long) fontSize1));
         if (extFont.HasSize2())
         {
-          OutAscii(wxString::Format(wxT("/Length2 %d /Length3 0"), extFont.GetSize2()));
+          OutAscii(wxString::Format(wxT("/Length2 %lu /Length3 0"), (unsigned long) extFont.GetSize2()));
         }
       }
       Out(">>");
@@ -1249,7 +1376,7 @@ wxPdfDocument::PutFonts()
         wxMemoryOutputStream* p = new wxMemoryOutputStream();
         /* size_t mapSize = */ font->WriteUnicodeMap(p);
         size_t mapLen = CalculateStreamLength(p->TellO());
-        OutAscii(wxString::Format(wxT("<</Length %ld"), mapLen));
+        OutAscii(wxString::Format(wxT("<</Length %lu"), (unsigned long) mapLen));
         Out("/Filter /FlateDecode");
         Out(">>");
         PutStream(*p);
@@ -1354,7 +1481,7 @@ wxPdfDocument::PutFonts()
       wxMemoryOutputStream* p = new wxMemoryOutputStream();
       /* size_t mapSize = */ font->WriteUnicodeMap(p);
       size_t mapLen = CalculateStreamLength(p->TellO());
-      OutAscii(wxString::Format(wxT("<</Length %ld"), mapLen));
+      OutAscii(wxString::Format(wxT("<</Length %lu"), (unsigned long) mapLen));
       if (compressed)
       {
         // Decompresses data encoded using the public-domain zlib/deflate compression
@@ -1465,7 +1592,7 @@ wxPdfDocument::PutImages()
           p->Write(currentImage->GetData(),currentImage->GetDataSize());
         }
         dataLen = CalculateStreamLength(p->TellO());
-        OutAscii(wxString::Format(wxT("/Length %lu>>"), (unsigned int) dataLen));
+        OutAscii(wxString::Format(wxT("/Length %lu>>"), (unsigned long) dataLen));
         PutStream(*p);
 
         Out("endobj");
@@ -1535,7 +1662,7 @@ wxPdfDocument::PutImages()
           OutAscii(wxString(wxT("/Mask [")) + trns + wxString(wxT("]")));
         }
 
-        OutAscii(wxString::Format(wxT("/Length %d>>"), CalculateStreamLength(currentImage->GetDataSize())));
+        OutAscii(wxString::Format(wxT("/Length %lu>>"), (unsigned long) CalculateStreamLength(currentImage->GetDataSize())));
 
         wxMemoryOutputStream* p = new wxMemoryOutputStream();
         p->Write(currentImage->GetData(),currentImage->GetDataSize());
@@ -1647,7 +1774,7 @@ wxPdfDocument::PutTemplates()
       p = &(currentTemplate->m_buffer);
     }
 
-    OutAscii(wxString::Format(wxT("/Length %ld >>"), CalculateStreamLength(p->TellO())));
+    OutAscii(wxString::Format(wxT("/Length %lu >>"), (unsigned long) CalculateStreamLength(p->TellO())));
     PutStream(*p);
     Out("endobj");
     if (m_compress)
@@ -1962,7 +2089,7 @@ wxPdfDocument::PutPatterns()
                      wxString::Format(wxT("/I%d Do Q"), image->GetIndex());
     wxMemoryOutputStream* p = new wxMemoryOutputStream;
     p->Write(sdata.ToAscii(), sdata.Length());
-    OutAscii(wxString(wxT("/Length ")) + wxString::Format(wxT("%ld"), CalculateStreamLength(p->TellO())));
+    OutAscii(wxString(wxT("/Length ")) + wxString::Format(wxT("%lu"), (unsigned long) CalculateStreamLength(p->TellO())));
     Out(">>");
     PutStream(*p);
     delete p;
@@ -2008,7 +2135,7 @@ wxPdfDocument::PutResources()
   PutLayers();
 
   // Resource dictionary
-  (*m_offsets)[2-1] = m_buffer.TellO();
+  (*m_offsets)[2-1] = m_buffer->TellO();
   Out("2 0 obj");
   Out("<<");
   PutResourceDict();
@@ -2078,9 +2205,9 @@ wxPdfDocument::ShowGlyph(wxUint32 glyph)
   {
 #if wxUSE_UNICODE
     wxMBConv* conv = m_currentFont->GetEncodingConv();
-    size_t len = conv->WC2MB(NULL, t, 0);
+    size_t len = conv->FromWChar(NULL, 0, t.wc_str(), 1);
     char* mbstr = new char[len+3];
-    len = conv->WC2MB(mbstr, t, len+3);
+    len = conv->FromWChar(mbstr, len+3, t.wc_str(), 1);
 #else
     size_t len = t.Length();;
     char* mbstr = new char[len+1];
@@ -2139,10 +2266,15 @@ wxPdfDocument::TextEscape(const wxString& s, bool newline)
   {
     wxString t = m_currentFont->ConvertCID2GID(s);
 #if wxUSE_UNICODE
+    size_t slen = s.length();
     wxMBConv* conv = m_currentFont->GetEncodingConv();
-    size_t len = conv->WC2MB(NULL, t, 0);
+    size_t len = conv->FromWChar(NULL, 0, t.wc_str(), slen);
     char* mbstr = new char[len+3];
-    len = conv->WC2MB(mbstr, t, len+3);
+    len = conv->FromWChar(mbstr, len+3, t.wc_str(), slen);
+    if (len == wxCONV_FAILED)
+    {
+      len = strlen(mbstr);
+    }
 #else
     size_t len = t.Length();;
     char* mbstr = new char[len+1];
@@ -2221,8 +2353,8 @@ wxPdfDocument::PutStream(wxMemoryOutputStream& s)
       }
       else
       {
-        m_buffer.Write(tmp);
-        m_buffer.Write("\n",1);
+        m_buffer->Write(tmp);
+        m_buffer->Write("\n",1);
       }
     }
   }
@@ -2269,13 +2401,14 @@ wxPdfDocument::OutTextstring(const wxString& s, bool newline)
   // Format a text string
   size_t ofs = CalculateStreamOffset();
 #if wxUSE_UNICODE
+  size_t slen = s.length();
   wxMBConvUTF16BE conv;
-  size_t len = conv.WC2MB(NULL, s, 0);
+  size_t len = conv.FromWChar(NULL, 0, s.wc_str(), slen);
   size_t lenbuf = CalculateStreamLength(len+2);
   char* mbstr = new char[lenbuf+3];
   mbstr[ofs+0] = '\xfe';
   mbstr[ofs+1] = '\xff';
-  len = 2 + conv.WC2MB(&mbstr[ofs+2], s, len+3);
+  len = 2 + conv.FromWChar(&mbstr[ofs+2], len+3, s.wc_str(), slen);
 #else
   size_t len = s.Length();;
   size_t lenbuf = CalculateStreamLength(len);
@@ -2422,10 +2555,10 @@ wxPdfDocument::Out(const char* s, size_t len, bool newline)
   }
   else
   {
-    m_buffer.Write(s,len);
+    m_buffer->Write(s,len);
     if (newline)
     {
-      m_buffer.Write("\n",1);
+      m_buffer->Write("\n",1);
     }
   }
 }
@@ -2471,7 +2604,7 @@ wxPdfDocument::OutLineRelative(double dx, double dy)
 void
 wxPdfDocument::OutCurve(double x1, double y1, double x2, double y2, double x3, double y3)
 {
-  // Draws a Bzier curve from last draw point
+  // Draws a Bezier curve from last draw point
   OutAscii(wxPdfUtility::Double2String(x1 * m_k,2) + wxString(wxT(" ")) +
            wxPdfUtility::Double2String(y1 * m_k,2) + wxString(wxT(" ")) +
            wxPdfUtility::Double2String(x2 * m_k,2) + wxString(wxT(" ")) +
@@ -2588,7 +2721,7 @@ wxPdfDocument::SetFillGradient(double x, double y, double w, double h, int gradi
     // paint the gradient
     OutAscii(wxString::Format(wxT("/Sh%d sh"), gradient));
     // restore previous Graphic State
-    Out("Q");
+    UnsetClipping();
   }
   else
   {
