@@ -1079,8 +1079,9 @@ wxPdfDocument::PrepareXmlCell(wxXmlNode* node, wxPdfCellContext& context)
         wxString s;
         if (name == wxT("msg"))
         {
-          s = wxGetTranslation(GetNodeContent(child));
-        }
+          s = GetNodeContent(child);
+          if (m_translate) s = wxGetTranslation(s);
+         }
         else
         {
           s = child->GetContent();
@@ -1185,7 +1186,8 @@ wxPdfDocument::WriteXml(const wxString& xmlString)
   {
     SetLineHeight(GetFontSize()*1.25 / GetScaleFactor());
   }
-  wxStringInputStream xmlStream(wxT("<xml>")+xmlString+wxT("</xml>"));
+  wxString xmlStringWithXmlRoot(wxT("<xml>")+xmlString+wxT("</xml>"));          
+  wxStringInputStream xmlStream(xmlStringWithXmlRoot);
   wxXmlDocument xmlDocument;
   bool loaded = xmlDocument.Load(xmlStream);
   if (loaded)
@@ -1216,6 +1218,22 @@ wxPdfDocument::WriteXml(const wxString& xmlString)
   }
 }
 
+void
+wxPdfDocument::WriteXml(wxXmlNode* node)
+{
+  if (GetLineHeight() == 0)
+  {
+    SetLineHeight(GetFontSize()*1.25 / GetScaleFactor());
+  }
+  double maxWidth = GetPageWidth() - GetRightMargin() - GetX();
+  wxPdfCellContext context(maxWidth, wxPDF_ALIGN_LEFT);
+  double saveX = GetX();
+  double saveY = GetY();
+  PrepareXmlCell(node, context);
+  SetXY(saveX, saveY);
+  WriteXmlCell(node, context);
+}
+
 void
 wxPdfDocument::WriteXmlCell(wxXmlNode* node, wxPdfCellContext& context)
 {
@@ -1330,7 +1348,7 @@ wxPdfDocument::WriteXmlCell(wxXmlNode* node, wxPdfCellContext& context)
           }
           else
           {
-            SetFont(wxT("arial"), wxT(""), saveSize);
+            SetFont(wxT("Helvetica"), wxT(""), saveSize);
           }
           SetLeftMargin(leftMargin);
           SetXY(leftMargin, GetY());
@@ -1425,7 +1443,7 @@ wxPdfDocument::WriteXmlCell(wxXmlNode* node, wxPdfCellContext& context)
           else
           {
             wxChar itemChar;
-            SetFont(wxT("arial"), wxT(""), saveSize);
+            SetFont(wxT("Helvetica"), wxT(""), saveSize);
             wxString item;
             switch (listType)
             {
@@ -1729,7 +1747,8 @@ wxPdfDocument::WriteXmlCell(wxXmlNode* node, wxPdfCellContext& context)
         wxString s;
         if (name == wxT("msg"))
         {
-          s = wxGetTranslation(GetNodeContent(child));
+          s = GetNodeContent(child);
+          if (m_translate) s = wxGetTranslation(s);
         }
         else
         {
